/*-----------------------------------------------------------------------
 * File: CMP_NODE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"
#include "ber_internal.h"

/*
 * return true iff two DER nodes are equivalent
 */
sint32 
DER_EqualNodes(DER_NODE_PTR A, DER_NODE_PTR B)
{
    uint32 length;
    sint16 i;

    if ((!A || !B) && (A != B)) return 0;
    if (A->Count != B->Count) return 0;
    length = BER_LengthOfTag(A->Tag);
    if (length != BER_LengthOfTag(B->Tag)) return 0;
    if (!cssm_memcmp(A->Tag, B->Tag, length)) return 0;
    if (A->Count != B->Count) return 0;
    for (i = 0; i < A->Count; i++) {
        if (A->Child[i].IsLeaf != B->Child[i].IsLeaf) return 0;
        if (A->Child[i].IsLeaf) {
            if (A->Child[i].X.Input.ContentLength != 
                B->Child[i].X.Input.ContentLength) 
                return 0;
            length = BER_LengthOfTag(A->Child[i].X.Input.Tag);
            if (length != BER_LengthOfTag(B->Child[i].X.Input.Tag)) return 0;
            if (!cssm_memcmp(A->Child[i].X.Input.Tag, B->Child[i].X.Input.Tag, length)) 
                return 0;
            if (!cssm_memcmp(A->Child[i].X.Input.Content, B->Child[i].X.Input.Content, 
                A->Child[i].X.Input.ContentLength))
                return 0;
        } else {
            if (!DER_EqualNodes(A->Child[i].X.Node, B->Child[i].X.Node)) return 0; 
        }
    }
    return 1;
}
