/*-----------------------------------------------------------------------
 * File: EXPAND.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* expand a single BER item; return updated "octets" if successful, 0 if not */
const uint8 *
BER_ExpandItem(                 /* return updated octets if successful, 0 if not */
    const uint8 *Octets,        /* input BER to parse */
    uint32 InputLength,         /* length of BER to parse (may not all be in memory) */
    BER_PARSED_ITEM_PTR Output) /* output is single structure */
{
    sint32 noctets;
    uint32 length;
    const uint8  *end;

    Output->Tag = 0;
    Output->ContentLength = 0;
    Output->Content = 0;
    if (!Octets) return 0;
    end = Octets + InputLength;
    Output->Tag = Octets;
    /* bypass tag */
    if ((*Octets++ & 31) == 31) { /* long tag */
        while ((*Octets++ & 128) != 0) if (Octets >= end) return 0;
    }
    if (Octets >= end) return 0;
    /* compute length of length and contents */
    if ((*Octets & 128) == 0) { 
        length = *Octets++;  /* short definite */
        Output->Content = Octets;
        Octets += length;
    } else {
        noctets = *Octets++ & 127;
        if (Octets >= end) return 0;
        if (noctets > 4)   return 0;
        length = 0;
        if (noctets > 0) {   /* long definite */
            while (noctets-- > 0) {
                length = 256 * length + *Octets++;
                if (Octets >= end) return 0;
            }
            Output->Content = Octets;
            Octets += length;
        } else {         /* constructed indefinite length */
            Output->Content = Octets;
            while (Octets[0] != 0 || Octets[1] != 0) { 
                noctets = BER_SizeofObject(Octets);
                if (length + noctets < length) 
                    return 0;
                length += noctets;
                Octets += noctets;
                if (Octets >= end) return 0;
            }
            Octets += 2;
        }
    }
    if (Octets > end) return 0;
    Output->ContentLength = length;
    return Octets;
}
