/*-----------------------------------------------------------------------
 * File: FLATTREE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// #define FIX_SORT_SET_BUG

#include "ber_der.h"

/* 
 * Copy parse tree to contiguous octet buffer--we assume output is 
 * sufficiently large.  It is also assumed that sizeofDERTree fixed 
 * up all the tags and default values. 
 */
uint32 
DER_CopyTree(
    DER_NODE_PTR Node, /* tree to linearize */
    uint8 *Octets)     /* output buffer */
{
    uint32 length, len;
    sint16 i;
    const uint8 *source;

    length = 0;
    /* output our tag */
    len = BER_OutputTag(Node->Tag, Octets);
    Octets += len;
    length += len;
    /* output our length */
    len = BER_OutputLength(Node->Length, Octets);
    Octets += len;
    length += len;
    /* now output our children as content */
    for (i = 0; i < Node->Count; i++) {
        if (Node->Child[i].IsLeaf) {
            /* output tag */
            source = Node->Child[i].X.Input.Tag;
            if (!source) continue;  /* optional */
            len = BER_OutputTag(source, Octets);
#ifdef FIX_SORT_SET_BUG
            Node->Child[i].X.Input.Tag = Octets;    /* for sort */
#endif
            Octets += len;
            length += len;
            /* output length */
            len = BER_OutputLength(Node->Child[i].X.Input.ContentLength, Octets);
            Octets += len;
            length += len;
            /* output content */
            length += Node->Child[i].X.Input.ContentLength;
            source = Node->Child[i].X.Input.Content;
            for (len = Node->Child[i].X.Input.ContentLength; len > 0; len--)
                *Octets++ = *source++;
        } else {
            if (!Node->Child[i].X.Node) {
#ifdef FIX_SORT_SET_BUG
                Node->Child[i].X.Input.Tag = 0;
#endif
                continue;   /* optional */
            }
            len = DER_CopyTree(Node->Child[i].X.Node, Octets);
#ifdef FIX_SORT_SET_BUG
            Node->Child[i].X.Input.Tag = Octets;    /* for sort */
#endif
            Octets += len;
            length += len;
        }
    }
#ifdef FIX_SORT_SET_BUG
    /* set entries should already be sorted by tag and then length -- now sort by contents */
    if (Node->OriginalTag[0] == (BER_CONSTRUCTED | BER_SET)) {
        /* use a simple bubble sort */
        int j, test;
        unsigned int k, minlen, lenj;
        unsigned char *valuei, *valuej;
        unsigned char temp;
        for (i = 0; i < Node->Count; i++) {
            if (!Node->Child[i].X.Input.Tag) 
                continue;  /* optional field is missing */
            len = BER_SizeofObject(Node->Child[i].X.Input.Tag);
            for (j = i + 1; j < Node->Count; j++) {
                valuei = (unsigned char *) Node->Child[i].X.Input.Tag;
                valuej = (unsigned char *) Node->Child[j].X.Input.Tag;
                if (!valuej) 
                    continue;  /* optional field is missing */
                lenj = BER_SizeofObject(Node->Child[j].X.Input.Tag);
                minlen = len < lenj ? len : lenj;
                test = 0;
                for (k = 0; k < minlen; k++) {
                    test = *valuei++ - *valuej++;
                    if (test != 0) 
                        break;
                }
                if (test > 0) {  /* exchange */
                    if (len != lenj)
                        return length; /* this should have been fixed by sizeofDERTree */
                    valuei = (unsigned char *) Node->Child[i].X.Input.Tag;
                    valuej = (unsigned char *) Node->Child[j].X.Input.Tag;
                    for (k = 0; k < len; k++) {
                        temp = *valuei;
                        *valuei++ = *valuej;
                        *valuej++ = temp;
                    }
                }
            }
        }
    }
#endif
    return length;
}
