/*-----------------------------------------------------------------------
 * File: GENERATX.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* 
 * Generate a DER-encoded, flat octet representation of a given parse tree.
 * The input parse tree is *not* modified (a temporary copy is modified).
 * A buffer containing the DER is returned if successful, NULL if unsuccessful.
 * The user is responsible for deallocating the buffer using the memory free function.
 */

uint8 *
DER_Generate_copyctxtag(
    BER_HANDLE AppHandle,
    BER_MEMORY_FUNCS_PTR MemoryFunctions,
    const DER_NODE *InputTree)
{
    DER_NODE *tree = 0;
    uint8 *result, *buffer = 0;
    uint32 DERLength, copyLength;
    for (result = 0; result == 0; result = buffer) {
        tree = DER_DuplicateParseTree(AppHandle, MemoryFunctions, InputTree);
        if (!tree)
            break;
        // encode (modifies tree)
        DERLength = DER_SizeofTree_copyctxtag(tree);
        if (DERLength <= 0) 
            break;
        buffer = BERDER_MALLOC_WRAPPER(MemoryFunctions->malloc_func, AppHandle, DERLength);
        if (!buffer)
            break;
        copyLength = DER_CopyTree(tree, buffer);
        if (DERLength != copyLength) 
            break;
    }
    // memory cleanup 
    if (tree) DER_RecycleTree(AppHandle, MemoryFunctions, tree);
    if (!result && buffer) 
        BERDER_FREE_WRAPPER(*MemoryFunctions->free_func, AppHandle, buffer);
    return result;
}

