/*-----------------------------------------------------------------------
 * File: NCODEBIT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"
#include "ber_internal.h"

/*
 * Function to DER-encode a bit string. The function result is the number of *BYTES* used
 * to DER-encode the bit string. If the output buffer is too small, -1 is returned and it
 * is not changed. Any odd bytes must be in the most significant bits of the trailing byte.
 */
sint32  
DER_EncodeBitString(const uint8 *Bytes,   /* pointer to input bit string */
    uint32 NumberOfBits,            /* length of bit string in *BITS* */
    uint8 *Octets,                  /* pointer to output buffer */
    uint32 SizeofOctets)            /* length of output buffer in BYTES */
{
    sint32 result;
    uint32 length, oddbits;
    length = (NumberOfBits + 7)/ 8 + 1;
    result = 1 + BER_LengthOfLength(length) + length;
    if (result > (sint32) SizeofOctets)
        return -1;
    *Octets++ = BER_BIT_STRING;
    Octets += BER_OutputLength(length, Octets);
    oddbits = NumberOfBits % 8;
    if (oddbits) {
        *Octets++ = (uint8) (8 - oddbits);
    } else *Octets++ = 0;
    cssm_memcpy(Octets, Bytes, length - 1);
    if (oddbits)     /* force padding bits to be zero */
        Octets[length-2] &= (0xff << (8 - oddbits));
    return result;
}
