/*-----------------------------------------------------------------------
 * File: OID2INTS.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* decode an definite length object identifier into an array of unsigned long integers */
sint32 
BER_OidDecode(          /* return number of integers found */
    const uint8  *Octets,         /* input BER OBJECT IDENTIFIER */
    uint32 *Numbers,        /* array of unsigned long for output */
    sint32 Dimension)       /* dimension of numbers array */
{
    BER_PARSED_ITEM oid;
    sint32 count;

    if (Dimension < 2 || !BER_ExpandItem(Octets, BER_SizeofObject(Octets), &oid) ||
        oid.Tag[0] != BER_OBJECT_IDENTIFIER || oid.ContentLength < 1)
        return 0;
    Octets = oid.Content;
    count = 0;
    /* first byte contains two multiplicatively packed numbers */
    oid.ContentLength--;
    Numbers[count++] = *Octets / 40;
    Numbers[count++] = *Octets++ % 40;
    while (oid.ContentLength > 0) {
        if (count >= Dimension) 
            return 0;
        Numbers[count] = 0;
        do {
            if (oid.ContentLength-- <= 0)
                return 0;
            Numbers[count] = Numbers[count] * 128 + (*Octets & 127);
        } while ((*Octets++ & 0x80) != 0);
        count++;
    }

    return count;
}
