/*-----------------------------------------------------------------------
 * File: TREESIZE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// #define FIX_SORT_SET_BUG

#include "ber_der.h"
#include "ber_internal.h"

/*
 * Add up sizes of parse tree nodes, so we can reserve a sufficiently 
 * large output buffer.  Eliminate instances of default values for 
 * sequence and set nodes (yes, a side effect).  While we're at it, 
 * sort tags of sets. 
 * return sizeof output buffer needed for tree
 */
uint32 
DER_SizeofTree(DER_NODE_PTR Node)
{
    uint32 length, lenTag;
    sint32 i, j;
    const uint8 *source;

    length = 0;
    if (!Node) return 0;
    for (i = 0; i < Node->Count; i++) {
        if (!Node->Child[i].IsLeaf) {
            if (Node->Type) {
              switch (Node->OriginalTag[0] & 31) {
              case BER_SET:
                /* omit child if content equal to default value for set type */
                if (Node->Type && Node->Type->Defaults) {
                    lenTag = BER_LengthOfTag(Node->Child[i].X.Input.Tag);
                    for (j = 0; j < Node->Type->NumberOfDefaults; j++) {    
                        if (lenTag == BER_LengthOfTag(Node->Type->ContextTags[j]) &&
                            !cssm_memcmp(Node->Child[i].X.Input.Tag,
                                Node->Type->ContextTags[j], 
                                lenTag) &&
                            Node->Child[i].X.Input.ContentLength == Node->Type->Defaults[j].ContentLength && 
                            !cssm_memcmp(Node->Child[i].X.Input.Content,
                                Node->Type->Defaults[j].Content, 
                                Node->Child[i].X.Input.ContentLength)) {
                            Node->Child[i].X.Input.Tag = 0;
                            break;
                        }
                    }
                    if (j < Node->Type->NumberOfDefaults)
                        continue;
                }
                break;
              default:
                /* omit child if content equal to default value for field */
                if (Node->Type->Defaults &&  
                    Node->Type->Defaults[i].Tag &&
                    Node->Child[i].X.Input.ContentLength == Node->Type->Defaults[i].ContentLength &&
                    !cssm_memcmp(Node->Child[i].X.Input.Content,
                        Node->Type->Defaults[i].Content, 
                        Node->Child[i].X.Input.ContentLength)) {
                    Node->Child[i].X.Input.Tag = 0;
                    continue;
                } else {    /* substitute context-sensitive tag if present */
                    if (Node->Type->ContextTags && Node->Type->ContextTags[i]) {
                        if (Node->Child[i].IsLeaf) {
                            if (Node->Child[i].X.Input.Tag)  /* optional */
                                Node->Child[i].X.Input.Tag = Node->Type->ContextTags[i];
                        } else {
                            if (Node->Child[i].X.Node)  /* optional */
                                Node->Child[i].X.Node->Tag = Node->Type->ContextTags[i];
                        }
                    }
                }
                break;
              }
            }
            length += DER_SizeofTree(Node->Child[i].X.Node);
        } else {    /* assume no complex default values */
            /* check for optional field absent */
            source = Node->Child[i].X.Input.Tag;
            if (!source) continue;  /* optional item not present */
            /* omit child if content equal to default value for field */
            if (Node->Type) {
                switch (Node->OriginalTag[0] & 31) {
                case BER_SET:
                    for (j = 0; j < Node->Type->NumberOfDefaults; j++) {
                        if (Node->Type->Defaults && Node->Type->Defaults[j].Tag &&
                            Node->Child[i].X.Input.ContentLength == Node->Type->Defaults[j].ContentLength &&
                            !cssm_memcmp(Node->Child[i].X.Input.Content,
                                Node->Type->Defaults[j].Content, 
                                Node->Child[i].X.Input.ContentLength)) {
                            Node->Child[i].X.Input.Tag = 0;
                            continue;
                        }
                    }
                    break;
                default:
                    if (Node->Type->Defaults && Node->Type->Defaults[i].Tag &&
                        Node->Child[i].X.Input.ContentLength == Node->Type->Defaults[i].ContentLength &&
                        !cssm_memcmp(Node->Child[i].X.Input.Content,
                            Node->Type->Defaults[i].Content, 
                            Node->Child[i].X.Input.ContentLength)) {
                        Node->Child[i].X.Input.Tag = 0;
                        continue;
                    /* substitute context-sensitive tag if sequence if present */
                    } else if (Node->Type && Node->Type->ContextTags && 
                        Node->Type->ContextTags[i]) {
                        Node->Child[i].X.Input.Tag = Node->Type->ContextTags[i];
                    }
                    break;
                }
            }
            source = Node->Child[i].X.Input.Tag;
            /* count length of tag */
            length += BER_LengthOfTag(source);
            /* count length of content length */
            length += BER_LengthOfLength(Node->Child[i].X.Input.ContentLength);
            /* count content */
            length += Node->Child[i].X.Input.ContentLength;
        }
    }
#ifdef FIX_SORT_SET_BUG
    /* sort set entries by tag and then length (defer sort of contents until CopyDERTree--not flat yet) */
    if (Node->OriginalTag[0] == (BER_CONSTRUCTED | BER_SET)) {
        /* use a simple bubble sort */
        int test;
        for (i = 0; i < Node->Count; i++) {
            if (!Node->Child[i].X.Input.Tag) 
                    continue;
            for (j = i + 1; j < Node->Count; j++) {
                if (!Node->Child[j].X.Input.Tag) 
                    continue;
                test = BER_TagCompare(Node->Child[i].X.Input.Tag, Node->Child[j].X.Input.Tag);
                if (test == 0) 
                    test = ((Node->Child[i].IsLeaf || !Node->Child[i].X.Node) ? Node->Child[i].X.Input.ContentLength : Node->Child[i].X.Node->Length) - 
                        ((Node->Child[j].IsLeaf || !Node->Child[j].X.Node) ? Node->Child[j].X.Input.ContentLength  : Node->Child[j].X.Node->Length);
                if (test > 0) {
                    struct der_node_child_struct temp;
                    temp = Node->Child[i];
                    Node->Child[i] = Node->Child[j];
                    Node->Child[j] = temp;
                }
            }
        }
    }
#endif
    /* now we have the length of the aggregated contents */
    Node->Length = length;  /* set length of contents only in node */
    length += BER_LengthOfLength(length);
    length += BER_LengthOfTag(Node->Tag);
    return length;  /* return total length */
}
