/*-----------------------------------------------------------------------
 * File: XPANDSET.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "ber_der.h"

/* 
 * expand a BER set one level; return count of items in set 
 * sucessfully parsed 
 */
sint32 
BER_ExpandSet(                  /* return count of items in set sucessfully parsed */
    const uint8  *Octets,                 /* input BER to parse */
    uint32 InputLength,             /* length of BER to parse (may not all be in memory) */
    sint32 MaxCount,                /* maximum number of items we can accept (card of output) */
    sint32 NumberOfTypes,           /* number of tags */
    const BER_PARSED_ITEM *Defaults,   /* default values */
    BER_PARSED_ITEM_PTR Output)     /* output: array of parsed items */
{
    uint32 remainingInput;
    const uint8  *currentPosition;
    sint32 i, j;

    remainingInput = InputLength;
    currentPosition = Octets;
    for (i = 0; i < MaxCount; i++) {
        currentPosition = BER_ExpandItem(currentPosition, 
            remainingInput, &Output[i]);
        if (!currentPosition) break;
        remainingInput = InputLength - (currentPosition - Octets);
    }
    if (Defaults) {
        for (j = 0; j < NumberOfTypes; j++) {
            /* check for missing default values */
            if (Defaults[j].Tag) Output[i++] = Defaults[j];
            if (i >= MaxCount) return 0;    
            /* output is too small return nothing */
        }
    }
    return i;
}
