/*-----------------------------------------------------------------------
 * File: AC_MAIN.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
 
/*
 * This file implements all of the global module functions
 */

#include <assert.h>

/* Windows & CSSM includes */
#include "cssm.h"       /* CSSM types, errors, and APIs */
#include "cssmspi.h"    /* CSSM service provider interfaces */
#include "cssmaci.h"    /* CSSM AuthCompute interfaces */
#include "maf_include.h"

/* Intel includes */

#include "guids.h"      /* guid definitions */
#include "mds_util_api.h"
#include "acm_aci.h"
#ifdef ISL_INCLUDED
#include "eislapi.h"    /* verification includes */
#endif

/* AC Includes */
#include "acm_aci.h"

#include "mempool.h"

PORT_TLS_INDEX TLS_MemoryPool;
MEMORY_POOL pool;
MEMORY_POOL_PTR poolptr = &pool;
extern MEMORY_POOL_FUNCS memory_pool_funcs;
extern const CSSM_API_MEMORY_FUNCS Addin_APIMemFuncs;

CSSM_GUID intel_acm_guid_3_0_0 = intel_acm_guid_3_0_0_init;

CSSM_RETURN 
Addin_callout_Initialize(void)
{
    
    CSSM_RETURN ret;
    ret = port_AllocateTlsIndex(&TLS_MemoryPool, NULL);
    if (CSSM_OK != ret)
    {
        return ret;
    }
    ret = (*memory_pool_funcs.init)((CSSM_MEMORY_FUNCS_PTR)&Addin_APIMemFuncs, NULL, poolptr);
    return ret;
}

CSSM_RETURN
Addin_callout_Terminate(void)
{
    
    (*memory_pool_funcs.recycle)(poolptr);
    return port_DeleteTlsIndex(TLS_MemoryPool);
}

CSSM_RETURN Addin_callout_ModuleLoad(
    MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
    CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
    const void* CssmNotifyCallbackCtx)
{
    /* Reference unused parameters to quiet the compiler */
    pLoadTracker;
    
    /* Trigger a callback into the CSSM to simulate an insert */
    if ( CssmNotifyCallback != NULL )
    {
        CssmNotifyCallback( &intel_acm_guid,
                            (void *)CssmNotifyCallbackCtx,
                            0,
                            CSSM_SERVICE_AC,
                            CSSM_NOTIFY_INSERT );
        return CSSM_OK;
    }

    return CSSMERR_CL_INTERNAL_ERROR;
}

CSSM_RETURN Addin_callout_ModuleUnload(
    MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker,
    CSSM_SPI_ModuleEventHandler CssmNotifyCallback,
    const void* CssmNotifyCallbackCtx)
{
    /* Reference unused parameters to quiet the compiler */
    pLoadTracker;
    CssmNotifyCallback;
    CssmNotifyCallbackCtx;

    return CSSM_OK;
}


CSSM_RETURN Addin_callout_ModuleAttach(
		MAF_MODULE_LOAD_TRACKER_PTR pLoadTracker, 
		MAF_MODULE_ATTACH_TRACKER_PTR pAttachTracker,
		CSSM_MODULE_FUNCS_PTR *FuncTbl )
{
    static CSSM_MODULE_FUNCS FunctionList;
    static CSSM_SPI_AC_FUNCS Functions;

    if ( !FuncTbl )
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }
    else if ( pAttachTracker->SubServiceType == CSSM_SERVICE_AC )
    {
        FunctionList.ServiceType = CSSM_SERVICE_AC;
        FunctionList.NumberOfServiceFuncs = 
                                      sizeof(CSSM_SPI_AC_FUNCS) /
                                          sizeof(CSSM_PROC_ADDR);
        FunctionList.ServiceFuncs = (CSSM_PROC_ADDR*)&Functions;

        Functions.AuthCompute = ACM_AuthCompute;
        Functions.PassThrough = ACM_PassThrough;

        *FuncTbl = &FunctionList;
    }
    else
    {
        return CSSMERR_CSSM_INVALID_SERVICE_MASK;
    }

    return CSSM_OK;
}
