/*-----------------------------------------------------------------------
 * File: CRTDESCR.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#define X509CL_MODULE

    /* Windows & CSSM includes */
#ifndef VMS
#pragma warning (disable:4201 4214 4115)
#include <stdio.h>
#pragma warning (default:4201 4214 4115)
#else
#include <stdio.h>
#endif
#include "cssm.h"
#include "cssmspi.h"
#include "guids.h"
#include "cssmport.h"
#include "cssmutil.h"

    /* Certificate-specific includes */
#include "x509defs.h"
#include "x_fndefs.h"

#include "oidsx509.h"
#include "oidsalg.h"
#include "x509hlpr.h"
#include "clutil.h"
#include "x_qtable.h"
#include "internal.h"

#include "maf_include.h"
#include "api_shim.h"


#include <cssmMemTrack.h>

#define X509_FORMAT_VERSION 2

/*-----------------------------------------------------------------------------
 * Name: CL_CertDescribeFormat
 *
 * Description:
 * This function returns a list of the field identifiers used to describe 
 * the X509 certificate type.
 *
 * Parameters: 
 * FieldIdList (output)   : A pointer to the array of the CSSM certificates 
 *                          field identifiers (tags).
 * NumberOfFields (output): The number of fields in the certificate.  
 *                          Also, the length of the FieldIdList array.
 *
 *---------------------------------------------------------------------------*/
CSSM_RETURN CSSMCLI CL_CertDescribeFormat (CSSM_CL_HANDLE CLHandle,
                                           uint32 *NumberOfFields,
                                           CSSM_OID_PTR *result_val)
{
    /* Initializations */
    CSSM_OID_PTR oid_array = NULL;
    uint32       i=0, j=0;
    if (port_IsBadWritePtr (result_val, sizeof(CSSM_OID_PTR)))
    {
        return CSSMERR_CL_INVALID_OUTPUT_POINTER;
    }

    
    /* Check inputs */
    /* CLHandle is checked by CSSM */
    /* Check that NumberOfFields is a valid pointer */
    if ( port_IsBadWritePtr (NumberOfFields, sizeof(uint32)) ) 
    {
        return CSSMERR_CL_INVALID_OUTPUT_POINTER;
        
    }
    *result_val = NULL;

    /* Allocate memory for the array of CSSM certificate OIDs */
    oid_array = (CSSM_OID_PTR) App_Malloc(CLHandle,sizeof(CSSM_OID) * NUMBER_X509_CERT_OIDS);
    if (!oid_array)
    {
        return CSSMERR_CL_MEMORY_ERROR;
        
    }

    /* Fill in the CSSM certificate OIDs and the NumberOfFields */
    for(i=0; i<NUMBER_X509_CERT_OIDS; i++)
    {
        oid_array[i].Length = X509_CERT_OIDS[i].oidval->Length;
        oid_array[i].Data   = App_Malloc(CLHandle, X509_CERT_OIDS[i].oidval->Length);
        if (!oid_array[i].Data)
        {
            for(j=0; j<i; j++)
            {
                App_Free(CLHandle, oid_array[j].Data);
            }
            App_Free(CLHandle, oid_array);
            return CSSMERR_CL_MEMORY_ERROR;
            
        }
        port_memcpy(oid_array[i].Data, X509_CERT_OIDS[i].oidval->Data, oid_array[i].Length);
    }

    /* Update the outputs and return */
    
    *NumberOfFields = NUMBER_X509_CERT_OIDS;
    *result_val = oid_array;
    return CSSM_OK;
}

