/*-----------------------------------------------------------------------
 * File: MAF_CONFIG.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


#ifndef MAF_CONFIG_H
#define MAF_CONFIG_H

#define NO_MODULE_STRING_NAMES
#include "guids.h"
#include "clutil.h"

/*Name of the addin module */
#define ADDIN_NAME                          "Intel X509v3 CL Module"

/* Manifest section name of the service provider */
#ifdef VMS
#define ADDIN_SELF_CHECK_SECTION            "cdsa$in509cl300_shr"
#else
#define ADDIN_SELF_CHECK_SECTION            "in509cl300"
#endif

/* GUID of the addin module */
#define ADDIN_GUID_INITIALIZER              intel_x509v3_clm_guid_init
#define MODULE_GUID                         intel_x509v3_clm_guid

/* Major version of the addin module */
#define ADDIN_VERSION_MAJOR                 CSSM_MAJOR

/* Minor version of the addin module */
#define ADDIN_VERSION_MINOR                 CSSM_MINOR

/* ProductVersion */
#define ADDIN_PRODUCT_VERSION_MAJOR         INTEL_X509V3_CLM_MAJOR_VER
#define ADDIN_PRODUCT_VERSION_MINOR         INTEL_X509V3_CLM_MINOR_VER

/* Standard Description */
#define ADDIN_STANDARD_DESCRIPTION          "X09v3 CA"
 
/* Standard Version */
#define ADDIN_STANDARD_VERSION_MAJOR        509
#define ADDIN_STANDARD_VERSION_MINOR        3

/* CL SSID */
#define ADDIN_CL_SUBSERVICE_ID              0

/* Vendor */
#ifdef VMS
#define ADDIN_VENDOR_NAME                   "Compaq Computer Corporation"
#else
#define ADDIN_VENDOR_NAME                   "Intel Corporation"
#endif
/* Is the addin module thread safe? */
#define ADDIN_THREAD_SAFE                   1

/* Does the addin module require the writer lock durring SPI functions on the load 
tracker node (is this node being modified?) */
#define ADDIN_NEED_LOAD_WRITER_LOCK         0

/* Does the addin module require the writer lock durring SPI functions on the attach 
tracker node (is this node being modified?) */
#define ADDIN_NEED_ATTACH_WRITER_LOCK       1

/* Does the addin module require the custom ADDIN_LOAD_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_LOAD_STRUCT        1

/* Does the addin module require the custom ADDIN_ATTACH_DATA structure (see below) */
#define ADDIN_NEED_ADDIN_ATTACH_STRUCT      1

/* Allow CL to use key_parsing functions */
#define MAF_NEED_KEYPARSE 1

/* Include ber_der.h*/
#define  MAF_CLM_INCLUDED 1

/* Macro used to define if a subservice pair (id, type) is valid for an addin */
#define ADDIN_IS_VALID_SUBSERVICE(SubServiceId, SubServiceType) \
    (((SubServiceId) == 0) && ((SubServiceType) == (CSSM_SERVICE_CL)))

#if (ADDIN_NEED_ADDIN_LOAD_STRUCT) 
/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_LOAD_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_load_data
{
    /* put custom fields here */

    /* include funcptrs.h which declares the external function pointers
     * required by this module.
     */
#include "funcptrs.h"
};
typedef struct addin_load_data ADDIN_LOAD_DATA;

#endif /* if ADDIN_NEED_ADDIN_LOAD_STRUCT */


#if (ADDIN_NEED_ADDIN_ATTACH_STRUCT)

/*-----------------------------------------------------------------------------
 * Description:
 * Structure containing custom fields in the CSSM_MODULE_ATTACH_TRACKER structure
 *---------------------------------------------------------------------------*/
struct addin_attach_data
{
    
    /* Declare an operation context for the CLUTIL library.*/
    CSSM_CLUTIL_CONTEXT clContext;


};
typedef struct addin_attach_data ADDIN_ATTACH_DATA;

#endif /* if ADDIN_NEED_ADDIN_ATTACH_STRUCT */

#endif
