/*-----------------------------------------------------------------------
 * File: FREEKEY.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "cspinc.h"

CSSM_RETURN CSSMAPI ADDIN_CSM_FreeKey(
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ACCESS_CREDENTIALS *AccessCred,
                        CSSM_KEY_PTR pKey,
                        CSSM_BOOL Delete )
{
    CSSM_RETURN rv = CSSM_OK;
    CSM_KM_KEY_HANDLE hKey = CSSM_INVALID_HANDLE;
    const ADDIN_LOAD_DATA *pLoadData = NULL;

    /* Fetch the load data for the key file handle */
    pLoadData = Addin_GetLoadData();
    assert( pLoadData );

    /* Should we delete the key permanently? */
    if ( Delete == CSSM_TRUE )
    {
        if ( pKey->KeyHeader.BlobType == CSSM_KEYBLOB_REFERENCE )        
        {
            if ( pKey->KeyHeader.Format == CSSM_KEYBLOB_REF_FORMAT_INTEGER )
            {
                /* The key is really in the key manager. Do all of the 
                 * remaining checks on that one.
                 */
                if ( pKey->KeyData.Length != sizeof(CSM_KM_KEY_HANDLE) )
                {
                    ERR( rv = CSSMERR_CSP_INVALID_KEY_REFERENCE );
                }
                else
                {
                    hKey = *(CSM_KM_KEY_HANDLE*)pKey->KeyData.Data;
                }
            }
            else if ( pKey->KeyHeader.Format == 
                                            CSSM_KEYBLOB_REF_FORMAT_STRING )
            {
                /* The key contains a label reference. Search the key
                 * manager using the label to get the key handle.
                 */
                rv = CSM_KMFindKeyByName( pLoadData->hKeyFile,
                                          &pKey->KeyData,
                                          NULL,
                                          NULL,
                                          NULL,
                                          &hKey );
            }
            else
            {
                ERR( rv = CSSMERR_CSP_UNSUPPORTED_KEY_FORMAT );
            } /* Find key handle */

            if ( rv == CSSM_OK )
            {
                rv = CSM_KMDeleteKey( pLoadData->hKeyFile,
                                      hKey,
                                      AccessCred );
            }
        } /* Blob is a reference */
    } /* Delete the key blob */

    /* Delete the key buffer if everything has succeeded so far */
    if ( ( rv == CSSM_OK ) &&
         ( pKey->KeyData.Data != NULL ) )
    {
        App_Free( CSPHandle, pKey->KeyData.Data );
    }

    return rv;
}

