/*-----------------------------------------------------------------------
 * File: CSP_MISC.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
 * File Description: Contains miscellaneous operations for the CSP.
 */

#include "geninc.h"

CSSM_RETURN CSSMAPI ADDIN_CSM_FreeKey( 
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ACCESS_CREDENTIALS *AccessCred,
                        CSSM_KEY_PTR KeyPtr,
                        CSSM_BOOL Delete )
{
    const ADDIN_LOAD_DATA   *loadData;
    CSSM_RETURN             rtn = CSSM_OK;

    if (Delete == CSSM_TRUE)
    {
        if (KeyPtr->KeyHeader.KeyClass == CSSM_KEYCLASS_PRIVATE_KEY)
        {
            /* Get the key from the key manager */
            loadData = Addin_GetLoadData();

            if (loadData == NULL)
            {
                rtn = CSSMERR_CSP_INTERNAL_ERROR;
            }

            if (rtn == CSSM_OK)
            {
                /* Remove the key from the key manager */
                rtn = CSM_KMDeleteKey(loadData->hKeyFile,
									  *(CSM_KM_KEY_HANDLE*)KeyPtr->KeyData.Data,
									  AccessCred);
            }
        }
    }

    if (rtn == CSSM_OK)
    {
        /* Free the memory */
        rtn = MAF_DeleteOutputBuffer(CSPHandle, &(KeyPtr->KeyData),  CSSM_TRUE);
    }

    return rtn;
}
