/*-----------------------------------------------------------------------
 * File: HRSAPI.C
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#include "precomp.h"


/*************************************************************************/
/*** Human Recognition Services ******************************************/
/*************************************************************************/

CSSM_RETURN CSSMAPI
CSSM_HRS_FreeBIRHandle(CSSM_HANDLE ModuleHandle,
                       CSSM_HRS_BIR_HANDLE BIRHandle)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->FreeBIRHandle,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->FreeBIRHandle(ModuleHandle,
                                             BIRHandle));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_GetBIRFromHandle(CSSM_HANDLE ModuleHandle,
                          CSSM_HRS_BIR_HANDLE BIRHandle,
                          CSSM_HRS_BIR_PTR *BIR)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->GetBIRFromHandle,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->GetBIRFromHandle(ModuleHandle,
                                                BIRHandle,
                                                BIR));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_GetHeaderFromHandle(CSSM_HANDLE ModuleHandle,
                             CSSM_HRS_BIR_HANDLE BIRHandle,
                             CSSM_HRS_BIR_HEADER_PTR Header)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->GetHeaderFromHandle,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->GetHeaderFromHandle(ModuleHandle,
                                                   BIRHandle,
                                                   Header));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_EnableEvents(CSSM_HANDLE ModuleHandle,
                      CSSM_HRS_MODULE_EVENT_MASK *Events)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->EnableEvents,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->EnableEvents(ModuleHandle,
                                            Events));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_SetGUICallbacks(CSSM_HANDLE ModuleHandle,
                         CSSM_HRS_GUI_STREAMING_CALLBACK GuiStreamingCallback,
                         void *GuiStreamingCallbackCtx,
                         CSSM_HRS_GUI_STATE_CALLBACK GuiStateCallback,
                         void *GuiStateCallbackCtx)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->SetGUICallbacks,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->SetGUICallbacks(ModuleHandle,
                                               GuiStreamingCallback,
                                               GuiStreamingCallbackCtx,
                                               GuiStateCallback,
                                               GuiStateCallbackCtx));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}


CSSM_RETURN CSSMAPI
CSSM_HRS_CancelGUICallbacks(CSSM_HANDLE ModuleHandle)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->CancelGUICallbacks,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->CancelGUICallbacks(ModuleHandle));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}




CSSM_RETURN CSSMAPI
CSSM_HRS_SetStreamCallback(CSSM_HANDLE ModuleHandle,
                           CSSM_HRS_STREAM_CALLBACK StreamCallback,
                           void *StreamCallbackCtx)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->SetStreamCallback,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->SetStreamCallback(ModuleHandle,
                                                 StreamCallback,
                                                 StreamCallbackCtx));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_CancelStreamCallbacks(CSSM_HANDLE ModuleHandle)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->CancelStreamCallbacks,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->CancelStreamCallbacks(ModuleHandle));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_StreamInputOutput(CSSM_HANDLE ModuleHandle,
                           CSSM_DATA_PTR InMessage,
                           CSSM_DATA_PTR OutMessage)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->StreamInputOutput,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->StreamInputOutput(ModuleHandle,
                                                 InMessage,
                                                 OutMessage));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Capture(CSSM_HANDLE ModuleHandle,
                 CSSM_HRS_BIR_PURPOSE Purpose,
                 CSSM_HRS_BIR_HANDLE_PTR CapturedBIR,
                 sint32 Timeout,
                 CSSM_HRS_BIR_HANDLE_PTR AuditData)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Capture,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Capture(ModuleHandle,
                                       Purpose,
                                       CapturedBIR,
                                       Timeout,
                                       AuditData));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_CreateTemplate(CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *CapturedBIR,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                        const CSSM_DATA *Payload)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->CreateTemplate,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->CreateTemplate(ModuleHandle,
                                              CapturedBIR,
                                              StoredTemplate,
                                              NewTemplate,
                                              Payload));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Process(CSSM_HANDLE ModuleHandle,
                 const CSSM_HRS_INPUT_BIR *CapturedBIR,
                 CSSM_HRS_BIR_HANDLE_PTR ProcessedBIR)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Process,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Process(ModuleHandle,
                                       CapturedBIR,
                                       ProcessedBIR));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_VerifyMatch(CSSM_HANDLE ModuleHandle,
                     const CSSM_HRS_FAR *MaxFARRequested,
                     const CSSM_HRS_FRR *MaxFRRRequested,
                     const CSSM_BOOL *FARPrecedence,
                     const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                     const CSSM_HRS_INPUT_BIR *StoredTemplate,
                     CSSM_HRS_BIR_HANDLE *AdaptedBIR,
                     CSSM_BOOL *Result,
                     CSSM_HRS_FAR_PTR FARAchieved,
                     CSSM_HRS_FRR_PTR FRRAchieved,
                     CSSM_DATA_PTR *Payload)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->VerifyMatch,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->VerifyMatch(ModuleHandle,
                                           MaxFARRequested,
                                           MaxFRRRequested,
                                           FARPrecedence,
                                           ProcessedBIR,
                                           StoredTemplate,
                                           AdaptedBIR,
                                           Result,
                                           FARAchieved,
                                           FRRAchieved,
                                           Payload));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_IdentifyMatch(CSSM_HANDLE ModuleHandle,
                       const CSSM_HRS_FAR *MaxFARRequested,
                       const CSSM_HRS_FRR *MaxFRRRequested,
                       const CSSM_BOOL *FARPrecedence,
                       const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                       const CSSM_HRS_IDENTIFY_POPULATION *Population,
                       CSSM_BOOL Binning,
                       uint32 MaxNumberOfResults,
                       uint32 *NumberOfResults,
                       CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                       sint32 Timeout)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->IdentifyMatch,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->IdentifyMatch(ModuleHandle,
                                             MaxFARRequested,
                                             MaxFRRRequested,
                                             FARPrecedence,
                                             ProcessedBIR,
                                             Population,
                                             Binning,
                                             MaxNumberOfResults,
                                             NumberOfResults,
                                             Candidates,
                                             Timeout));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Enroll(CSSM_HANDLE ModuleHandle,
                CSSM_HRS_BIR_PURPOSE Purpose,
                const CSSM_HRS_INPUT_BIR *StoredTemplate,
                CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                const CSSM_DATA *Payload,
                sint32 Timeout,
                CSSM_HRS_BIR_HANDLE_PTR AuditData)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Enroll,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Enroll(ModuleHandle,
                                      Purpose,
                                      StoredTemplate,
                                      NewTemplate,
                                      Payload,
                                      Timeout,
                                      AuditData));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Verify(CSSM_HANDLE ModuleHandle,
                const CSSM_HRS_FAR *MaxFARRequested,
                const CSSM_HRS_FRR *MaxFRRRequested,
                const CSSM_BOOL *FARPrecedence,
                const CSSM_HRS_INPUT_BIR *StoredTemplate,
                CSSM_HRS_BIR_HANDLE_PTR AdaptedBIR,
                CSSM_BOOL *Result,
                CSSM_HRS_FAR_PTR FARAchieved,
                CSSM_HRS_FRR_PTR FRRAchieved,
                CSSM_DATA_PTR *Payload,
                sint32 Timeout,
                CSSM_HRS_BIR_HANDLE_PTR AuditData)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Verify,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Verify(ModuleHandle,
                                      MaxFARRequested,
                                      MaxFRRRequested,
                                      FARPrecedence,
                                      StoredTemplate,
                                      AdaptedBIR,
                                      Result,
                                      FARAchieved,
                                      FRRAchieved,
                                      Payload,
                                      Timeout,
                                      AuditData));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Identify(CSSM_HANDLE ModuleHandle,
                  const CSSM_HRS_FAR *MaxFARRequested,
                  const CSSM_HRS_FRR *MaxFRRRequested,
                  const CSSM_BOOL *FARPrecedence,
                  const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                  const CSSM_HRS_IDENTIFY_POPULATION *Population,
                  CSSM_BOOL Binning,
                  uint32 MaxNumberOfResults,
                  uint32 *NumberOfResults,
                  CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                  sint32 Timeout,
                  CSSM_HRS_BIR_HANDLE_PTR AuditData)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Identify,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Identify(ModuleHandle,
                                        MaxFARRequested,
                                        MaxFRRRequested,
                                        FARPrecedence,
                                        ProcessedBIR,
                                        Population,
                                        Binning,
                                        MaxNumberOfResults,
                                        NumberOfResults,
                                        Candidates,
                                        Timeout,
                                        AuditData));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_Import(CSSM_HANDLE ModuleHandle,
                const CSSM_DATA *InputData,
                CSSM_HRS_BIR_BIOMETRIC_DATA_FORMAT InputFormat,
                CSSM_HRS_BIR_PURPOSE Purpose,
                CSSM_HRS_BIR_HANDLE_PTR ConstructedBIR)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->Import,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->Import(ModuleHandle,
                                      InputData,
                                      InputFormat,
                                      Purpose,
                                      ConstructedBIR));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_SetPowerMode(CSSM_HANDLE ModuleHandle,
                      CSSM_HRS_POWER_MODE PowerMode)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->SetPowerMode,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->SetPowerMode(ModuleHandle,
                                            PowerMode));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbOpen(CSSM_HANDLE ModuleHandle,
                const uint8 *DbName,
                CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                CSSM_HRS_DB_HANDLE_PTR DbHandle,
                CSSM_HRS_DB_CURSOR_PTR Cursor)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbOpen,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbOpen(ModuleHandle,
                                      DbName,
                                      AccessRequest,
                                      DbHandle,
                                      Cursor));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbClose(CSSM_HANDLE ModuleHandle,
                 CSSM_HRS_DB_HANDLE DbHandle)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbClose,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbClose(ModuleHandle,
                                       DbHandle));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbCreate(CSSM_HANDLE ModuleHandle,
                  const uint8 *DbName,
                  CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                  CSSM_HRS_DB_HANDLE_PTR DbHandle)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbCreate,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbCreate(ModuleHandle,
                                        DbName,
                                        AccessRequest,
                                        DbHandle));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbDelete(CSSM_HANDLE ModuleHandle,
                  const uint8 *DbName)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbDelete,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbDelete(ModuleHandle,
                                        DbName));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbSetCursor(CSSM_HANDLE ModuleHandle,
                     CSSM_HRS_DB_HANDLE DbHandle,
                     const CSSM_GUID *KeyValue,
                     CSSM_HRS_DB_CURSOR_PTR Cursor)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbSetCursor,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbSetCursor(ModuleHandle,
                                           DbHandle,
                                           KeyValue,
                                           Cursor));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbFreeCursor(CSSM_HANDLE ModuleHandle,
                      CSSM_HRS_DB_CURSOR_PTR Cursor)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbFreeCursor,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbFreeCursor(ModuleHandle,
                                            Cursor));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbStoreBIR(CSSM_HANDLE ModuleHandle,
                    const CSSM_HRS_INPUT_BIR *BIRToStore,
                    CSSM_HRS_DB_HANDLE DbHandle,
                    CSSM_GUID_PTR Uuid)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbStoreBIR,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbStoreBIR(ModuleHandle,
                                          BIRToStore,
                                          DbHandle,
                                          Uuid));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbGetBIR(CSSM_HANDLE ModuleHandle,
                  CSSM_HRS_DB_HANDLE DbHandle,
                  const CSSM_GUID *KeyValue,
                  CSSM_HRS_BIR_HANDLE_PTR RetrievedBIR,
                  CSSM_HRS_DB_CURSOR_PTR Cursor)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbGetBIR,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbGetBIR(ModuleHandle,
                                        DbHandle,
                                        KeyValue,
                                        RetrievedBIR,
                                        Cursor));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbGetNextBIR(CSSM_HANDLE ModuleHandle,
                      CSSM_HRS_DB_CURSOR_PTR Cursor,
                      CSSM_HRS_BIR_HANDLE_PTR RetievedBIR,
                      CSSM_GUID_PTR Uuid)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbGetNextBIR,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbGetNextBIR(ModuleHandle,
                                            Cursor,
                                            RetievedBIR,
                                            Uuid));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbQueryBIR(CSSM_HANDLE ModuleHandle,
                    CSSM_HRS_DB_HANDLE DbHandle,
                    const CSSM_HRS_INPUT_BIR *BIRToQuery,
                    CSSM_GUID_PTR Uuid)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbQueryBIR,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbQueryBIR(ModuleHandle,
                                          DbHandle,
                                          BIRToQuery,
                                          Uuid));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}

CSSM_RETURN CSSMAPI
CSSM_HRS_DbDeleteBIR(CSSM_HANDLE ModuleHandle,
                     CSSM_HRS_DB_HANDLE DbHandle,
                     const CSSM_GUID *KeyValue)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_RETURN tempErr;

    CSSM_STATE_FUNCS_PTR CSSMFuncs = NULL;
    CSSM_SPI_HRS_FUNCS_PTR SPFuncs = NULL;

    CSSM_GUID SPGuid;

#ifdef ISL_INCLUDED
    /* now we get the return address */
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress(retAddress);
#else
    CSSM_PROC_ADDR retAddress = NULL;   /* dummy for parameter passing */
#endif

    ERR(err = mef_SetUpAndGetAttachFuncs(NULL,
                                         ModuleHandle,
                                         CSSM_SERVICE_HRS,
                                         &CSSMFuncs,
                                         (void**)(&SPFuncs),
                                         &SPGuid));

    if(CSSM_OK == err)
    {
        /* Confirm that the SP implements this function.  Also do PVC check if
           everything is checking out.
        */
        ERR(err = mef_CheckFuncCallValidity(CSSMFuncs,
                                            ModuleHandle,
                                            (CSSM_PROC_ADDR)retAddress,
                                            (CSSM_PROC_ADDR)SPFuncs->DbDeleteBIR,
                                            0,    /* no privilege */
                                            NULL, /* don't need privilege */
                                            CSSM_HINT_ADDRESS_APP));
        if(CSSM_OK == err)
        {
            /* call the service provider */
            ERR(err = SPFuncs->DbDeleteBIR(ModuleHandle,
                                           DbHandle,
                                           KeyValue));
        }

        ERR(tempErr = mef_CleanUpAndReleaseAttachFuncs(ModuleHandle,
                                                       CSSMFuncs));
        if(CSSM_OK == err)
        {
            err = tempErr;
        }
    }

    return (err);
}
