/*-----------------------------------------------------------------------
 * File: hrspwbspi.c
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */


#include "precomp.h"

/*-----------------*
 * "Magic Numbers" *
 *-----------------*/

/* Buffer for password to be captured to; declared here 
 * to allow access by multiple functions */
#define MAX_PASSWORD_SIZE 32
char inputPassword[MAX_PASSWORD_SIZE + 1];   /* +1 for null terminator */
char confirmPassword[MAX_PASSWORD_SIZE + 1]; /* +1 for null terminator */

/* Some parameters for the password based encryption of payloads */
const uint32 PBEKeyLength        = 64;   /* in bits */
const uint32 AlgorithmIterations = 1000; /* This must be sufficiently large to make
                                          * generating all the keys for a large password
                                          * dictionary noticibly more painful, while
                                          * still being small enough to remain uncostly
                                          * for our purposes. */


/*----------------------------------------------------------------------------------*
 *  Helper Functions and Data structures                                            *
 *----------------------------------------------------------------------------------*/

#define SPI_RETURN( errorCode ) \
{									\
	Addin_SPIEnd(pAddinContext);	\
	return errorCode;				\
}

#ifdef LINUX

#include <stdio.h>

#endif /* LINUX defined */

#ifdef WIN32

#define WIN32_LEAN_AND_MEAN		/* Trim headers to only most commonly used stuff */
#include <windows.h>			/* Include Win API for password dialog           */
#include "resource.h"			/* Include dialog resource definitions           */

/*-----------------------------------------*
 *  Callback for Dialog box event handling *
 *-----------------------------------------*/
#define CAPTURE_TIMER_ID 0x123 /* number doesn't matter; just want one unlikely to
                                * collide with ones the system might possibly use    */
UINT      timeoutValue;        /* a buffer for the capture function to pass the 
                                * timeout duration to the dialog handler             */
CSSM_BOOL timeoutFlag;         /* flag to return to the Capture function to indicate
                                * whether or not the operation timed out             */
CSSM_BOOL winError;            /* flag to check whether or not the WinAPI calls 
                                * errored                                            */

BOOL CALLBACK CapturePasswordForVerification
    (HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{ 
	RECT rcDlg, rcDesktop;  
    UINT timerID = 0;

    switch (message) 
    { 
		/*  This message is sent when the dialog gets created */
		case WM_INITDIALOG:		
 			
			GetWindowRect(GetDesktopWindow(), &rcDesktop); 
			GetWindowRect(hwndDlg, &rcDlg); 
			 
			/* Center Dialog on the desktop */			

			SetWindowPos(hwndDlg, 
				HWND_TOP, 
				rcDesktop.right/2  - ((rcDlg.right   - rcDlg.left)  / 2), 
				rcDesktop.bottom/2 - ((rcDlg.bottom  - rcDlg.top)   / 2), 
				0, 0,  /* ignores size arguments; no resize desired */
				SWP_NOSIZE); 

			/* Start dialog with focus in the password edit box */
			SetFocus(GetDlgItem(hwndDlg, IDC_PWEDIT)); 

            /* Start timer for time-out of the dialog */
            timeoutFlag = CSSM_FALSE;
            timerID     = SetTimer(hwndDlg, CAPTURE_TIMER_ID, timeoutValue, NULL);
            
            if (timerID != CAPTURE_TIMER_ID)
            {
                EndDialog(hwndDlg, wParam);
                return TRUE;
            }
			break;

		case WM_COMMAND: 
            switch (LOWORD(wParam)) 
            { 
                case IDOK: 
					/*  Dump the password into BSP memory */
                    if (!GetDlgItemText(hwndDlg, IDC_PWEDIT, inputPassword, 
							MAX_PASSWORD_SIZE + 1)) 
                        /* if call fails, null out the password cache */
                    {
                        *inputPassword=0;
                        winError = CSSM_TRUE;
                    }
						
				/* fall through after ok is pressed */
                case IDCANCEL: 
                    KillTimer(hwndDlg, timerID);
                    EndDialog(hwndDlg, wParam); 
                    return TRUE; 
            } 
            break;

        case WM_TIMER:
            timeoutFlag = CSSM_TRUE;
            KillTimer(hwndDlg, timerID);
            EndDialog(hwndDlg, wParam);
            return TRUE;
    } 
    return FALSE; 
}

BOOL CALLBACK CapturePasswordForEnrollment
    (HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{ 
	RECT rcDlg, rcDesktop;  
    UINT timerID = 0;

    switch (message) 
    { 
		/*  This message is sent when the dialog gets created */
		case WM_INITDIALOG:		
 			
			GetWindowRect(GetDesktopWindow(), &rcDesktop); 
			GetWindowRect(hwndDlg, &rcDlg); 
			 
			/* Center Dialog on the desktop */			

			SetWindowPos(hwndDlg, 
				HWND_TOP, 
				rcDesktop.right/2  - ((rcDlg.right   - rcDlg.left)  / 2), 
				rcDesktop.bottom/2 - ((rcDlg.bottom  - rcDlg.top)   / 2), 
				0, 0,  /* ignores size arguments; no resize desired */
				SWP_NOSIZE); 

			/* Start dialog with focus in the password edit box */
			SetFocus(GetDlgItem(hwndDlg, IDC_PWEDIT)); 

            /* Start timer for time-out of the dialog */
            timeoutFlag = CSSM_FALSE;
            timerID     = SetTimer(hwndDlg, CAPTURE_TIMER_ID, timeoutValue, NULL);
            
            if (timerID != CAPTURE_TIMER_ID)
            {
                EndDialog(hwndDlg, wParam);
                return TRUE;
            }
			break;

		case WM_COMMAND: 
            switch (LOWORD(wParam)) 
            { 
                case IDOK: 
					/*  Dump the password into BSP memory */
                    if (!GetDlgItemText(hwndDlg, IDC_PWEDIT, inputPassword, 
							MAX_PASSWORD_SIZE + 1)                         ||
                        !GetDlgItemText(hwndDlg, IDC_PWEDIT2, confirmPassword,
                            MAX_PASSWORD_SIZE + 1))
                    {
                        /* if call fails, null out the password cache */
						*inputPassword = 0; 
                        winError = CSSM_TRUE;
						
                    }
                    else if ((strlen(inputPassword) != strlen(confirmPassword)) ||
                             (memcmp(inputPassword, 
                                     confirmPassword,
                                     strlen(inputPassword))))
                    {
                        SetDlgItemText(hwndDlg, IDC_PWERRLABEL, "Passwords do not match!");
                        break;
                    }
                        
				/* fall through after ok is pressed */
                case IDCANCEL: 
                    KillTimer(hwndDlg, timerID);
                    EndDialog(hwndDlg, wParam); 
                    return TRUE; 
            } 
            break;

        case WM_TIMER:
            timeoutFlag = CSSM_TRUE;
            KillTimer(hwndDlg, timerID);
            EndDialog(hwndDlg, wParam);
            return TRUE;
    } 
    return FALSE; 
} 

#endif /* WIN32 defined for password dialog callback */



/*--------------------------------------------------------------------------------*
 * Private Helper functions:  These allow the functionality of the SPI to be used *
 * internally without violating the integrity checks imposed by EISL, since the   *
 * BSP is not that app to which the calls should be returning!                    *
 *--------------------------------------------------------------------------------*/

CSSM_RETURN _hrs_FreeBIRHandle(
                    ADDIN_ATTACH_DATA   *pAttachData,                    
                    CSSM_HRS_BIR_HANDLE  BIRHandle )
{
    BIR_LIST_NODE *pCurListNode  = NULL, 
		          *pPrevListNode = NULL,
				  *pTempListNode = NULL;

    if (NULL == (pCurListNode = pPrevListNode = pAttachData->BIRList))
		/* return (CSSMERR_HRS_INVALID_BIR_HANDLE); */  
		return (CSSMERR_HRS_INTERNAL_ERROR);

	/* Burn through the linked list of BIRs until the correct 
	 * header is found or nothing remains in the list */
	while (pCurListNode->BIRHandle != BIRHandle && pCurListNode->NextNode != NULL)
	{
		pPrevListNode = pCurListNode;
		pCurListNode  = (BIR_LIST_NODE*)pCurListNode->NextNode;		
	}
	
	/* if the BIR was successfully found */
	if (pCurListNode->BIRHandle == BIRHandle)
	{
		pTempListNode = (BIR_LIST_NODE*)pCurListNode->NextNode;
		
		/* Free the dynamically allocated biometric data   
		 * field of the stored BIR */
		Addin_free(pCurListNode->BIR.BiometricData, NULL);

		/* If the BIR to free is the first one in the list */
		if (pCurListNode == pPrevListNode)
		{			
			Addin_free(pAttachData->BIRList, NULL);
			pAttachData->BIRList = pTempListNode;
		}
		else 
		{
			Addin_free(pCurListNode, NULL);

			pPrevListNode->NextNode = pTempListNode;
		}
	}
	else 
		/* TODO: return (CSSMERR_HRS_INVALID_BIR_HANDLE); */
		return (CSSMERR_HRS_INTERNAL_ERROR);
    
    return CSSM_OK;
}


CSSM_RETURN _hrs_GetBIRFromHandle(
                    const ADDIN_ATTACH_DATA *pAttachData,                  
                    CSSM_HRS_BIR_HANDLE      BIRHandle,
                    CSSM_HRS_BIR_PTR        *BIR )
{
    BIR_LIST_NODE   *pCurListNode   = NULL; 

	if (NULL == (pCurListNode = pAttachData->BIRList))
		/* return (CSSMERR_HRS_INVALID_BIR_HANDLE); */
		return (CSSMERR_HRS_INTERNAL_ERROR);		

	/* Burn through the linked list of BIRs until the correct 
	 * header is found or nothing remains in the list */
	while (pCurListNode->BIRHandle != BIRHandle && 
		   pCurListNode->NextNode  != NULL)
		pCurListNode = (BIR_LIST_NODE*)pCurListNode->NextNode;

	/* If BIR was successfully found, capture it in the pointer for return
     * to SPI functions */
	if (pCurListNode->BIRHandle == BIRHandle)
    {
        *BIR = &pCurListNode->BIR;
        return CSSM_OK;
    }
	else 
		/* TODO: SPI_RETURN(CSSMERR_HRS_INVALID_BIR_HANDLE); */
		return (CSSMERR_HRS_INTERNAL_ERROR);
}


CSSM_RETURN _hrs_Capture(
                    ADDIN_ATTACH_DATA       *pAttachData,
                    CSSM_HRS_BIR_PURPOSE     Purpose,
                    CSSM_HRS_BIR_HANDLE_PTR  CapturedBIR,
                    sint32                   Timeout)
{
    CSSM_RETURN      ret               = CSSM_OK;
    BIR_LIST_NODE   *pOldList          = NULL,
	                *pHeadNode         = NULL;
	uint32           capturedBIRLength = 0;
    BIO_DATA_HEADER  bioDataHeader;

#ifdef WIN32
	
	INT_PTR dialogRetVal = 0;
	HANDLE  dllHandle;

#endif
	
    /*---------------------------------------*
     * Lock mutex to avoid multiple threads  *
     * wiping each other's passwords out of  *
     * the buffer.                           *
     *---------------------------------------*/
    
    /* allow for maximum timeout value.  The capture method 
     * should time out itself - the mutex shouldn't be doing
     * that work */
    if (CSSM_OK !=
            (ret = port_LockMutex(BSPMutex, 0xFFFFFFFF)))
        return ret;

	/*---------------------------------------*
	 * Get the password from the user.       *
	 * Start of platform specific code here! *
	 *---------------------------------------*/

/*------------------------------------------------------------------------
 * WIN32
 *-----------------------------------------------------------------------*/

	#ifdef WIN32
	/*---------------------------------------*
	 * Create Dialog and prompt for password *
	 *---------------------------------------*/
			
	/* Windows will find this library in the system directory 
	 * where the .dll is installed */
	dllHandle = LoadLibrary(ADDIN_SELF_CHECK_SECTION);

    /* set timeout duration for the dialog box */
    timeoutValue = Timeout;

    if (CSSM_HRS_PURPOSE_ENROLL_FOR_VERIFICATION_ONLY == Purpose)
    {
        dialogRetVal = DialogBox(
		    dllHandle,	MAKEINTRESOURCE(IDD_ENROLLDLG), 
		    NULL,		(DLGPROC)CapturePasswordForEnrollment);
    }
    else if (CSSM_HRS_PURPOSE_VERIFY == Purpose)
    {
       dialogRetVal = DialogBox(
			dllHandle,	MAKEINTRESOURCE(IDD_PWDIALOG), 
			NULL,		(DLGPROC)CapturePasswordForVerification);
    }
    else 
    {
        /* Unlock the mutex for the next capture call */
        port_UnlockMutex(BSPMutex);

        return (CSSMERR_HRS_PURPOSE_NOT_SUPPORTED);
    }

	if (IDOK != dialogRetVal) 
	{
	    /* Operation failed; password wasn't captured */
    
        /* Unlock the mutex for the next capture call */
        port_UnlockMutex(BSPMutex);

		if (CSSM_FALSE == timeoutFlag)
            return (CSSMERR_HRS_UNABLE_TO_CAPTURE);
        else 
            return (CSSMERR_HRS_TIMEOUT_EXPIRED);
	}

	#endif	/* WIN32 defined */

/*------------------------------------------------------------------------*/

/*------------------------------------------------------------------------*
 * Linux
 *------------------------------------------------------------------------*/

#ifdef LINUX

    memset(inputPassword, 0, MAX_PASSWORD_SIZE + 1);

    if (CSSM_HRS_PURPOSE_ENROLL_FOR_VERIFICATION_ONLY == Purpose)
    {
        printf("Please enter your password for verification:\n");
        fgets(inputPassword, MAX_PASSWORD_SIZE+1, stdin);
    }
    else if (CSSM_HRS_PURPOSE_VERIFY == Purpose)
    {
        memset(confirmPassword, 0, MAX_PASSWORD_SIZE + 1);

        printf("Please enter your password for enrollment:\n");
        fgets(inputPassword, MAX_PASSWORD_SIZE+1, stdin);
        printf("Please re-enter your password for confirmation:\n");
        fgets(confirmPassword, MAX_PASSWORD_SIZE+1, stdin);

        if (strlen(inputPassword) != strlen(confirmPassword) ||
            memcmp(inputPassword, confirmPassword, strlen(inputPassword)))
        {
            /* Unlock the mutex for the next capture call */
            port_UnlockMutex(BSPMutex);

            return (CSSMERR_HRS_UNABLE_TO_CAPTURE);
        }
    }
    else
    {
        /* Unlock the mutex for the next capture call */
        port_UnlockMutex(BSPMutex);

        return (CSSMERR_HRS_PURPOSE_NOT_SUPPORTED);
    }

#endif /* LINUX defined */

/*------------------------------------------------------------------------*/

	/*---------------------------------------*
	 * End of platform specific code. Assume *
	 * now that the password is stored in    *
	 * the global buffer inputPassword.      *
	 *---------------------------------------*/

	/* Keep track of BIR in addin until function FreeBIRHandle(...) 
	 * releases it */
	
	pOldList = pAttachData->BIRList;
	pAttachData->BIRList = pHeadNode = 
		(BIR_LIST_NODE*) Addin_malloc(sizeof(BIR_LIST_NODE), NULL);
    if (NULL == pHeadNode)
    {
        /* Unlock the mutex for the next capture call */
        port_UnlockMutex(BSPMutex);

        return CSSMERR_HRS_MEMORY_ERROR;
    }
	pHeadNode->NextNode     = (void*)pOldList;
	
	/*------------------*
	 * Build BIR Header *
	 *------------------*/
	pHeadNode->BIRHandle  = *CapturedBIR     = pAttachData->HandleIndex++;
	pHeadNode->BIR.Header.HeaderVersion      = BIR_HEADER_VERSION;
	pHeadNode->BIR.Header.Type = CSSM_HRS_BIR_DATA_TYPE_RAW;
	pHeadNode->BIR.Header.Format.FormatOwner = BIO_DATA_FORMAT_OWNER;
	pHeadNode->BIR.Header.Format.FormatID    = BIO_DATA_FORMAT_ID;
	pHeadNode->BIR.Header.Quality            = 100; /* by definition */
	pHeadNode->BIR.Header.FactorsMask        = CSSM_HRS_FACTOR_PASSWORD;

	/* The Purpose will be used in the HRS Process function
	 * to make sure that BIRs which are captured for verification
	 * are not hashed; otherwise, they're useless. */
	pHeadNode->BIR.Header.PurposeMask        = Purpose;	
	
	bioDataHeader.PWLength         = capturedBIRLength = strlen(inputPassword);
	bioDataHeader.PayloadLength    = 0;
	memset(bioDataHeader.HashSalt,   0, HASH_SALT_SIZE);
	memset(bioDataHeader.PBESalt,    0, PBE_SALT_SIZE);
	memset(bioDataHeader.InitVector, 0, INIT_VECTOR_SIZE);
	pHeadNode->BIR.Header.Length   = sizeof(CSSM_HRS_BIR_HEADER) +
	                                 sizeof(BIO_DATA_HEADER) + capturedBIRLength ;

	/* Finish BIR */
	pHeadNode->BIR.Signature     = NULL;
	pHeadNode->BIR.BiometricData = 
		(CSSM_HRS_BIR_BIOMETRIC_DATA*) 
			Addin_malloc(capturedBIRLength + sizeof(BIO_DATA_HEADER), NULL);
    if (NULL == pHeadNode->BIR.BiometricData)
    {
        /* Unlock the mutex for the next capture call */
        port_UnlockMutex(BSPMutex);

        return CSSMERR_HRS_MEMORY_ERROR;
    }

	memcpy((void*)pHeadNode->BIR.BiometricData, (void*)&bioDataHeader,
	       sizeof(BIO_DATA_HEADER));
	memcpy((void*)(pHeadNode->BIR.BiometricData + sizeof(BIO_DATA_HEADER)), 
           inputPassword, capturedBIRLength);		

	/* Wipe password out of buffer */
	memset((void*)inputPassword, 0, MAX_PASSWORD_SIZE);
    memset((void*)confirmPassword, 0, MAX_PASSWORD_SIZE);

    /* Unlock the mutex for the next capture call */
    port_UnlockMutex(BSPMutex);

    return CSSM_OK;
}

CSSM_RETURN _hrs_CreateTemplate(
            ADDIN_ATTACH_DATA           *pAttachData,
            const ADDIN_LOAD_DATA       *pLoadData,
            const CSSM_HRS_BIR          *pBIRtoWrap,            
            CSSM_HRS_BIR_HANDLE_PTR      NewTemplate,
            const CSSM_DATA             *Payload )
{
    CSSM_RETURN     ret                          = CSSM_OK;    
	BIR_LIST_NODE  *pOldList                     = NULL,
				   *pHeadNode                    = NULL;	
	uint8          *pNewBioData                  = NULL;
	uint32          newBioDataLength             = 0;			
	uint32          EncryptedPayloadLength       = 0;
	CSSM_CC_HANDLE  PBEContext                   = 0;
	CSSM_CC_HANDLE  EncryptionContext            = 0;
	BIO_DATA_HEADER bioDataHeader;
	CSSM_KEY        PBEKey;	
	CSSM_DATA       EncryptedPayload;
	CSSM_DATA       DeriveParameter;
	CSSM_DATA       password;
	CSSM_DATA		HashedPW                     = {0, NULL};
	CSSM_DATA       EncryptionOverflow           = {0, NULL};	
	CSSM_DATA       HashSalt, PBESalt;
	CSSM_DATA       InitVectorForIterations;	
	uint8			HashSaltData[HASH_SALT_SIZE];
	uint8           PBESaltData[PBE_SALT_SIZE];
	uint8           InitVectorData[INIT_VECTOR_SIZE];
	CSSM_PKCS5_PBKDF1_PARAMS PBEParameters;


    /*---------------------------------------------------------*
	 * Only BIRs with the purpose of enrollment should be used *
	 *---------------------------------------------------------*/

	if (CSSM_HRS_PURPOSE_ENROLL_FOR_VERIFICATION_ONLY
			!= pBIRtoWrap->Header.PurposeMask)
		return (CSSMERR_HRS_INCONSISTENT_PURPOSE);

	/*----------------------------------------------------------------------*
	 * Only BIRs created using the proper biometric data format can be used *
	 *----------------------------------------------------------------------*/

	if (BIO_DATA_FORMAT_ID    != pBIRtoWrap->Header.Format.FormatID ||
		BIO_DATA_FORMAT_OWNER != pBIRtoWrap->Header.Format.FormatOwner)
		return (CSSMERR_HRS_UNSUPPORTED_FORMAT);

	/*------------------------------------------------*
	 * Only raw and processed BIRs should be accepted *
	 *------------------------------------------------*/

	/*----------------------------------------------------------------------------*
	 * Note about signed BIRs: if this BSP could somehow support signed BIRs, the *
	 * signature would be verified at this point.  Note that this means that flag *
	 * can be discarded below to account for the check which could occur at this  *
	 * point in the future, if signatures were implemented here.                  *
	 *----------------------------------------------------------------------------*/

	/* Signing is checked above and doesn't matter here, and if the encryption
	 * flag is set, for this BIR format it only pertains to the payload data.  This
	 * allows for support of signed BIRs in this format later, in the situation     
	 * where the signing flag is set, or the association of a new payload with a BIR,
	 * in the situation where the encryption flag is set. */
	if (CSSM_HRS_BIR_DATA_TYPE_PROCESSED == 
		(~(CSSM_HRS_BIR_DATA_TYPE_SIGNED & CSSM_HRS_BIR_DATA_TYPE_ENCRYPTED) &
		 pBIRtoWrap->Header.Type))
	{
		/*---------------------------------------------------*
		 * Rewrap with payload, but don't encrypt - we no    *
		 * longer have plaintext password for key derivation *
		 *---------------------------------------------------*/
		
		/* New biometric data will include the old data plus the payload */
		memcpy((void*)&bioDataHeader, (void*)pBIRtoWrap->BiometricData,
			   sizeof(BIO_DATA_HEADER));
				
		bioDataHeader.PayloadLength = Payload->Length;		
		newBioDataLength            = bioDataHeader.PWLength + 
			                          bioDataHeader.PayloadLength +
			                          sizeof(BIO_DATA_HEADER);
		pNewBioData                 = Addin_malloc(newBioDataLength, NULL);

        if (NULL == pNewBioData)
            return CSSMERR_HRS_MEMORY_ERROR;

		/*---------------------------------------------------*
		 * Move the payload into the addin's memory space,   *
		 * so that it can be included in a new template BIR  *
		 *---------------------------------------------------*/

		/* In this biometric format, the header describing the length 
		 * of the password and payload, the cryptographic salts, and the
		 * cryptographic initialization data vector is included in the 
		 * opaque biometric data first, followed by the password, then 
		 * the payload.  We copy the headers and PW data over first */
		memcpy((void*)pNewBioData, (void*)&bioDataHeader,
			   sizeof(BIO_DATA_HEADER));
				
		memcpy((void*)(pNewBioData + sizeof(BIO_DATA_HEADER)), 
			   (void*)(pBIRtoWrap->BiometricData + sizeof(BIO_DATA_HEADER)), 
		       bioDataHeader.PWLength);
		
		/* Then we copy over the payload */
		memcpy((void*)(pNewBioData + newBioDataLength - Payload->Length),
		       (void*)Payload->Data, Payload->Length);		
	}
	/* if data is raw, then we are assuming it is the plaintext password.  This
	 * shouldn't be encrypted, as this BSP only returns processed BIRs with that 
	 * flag set.  Signing is checked above - strip that flag and check for raw. */
	else if ((pBIRtoWrap->Header.Type & ~CSSM_HRS_BIR_DATA_TYPE_SIGNED)
			== CSSM_HRS_BIR_DATA_TYPE_RAW)
	{
         /* Generate the salt for the password hashing */
		HashSalt.Length = HASH_SALT_SIZE;
		HashSalt.Data   = HashSaltData;		

		if (CSSM_OK != (ret = pLoadData->GenerateRandom(
								pLoadData->HashRandomContext, &HashSalt)))
			return (ret);

        memcpy(bioDataHeader.HashSalt,   HashSalt.Data, HASH_SALT_SIZE);
        memset(bioDataHeader.PBESalt,    0,  PBE_SALT_SIZE);
		memset(bioDataHeader.InitVector, 0,  INIT_VECTOR_SIZE);

        if ((Payload != NULL) && (Payload->Length > 0))
        {
		    /*-----------------------------------------------------*
		     * Encrypt the application's payload for wrapping into *
		     * the new template BIR                                *
		     *-----------------------------------------------------*/

		    /* We assume that the bio data is the plaintext password.  Since
		     * we can get the exact password back each time, we can use this 
		     * to derive a key and encrypt the payload, using the same pw to
		     * derive the key to decrypt later during verification.  Here 
		     * then, we encrypt the payload before wrapping up the template,
		     * and hash the password before putting it back into the new 
		     * template.     */		   

		    /* Generate the salt for the PBE key derivation */
		    PBESalt.Length  = PBE_SALT_SIZE;
		    PBESalt.Data    = PBESaltData;

		    if (CSSM_OK != (ret = pLoadData->GenerateRandom(
								    pLoadData->KeyDeriveRandomContext, &PBESalt)))
			    return (ret);
		    
		    /* Generate the initialization vector that will be used for 
		     * the PBE key derivation and the PB encryption */
		    InitVectorForIterations.Length = INIT_VECTOR_SIZE;
		    InitVectorForIterations.Data   = InitVectorData;

		    if (CSSM_OK != (ret = pLoadData->GenerateRandom(
								    pLoadData->InitRandomContext, &InitVectorForIterations)))
			    return (ret);

		    /* Copy the data to the header in the BIR so it can be used
		     * later for verification and decryption */		    
		    memcpy(bioDataHeader.PBESalt,    PBESalt.Data,  PBE_SALT_SIZE);
		    memcpy(bioDataHeader.InitVector, InitVectorForIterations.Data, 
					    INIT_VECTOR_SIZE);

		    if (CSSM_OK == (ret = pLoadData->CreateDeriveKeyContext(
										    pLoadData->CSPHandle,
										    CSSM_ALGID_PKCS5_PBKDF1_SHA1,
										    CSSM_ALGID_DES,
										    PBEKeyLength,
										    NULL,
										    NULL,
										    AlgorithmIterations,
										    &PBESalt,
										    NULL,
										    &PBEContext))) 
		    {
			    /* Initialize key struct to receive derived key */
			    memset(&PBEKey, 0, sizeof(CSSM_KEY));
			    
			    /* Wrap up password to pass to the CSP */
			    password.Length = pBIRtoWrap->Header.Length - sizeof(CSSM_HRS_BIR_HEADER) 
														    - sizeof(BIO_DATA_HEADER);
			    password.Data   = pBIRtoWrap->BiometricData + sizeof(BIO_DATA_HEADER);
			    
			    PBEParameters.Passphrase = password;
			    PBEParameters.InitVector = InitVectorForIterations;
			    DeriveParameter.Data     = (uint8*)&PBEParameters;
			    DeriveParameter.Length   = sizeof(CSSM_PKCS5_PBKDF1_PARAMS);

			    /* Get the key */
			    if (CSSM_OK == (ret = pLoadData->DeriveKey(
											    PBEContext,
											    &DeriveParameter,
											    CSSM_KEYUSE_ENCRYPT,
											    CSSM_KEYATTR_RETURN_DEFAULT,
											    NULL,
											    NULL,
											    &PBEKey)))
			    {								
				    /* Create cryptographic context for the payload encryption */
				    if (CSSM_OK == (ret = pLoadData->CreateSymmetricContext(
												    pLoadData->CSPHandle,
												    CSSM_ALGID_DES,
												    CSSM_ALGMODE_CBCPadIV8,
												    NULL,
												    &PBEKey,
												    &InitVectorForIterations,
												    CSSM_PADDING_PKCS5,
												    NULL,
												    &EncryptionContext)))
				    {
					    /* Prepare buffer to receive the encrypted payload. */
					    EncryptedPayload.Length = Payload->Length;
					    EncryptedPayload.Data   = Addin_malloc(Payload->Length, NULL);
                        if (NULL == EncryptedPayload.Data)
                            return CSSMERR_HRS_MEMORY_ERROR;

					    /* Encrypt the payload */
					    ret = pLoadData->EncryptData(
												    EncryptionContext,
												    Payload,
												    1,
												    &EncryptedPayload,
												    1,
												    &EncryptedPayloadLength,
												    &EncryptionOverflow);
				    
					    /* Release the encryption context */
					    pLoadData->DeleteContext(EncryptionContext);					
				    }

				    /* Release the key */
				    pLoadData->FreeKey(
						    	    pLoadData->CSPHandle,
								    NULL,
								    &PBEKey,
								    CSSM_TRUE);					
			    }

			    /* Release the key derivation context */
			    pLoadData->DeleteContext(PBEContext);
		    }
		    /* Check for errors */
            if (CSSM_OK != ret)	
            {
                if (NULL != EncryptedPayload.Data)
                    Addin_free(EncryptedPayload.Data, NULL);
                return (ret);
            }

        }
        else /* There is no payload */
        {
            EncryptedPayloadLength  = 0;
            EncryptedPayload.Length = 0;
            EncryptedPayload.Data   = NULL;
        }

		/*---------------------------------------------------------------*
		 * Hash the BIRs password for wrapping into the new template BIR *
		 *---------------------------------------------------------------*/
		
		/* Prepare the buffer with the salted password for hashing */
		password.Length = pBIRtoWrap->Header.Length - sizeof(CSSM_HRS_BIR_HEADER) 
								- sizeof(BIO_DATA_HEADER) + HASH_SALT_SIZE;
		password.Data   = Addin_malloc(password.Length, NULL);
        if (NULL == password.Data)
        {
            if (NULL != EncryptedPayload.Data)
                Addin_free(EncryptedPayload.Data, NULL);

            return CSSMERR_HRS_MEMORY_ERROR;
        }

		memcpy(password.Data,
			   pBIRtoWrap->BiometricData + sizeof(BIO_DATA_HEADER), 
			   password.Length - HASH_SALT_SIZE);
		memcpy(password.Data + password.Length - HASH_SALT_SIZE, 
			   HashSalt.Data,
			   HASH_SALT_SIZE);

		/* This function will allocate the memory required for storing 
		 * the hashed password in the addin's memory space.  It 
		 * must be EXPLICITLY freed when it is no longer needed.       */
		ret = pLoadData->DigestData(pLoadData->DigestContext, &password, 1, &HashedPW);
				
		/* Free the salted password's data buffer */
		Addin_free(password.Data, NULL);

        if (CSSM_OK != ret)
        {
            if (NULL != EncryptedPayload.Data)
                Addin_free(EncryptedPayload.Data, NULL);
            return (ret);
        }

		/* condense all this data into one opaque field
		 * for the new BIR template                     */
		bioDataHeader.PWLength      = HashedPW.Length;
		bioDataHeader.PayloadLength = EncryptedPayloadLength;

		newBioDataLength = HashedPW.Length + EncryptedPayloadLength +
		                   sizeof(BIO_DATA_HEADER);
		pNewBioData      = Addin_malloc(newBioDataLength, NULL);
        if (NULL == pNewBioData)
        {
            Addin_free(HashedPW.Data, NULL);
            if (NULL != EncryptedPayload.Data)
                Addin_free(EncryptedPayload.Data, NULL);

            return CSSMERR_HRS_MEMORY_ERROR;
        }

		/* move bio data header into addin's new bio data field */
		memcpy((void*)pNewBioData, &bioDataHeader, sizeof(BIO_DATA_HEADER));

		/* move password into addin's new bio data field */
		memcpy((void*)(pNewBioData + sizeof(BIO_DATA_HEADER)), 
		  	   (void*)HashedPW.Data, HashedPW.Length);

        if (EncryptedPayloadLength > 0)
        {
		    /* move payload  into addin's new bio data field */
		    memcpy((void*)(pNewBioData + sizeof(BIO_DATA_HEADER) + HashedPW.Length),
			    (void*)EncryptedPayload.Data, EncryptedPayload.Length);

		    /* Note here that the encrypted payload may span TWO 
		     * data structs; handle this possibility             */
		    if (EncryptedPayload.Length < EncryptedPayloadLength)
		    {
			    memcpy(
				    (void*)(pNewBioData + sizeof(BIO_DATA_HEADER) +
				    HashedPW.Length     + EncryptedPayload.Length),
				    (void*)EncryptionOverflow.Data, EncryptionOverflow.Length);

			    /* overflow buffer needs to be freed */
			    Addin_free(EncryptionOverflow.Data, NULL);

			    /* Update bio data header for the opaque field */
			    bioDataHeader.PayloadLength = EncryptedPayloadLength;
			    bioDataHeader.PWLength      = HashedPW.Length;
		    }
        }

		/* buffers for password hash and encryption need to be freed */
		Addin_free(HashedPW.Data, NULL);
		if (NULL != EncryptedPayload.Data)
            Addin_free(EncryptedPayload.Data,	NULL);
	}
	else 
	{
		/* Error - data type specifiers for the biometric data 
		 * don't match either acceptable case */
		
		return (CSSMERR_HRS_INVALID_BIR);
	}
	
	/*----------------------------------------------------*
	 * Build template with new biometric data and payload *
	 *----------------------------------------------------*/

	/* The assumption, at this point, is that for either case of BIR
	 * input, raw or processed, that pNewBioData contains the opaque
	 * data field to be stored, that newBioDataLength is the length 
	 * of that field. */

	/* Keep track of BIR in addin until function FreeBIRHandle(...) releases it */

	pOldList = pAttachData->BIRList;
	pAttachData->BIRList = pHeadNode = 
		(BIR_LIST_NODE*) Addin_malloc(sizeof(BIR_LIST_NODE), NULL);
    if (NULL == pHeadNode)
        return CSSMERR_HRS_MEMORY_ERROR;

	pHeadNode->NextNode     = (void*)pOldList;
	 
	/* Build BIR Header */
	pHeadNode->BIRHandle  = *NewTemplate     = pAttachData->HandleIndex++;
	pHeadNode->BIR.Header.HeaderVersion      = BIR_HEADER_VERSION;	
	pHeadNode->BIR.Header.Type = CSSM_HRS_BIR_DATA_TYPE_PROCESSED;		
	pHeadNode->BIR.Header.Format.FormatOwner = BIO_DATA_FORMAT_OWNER;
	pHeadNode->BIR.Header.Format.FormatID    = BIO_DATA_FORMAT_ID;
	pHeadNode->BIR.Header.Quality            = 100; /* by definition */		 
	pHeadNode->BIR.Header.FactorsMask        = CSSM_HRS_FACTOR_PASSWORD;
	pHeadNode->BIR.Header.PurposeMask        = CSSM_HRS_PURPOSE_VERIFY;

	/* Only captured raw BIRs are wrapped into proccessed BIRs
	 * with encrypted payloads */
	if (CSSM_HRS_BIR_DATA_TYPE_RAW == 
		(pBIRtoWrap->Header.Type & ~CSSM_HRS_BIR_DATA_TYPE_SIGNED))  
		pHeadNode->BIR.Header.Type |= CSSM_HRS_BIR_DATA_TYPE_ENCRYPTED;

	/* bio data length includes the format header, the password, and the payload,
	 * while the BIR header length includes all this and the HRS BIR header's length */	
	pHeadNode->BIR.Header.Length = newBioDataLength + sizeof(CSSM_HRS_BIR_HEADER);

	/* Finish the BIR */
	pHeadNode->BIR.Signature     = NULL;
	pHeadNode->BIR.BiometricData = (CSSM_HRS_BIR_BIOMETRIC_DATA*)pNewBioData;
	
    return CSSM_OK;
}

CSSM_RETURN _hrs_Process(
            ADDIN_ATTACH_DATA           *pAttachData,
            const ADDIN_LOAD_DATA       *pLoadData,
            const CSSM_HRS_BIR          *pBIRtoProcess,
            CSSM_HRS_BIR_HANDLE_PTR      ProcessedBIR )
{
    CSSM_RETURN     ret                  = CSSM_OK;
	BIR_LIST_NODE  *pOldList             = NULL,
				   *pHeadNode            = NULL;
	CSSM_DATA       PWtoHash, HashedPW   = {0, NULL};
	CSSM_DATA       HashSalt;
	uint8           HashSaltData[8];
	BIO_DATA_HEADER bioDataHeader;

	/*--------------------------------*
	 * Check for the proper BIR flags *
	 *--------------------------------*/

	/*----------------------------------------------------------------------------*
	 * Note about signed BIRs: if this BSP could somehow support signed BIRs, the *
	 * signature would be verified at this point.  Note that this means that flag *
	 * can be discarded below to account for the check which could occur at this  *
	 * point in the future, if signatures were implemented here.                  *
	 *----------------------------------------------------------------------------*/

	/* Only raw BIRs should be processed. Signed raw data is OK, 
	 * so strip that flag off before the check.  This is to allow 
	 * for the possibility of signed BIRs in the future; see above. */
	if (pBIRtoProcess->Header.Type & ~CSSM_HRS_BIR_DATA_TYPE_SIGNED
			!= CSSM_HRS_BIR_DATA_TYPE_RAW)
		return (CSSMERR_HRS_INVALID_BIR);

	/* Only enrollment purpose is supported.  Since the Verify
	 * function requires that the BIR captured for verification comes
	 * to it with the password in plaintext, even the 
	 * CSSM_HRS_PURPOSE_VERIFY is not supported in this function. 
	 * When a BIR is "processed" within the context of this BSP, 
	 * it can then be used to "enroll" a BIR for a payload which is
	 * stored in plaintext, instead of encrypted. */
	if (CSSM_HRS_PURPOSE_ENROLL_FOR_VERIFICATION_ONLY != 
			pBIRtoProcess->Header.PurposeMask)
		return (CSSMERR_HRS_PURPOSE_NOT_SUPPORTED);

	/* Only BIRs utilizing the proper biometric data format can be used */
	if (BIO_DATA_FORMAT_ID    != pBIRtoProcess->Header.Format.FormatID ||
		BIO_DATA_FORMAT_OWNER != pBIRtoProcess->Header.Format.FormatOwner)
		return (CSSMERR_HRS_UNSUPPORTED_FORMAT);

	/*-------------------*
	 * Hash the password *
	 *-------------------*/

	/* Generate a salt for the password hashing */
	HashSalt.Length = HASH_SALT_SIZE;
	HashSalt.Data   = HashSaltData;

	if (CSSM_OK != (ret = pLoadData->GenerateRandom(
						pLoadData->HashRandomContext, &HashSalt)))
		return (ret);

	/* Keep the salt for later password verification.  Null out the others,
	 * as they are not yet in use. */
	memcpy(bioDataHeader.HashSalt, HashSalt.Data, HASH_SALT_SIZE);
    memset(bioDataHeader.PBESalt,    0, PBE_SALT_SIZE);
	memset(bioDataHeader.InitVector, 0, INIT_VECTOR_SIZE);

	/* Prepare the buffer with the salted password for hashing */
	PWtoHash.Length = pBIRtoProcess->Header.Length - sizeof(CSSM_HRS_BIR_HEADER) 
							- sizeof(BIO_DATA_HEADER) + HASH_SALT_SIZE;
	PWtoHash.Data   = Addin_malloc(PWtoHash.Length, NULL);
    if (NULL == PWtoHash.Data)
    {
        return CSSMERR_HRS_MEMORY_ERROR;
    }

	memcpy(PWtoHash.Data,
           pBIRtoProcess->BiometricData + sizeof(BIO_DATA_HEADER), 
           PWtoHash.Length - HASH_SALT_SIZE);
	memcpy(PWtoHash.Data + PWtoHash.Length - HASH_SALT_SIZE, 
           HashSalt.Data,
           HASH_SALT_SIZE);

	/* This function will allocate the memory for the hashed
	 * password within the addin's memory space.  It must be
	 * EXPLICITLY freed later when it's no longer needed.     */
	ret = pLoadData->DigestData(pLoadData->DigestContext, &PWtoHash, 1, &HashedPW);    
	
    /* Free the salted password's data buffer */
	Addin_free(PWtoHash.Data, NULL);
    
    if (CSSM_OK != ret) return (ret);


	/*----------------------*
	 * Create processed BIR *
	 *----------------------*/

	/* Keep track of BIR in addin until function FreeBIRHandle(...) 
	 * releases it */

	pOldList = pAttachData->BIRList;
	pAttachData->BIRList = pHeadNode = 
		(BIR_LIST_NODE*) Addin_malloc(sizeof(BIR_LIST_NODE), NULL);
    if (NULL == pHeadNode)
    {
        Addin_free(HashedPW.Data, NULL);

        return CSSMERR_HRS_MEMORY_ERROR;
    }

	pHeadNode->NextNode     = (void*)pOldList;
	
	/* Build BIR Header */
	pHeadNode->BIRHandle  = *ProcessedBIR    = pAttachData->HandleIndex++;
	pHeadNode->BIR.Header.HeaderVersion      = BIR_HEADER_VERSION;
	pHeadNode->BIR.Header.Type = CSSM_HRS_BIR_DATA_TYPE_PROCESSED;
	pHeadNode->BIR.Header.Format.FormatOwner = BIO_DATA_FORMAT_OWNER;
	pHeadNode->BIR.Header.Format.FormatID    = BIO_DATA_FORMAT_ID;
	pHeadNode->BIR.Header.Quality            = 100; /* by definition */		 
	pHeadNode->BIR.Header.FactorsMask        = CSSM_HRS_FACTOR_PASSWORD;

	pHeadNode->BIR.Header.Length   = sizeof(CSSM_HRS_BIR_HEADER) +
	                                 sizeof(BIO_DATA_HEADER) + HashedPW.Length;
	
	/* Finish the bio data header here; salts were set above */
	bioDataHeader.PWLength         = HashedPW.Length;
	bioDataHeader.PayloadLength    = 0;

	/* This BIR is just another incarnation of the input BIR -
	 * it has the same purpose */
	pHeadNode->BIR.Header.PurposeMask        = pBIRtoProcess->Header.PurposeMask;

	/* Finish BIR */
	pHeadNode->BIR.Signature     = NULL;
	pHeadNode->BIR.BiometricData = 
		(CSSM_HRS_BIR_BIOMETRIC_DATA*) 
			Addin_malloc(HashedPW.Length + sizeof(BIO_DATA_HEADER), NULL);
    if (NULL == pHeadNode->BIR.BiometricData)
    {
        Addin_free(HashedPW.Data, NULL);

        return CSSMERR_HRS_MEMORY_ERROR;
    }

	memcpy((void*)pHeadNode->BIR.BiometricData, (void*)&bioDataHeader,
	       sizeof(BIO_DATA_HEADER));
	memcpy((void*)(pHeadNode->BIR.BiometricData + sizeof(BIO_DATA_HEADER)), 
           (void*)HashedPW.Data, HashedPW.Length);	
	
	/* Free the memory that the CSSM_DigestData(...) function 
	 * allocated for the hashed password.                     */
	Addin_free(HashedPW.Data, NULL);

    return CSSM_OK;
}


CSSM_RETURN _hrs_VerifyMatch(
                    const ADDIN_ATTACH_DATA     *pAttachData,
                    const ADDIN_LOAD_DATA       *pLoadData,
                    CSSM_HANDLE                  ModuleHandle,
                    const CSSM_HRS_BIR          *pCapturedBIR,
                    const CSSM_HRS_BIR          *pTemplateBIR,
                    CSSM_BOOL *Result,
                    CSSM_DATA_PTR *Payload )
{
    CSSM_RETURN			 ret						= CSSM_OK;
	CSSM_DATA            PWtoHash;
	CSSM_DATA            HashedPW					= {0, NULL};
	CSSM_CC_HANDLE		 PBEContext					= 0;
	CSSM_CC_HANDLE		 DecryptionContext			= 0;
	CSSM_KEY			 PBEKey;
	CSSM_DATA			 DecryptedPayload;	
	CSSM_DATA			 EncryptedPayload;
	CSSM_DATA			 DecryptionOverflow			= {0, NULL};
	CSSM_DATA			 DeriveParameter;
	BIO_DATA_HEADER      bioDataHeader;
	uint32				 DecryptedPayloadLength		= 0;	
	CSSM_DATA            PBESalt;
	CSSM_DATA            InitVectorForIterations;	
	uint8				 PBESaltData[PBE_SALT_SIZE];
	uint8				 InitVectorData[INIT_VECTOR_SIZE];
	CSSM_PKCS5_PBKDF1_PARAMS	 PBEParameters;


    /*----------------------------------------------------------------------*
	 * In this BSP, the ProcessedBIR parameter is something of a misnomer - *
	 * if the captured BIR has been "processed", then the hash is the only  *
	 * thing included in the BIR passed in.  This is NOT acceptable,        *
	 * because we have no idea if the BIR wasn't just pulled out of the     *
	 * application's database and sent over twice - of course that would    *
	 * be verified.  The only way we have of knowing the person being       *
	 * represented is who they truly say they are is to get some kind of    *
	 * "secret" from them - in this case the PLAINTEXT password.  If we     *
	 * don't do the hash within this function, then we've got no guarantee  *
	 * that the person knows the secret at all.  Therefore, the BIR given   *
	 * for verification MUST be of the RAW data type, and NOT the PROCESSED *
	 * data type!                                                           *
	 *----------------------------------------------------------------------*/

	/* Check for RAW data type - signed flag doesn't pertain to this check
	 * and can be stripped */
	if (CSSM_HRS_BIR_DATA_TYPE_RAW != 
		(pCapturedBIR->Header.Type & ~CSSM_HRS_BIR_DATA_TYPE_SIGNED))
		return (CSSMERR_HRS_INVALID_BIR);

	/* Check that both BIRs are using the proper biometric data format */
	if (BIO_DATA_FORMAT_ID    != pCapturedBIR->Header.Format.FormatID ||
		BIO_DATA_FORMAT_ID    != pTemplateBIR->Header.Format.FormatID ||
		BIO_DATA_FORMAT_OWNER != pCapturedBIR->Header.Format.FormatOwner ||
		BIO_DATA_FORMAT_OWNER != pTemplateBIR->Header.Format.FormatOwner)
		return (CSSMERR_HRS_UNSUPPORTED_FORMAT);

	/* Check that both BIRs have the proper purpose flags */
	if (CSSM_HRS_PURPOSE_VERIFY != pCapturedBIR->Header.PurposeMask ||
		CSSM_HRS_PURPOSE_VERIFY != pTemplateBIR->Header.PurposeMask)
		return (CSSMERR_HRS_INCONSISTENT_PURPOSE);

	/*-----------------------------------------------------*
	 * Hash the raw password for comparison to stored hash *
	 *-----------------------------------------------------*/

	/* Prepare the buffer with the salted password for hashing */
	memcpy((void*)&bioDataHeader, (void*)pTemplateBIR->BiometricData,
	       sizeof(BIO_DATA_HEADER));

	PWtoHash.Length = pCapturedBIR->Header.Length - sizeof(CSSM_HRS_BIR_HEADER) 
							- sizeof(BIO_DATA_HEADER) + HASH_SALT_SIZE;
	PWtoHash.Data   = Addin_malloc(PWtoHash.Length, NULL);
    if (NULL == PWtoHash.Data)
    {
        return CSSMERR_HRS_MEMORY_ERROR;
    }

	memcpy(PWtoHash.Data,
           pCapturedBIR->BiometricData + sizeof(BIO_DATA_HEADER), 
           PWtoHash.Length - HASH_SALT_SIZE);
	memcpy(PWtoHash.Data + PWtoHash.Length - HASH_SALT_SIZE, 
           bioDataHeader.HashSalt,
           HASH_SALT_SIZE);

	/* This function will allocate the memory for the hashed
	 * password within the addin's memory space.  It must be
	 * EXPLICITLY freed later when it's no longer needed.     */
	ret = pLoadData->DigestData(pLoadData->DigestContext, &PWtoHash, 1, &HashedPW);
	
    /* Free the salted password's data buffer */
	Addin_free(PWtoHash.Data, NULL);

    if (CSSM_OK != ret) return (ret);

	
	/*------------------------------------------------------------*
	 * Check to see if this new hash and the old one are the same *
	 *------------------------------------------------------------*/
	
	if ( (HashedPW.Length != bioDataHeader.PWLength)  ||
	     (memcmp((void*)HashedPW.Data, 
		         (void*)(pTemplateBIR->BiometricData + sizeof(BIO_DATA_HEADER)),
                 HashedPW.Length)))	 
	{
		/* Hashes were NOT the same; release no info or privileges */
		*Result  = CSSM_FALSE;
        if (NULL != Payload)
            *Payload = NULL;

		/* Free the password's hash data */
		Addin_free(HashedPW.Data, NULL);

		return (CSSM_OK);
	}
	
	/* Free the password's hash data */
	Addin_free(HashedPW.Data, NULL);

	/* Hashes WERE the same; verify user! */
	*Result = CSSM_TRUE;

    if (NULL != Payload)
    {
	    /*-------------------------------------------*
	     * Return payload to the calling application *
	     *-------------------------------------------*/

	    *Payload = (CSSM_DATA*)App_Malloc(ModuleHandle, sizeof(CSSM_DATA));
        if (NULL == *Payload)
        {
            return CSSMERR_HRS_MEMORY_ERROR;
        }

        /* If there is no payload, we're done */
        if (bioDataHeader.PayloadLength <= 0)
        {
            (*Payload)->Data   = NULL;
            (*Payload)->Length = 0;

            return (CSSM_OK);
        }

	    /* If the payload wasn't encrypted, just copy it into the 
	     * application's memory space now */
	    if (!(CSSM_HRS_BIR_DATA_TYPE_ENCRYPTED & pTemplateBIR->Header.Type))
	    {	
		    (*Payload)->Data   = (uint8*)App_Malloc(ModuleHandle, bioDataHeader.PayloadLength);
            if (NULL == (*Payload)->Data)
            {
                App_Free(ModuleHandle, *Payload);

                return CSSMERR_HRS_MEMORY_ERROR;
            }

		    (*Payload)->Length = bioDataHeader.PayloadLength;
		    memcpy((void*)(*Payload)->Data, 
		           (void*)(pTemplateBIR->BiometricData + sizeof(BIO_DATA_HEADER) + 
			               bioDataHeader.PWLength),
					       bioDataHeader.PayloadLength);

		    return (CSSM_OK);
	    }

	    /* If the payload WAS encrypted, it must be decrypted first, 
	     * and then moved to the application's memory space */
	    
	    /* Create a crypto context for the Password Based Encryption (PBE)
	     * symmetric key derivation */
	    PBESalt.Length = PBE_SALT_SIZE;
	    PBESalt.Data   = PBESaltData;
	    memcpy(PBESalt.Data, bioDataHeader.PBESalt, PBE_SALT_SIZE);

	    if (CSSM_OK == (ret = pLoadData->CreateDeriveKeyContext(
									    pLoadData->CSPHandle,
									    CSSM_ALGID_PKCS5_PBKDF1_SHA1,
									    CSSM_ALGID_DES,
									    PBEKeyLength,
									    NULL,
									    NULL,
									    AlgorithmIterations,
									    &PBESalt,
									    NULL,
									    &PBEContext))) 
	    {
		    /* Initialize key struct to receive derived key */
		    memset(&PBEKey, 0, sizeof(CSSM_KEY));
	    
		    /* Wrap up password to pass to the CSP */	
		    InitVectorForIterations.Length = INIT_VECTOR_SIZE;
		    InitVectorForIterations.Data   = InitVectorData;
		    memcpy(InitVectorForIterations.Data, bioDataHeader.InitVector, 
			    INIT_VECTOR_SIZE);

		    PWtoHash.Length = pCapturedBIR->Header.Length - sizeof(CSSM_HRS_BIR_HEADER) 
							    - sizeof(BIO_DATA_HEADER);
		    PWtoHash.Data   = pCapturedBIR->BiometricData + sizeof(BIO_DATA_HEADER);
		    
		    PBEParameters.Passphrase = PWtoHash; 
		    PBEParameters.InitVector = InitVectorForIterations;
		    DeriveParameter.Data     = (uint8*)&PBEParameters;
		    DeriveParameter.Length   = sizeof(CSSM_PKCS5_PBKDF1_PARAMS);

		    /* Get the key */
		    if (CSSM_OK == (ret = pLoadData->DeriveKey(
										    PBEContext,
										    &DeriveParameter,
										    CSSM_KEYUSE_DECRYPT,
										    CSSM_KEYATTR_RETURN_DEFAULT,
										    NULL,
										    NULL,
										    &PBEKey)))
		    {					
			    /* Create cryptographic context for the payload encryption */
			    if (CSSM_OK == (ret = pLoadData->CreateSymmetricContext(
											    pLoadData->CSPHandle,
											    CSSM_ALGID_DES,
											    CSSM_ALGMODE_CBCPadIV8,
											    NULL,
											    &PBEKey,
											    &InitVectorForIterations,
											    CSSM_PADDING_PKCS5,
											    NULL,
											    &DecryptionContext)))
			    {
				    /* Prepare buffer to receive the decrypted payload.  DES should not
				     * produce a plaintext larger than the ciphertext.                  */
				    DecryptedPayload.Length = bioDataHeader.PayloadLength;
				    DecryptedPayload.Data   = Addin_malloc(DecryptedPayload.Length, NULL);
                    if (NULL != DecryptedPayload.Data)
                    {
				        /* Prepare a buffer for the encrypted payload to pass to the CSP */
				        EncryptedPayload.Length = bioDataHeader.PayloadLength;
				        EncryptedPayload.Data   = pTemplateBIR->BiometricData + 
										          sizeof(BIO_DATA_HEADER) + bioDataHeader.PWLength;

				        /* Decrypt the payload */
				        ret = pLoadData->DecryptData(
										        DecryptionContext,
										        &EncryptedPayload,
										        1,
										        &DecryptedPayload,
										        1,
										        &DecryptedPayloadLength,
										        &DecryptionOverflow);					
                    }
                    else
                        ret = CSSMERR_HRS_MEMORY_ERROR;
                
				    /* Release the decryption context */
				    pLoadData->DeleteContext(DecryptionContext);
			    }

			    /* Release the key and the crypto contexts now */
			    pLoadData->FreeKey(
							    pLoadData->CSPHandle,
							    NULL,
							    &PBEKey,
							    CSSM_TRUE);	
		    }

		    /* Release the key derivation context */
		    pLoadData->DeleteContext(PBEContext);			
	    }
	    /* Errors here */
	    if (CSSM_OK != ret)
        {
            App_Free(ModuleHandle, *Payload);

            if (NULL != DecryptedPayload.Data)
                Addin_free(DecryptedPayload.Data, NULL);
            if (NULL != DecryptionOverflow.Data)
                Addin_free(DecryptionOverflow.Data, NULL);

            return (ret);
        }


	    /* Move the encrypted payload into the application's memory space */
	    (*Payload)->Length = DecryptedPayloadLength;
	    (*Payload)->Data   = App_Malloc(ModuleHandle, DecryptedPayloadLength);
        if (NULL == (*Payload)->Data)
        {
            App_Free(ModuleHandle, *Payload);   
            Addin_free(DecryptedPayload.Data, NULL);
        
            if (NULL != DecryptionOverflow.Data)
                Addin_free(DecryptionOverflow.Data, NULL);

            return CSSMERR_HRS_MEMORY_ERROR;
        }

        /* if the actual payload decryption blob is smaller than 
         * the buffer allocated to the CSP to return it */
        if (DecryptedPayloadLength < DecryptedPayload.Length)
        {
            memcpy((void*)(*Payload)->Data, (void*)DecryptedPayload.Data,
		           DecryptedPayloadLength);

            Addin_free(DecryptedPayload.Data,	NULL);
        }
        else /* the blob is at least as large as the buffer */
        {
	        memcpy((void*)(*Payload)->Data, (void*)DecryptedPayload.Data,
		           DecryptedPayload.Length);

	        /* It's possible that the plaintext didn't fit into the buffer
	        * which was allocated to the CSP.  If so, copy the overflowed data 
	        * and deallocate the decryption buffers and password buffer now   */
	        memcpy((void*)((*Payload)->Data + DecryptedPayload.Length),
    	           (void*)DecryptionOverflow.Data, DecryptionOverflow.Length);

	        Addin_free(DecryptedPayload.Data,	NULL);
	        Addin_free(DecryptionOverflow.Data,	NULL);
        }
    }
    return CSSM_OK;
}

/*----------------------------------------------------------------------------------*
 *  HRS SPI Function Implementations                                                *
 *----------------------------------------------------------------------------------*/

CSSM_RETURN CSSMHRSI HRS_FreeBIRHandle(
                    CSSM_HANDLE         ModuleHandle,
                    CSSM_HRS_BIR_HANDLE BIRHandle )
{
    CSSM_RETURN          ret           = CSSM_OK;
    void                *pAddinContext = NULL;
	ADDIN_ATTACH_DATA   *pAttachData   = NULL;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

	/* Note to keep in mind from here on:  after MAF has been initialized
	 * the function must release the context with ADDIN_SPIEnd(...).  The
	 * return macro for doing this is SPI_RETURN(<errorcode>).  This will
	 * be used to return in each function in each case after MAF is in use. */

    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if (NULL == (pAttachData = Addin_GetAttachData()))	
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
    ret = _hrs_FreeBIRHandle(pAttachData, BIRHandle);
    if (CSSM_OK != ret) SPI_RETURN(ret);
	
	SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_GetBIRFromHandle(
                    CSSM_HANDLE          ModuleHandle,
                    CSSM_HRS_BIR_HANDLE  BIRHandle,
                    CSSM_HRS_BIR_PTR    *BIR )
{
    CSSM_RETURN        ret           = CSSM_OK;
    void              *pAddinContext = NULL;	
    CSSM_HRS_BIR      *pFoundBIR     = NULL;
	ADDIN_ATTACH_DATA *pAttachData   = NULL;

	/*--------------------------*
	 * Check for bad parameters *
	 *--------------------------*/

	if (NULL == BIR)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

	ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if (NULL == (pAttachData = Addin_GetAttachData()))	
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
     ret = _hrs_GetBIRFromHandle(pAttachData, BIRHandle, &pFoundBIR);
     if (CSSM_OK != ret) SPI_RETURN(ret);

    /* allocate memory in the Application's memory space
	 * for the BIR to be stored */
	*BIR = App_Malloc(ModuleHandle, sizeof(CSSM_HRS_BIR));
    if (NULL == *BIR)
        SPI_RETURN(CSSMERR_HRS_MEMORY_ERROR);

	/* allocate memory in the Application's memory space
	 * for the biometric data to be stored */
	(*BIR)->BiometricData = (CSSM_HRS_BIR_BIOMETRIC_DATA*)
		App_Malloc(ModuleHandle, pFoundBIR->Header.Length - sizeof(CSSM_HRS_BIR_HEADER));
    if (NULL == (*BIR)->BiometricData)
    {
        App_Free(ModuleHandle, *BIR);

        SPI_RETURN(CSSMERR_HRS_MEMORY_ERROR);
    }

    /* move BIR header to Application's memory space */
	memcpy((void*)&(*BIR)->Header, (void*)&(pFoundBIR->Header),
				sizeof(CSSM_HRS_BIR_HEADER));

	/* move the BIR biometric data to the Application's memory space */
	memcpy((void*)(*BIR)->BiometricData, 
           (void*)pFoundBIR->BiometricData,
           (*BIR)->Header.Length - sizeof(CSSM_HRS_BIR_HEADER));
	
	/* This BSP cannot support signatures */
	(*BIR)->Signature = NULL;

    /*------------------------------------------------------------*
     * Free the BIR now that it's been yielded to the application *
     *------------------------------------------------------------*/
    ret = _hrs_FreeBIRHandle(pAttachData, BIRHandle);
    if (CSSM_OK != ret) SPI_RETURN(ret);

    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_GetHeaderFromHandle(
                    CSSM_HANDLE             ModuleHandle,
                    CSSM_HRS_BIR_HANDLE     BIRHandle,
                    CSSM_HRS_BIR_HEADER_PTR Header )
{
    CSSM_RETURN        ret           = CSSM_OK;
    void              *pAddinContext = NULL;	
	CSSM_HRS_BIR      *pFoundBIR     = NULL;
	ADDIN_ATTACH_DATA *pAttachData   = NULL;

	/*--------------------------*
	 * Check for bad parameters *
	 *--------------------------*/

	if (NULL == Header)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (ret != CSSM_OK) return ret;

	if ((pAttachData = Addin_GetAttachData()) == NULL)	
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

    /*-------------------------------------------------*
     * Get the BIR from which the header will be taken *
     *-------------------------------------------------*/
    ret = _hrs_GetBIRFromHandle(pAttachData, BIRHandle, &pFoundBIR);
    if (CSSM_OK != ret) SPI_RETURN(ret);

	/* Copy the header into the application's memory space */
	memcpy((void*)Header, (void*)&(pFoundBIR->Header), 
		   sizeof(CSSM_HRS_BIR_HEADER));
    
    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_EnableEvents(
                    CSSM_HANDLE                 ModuleHandle,
                    CSSM_HRS_MODULE_EVENT_MASK *Events )
{
	return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_SetGUICallbacks(
                    CSSM_HANDLE ModuleHandle,
                    CSSM_HRS_GUI_STREAMING_CALLBACK GuiStreamingCallback,
                    void *GuiStreamingCallbackCtx,
                    CSSM_HRS_GUI_STATE_CALLBACK GuiStateCallback,
                    void *GuiStateCallbackCtx )
{
	return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_CancelGUICallbacks(
                    CSSM_HANDLE ModuleHandle )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_SetStreamCallback(
                    CSSM_HANDLE               ModuleHandle,
                    CSSM_HRS_STREAM_CALLBACK  StreamCallback,
                    void                     *StreamCallbackCtx )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_CancelStreamCallbacks(
                    CSSM_HANDLE ModuleHandle )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_StreamInputOutput(
                    CSSM_HANDLE   ModuleHandle,
                    CSSM_DATA_PTR InMessage,
                    CSSM_DATA_PTR OutMessage )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_Capture(
                    CSSM_HANDLE             ModuleHandle,
                    CSSM_HRS_BIR_PURPOSE    Purpose,
                    CSSM_HRS_BIR_HANDLE_PTR CapturedBIR,
                    sint32                  Timeout,
                    CSSM_HRS_BIR_HANDLE_PTR AuditData )
{
    CSSM_RETURN        ret               = CSSM_OK;
    void              *pAddinContext     = NULL;	
	ADDIN_ATTACH_DATA *pAttachData       = NULL;
	
	/*------------------------------------------------------------------*
	 * Check for bad paramters and initialize output for the error case *
	 *------------------------------------------------------------------*/
	
	if (NULL == CapturedBIR)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
	else
		*CapturedBIR = CSSM_HRS_INVALID_HANDLE;

    if (Timeout < -1)
        return CSSMERR_HRS_INVALID_DATA;
    if (-1 == Timeout)
        Timeout = DEFAULT_TIMEOUT;

	if (NULL != AuditData)	
		/* AuditData not supported */
		*AuditData = CSSM_HRS_UNSUPPORTED_HANDLE;
	
	if (Purpose != CSSM_HRS_PURPOSE_VERIFY &&		
		Purpose != CSSM_HRS_PURPOSE_ENROLL_FOR_VERIFICATION_ONLY)	
		return CSSMERR_HRS_PURPOSE_NOT_SUPPORTED;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (ret != CSSM_OK) return ret;

	if ((pAttachData = Addin_GetAttachData()) == NULL)	
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);
	
	/*-----------------------------------------------------------------*
	 * Check to ensure there are still handles available for more BIRs *
	 *-----------------------------------------------------------------*/
	if (pAttachData->HandleIndex < 0)
		/* Overflow, we've run out of handles */
		SPI_RETURN(CSSMERR_HRS_TOO_MANY_HANDLES);

    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
    ret = _hrs_Capture(pAttachData, Purpose, CapturedBIR, Timeout);
    if (CSSM_OK != ret) SPI_RETURN(ret);

    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_CreateTemplate(
                    CSSM_HANDLE               ModuleHandle,
                    const CSSM_HRS_INPUT_BIR *CapturedBIR,
                    const CSSM_HRS_INPUT_BIR *StoredTemplate,
                    CSSM_HRS_BIR_HANDLE_PTR   NewTemplate,
                    const CSSM_DATA          *Payload )
{
	CSSM_RETURN            ret           = CSSM_OK;
	void                  *pAddinContext = NULL;
    CSSM_HRS_BIR          *pBIRtoWrap    = NULL;
	ADDIN_ATTACH_DATA     *pAttachData   = NULL;
	const ADDIN_LOAD_DATA *pLoadData     = NULL;

	/*---------------------------------------------------------------*
	 * Check for bad paramters and initialize output for error cases *
	 *---------------------------------------------------------------*/
	if (NULL == CapturedBIR)
		return CSSMERR_HRS_INVALID_INPUT_POINTER;

	if (NULL != StoredTemplate)
		return CSSMERR_HRS_ADAPTATION_NOT_SUPPORTED;
	
	if (NULL == NewTemplate)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
	else
		*NewTemplate = CSSM_HRS_INVALID_HANDLE;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if ((NULL == (pAttachData = Addin_GetAttachData())) ||
        (NULL == (pLoadData   = Addin_GetLoadData())) )
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

	/*-----------------------------------------------------------------*
	 * Check to ensure there are still handles available for more BIRs *
	 *-----------------------------------------------------------------*/
	if (pAttachData->HandleIndex < 0)
		/* Overflow, we've run out of handles */
		SPI_RETURN(CSSMERR_HRS_TOO_MANY_HANDLES);
    
	/*-----------------------------------------*
	 * Find the BIR to be used in the template *
	 *-----------------------------------------*/

	if (CSSM_HRS_FULLBIR_INPUT == CapturedBIR->Form)
		pBIRtoWrap = CapturedBIR->InputBIR.BIR;
	else if (CSSM_HRS_BIR_HANDLE_INPUT == CapturedBIR->Form)
	{
		ret = _hrs_GetBIRFromHandle(
            pAttachData, *(CapturedBIR->InputBIR.BIRinBSP), &pBIRtoWrap);
        if (CSSM_OK != ret) SPI_RETURN(ret);
	}
	else 
		/* something went wrong - these are the only
		 * two forms supported by this BSP */
		SPI_RETURN(CSSMERR_HRS_INVALID_BIR);
	
    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
	ret = _hrs_CreateTemplate(  pAttachData, pLoadData, pBIRtoWrap, 
                                NewTemplate, Payload);
    if (CSSM_OK != ret) SPI_RETURN(ret);

	SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_Process(
                    CSSM_HANDLE               ModuleHandle,
                    const CSSM_HRS_INPUT_BIR *CapturedBIR,
                    CSSM_HRS_BIR_HANDLE_PTR   ProcessedBIR )
{
    CSSM_RETURN           ret            = CSSM_OK;
    void                  *pAddinContext = NULL;	
	CSSM_HRS_BIR          *pBIRtoProcess = NULL;	
	ADDIN_ATTACH_DATA     *pAttachData   = NULL;
    const ADDIN_LOAD_DATA *pLoadData     = NULL;
	

	/*-----------------------------------------------------------------------*
	 * Check for bad parameters and initialize output for the case of errors *
	 *-----------------------------------------------------------------------*/
	if (NULL == CapturedBIR)
		return CSSMERR_HRS_INVALID_INPUT_POINTER;

	if (NULL == ProcessedBIR)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
	else
		*ProcessedBIR = CSSM_HRS_INVALID_HANDLE;

	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/

	ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if ((NULL == (pAttachData = Addin_GetAttachData())) ||
        (NULL == (pLoadData   = Addin_GetLoadData())) )
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

	/*-----------------------------------------------------------------*
	 * Check to ensure there are still handles available for more BIRs *
	 *-----------------------------------------------------------------*/
	if (pAttachData->HandleIndex < 0)
		/* Overflow, we've run out of handles */
		SPI_RETURN(CSSMERR_HRS_TOO_MANY_HANDLES);

	/*-------------------------*
	 * Find the BIR to process *
	 *-------------------------*/

	if (CSSM_HRS_FULLBIR_INPUT == CapturedBIR->Form)
		pBIRtoProcess = CapturedBIR->InputBIR.BIR;
	else if (CSSM_HRS_BIR_HANDLE_INPUT == CapturedBIR->Form)
	{			
		ret = _hrs_GetBIRFromHandle(
            pAttachData, *(CapturedBIR->InputBIR.BIRinBSP), &pBIRtoProcess);
        if (CSSM_OK != ret) SPI_RETURN(ret);
	}
	else
		/* something went wrong - these are the only two 
		 * forms supported by this BSP */
		SPI_RETURN(CSSMERR_HRS_INVALID_BIR);

    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
	ret = _hrs_Process(pAttachData, pLoadData, pBIRtoProcess, ProcessedBIR);
    if (CSSM_OK != ret) SPI_RETURN(ret);


    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_VerifyMatch(
                    CSSM_HANDLE               ModuleHandle,
                    const CSSM_HRS_FAR       *MaxFARRequested,
                    const CSSM_HRS_FRR       *MaxFRRRequested,
                    const CSSM_BOOL          *FARPrecedence,
                    const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                    const CSSM_HRS_INPUT_BIR *StoredTemplate,
                    CSSM_HRS_BIR_HANDLE      *AdaptedBIR,
                    CSSM_BOOL                *Result,
                    CSSM_HRS_FAR_PTR          FARAchieved,
                    CSSM_HRS_FRR_PTR          FRRAchieved,
                    CSSM_DATA_PTR            *Payload )
{
    CSSM_RETURN               ret           = CSSM_OK;
    void                     *pAddinContext = NULL;	
	CSSM_HRS_BIR             *pCapturedBIR  = NULL;
	CSSM_HRS_BIR             *pTemplateBIR  = NULL;
	const ADDIN_ATTACH_DATA  *pAttachData   = NULL;
	const ADDIN_LOAD_DATA    *pLoadData     = NULL;
	
	/*-----------------------------------------------------------------------*
	 * Check for bad parameters and initialize output for the case of errors *
	 *-----------------------------------------------------------------------*/
	
	/* MaxFARRequested, MaxFRRRequested, and FARPrecedence are ignored 
	 * anyway, so we won't quibble over those. See note below! */

	if (NULL == ProcessedBIR || NULL == StoredTemplate)
		return CSSMERR_HRS_INVALID_INPUT_POINTER;
	
	if (NULL != AdaptedBIR)
		*AdaptedBIR = CSSM_HRS_UNSUPPORTED_HANDLE;

	if (NULL == FARAchieved || NULL == Result)
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
	
    *Result  = CSSM_FALSE;

    if (NULL != Payload)
        *Payload = NULL;
	
	/*--------------------------------------------------------------------------*
	 * A note to begin this function, the false accept rate (FAR) for this BSP  *
	 * is by nature next to nothing.  The chances of someone being validated    *
	 * by entering an incorrect password whose hash collides with the correct   *
	 * one seems to me to have about the same odds as a person getting hit by   *
	 * lightning 3 times in each of 3 states where he's won the lotto.  We'll   *
	 * just consider this to be zero, and ignore the parameter.  The false      *
	 * reject rate (FRR) is most certainly zero - if someone enters the correct *
	 * pasword, they WILL get access.  No question there, so this parameter can *
	 * also be ignored.  Similarly, the returned FAR and FRR achieved will      *
	 * always be 0 and 0.                                                       *
	 *--------------------------------------------------------------------------*/
	*FARAchieved = 0; 
    if (NULL != FRRAchieved)
        *FRRAchieved = 0;
	
	/*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/
	
    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if ((NULL == (pAttachData = Addin_GetAttachData())) ||
        (NULL == (pLoadData   = Addin_GetLoadData())) )
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

	/*------------------------*
	 * Find the BIR to verify *
	 *------------------------*/

	if (CSSM_HRS_FULLBIR_INPUT == ProcessedBIR->Form)
		pCapturedBIR = ProcessedBIR->InputBIR.BIR;
	else if (CSSM_HRS_BIR_HANDLE_INPUT == ProcessedBIR->Form)
	{
        ret = _hrs_GetBIRFromHandle(
            pAttachData, *(ProcessedBIR->InputBIR.BIRinBSP), &pCapturedBIR);
        if (CSSM_OK != ret) SPI_RETURN(ret);
	}
	else
		/* something went wrong - these are the only two 
		 * forms supported by this BSP */
		SPI_RETURN(CSSMERR_HRS_INVALID_BIR);

	/*-----------------------------------------*
	 * Find the template BIR to verify against *
	 *-----------------------------------------*/

	if (CSSM_HRS_FULLBIR_INPUT == StoredTemplate->Form)
		pTemplateBIR = StoredTemplate->InputBIR.BIR;
	else if (CSSM_HRS_BIR_HANDLE_INPUT == StoredTemplate->Form)
	{
        ret = _hrs_GetBIRFromHandle(
            pAttachData, *(StoredTemplate->InputBIR.BIRinBSP), &pTemplateBIR);
        if (CSSM_OK != ret) SPI_RETURN(ret);
	}
	else
		/* something went wrong - these are the only two 
		 * forms supported by this BSP */
		SPI_RETURN(CSSMERR_HRS_INVALID_BIR);

    /*----------------------------------------------------------------*
     * Call to the internal routine where the real functionality lies *
     *----------------------------------------------------------------*/
    ret = _hrs_VerifyMatch( pAttachData,    pLoadData,  ModuleHandle, pCapturedBIR, 
                            pTemplateBIR,   Result,     Payload);
    if (CSSM_OK != ret) SPI_RETURN(ret);

	SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_IdentifyMatch(
                    CSSM_HANDLE                         ModuleHandle,
                    const CSSM_HRS_FAR                 *MaxFARRequested,
                    const CSSM_HRS_FRR                 *MaxFRRRequested,
                    const CSSM_BOOL                    *FARPrecedence,
                    const CSSM_HRS_INPUT_BIR           *ProcessedBIR,
                    const CSSM_HRS_IDENTIFY_POPULATION *Population,
                    CSSM_BOOL                           Binning,
                    uint32                              MaxNumberOfResults,
                    uint32                             *NumberOfResults,
                    CSSM_HRS_CANDIDATE_ARRAY_PTR       *Candidates,
                    sint32                              Timeout )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_Enroll(
                    CSSM_HANDLE               ModuleHandle,
                    CSSM_HRS_BIR_PURPOSE      Purpose,
                    const CSSM_HRS_INPUT_BIR *StoredTemplate,
                    CSSM_HRS_BIR_HANDLE_PTR   NewTemplate,
                    const CSSM_DATA          *Payload,
                    sint32                    Timeout,
                    CSSM_HRS_BIR_HANDLE_PTR   AuditData )
{
    CSSM_RETURN              ret                = CSSM_OK;    
    void                    *pAddinContext      = NULL;
	CSSM_HRS_BIR_HANDLE      CapturedBIR;
	CSSM_HRS_BIR            *pInputCapturedBIR;
    ADDIN_ATTACH_DATA       *pAttachData        = NULL;
    const ADDIN_LOAD_DATA   *pLoadData          = NULL;

    /*-----------------------------------------------------------------------*
	 * Check for bad parameters and initialize output for the case of errors *
	 *-----------------------------------------------------------------------*/

    if (NULL != StoredTemplate)
		return CSSMERR_HRS_ADAPTATION_NOT_SUPPORTED;

    if (NULL == NewTemplate)
        return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
    else
        *NewTemplate = CSSM_HRS_INVALID_HANDLE;

    if (Timeout < -1)
        return CSSMERR_HRS_INVALID_DATA;
    if (-1 == Timeout)
        Timeout = DEFAULT_TIMEOUT;

	if (NULL != AuditData)
		*AuditData = CSSM_HRS_UNSUPPORTED_HANDLE;
	
    /*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/
	
    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if ((NULL == (pAttachData = Addin_GetAttachData())) ||
        (NULL == (pLoadData   = Addin_GetLoadData())) )
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

	/*---------------------------------------------*
	 * Capture the BIR that will be used to create *
	 * the new enrollment template                 *
	 *---------------------------------------------*/

	ret = _hrs_Capture(pAttachData, Purpose, &CapturedBIR, Timeout);
	if (CSSM_OK != ret) SPI_RETURN(ret);

	/*---------------------------------------------*
	 * Create the new enrollment template and      *
	 * associate payload with the captured BIR     *
	 *---------------------------------------------*/

	ret = _hrs_GetBIRFromHandle(pAttachData, CapturedBIR, 
                                &pInputCapturedBIR);
    if (CSSM_OK != ret) SPI_RETURN(ret);

	ret = _hrs_CreateTemplate(  pAttachData,        pLoadData,
                                pInputCapturedBIR,  NewTemplate,
								Payload);
	if (CSSM_OK != ret) SPI_RETURN(ret);

	/*---------------------------------------------------------*
	 * Release the BIR which was captured to perform the       *
	 * enrollment                                              *
	 *---------------------------------------------------------*/

	ret = _hrs_FreeBIRHandle(pAttachData, CapturedBIR);
	if (CSSM_OK != ret) SPI_RETURN(ret);

    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_Verify(
                    CSSM_HANDLE               ModuleHandle,
                    const CSSM_HRS_FAR       *MaxFARRequested,
                    const CSSM_HRS_FRR       *MaxFRRRequested,
                    const CSSM_BOOL          *FARPrecedence,
                    const CSSM_HRS_INPUT_BIR *StoredTemplate,
                    CSSM_HRS_BIR_HANDLE_PTR   AdaptedBIR,
                    CSSM_BOOL                *Result,
                    CSSM_HRS_FAR_PTR          FARAchieved,
                    CSSM_HRS_FRR_PTR          FRRAchieved,
                    CSSM_DATA_PTR            *Payload,
                    sint32                    Timeout,
                    CSSM_HRS_BIR_HANDLE_PTR   AuditData )
{
    CSSM_RETURN              ret                = CSSM_OK;    
    void                    *pAddinContext      = NULL;
	CSSM_HRS_BIR_HANDLE      CapturedBIR;
	CSSM_HRS_BIR            *pInputCapturedBIR;
    CSSM_HRS_BIR            *pInputTemplateBIR;
    ADDIN_ATTACH_DATA       *pAttachData        = NULL;
    const ADDIN_LOAD_DATA   *pLoadData          = NULL;
        
    /*-----------------------------------------------------------------------*
	 * Check for bad parameters and initialize output for the case of errors *
	 *-----------------------------------------------------------------------*/
	
	/* MaxFARRequested, MaxFRRRequested, and FARPrecedence are ignored 
	 * anyway, so we won't quibble over those. See note in HRS_VerifyMatch   */

	if (NULL == StoredTemplate)
		return CSSMERR_HRS_INVALID_INPUT_POINTER;
	
	if (NULL != AdaptedBIR)
		*AdaptedBIR = CSSM_HRS_UNSUPPORTED_HANDLE;

	if (NULL == FARAchieved || NULL == Result)        
		return CSSMERR_HRS_INVALID_OUTPUT_POINTER;
    
    if (Timeout < -1)
        return CSSMERR_HRS_INVALID_DATA;
    if (-1 == Timeout)
        Timeout = DEFAULT_TIMEOUT;

    if (NULL != AuditData)
		*AuditData = CSSM_HRS_UNSUPPORTED_HANDLE;

    *Result      = CSSM_FALSE;
    *FARAchieved = 0;

	if (NULL != Payload)
        *Payload = NULL;

    if (NULL != FRRAchieved)
        *FRRAchieved = 0;
	
    /*-------------------------*
	 * Initialize MAF Contexts *
	 *-------------------------*/
	
    ADDIN_SPI_BEGIN(ret, ModuleHandle, pAddinContext);
    if (CSSM_OK != ret) return ret;

	if ((NULL == (pAttachData = Addin_GetAttachData())) ||
        (NULL == (pLoadData   = Addin_GetLoadData())) )
		SPI_RETURN(CSSMERR_HRS_INTERNAL_ERROR);

	/*---------------------------------------------------------*
	 * Capture the BIR that will be used as verification       *
	 * against the input template BIR                          *
	 *---------------------------------------------------------*/

	ret = _hrs_Capture( pAttachData, CSSM_HRS_PURPOSE_VERIFY, &CapturedBIR, Timeout);
	if (CSSM_OK != ret) SPI_RETURN(ret);

	/*---------------------------------------------------------*
	 * Verify that the two BIRs had the same plaintext         *
	 * password, and if so, grab the payload from the template *
	 *---------------------------------------------------------*/

    ret = _hrs_GetBIRFromHandle(pAttachData, CapturedBIR, &pInputCapturedBIR);
    if (CSSM_OK != ret) SPI_RETURN(ret);

	/*-----------------------------------------*
	 * Find the template BIR to verify against *
	 *-----------------------------------------*/

	if (CSSM_HRS_FULLBIR_INPUT == StoredTemplate->Form)
		pInputTemplateBIR = StoredTemplate->InputBIR.BIR;
	else if (CSSM_HRS_BIR_HANDLE_INPUT == StoredTemplate->Form)
	{
        ret = _hrs_GetBIRFromHandle(
            pAttachData, *(StoredTemplate->InputBIR.BIRinBSP), &pInputTemplateBIR);
        if (CSSM_OK != ret) SPI_RETURN(ret);
	}
	else
		/* something went wrong - these are the only two 
		 * forms supported by this BSP */
		SPI_RETURN(CSSMERR_HRS_INVALID_BIR);

	ret = _hrs_VerifyMatch(pAttachData, pLoadData, ModuleHandle, 
						   pInputCapturedBIR, 
						   pInputTemplateBIR, 
						   Result, 
						   Payload);
	if (CSSM_OK != ret) SPI_RETURN(ret);

	/*---------------------------------------------------------*
	 * Release the BIR which was captured to perform the       *
	 * verification                                            *
	 *---------------------------------------------------------*/

	ret = _hrs_FreeBIRHandle(pAttachData, CapturedBIR);
	if (CSSM_OK != ret) SPI_RETURN(ret);

    SPI_RETURN(CSSM_OK);
}

CSSM_RETURN CSSMHRSI HRS_Identify(
                    CSSM_HANDLE                   ModuleHandle,
                    const CSSM_HRS_FAR           *MaxFARRequested,
                    const CSSM_HRS_FRR           *MaxFRRRequested,
                    const CSSM_BOOL              *FARPrecedence,
                    const CSSM_HRS_INPUT_BIR     *ProcessedBIR,
                    const CSSM_HRS_IDENTIFY_POPULATION *Population,
                    CSSM_BOOL                     Binning,
                    uint32                        MaxNumberOfResults,
                    uint32                       *NumberOfResults,
                    CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                    sint32                        Timeout,
                    CSSM_HRS_BIR_HANDLE_PTR       AuditData )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_Import(
                    CSSM_HANDLE                         ModuleHandle,
                    const CSSM_DATA                    *InputData,
                    CSSM_HRS_BIR_BIOMETRIC_DATA_FORMAT  InputFormat,
                    CSSM_HRS_BIR_PURPOSE                Purpose,
                    CSSM_HRS_BIR_HANDLE_PTR             ConstructedBIR )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_SetPowerMode(
                    CSSM_HANDLE         ModuleHandle,
                    CSSM_HRS_POWER_MODE PowerMode )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbOpen(
                    CSSM_HANDLE              ModuleHandle,
                    const uint8             *DbName,
                    CSSM_HRS_DB_ACCESS_TYPE  AccessRequest,
                    CSSM_HRS_DB_HANDLE_PTR   DbHandle,
                    CSSM_HRS_DB_CURSOR_PTR   Cursor )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbClose(
                    CSSM_HANDLE         ModuleHandle,
                    CSSM_HRS_DB_HANDLE  DbHandle )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbCreate(
                    CSSM_HANDLE               ModuleHandle,
                    const uint8              *DbName,
                    CSSM_HRS_DB_ACCESS_TYPE   AccessRequest,
                    CSSM_HRS_DB_HANDLE_PTR    DbHandle )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbDelete(
                    CSSM_HANDLE  ModuleHandle,
                    const uint8 *DbName )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbSetCursor(
                    CSSM_HANDLE             ModuleHandle,
                    CSSM_HRS_DB_HANDLE      DbHandle,
                    const CSSM_GUID        *KeyValue,
                    CSSM_HRS_DB_CURSOR_PTR  Cursor )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbFreeCursor(
                    CSSM_HANDLE            ModuleHandle,
                    CSSM_HRS_DB_CURSOR_PTR Cursor )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbStoreBIR(
                    CSSM_HANDLE               ModuleHandle,
                    const CSSM_HRS_INPUT_BIR *BIRToStore,
                    CSSM_HRS_DB_HANDLE        DbHandle,
                    CSSM_GUID_PTR             Uuid )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbGetBIR(
                    CSSM_HANDLE              ModuleHandle,
                    CSSM_HRS_DB_HANDLE       DbHandle,
                    const CSSM_GUID         *KeyValue,
                    CSSM_HRS_BIR_HANDLE_PTR  RetrievedBIR,
                    CSSM_HRS_DB_CURSOR_PTR   Cursor )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbGetNextBIR(
                    CSSM_HANDLE             ModuleHandle,
                    CSSM_HRS_DB_CURSOR_PTR  Cursor,
                    CSSM_HRS_BIR_HANDLE_PTR RetievedBIR,
                    CSSM_GUID_PTR           Uuid )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbQueryBIR(
                    CSSM_HANDLE               ModuleHandle,
                    CSSM_HRS_DB_HANDLE        DbHandle,
                    const CSSM_HRS_INPUT_BIR *BIRToQuery,
                    CSSM_GUID_PTR             Uuid )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}

CSSM_RETURN CSSMHRSI HRS_DbDeleteBIR(
                    CSSM_HANDLE         ModuleHandle,
                    CSSM_HRS_DB_HANDLE  DbHandle,
                    const CSSM_GUID    *KeyValue )
{
    return CSSMERR_HRS_FUNCTION_NOT_IMPLEMENTED;
}


/* Define the attach function.
 */
CSSM_RETURN HRS_ModuleAttach(
                   CSSM_MODULE_FUNCS_PTR *FuncTbl )
{
    static CSSM_MODULE_FUNCS FunctionList;
    static CSSM_SPI_HRS_FUNCS Functions;
    if (!FuncTbl)
    {
        return CSSMERR_CSSM_INVALID_POINTER;
    }
    FunctionList.ServiceType = CSSM_SERVICE_HRS;
    FunctionList.NumberOfServiceFuncs = sizeof(CSSM_SPI_HRS_FUNCS) /
    sizeof(
    CSSM_PROC_ADDR );
    FunctionList.ServiceFuncs = (CSSM_PROC_ADDR*)&Functions;
    Functions.FreeBIRHandle = HRS_FreeBIRHandle;
    Functions.GetBIRFromHandle = HRS_GetBIRFromHandle;
    Functions.GetHeaderFromHandle = HRS_GetHeaderFromHandle;
    Functions.EnableEvents = HRS_EnableEvents;
    Functions.SetGUICallbacks = HRS_SetGUICallbacks;
    Functions.CancelGUICallbacks = HRS_CancelGUICallbacks;
    Functions.SetStreamCallback = HRS_SetStreamCallback;
    Functions.CancelStreamCallbacks = HRS_CancelStreamCallbacks;
    Functions.StreamInputOutput = HRS_StreamInputOutput;
    Functions.Capture = HRS_Capture;
    Functions.CreateTemplate = HRS_CreateTemplate;
    Functions.Process = HRS_Process;
    Functions.VerifyMatch = HRS_VerifyMatch;
    Functions.IdentifyMatch = HRS_IdentifyMatch;
    Functions.Enroll = HRS_Enroll;
    Functions.Verify = HRS_Verify;
    Functions.Identify = HRS_Identify;
    Functions.Import = HRS_Import;
    Functions.SetPowerMode = HRS_SetPowerMode;
    Functions.DbOpen = HRS_DbOpen;
    Functions.DbClose = HRS_DbClose;
    Functions.DbCreate = HRS_DbCreate;
    Functions.DbDelete = HRS_DbDelete;
    Functions.DbSetCursor = HRS_DbSetCursor;
    Functions.DbFreeCursor = HRS_DbFreeCursor;
    Functions.DbStoreBIR = HRS_DbStoreBIR;
    Functions.DbGetBIR = HRS_DbGetBIR;
    Functions.DbGetNextBIR = HRS_DbGetNextBIR;
    Functions.DbQueryBIR = HRS_DbQueryBIR;
    Functions.DbDeleteBIR = HRS_DbDeleteBIR;
    *FuncTbl = &FunctionList;
    return CSSM_OK;
}


