/*-----------------------------------------------------------------------
 * File: ACM_ACI.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/* Implements the SPI shims needed to provide resource protection around
 * existing AC functionality using MAF.
 */

/* MAF stuff */
#include "maf_include.h"

/* Generic CSSM stuff */
#include "cssm.h"
#include "cssmspi.h"
#include "cssmcli.h"

#ifdef ISL_INCLUDED
/* EISL stuff */
#include "eislapi.h"
#endif

/* Local headers */
#include "acm_aci.h"

CSSM_RETURN CSSMAPI 
ACM_AuthCompute(CSSM_AC_HANDLE ACHandle,
                const CSSM_TUPLEGROUP *BaseAuthorizations,
                const CSSM_TUPLEGROUP *Credentials,
                uint32 NumberOfRequestors,
                const CSSM_LIST *Requestors,
                const CSSM_LIST *RequestedAuthorizationPeriod,
                const CSSM_LIST *RequestedAuthorization,
                CSSM_TUPLEGROUP_PTR AuthorizationResult)
{
    void *pLockRef = NULL;
    
    CSSM_RETURN ret = CSSM_OK;

    /* Setup the current execution context */
    ADDIN_SPI_BEGIN(ret, ACHandle, pLockRef );
    if (CSSM_OK != ret)
    {
        return ret;
    }
    if (pLockRef == NULL)
    {
        return CSSMERR_AC_INTERNAL_ERROR;
    }
    
    /* Perform the actual service */
    ret = AC_AuthCompute(ACHandle, 
                         BaseAuthorizations,
                         Credentials,
                         NumberOfRequestors,
                         Requestors,
                         RequestedAuthorizationPeriod,
                         RequestedAuthorization,
                         AuthorizationResult);

    /* Shut down the SPI handler */
    Addin_SPIEnd( pLockRef );

    /* Return the proper value */
    return ret;
}

CSSM_RETURN CSSMAPI ACM_PassThrough(CSSM_AC_HANDLE ACHandle, 
                                    CSSM_TP_HANDLE TPHandle, 
                                    CSSM_CL_HANDLE CLHandle, 
                                    CSSM_CC_HANDLE CCHandle, 
                                    const CSSM_DL_DB_LIST *DBList,
                                    uint32 PassThroughId, 
                                    const void * InputParams,
                                    void **OutputParams)
{
    void *pLockRef = NULL;
    
    CSSM_RETURN ret = CSSM_OK;
    /* Setup the current execution context */
    ADDIN_SPI_BEGIN(ret, ACHandle, pLockRef );
    if (CSSM_OK != ret)
    {
        return ret;
    }
    if (pLockRef == NULL)
    {
        return CSSMERR_AC_INTERNAL_ERROR;
    }
    
    /* Perform the actual service */
    ret = AC_PassThrough (ACHandle, 
                          TPHandle, 
                          CLHandle, 
                          CCHandle, 
                          DBList,
                          PassThroughId, 
                          InputParams,
                          OutputParams);

    /* Shut down the SPI handler */
    Addin_SPIEnd( pLockRef );

    /* Return the proper value */
    return ret;
}

