/*-----------------------------------------------------------------------
 * File: CSM_KEYMGR_INTERNAL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef __CSM_KEYMGR_INTERNAL_H__
#define __CSM_KEYMGR_INTERNAL_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _CSSMTYPE_H
#error cssmtype.h must be included before this file
#endif
#ifndef __COLLECTN_H__
#error maf_collectn.h must be included before this file
#endif
#ifndef __CSM_KEYMGR_H__
#error csm_keymgr.h must be included before this file
#endif
#ifndef __CSM_KEYMGR_DB_H__
#error csm_keymgr_db.h must be included before this file
#endif

#ifdef VMS
/*
        123456789012345678901234567890123456789012345678901234567890
                 ^         ^         ^         ^         ^         ^
                 10        20        30        40        50        60
*/
#define csm_kmFindKeyByHash        csmint_kmFindKeyByHash
#define csm_kmFindKeyByName        csmint_kmFindKeyByName
#define csm_kmFindKeyByHandle      csmint_kmFindKeyByHandle
#endif
/*
 * DB record creation.
 */
CSSM_RETURN csm_kmBuildNewDBRecord(
                        const CSSM_KEY *pKey,
                        const CSSM_KEY *pRefKey,
                        const CSSM_DATA *pName,
                        const CSSM_RESOURCE_CONTROL_CONTEXT *pCredAndAclEntry,
                        CSM_KM_DB_RECORD **ppRecord );

/*
 * Key hashing function.
 */
CSSM_RETURN csm_kmHashKey(
                        const CSSM_KEY * pKey,
                        uint8 * pDigest );

/*
 * Key wrapping and unwrapping.
 */
#define CSM_KM_PBE_ITERATIONS               (1000)
#define CSM_KM_PBE_MAX_KEY_LENGTH           (64)

CSSM_RETURN csm_kmWrapKey(
                        CSSM_HANDLE hModule,
                        const CSSM_KEY *pKey,
                        CSSM_KEY *pWrappedKey,
                        const CSSM_DATA *pDeriveSrc,
                        CSSM_DATA_PTR pPbeSalt,
                        CSSM_DATA_PTR pIV );

CSSM_RETURN csm_kmUnwrapKey(
                        CSSM_HANDLE hModule,
                        const CSSM_KEY *pWrappedKey,
                        CSSM_KEY *pKey,
                        const CSSM_ACCESS_CREDENTIALS *pAccessCred,
                        CSSM_SAMPLE_TYPE ExpectedSample,
                        const CSSM_DATA *pPrompt,
                        const CSSM_DATA *pPbeSalt,
                        const CSSM_DATA *pIV );

/*
 * Handle Support
 * --------------
 */

typedef struct _csm_km_handles_node
{
    CSM_KM_HANDLE handle;
    struct _csm_km_handles_node * next;
} CSM_KM_HANDLES_NODE;

typedef struct _csm_km_handles
{
    CSM_KM_HANDLES_NODE * handles;
} CSM_KM_HANDLES;

CSSM_RETURN csm_kmInitHandles(
                        CSM_KM_HANDLES * pHandles );

CSSM_RETURN csm_kmNoHandles(
                        CSM_KM_HANDLES * pHandles,
                        CSSM_BOOL * pNoHandles );

CSSM_RETURN csm_kmInsertHandle(
                        CSM_KM_HANDLES * pHandles,
                        CSM_KM_HANDLE handle );

CSSM_RETURN csm_kmGetHandle(
                        CSM_KM_HANDLES * pHandles,
                        CSM_KM_HANDLE * pHandle );

CSSM_RETURN csm_kmDeleteAllHandles(
                        CSM_KM_HANDLES * pHandles );

CSSM_RETURN csm_kmTermHandles(
                        CSM_KM_HANDLES * pHandles );


/*
 * CSM_KM_KEY_FILE_HANDLES related data types and functions.
 */

typedef struct _csm_km_key_file_handles_node
{
    CSM_KM_HANDLE hKeyFile;
    int fd;
    MAF_LIST_COLLECTION keys;
    CSM_KM_HANDLES discardedKeyHandles;
    CSM_KM_HANDLE keysNextHandle;
    /*
     * There should be some way to link all processes accessing
     * the key file together.
     */
} CSM_KM_KEY_FILE_HANDLES_NODE;

/*
 * CSM_KM_KEYS related data types and functions.
 */

typedef struct _csm_km_keys_node
{
    CSM_KM_HANDLE hKey;
    long dbIndex;               /* Easier to delete, but delete is dumb. */
    CSM_KM_DB_RECORD * pRecord;
} CSM_KM_KEYS_NODE;

typedef struct _csm_km_keys_find_duplicate
{
    uint8 * pHash;
    const CSSM_DATA * pName;
} CSM_KM_KEYS_FIND_DUPLICATE;


/*
 * Functions that support Key Manager's use of MLC.
 */

void MLC_FASTCALL csm_kmTearDownKeyFileHandle(
                        void *pItem );      /* CSM_KM_KEY_FILE_HANDLES_NODE * */

int MLC_FASTCALL csm_kmFindKeyFileHandle(
                        void *pItem,        /* CSM_KM_KEY_FILE_HANDLES_NODE * */
                        void *pKey );       /* CSM_KM_HANDLE *                */

void MLC_FASTCALL csm_kmTearDownKeys(
                        void *pItem );      /* CSM_KM_KEYS_NODE *             */

int MLC_FASTCALL csm_kmFindKeyByHash(
                        void *pItem,        /* CSM_KM_KEYS_NODE *             */
                        void *pKey );       /* CSSM_DATA *                    */

int MLC_FASTCALL csm_kmFindKeyByName(
                        void *pItem,        /* CSM_KM_KEYS_NODE *             */
                        void *pKey );       /* CSSM_DATA *                    */

int MLC_FASTCALL csm_kmFindKeyByHandle(
                        void *pItem,        /* CSM_KM_KEYS_NODE *             */
                        void *pKey );       /* CSM_KM_KEY_HANDLE *            */

int MLC_FASTCALL csm_kmFindKeyDuplicate(
                        void *pItem,        /* CSM_KM_KEYS_NODE *             */
                        void *pKey );       /* CSM_KM_KEYS_FIND_DUPLICATE *   */

#ifdef __cplusplus
}
#endif

#endif /* ifndef __CSM_KEYMGR_INTERNAL_H__ */
