/*-----------------------------------------------------------------------
 * File: TPM_CSPI.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#ifndef __TPM_CSPI_H__
#define __TPM_CSPI_H__

/* MAF Interfaces */
CSSM_RETURN CSSMAPI TPM_SubmitCredRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_AUTHORITY_REQUEST_TYPE RequestType,
                        const CSSM_TP_REQUEST_SET *RequestInput,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthContext,
                        sint32 *EstimatedTime,
                        CSSM_DATA_PTR ReferenceIdentifier );

CSSM_RETURN CSSMAPI TPM_RetrieveCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        sint32 *EstimatedTime,
                        CSSM_BOOL *ConfirmationRequired,
                        CSSM_TP_RESULT_SET_PTR *RetrieveOutput );

CSSM_RETURN CSSMAPI TPM_ConfirmCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        const CSSM_TP_CONFIRM_RESPONSE *Responses,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority );

CSSM_RETURN CSSMAPI TPM_ReceiveConfirmation(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        CSSM_TP_CONFIRM_RESPONSE_PTR *Responses,
                        sint32 *ElapsedTime );

CSSM_RETURN CSSMAPI TPM_CertReclaimKey(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        uint32 CertIndex,
                        CSSM_LONG_HANDLE KeyCacheHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry );

CSSM_RETURN CSSMAPI TPM_CertReclaimAbort(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_LONG_HANDLE KeyCacheHandle );

CSSM_RETURN CSSMAPI TPM_FormRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_FORM_TYPE FormType,
                        CSSM_DATA_PTR BlankForm );

CSSM_RETURN CSSMAPI TPM_FormSubmit(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_TP_FORM_TYPE FormType,
                        const CSSM_DATA *Form,
                        const CSSM_TP_AUTHORITY_ID *ClearanceAuthority,
                        const CSSM_TP_AUTHORITY_ID *RepresentedAuthority,
                        CSSM_ACCESS_CREDENTIALS_PTR Credentials );

CSSM_RETURN CSSMAPI TPM_CertGroupVerify(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_CERTGROUP *CertGroupToBeVerified,
                        const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR VerifyContextResult );

CSSM_RETURN CSSMAPI TPM_CertCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CertFields,
                        CSSM_DATA_PTR CertTemplate );

CSSM_RETURN CSSMAPI TPM_CertGetAllTemplateFields(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DATA *CertTemplate,
                        uint32 *NumberOfFields,
                        CSSM_FIELD_PTR *CertFields );

CSSM_RETURN CSSMAPI TPM_CertSign(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_DATA *CertTemplateToBeSigned,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
                        CSSM_DATA_PTR SignedCert );


CSSM_RETURN CSSMAPI TPM_CrlVerify(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ENCODED_CRL *CrlToBeVerified,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult );

CSSM_RETURN CSSMAPI TPM_CrlCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CrlFields,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TPM_CertRevoke(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRevoked,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_TP_CERTCHANGE_REASON Reason,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TPM_CertRemoveFromCrlTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRemoved,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TPM_CrlSign(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_ENCODED_CRL *CrlToBeSigned,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
                        CSSM_DATA_PTR SignedCrl );

CSSM_RETURN CSSMAPI TPM_ApplyCrlToDb(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ENCODED_CRL *CrlToBeApplied,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *ApplyCrlVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR ApplyCrlVerifyResult );

CSSM_RETURN CSSMAPI TPM_CertGroupConstruct(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        const void *ConstructParams,
                        const CSSM_CERTGROUP *CertGroupFrag,
                        CSSM_CERTGROUP_PTR *CertGroup );

CSSM_RETURN CSSMAPI TPM_CertGroupPrune(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        const CSSM_CERTGROUP *OrderedCertGroup,
                        CSSM_CERTGROUP_PTR *PrunedCertGroup );

CSSM_RETURN CSSMAPI TPM_CertGroupToTupleGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        CSSM_TUPLEGROUP_PTR *TupleGroup );

CSSM_RETURN CSSMAPI TPM_TupleGroupToCertGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_TUPLEGROUP *TupleGroup,
                        CSSM_CERTGROUP_PTR *CertTemplates );

CSSM_RETURN CSSMAPI TPM_PassThrough(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        uint32 PassThroughId,
                        const void *InputParams,
                        void **OutputParams );

/* TP interfaces */
CSSM_RETURN CSSMAPI TP_SubmitCredRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_AUTHORITY_REQUEST_TYPE RequestType,
                        const CSSM_TP_REQUEST_SET *RequestInput,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthContext,
                        sint32 *EstimatedTime,
                        CSSM_DATA_PTR ReferenceIdentifier );

CSSM_RETURN CSSMAPI TP_RetrieveCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        sint32 *EstimatedTime,
                        CSSM_BOOL *ConfirmationRequired,
                        CSSM_TP_RESULT_SET_PTR *RetrieveOutput );

CSSM_RETURN CSSMAPI TP_ConfirmCredResult(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        const CSSM_TP_CALLERAUTH_CONTEXT *CallerAuthCredentials,
                        const CSSM_TP_CONFIRM_RESPONSE *Responses,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority );

CSSM_RETURN CSSMAPI TP_ReceiveConfirmation(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_DATA *ReferenceIdentifier,
                        CSSM_TP_CONFIRM_RESPONSE_PTR *Responses,
                        sint32 *ElapsedTime );

CSSM_RETURN CSSMAPI TP_CertReclaimKey(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        uint32 CertIndex,
                        CSSM_LONG_HANDLE KeyCacheHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry );

CSSM_RETURN CSSMAPI TP_CertReclaimAbort(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_LONG_HANDLE KeyCacheHandle );

CSSM_RETURN CSSMAPI TP_FormRequest(
                        CSSM_TP_HANDLE TPHandle,
                        const CSSM_TP_AUTHORITY_ID *PreferredAuthority,
                        CSSM_TP_FORM_TYPE FormType,
                        CSSM_DATA_PTR BlankForm );

CSSM_RETURN CSSMAPI TP_FormSubmit(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_TP_FORM_TYPE FormType,
                        const CSSM_DATA *Form,
                        const CSSM_TP_AUTHORITY_ID *ClearanceAuthority,
                        const CSSM_TP_AUTHORITY_ID *RepresentedAuthority,
                        CSSM_ACCESS_CREDENTIALS_PTR Credentials );

CSSM_RETURN CSSMAPI TP_CertGroupVerify(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_CERTGROUP *CertGroupToBeVerified,
                        const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR VerifyContextResult );

CSSM_RETURN CSSMAPI TP_CertCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CertFields,
                        CSSM_DATA_PTR CertTemplate );

CSSM_RETURN CSSMAPI TP_CertGetAllTemplateFields(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DATA *CertTemplate,
                        uint32 *NumberOfFields,
                        CSSM_FIELD_PTR *CertFields );

CSSM_RETURN CSSMAPI TP_CertSign(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_DATA *CertTemplateToBeSigned,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
                        CSSM_DATA_PTR SignedCert );


CSSM_RETURN CSSMAPI TP_CrlVerify(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ENCODED_CRL *CrlToBeVerified,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *VerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult );

CSSM_RETURN CSSMAPI TP_CrlCreateTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        uint32 NumberOfFields,
                        const CSSM_FIELD *CrlFields,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TP_CertRevoke(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRevoked,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_TP_CERTCHANGE_REASON Reason,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TP_CertRemoveFromCrlTemplate(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DATA *OldCrlTemplate,
                        const CSSM_CERTGROUP *CertGroupToBeRemoved,
                        const CSSM_CERTGROUP *RevokerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *RevokerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR RevokerVerifyResult,
                        CSSM_DATA_PTR NewCrlTemplate );

CSSM_RETURN CSSMAPI TP_CrlSign(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_ENCODED_CRL *CrlToBeSigned,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *SignerVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR SignerVerifyResult,
                        CSSM_DATA_PTR SignedCrl );

CSSM_RETURN CSSMAPI TP_ApplyCrlToDb(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_ENCODED_CRL *CrlToBeApplied,
                        const CSSM_CERTGROUP *SignerCertGroup,
                        const CSSM_TP_VERIFY_CONTEXT *ApplyCrlVerifyContext,
                        CSSM_TP_VERIFY_CONTEXT_RESULT_PTR ApplyCrlVerifyResult );

CSSM_RETURN CSSMAPI TP_CertGroupConstruct(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CSP_HANDLE CSPHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        const void *ConstructParams,
                        const CSSM_CERTGROUP *CertGroupFrag,
                        CSSM_CERTGROUP_PTR *CertGroup );

CSSM_RETURN CSSMAPI TP_CertGroupPrune(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        const CSSM_CERTGROUP *OrderedCertGroup,
                        CSSM_CERTGROUP_PTR *PrunedCertGroup );

CSSM_RETURN CSSMAPI TP_CertGroupToTupleGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_CERTGROUP *CertGroup,
                        CSSM_TUPLEGROUP_PTR *TupleGroup );

CSSM_RETURN CSSMAPI TP_TupleGroupToCertGroup(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_TUPLEGROUP *TupleGroup,
                        CSSM_CERTGROUP_PTR *CertTemplates );

CSSM_RETURN CSSMAPI TP_PassThrough(
                        CSSM_TP_HANDLE TPHandle,
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_CC_HANDLE CCHandle,
                        const CSSM_DL_DB_LIST *DBList,
                        uint32 PassThroughId,
                        const void *InputParams,
                        void **OutputParams );


#endif /* __TPM_CSPI_H__ */
