/*-----------------------------------------------------------------------
 * File: MAF_API.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
#include "maf_include.h"

CSSM_RETURN CSSMAPI MAFCALL_ModuleLoad(
                        const CSSM_GUID *ModuleGuid,
                        CSSM_KEY_HIERARCHY KeyHierarchy,
                        CSSM_API_ModuleEventHandler AppNotifyCallback,
                        const void *AppNotifyCallbackCtx )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->ModuleLoad( ModuleGuid,
                                    KeyHierarchy,
                                    AppNotifyCallback,
                                    AppNotifyCallbackCtx );
    }
    
    return rv;
}

CSSM_RETURN CSSMAPI MAFCALL_ModuleUnload(
                        const CSSM_GUID *ModuleGuid,
                        CSSM_API_ModuleEventHandler AppNotifyCallback,
                        const void *AppNotifyCallbackCtx )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->ModuleUnload( ModuleGuid,
                                      AppNotifyCallback,
                                      AppNotifyCallbackCtx );
    }
    
    return rv;
}

CSSM_RETURN CSSMAPI MAFCALL_ModuleAttach(
                        const CSSM_GUID *ModuleGuid,
                        const CSSM_VERSION *Version,
                        const CSSM_API_MEMORY_FUNCS *MemoryFuncs,
                        uint32 SubserviceID,
                        CSSM_SERVICE_TYPE SubServiceType,
                        CSSM_ATTACH_FLAGS AttachFlags,
                        CSSM_KEY_HIERARCHY KeyHierarchy,
                        CSSM_FUNC_NAME_ADDR *FunctionTable,
                        uint32 NumFunctionTable,
                        const void *Reserved,
                        CSSM_MODULE_HANDLE_PTR NewModuleHandle )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->ModuleAttach( ModuleGuid,
                                      Version,
                                      MemoryFuncs,
                                      SubserviceID,
                                      SubServiceType,
                                      AttachFlags,
                                      KeyHierarchy,
                                      FunctionTable,
                                      NumFunctionTable,
                                      Reserved,
                                      NewModuleHandle );
    }
    
    return rv;
}

CSSM_RETURN CSSMAPI MAFCALL_ModuleDetach(
                        CSSM_MODULE_HANDLE ModuleHandle )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->ModuleDetach( ModuleHandle );
    }
    
    return rv;
}

CSSM_RETURN CSSMAPI MAFCALL_CL_CertGetFirstFieldValue(
                        CSSM_CL_HANDLE CLHandle,
                        const CSSM_DATA *Cert,
                        const CSSM_OID *CertField,
                        CSSM_HANDLE_PTR ResultsHandle,
                        uint32 *NumberOfMatchedFields,
                        CSSM_DATA_PTR *Value )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->CL_CertGetFirstFieldValue( CLHandle,
                                                   Cert,
                                                   CertField,
                                                   ResultsHandle,
                                                   NumberOfMatchedFields,
                                                   Value );
    }
    
    return rv;
}

CSSM_RETURN CSSMAPI MAFCALL_CL_CertAbortQuery(
                        CSSM_CL_HANDLE CLHandle,
                        CSSM_HANDLE ResultsHandle )
{
    const ADDIN_LOAD_DATA *pLoadData = NULL;
    CSSM_RETURN rv = CSSM_OK;

    /* Fetch the entry point */
    pLoadData = Addin_GetLoadData();
    if ( pLoadData == NULL )
    {
        ERR( rv = CSSM_ERRCODE_INTERNAL_ERROR );
    }
    else
    {
        /* Call the API function */
        rv = pLoadData->CL_CertAbortQuery( CLHandle,
                                           ResultsHandle );
    }
    
    return rv;
}
