/*-----------------------------------------------------------------------
 * File: MEMPOOL.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
 
/* definitions for using the memory pool class*/
#ifndef MEMORY_POOL_H
#define MEMORY_POOL_H
#include "cssm.h"
#include "cssmport.h"
#include "cssmMemTrack.h"
#include "ber_der.h"

#define MEMORY_POOL_MIN_ALLOCATION_SIZE 4096

typedef struct memory_pool {
    struct bufferList {
        struct bufferList *next;
        uint8 * buffer;
        uint32 size;
    } *BufferListHead, *BufferListTail, *CurrentBufferListNode;
    uint32 CurrentOffset;
    void * ParentObject;
    CSSM_MEMORY_FUNCS *ParentFuncs;
} MEMORY_POOL, *MEMORY_POOL_PTR;

/*  
    The cssm_memory_pool pointer is passed in places where the void * parameter (not the buffer, 
    but the undefined parameter) is used.
    The memory pool includes additional constructor, destruct, and free all buffers interfaces.
    This function initializes the memory pool object.  The underlying allocator is 
    used to obtain memory (albeit in large chunks). CSSM_OK is 
    returned if successful--otherwise an error code is returned.
*/

typedef CSSM_RETURN ( CSSMAPI * MEMORY_POOL_INIT ) (
    const CSSM_MEMORY_FUNCS_PTR underlying_allocation_funcs,    /* input */
    void *underlying_allocation_object,                 /* input */
    MEMORY_POOL_PTR new_memory_pool_object);            /* updated object*/

/*
This function logically frees all the buffers ever allocated from this pool.  
The memory pool may or may not free the actual buffers to the underlying memory allocator 
that was specificed when it was constructed. 
*/
typedef CSSM_RETURN (CSSMAPI * MEMORY_POOL_RESET) (MEMORY_POOL_PTR);

/*
This function frees all the buffers ever allocated in the memory pool and also releases 
any internal housekeeping memory. The memory pool is destroyed (in the C++ sense) by 
this operation, and the memory of the pool itself can then be recycled.
*/

typedef CSSM_RETURN (CSSMAPI * MEMORY_POOL_RECYCLE) (MEMORY_POOL_PTR);
 
/*  The function table includes the extra entries: */
typedef struct memory_pool_funcs {
    CSSM_MEMORY_FUNCS cssm_memory_funcs;
    MEMORY_POOL_INIT init;
    MEMORY_POOL_RESET reset;
    MEMORY_POOL_RECYCLE recycle;
} MEMORY_POOL_FUNCS, *MEMORY_POOL_FUNCS_PTR;

/* The cssm_memory_funcs may be passed to existing CSSM functions in place 
of the functions of the underlying memory manager*/

#endif 
