/***************************************************************************/
/* clutil.h                                                                */
/*                                                                         */
/* Utility routine for manipulating CSSM_LIST and CSSM_TUPLE data          */
/* structures.  These are primarily of interest to those who interact      */
/* with the SPKI CL or TP or the AC (AuthCompute) module.                  */
/***************************************************************************/

#if !defined( __CLUTIL_H__ )
#define __CLUTIL_H__

#include "cssmtype.h"

/***************************************************************************/
/* drop_data                                                               */
/*                                                                         */
/* Drop a CSSM_DATA.                                                       */
/***************************************************************************/

void drop_data( CSSM_DATA *cd ) ;

/***************************************************************************/
/* new_data                                                                */
/*                                                                         */
/* Create a CSSM_DATA block,  initializes with the given params.           */
/***************************************************************************/

CSSM_DATA_PTR new_data( uint32 lth, uint8 *data ) ;

/***************************************************************************/
/* new_element                                                             */
/*                                                                         */
/* Carve a new cssm_list_element and zero it.                              */
/*                                                                         */
/***************************************************************************/

CSSM_LIST_ELEMENT_PTR new_element( void ) ;

/***************************************************************************/
/* drop_list                                                               */
/*                                                                         */
/* Drop a CSSM_LIST.                                                       */
/***************************************************************************/

void drop_list( CSSM_LIST *cl )  ;

/***************************************************************************/
/* init_list                                                               */
/*                                                                         */
/* Init a list block.                                                      */
/***************************************************************************/

void init_list( CSSM_LIST *cl ) ;

/***************************************************************************/
/* new_tuple                                                               */
/*                                                                         */
/* Carve and init a new tuple.                                             */
/***************************************************************************/

CSSM_TUPLE_PTR new_cssm_tuple( void ) ;

/***************************************************************************/
/* drop_tuple                                                              */
/*                                                                         */
/* Drop a CSSM_TUPLE                                                       */
/***************************************************************************/

void drop_cssm_tuple( CSSM_TUPLE *t ) ;

/***************************************************************************/
/* drop_tuplegroup                                                         */
/*                                                                         */
/* Drop a CSSM_TUPLEGROUP.                                                 */
/***************************************************************************/

void drop_tuplegroup( CSSM_TUPLEGROUP *tg ) ;

/***************************************************************************/
/* fill_params                                                             */
/*                                                                         */
/* Parse the incoming canonical S-expression.  It should be a set of test  */
/* parameters.  Extract those into parameters suitable for passing         */
/* to AuthCompute.                                                         */
/***************************************************************************/

CSSM_RETURN fill_params( uint32 inblth, uint8 inbuff[],
                         CSSM_TUPLEGROUP *acl,
                         CSSM_TUPLEGROUP *cred,
                         uint32 max_nreq, /* allocation length for req */
                         uint32 *nreq,
                         CSSM_LIST req[],
                         CSSM_LIST *valid,
                         CSSM_LIST *tag ) ;

/***************************************************************************/
/* size_tuple                                                              */
/*                                                                         */
/* Return the size of this tuple, in canonical form.                       */
/***************************************************************************/

uint32 size_tuple( CSSM_TUPLE *t );

/***************************************************************************/
/* pack_tuple                                                              */
/*                                                                         */
/* Return the size of this tuple, in canonical form.                       */
/***************************************************************************/

void pack_tuple( uint8 **pW, CSSM_TUPLE *t );

/***************************************************************************/
/* pack_certuple                                                           */
/*                                                                         */
/* Pack this tuple in canonical form.                                      */
/***************************************************************************/

void pack_certuple( uint8 **pW, CSSM_TUPLE *t );

/***************************************************************************/
/* size_list                                                               */
/*                                                                         */
/* Return the size of this list, in canonical form.                        */
/***************************************************************************/

uint32 size_list( CSSM_LIST *s );

/***************************************************************************/
/* pack_list                                                               */
/*                                                                         */
/* Pack this list in canonical form.                                       */
/***************************************************************************/

void pack_list( uint8 **pW, CSSM_LIST *s );

/***************************************************************************/
/* tg_to_acl                                                               */
/*                                                                         */
/* Take a tuplegroup, as output by AuthCompute, and form a packed canonical*/
/* S-expression ACL from it.                                               */
/***************************************************************************/

CSSM_RETURN tg_to_acl( CSSM_TUPLEGROUP *tgdout, CSSM_DATA *dout ) ;

#endif /* __CLUTIL_H__ */
