/*-----------------------------------------------------------------------
 * File: ADDUTIL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


/*
 * This file contains the functions that are contained in the add-in util
 * of the CSSM functions.
 */

#include "precomp.h"

CSSM_RETURN
cssm_CallChallengeCallbackModuleHandle(const CSSM_LIST *Challenge,
                           CSSM_SAMPLEGROUP_PTR Response,
                           CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ChallengeProxy,
                           const CSSM_MEMORY_FUNCS *MemFuncs)
{
    CSSM_RETURN err = CSSM_OK;

    if((NULL != ChallengeProxy) &&
       (CSSM_FALSE == port_IsBadCodePtr((CSSM_PROC_ADDR)ChallengeProxy->Callback)))
    {

#ifdef ISL_INCLUDED
        /*
         * Check that callback function is in the app's verified space.
         */
        if(PvcMode & CSSM_PVC_APP)
        {
            if(CSSM_OK != ERR(cssm_FindCallerListEntryAndMultiLock((CSSM_PROC_ADDR)ChallengeProxy->Callback,
                                                                   CSSM_NO_LOCK,
                                                                   NULL,
                                                                   CSSM_NO_LOCK,
                                                                   NULL,
                                                                   CSSM_NO_LOCK,
                                                                   NULL)))
            {
                ERR(err = CSSMERR_CSSM_FUNCTION_INTEGRITY_FAIL);
            }
        }

#endif

    }
    else
    {
        ERR(err = CSSMERR_CSSM_INVALID_POINTER);
    }

    if( CSSM_OK == err )
    {
    /* now we actually call the callback */
        ERR(err = ChallengeProxy->Callback(Challenge,Response,ChallengeProxy->CallbackCtx,MemFuncs));
    }
    else
    {
        if(CSSM_OK == err)
        {
            /* we should've gotten a callback if it was a complete and
                   valid structure pointed to
                */
            ERR(err = CSSMERR_CSSM_INVALID_POINTER);
        }
    }

    RETURN (err)
}

/* CSSM proxy for challenge callback*/
CSSM_RETURN CSSMAPI
CSSM_CHALLENGE_CALLBACK_API_PROXY(const CSSM_LIST *Challenge,
                                  CSSM_SAMPLEGROUP_PTR Response,
                                  void *CallerCtx,
                                  const CSSM_MEMORY_FUNCS *MemFuncs)
{
    CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ProxyCtx;
    CSSM_RETURN err = CSSM_OK;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;

#ifdef ISL_INCLUDED
    uint32 i;
    ISL_STATUS callValid = ISL_FAIL;
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress( retAddress);
#endif


    if(CSSM_TRUE == port_IsBadReadPtr(CallerCtx,
                                      sizeof(CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE)))
    {
        ERR(err = CSSMERR_CSSM_INVALID_CONTEXT_HANDLE);
    } 
    else 
    {
        ProxyCtx = (CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR) CallerCtx;
        /* we need to verify that the caller is the valid service provider */
         /* we need to verify that the caller is the valid service provider */
        ERR(err = cssm_FindServiceAndMultiLock(ProxyCtx->ModuleHandle,
                                               CSSM_NO_LOCK,
                                               NULL,
                                               CSSM_NO_LOCK,
                                               &pService,
                                               CSSM_READER_LOCK));
        if(CSSM_OK == err)
        {

#ifdef ISL_INCLUDED
            if( cssm_GetPVCPolicy() & CSSM_PVC_SP )
            {
                if(NULL != pService->SPCredentialPtr)
                {
                    i = 0;

                    /* Loop through checking all components in the manifest -
                       it's likely that there's only one, but the spec allows
                       for more.
                    */
                    while((ISL_FAIL == callValid) &&
                          (i < pService->NumSPCreds))
                    {
                        callValid = EISL_CheckAddressWithinModule(pService->SPCredentialPtr[i],
                                                                  retAddress);
                        if(ISL_FAIL == callValid)
                        {
                            i++;
                        }
                    }
                }


                /* if we couldn't validate the caller, we'll be failing the call */
                if(ISL_FAIL == callValid)
                {
                    ERR(err = CSSMERR_CSSM_FUNCTION_INTEGRITY_FAIL);
                }
            }
#endif
            /* release the service record */
            cssm_ReleaseServiceLock (pService, CSSM_READER_LOCK);

        }
        else
        {
            ERR(err = CSSMERR_CSSM_INTERNAL_ERROR);
        }

        /* if everything's OK, we'll call the callback now */
        if(CSSM_OK == err)
        {
            ERR(err = cssm_CallChallengeCallbackModuleHandle(Challenge,Response,ProxyCtx,MemFuncs));
        }

    }

    RETURN (err)
}

CSSM_RETURN CSSMAPI cssm_SetAccessCredProxyModuleHandle( CSSM_MODULE_HANDLE ModuleHandle,
                                             const CSSM_ACCESS_CREDENTIALS* AccessCred,
                                             CSSM_ACCESS_CREDENTIALS_PTR ProxyCallback,
                                             CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ProxyCtx)

{
    /* Do we have a callback to proxy? */
    if ( AccessCred != NULL )
    {
        ProxyCtx->Callback = AccessCred->Callback;
        ProxyCtx->CallbackCtx = AccessCred->CallerCtx;
        ProxyCtx->ModuleHandle = ModuleHandle;
        port_memcpy( ProxyCallback->EntryTag, AccessCred->EntryTag, (uint32)sizeof(CSSM_STRING));
        port_memcpy ( &ProxyCallback->BaseCerts, &AccessCred->BaseCerts, (uint32)sizeof( CSSM_BASE_CERTS));
        port_memcpy ( &ProxyCallback->Samples, &AccessCred->Samples, (uint32)sizeof( CSSM_SAMPLEGROUP));
        if ( AccessCred->Callback != NULL )
        {
            /* The CSSM_ACCESS_CREDENTIALS contains a callback to proxy */
            ProxyCallback->Callback = CSSM_CHALLENGE_CALLBACK_API_PROXY;
            ProxyCallback->CallerCtx = &ProxyCtx;
        }
        else
        {
            /* The CSSM_ACCESS_CREDENTIALS does not contain a callback to proxy */
             ProxyCallback->Callback = NULL;
             ProxyCallback->CallerCtx = AccessCred->CallerCtx;
        }
     }
    RETURN (CSSM_OK)
}


CSSM_RETURN
cssm_CallSubjectCallbackModuleHandle(const CSSM_LIST *SubjectRequest,
                         CSSM_LIST_PTR SubjectResponse,
                         CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR SubjectProxy,
                         const CSSM_MEMORY_FUNCS *MemFuncs)
{
    CSSM_RETURN err = CSSM_OK;

    if((NULL != SubjectProxy) &&
       (CSSM_FALSE == port_IsBadCodePtr((CSSM_PROC_ADDR)SubjectProxy->Callback)))
    {

#ifdef ISL_INCLUDED
        /*
         * Check that callback function is in the app's verified space.
         */
        if(PvcMode & CSSM_PVC_APP)
        {
            if(CSSM_OK != ERR(cssm_FindCallerListEntryAndMultiLock((CSSM_PROC_ADDR)SubjectProxy->Callback,
                                                                   CSSM_NO_LOCK,
                                                                   NULL,
                                                                   CSSM_NO_LOCK,
                                                                   NULL,
                                                                   CSSM_NO_LOCK,
                                                                   NULL)))
            {
                ERR(err = CSSMERR_CSSM_FUNCTION_INTEGRITY_FAIL);
            }
        }

#endif

    }
    else
    {
        ERR(err = CSSMERR_CSSM_INVALID_POINTER);
    }

    if( CSSM_OK == err )
    {
    /* now we actually call the callback */
        ERR(err = SubjectProxy->Callback(SubjectRequest,SubjectResponse,SubjectProxy->CallbackCtx,MemFuncs));
    }
    else
    {
        if(CSSM_OK == err)
        {
            /* we should've gotten a callback if it was a complete and
                   valid structure pointed to
                */
            ERR(err = CSSMERR_CSSM_INVALID_POINTER);
        }
    }

    RETURN (err)
}

/* CSSM proxy for seed callback*/
CSSM_RETURN CSSMAPI
CSSM_SUBJECT_CALLBACK_API_PROXY(const CSSM_LIST *SubjectRequest,
                                  CSSM_LIST_PTR SubjectResponse,
                                  void *CallerCtx,
                                  const CSSM_MEMORY_FUNCS *MemFuncs)
{
    CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR ProxyCtx;
    CSSM_RETURN err = CSSM_OK;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;

#ifdef ISL_INCLUDED
    uint32 i;
    ISL_STATUS callValid = ISL_FAIL;
    ISL_FUNCTION_PTR retAddress = NULL;
    EISL_GetReturnAddress( retAddress);
#endif


    if(CSSM_TRUE == port_IsBadReadPtr(CallerCtx,
                                      sizeof(CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE)))
    {
        ERR(err = CSSMERR_CSSM_INVALID_CONTEXT_HANDLE);
    } 
    else 
    {
        ProxyCtx = (CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR) CallerCtx;
         /* we need to verify that the caller is the valid service provider */
        ERR(err = cssm_FindServiceAndMultiLock(ProxyCtx->ModuleHandle,
                                               CSSM_NO_LOCK,
                                               NULL,
                                               CSSM_NO_LOCK,
                                               &pService,
                                               CSSM_READER_LOCK));
        if(CSSM_OK == err)
        {

#ifdef ISL_INCLUDED
            if( cssm_GetPVCPolicy() & CSSM_PVC_SP )
            {
                if(NULL != pService->SPCredentialPtr)
                {
                    i = 0;

                    /* Loop through checking all components in the manifest -
                       it's likely that there's only one, but the spec allows
                       for more.
                    */
                    while((ISL_FAIL == callValid) &&
                          (i < pService->NumSPCreds))
                    {
                        callValid = EISL_CheckAddressWithinModule(pService->SPCredentialPtr[i],
                                                                  retAddress);
                        if(ISL_FAIL == callValid)
                        {
                            i++;
                        }
                    }
                }


                /* if we couldn't validate the caller, we'll be failing the call */
                if(ISL_FAIL == callValid)
                {
                    ERR(err = CSSMERR_CSSM_FUNCTION_INTEGRITY_FAIL);
                }
            }
#endif
            /* release the service record */
            cssm_ReleaseServiceLock (pService, CSSM_READER_LOCK);

        }
        else
        {
            ERR(err = CSSMERR_CSSM_INTERNAL_ERROR);
        }

        /* if everything's OK, we'll call the callback now */
        if(CSSM_OK == err)
        {
            ERR(err = cssm_CallSubjectCallbackModuleHandle(SubjectRequest,SubjectResponse,ProxyCtx,MemFuncs));
        }

    }

    RETURN (err)
}



CSSM_RETURN CSSMAPI cssm_SetResourceControlProxyModuleHandle(CSSM_MODULE_HANDLE ModuleHandle,
                                                 const CSSM_RESOURCE_CONTROL_CONTEXT* CredAndAclEntry,
                                                 CSSM_CHALLENGE_CALLBACK_PROXY_MODULEHANDLE_PTR ChallengeProxyCtx,
                                                 CSSM_SUBJECT_CALLBACK_PROXY_MODULEHANDLE_PTR SubjectProxyCtx,
                                                 CSSM_RESOURCE_CONTROL_CONTEXT* ProxyCallback)
{
    if ( CredAndAclEntry != NULL )
    {
        if( (CredAndAclEntry->AccessCred != NULL) && 
            (!port_IsBadReadPtr( CredAndAclEntry->AccessCred, (uint32) sizeof( CSSM_ACCESS_CREDENTIALS)) ) )
        {
            if( CredAndAclEntry->AccessCred->Callback != NULL)
            {
                ChallengeProxyCtx->Callback = CredAndAclEntry->AccessCred->Callback;
                ChallengeProxyCtx->CallbackCtx = CredAndAclEntry->AccessCred->CallerCtx;
                ChallengeProxyCtx->ModuleHandle = ModuleHandle;
            }
        }
        if( CredAndAclEntry->InitialAclEntry.Callback != NULL )
        {
            SubjectProxyCtx->Callback = CredAndAclEntry->InitialAclEntry.Callback;
            SubjectProxyCtx->CallbackCtx = CredAndAclEntry->InitialAclEntry.CallerContext;
            SubjectProxyCtx->ModuleHandle = ModuleHandle;
        }
        if( CredAndAclEntry->AccessCred != NULL)
            ProxyCallback->AccessCred = CredAndAclEntry->AccessCred;
        else
            ProxyCallback->AccessCred = NULL;

        port_memcpy ( &ProxyCallback->InitialAclEntry, &CredAndAclEntry->InitialAclEntry, (uint32)sizeof( CSSM_ACL_ENTRY_INPUT));

        if( (CredAndAclEntry->AccessCred != NULL) && 
            (!port_IsBadReadPtr( CredAndAclEntry->AccessCred, (uint32) sizeof( CSSM_ACCESS_CREDENTIALS)) ) )
        {
            if ( CredAndAclEntry->AccessCred->Callback != NULL )
            {
                /* The CSSM_ACCESS_CREDENTIALS contains a callback to proxy */
                ProxyCallback->AccessCred->Callback = CSSM_CHALLENGE_CALLBACK_API_PROXY;
                ProxyCallback->AccessCred->CallerCtx = ChallengeProxyCtx;
            }
            else
            {
            /* The CSSM_ACCESS_CREDENTIALS does not contain a callback to proxy */
                ProxyCallback->AccessCred->Callback = NULL;
                ProxyCallback->AccessCred->CallerCtx = CredAndAclEntry->AccessCred->CallerCtx;
            }
        }
        if ( CredAndAclEntry->InitialAclEntry.Callback != NULL )
        {
        /* The CSSM_ACCESS_CREDENTIALS contains a callback to proxy */
            ProxyCallback->InitialAclEntry.Callback = CSSM_SUBJECT_CALLBACK_API_PROXY;
            ProxyCallback->InitialAclEntry.CallerContext = SubjectProxyCtx;
        }
        else
        {
                /* The CSSM_ACCESS_CREDENTIALS does not contain a callback to proxy */
                ProxyCallback->InitialAclEntry.Callback = NULL;
                ProxyCallback->InitialAclEntry.CallerContext = CredAndAclEntry->InitialAclEntry.CallerContext;
        }
    }
    RETURN (CSSM_OK)
}


/*----------------------------------------------------------
*Name: cssm_Rand
*
*Description:
* Generate a 32 bit random number.  Note: this function no
* longer guarrantees uniqueness for handles.
*
*Parameters: 
* None
*
*Returns: 
* 32 bit random number
*
*--------------------------------------------------------*/
uint32 cssm_Rand ()
{
    static uint32 value=0;
    TRACE(cssm_trace_info, FNCODE_cssm_Rand);
    
    /* use C runtime functions */
    srand ((uint32)time (NULL));
    value ^= (uint32)rand();
#ifndef VMS
    value <<= 16;
    srand (value);
    value ^= (uint32)rand();
#endif
    RETURN(value)
}

/*----------------------------------------------------------
*Name: cssm_CheckAPIMemFuncs
*
*Description:
* Check to make sure that memory functions are initialize
*
*Parameters: 
* MemoryFunctions (input) - pointer to memory function structure
*
*Returns: 
* CSSM_FAIL - bad pointer somewhere
* CSSM_OK - all pointers OK
*
*--------------------------------------------------------*/
CSSM_RETURN cssm_CheckAPIMemFuncs (
                                   const CSSM_API_MEMORY_FUNCS *MemoryFunctions)
{
    TRACE(cssm_trace_info, FNCODE_cssm_CheckAPIMemFuncs);
    
    /* Check validity of pointers passed in */
    if (port_IsBadReadPtr (MemoryFunctions, (uint32)sizeof (CSSM_API_MEMORY_FUNCS))) {
        RETURN(CSSMERR_CSSM_INVALID_POINTER)
    }
    
    /* Check validity of callbacks */
    if (port_IsBadCodePtr((CSSM_PROC_ADDR)MemoryFunctions->malloc_func)) {
        RETURN(CSSMERR_CSSM_INVALID_POINTER)
    }
    
    if (port_IsBadCodePtr((CSSM_PROC_ADDR)MemoryFunctions->realloc_func)) {
        RETURN(CSSMERR_CSSM_INVALID_POINTER)
    }
    
    if (port_IsBadCodePtr((CSSM_PROC_ADDR)MemoryFunctions->free_func)) {
        RETURN(CSSMERR_CSSM_INVALID_POINTER)
    }
    
    if (port_IsBadCodePtr((CSSM_PROC_ADDR)MemoryFunctions->calloc_func)) {
        RETURN(CSSMERR_CSSM_INVALID_POINTER)
    }
    
    RETURN(CSSM_OK)
}

CSSM_RETURN
cssm_FindModuleAndMultiLock(CSSM_MODULE_HANDLE hModule,
                            CSSM_LOCK_TYPE ModuleListLock,
                            cssm_INTERNAL_MODULE_LIST_PTR *ppModule,
                            CSSM_LOCK_TYPE ModuleEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;
    CSSM_RETURN LockRet;
    CSSM_MODULE_HANDLE hModuleIndex = cssm_ExtractModuleIndex(hModule);
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    CSSM_BOOL ModuleFound;
    
    /* Check return parameter */
    if(ppModule == NULL)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    *ppModule = NULL;
    
    /* Get a reader lock on the module list */
    if(CSSM_WRITER_LOCK == ModuleListLock)
    {
        LockRet = cssm_GetModuleListLock(CSSM_WRITER_LOCK);
    }
    else
    {
        LockRet = cssm_GetModuleListLock(CSSM_READER_LOCK);
    }
    
    if(CSSM_OK != LockRet)
    {
        retCode = LockRet;
        goto bailout;
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pModule = hModuleListHead;
    ModuleFound = CSSM_FALSE;
    while((NULL != pModule) &&
        (CSSM_FALSE == ModuleFound))
    {
        /* Test to see if the GUIDs match */
        if(pModule->ModuleHandleIndex == hModuleIndex)
        {
            ModuleFound = CSSM_TRUE;
        }
        else
        {
            /* Move to the next record */
            pModule = pModule->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL == pModule)
    {
        retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
        goto droplock_bailout;
    }
    
    /* Obtain the proper lock on the record */
    if(CSSM_READER_LOCK == ModuleEntryLock)
    {
        /* Get a reader lock on this entry in the list */
        if ((retCode = cssm_ReaderLock (pModule->hModuleSWMRLock)) != CSSM_OK)       
        {
            goto droplock_bailout;
        }
        else
        {
            /* make sure another thread hasn't changed the index value on us */
            if(pModule->ModuleHandleIndex != hModuleIndex)
            {
                cssm_ReaderUnlock(pModule->hModuleSWMRLock);
                retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
            }
        }
    }
    else
    {
        if(CSSM_WRITER_LOCK == ModuleEntryLock)
        {
            /* Get a writer lock on the record */
            if ((retCode = cssm_WriterLock (pModule->hModuleSWMRLock)) != CSSM_OK)          
            {
                goto droplock_bailout;
            }
            else
            {
                /* make sure another thread hasn't changed the index value on us */
                if(pModule->ModuleHandleIndex != hModuleIndex)
                {
                    cssm_WriterUnlock(pModule->hModuleSWMRLock);
                    retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
                }
            }
        }
    }
    
    *ppModule = pModule;
    
droplock_bailout:
    if(CSSM_NO_LOCK == ModuleListLock)
    {
        /* Drop the reader lock on the module list. */
        cssm_ReaderUnlock(hModuleListSWMRLock);
    }
    else
    {
        if(CSSM_OK != retCode)
        {
            /* now we drop either a reader or writer lock */
            if(CSSM_READER_LOCK == ModuleListLock)
            {
                cssm_ReaderUnlock(hModuleListSWMRLock);
            }
            else
            {
                cssm_WriterUnlock(hModuleListSWMRLock);
            }
        }
    }
    
bailout:
    
    /* Return the error code */
    
    RETURN (retCode)
}



CSSM_RETURN
cssm_FindModuleByGUIDAndMultiLock(const CSSM_GUID *GuidPtr,
                                  CSSM_LOCK_TYPE ModuleListLock,
                                  cssm_INTERNAL_MODULE_LIST_PTR *ppModule,
                                  CSSM_LOCK_TYPE ModuleEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;
    CSSM_RETURN LockRet;
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    CSSM_BOOL ModuleFound;
    
    /* Check return parameter */
    if(ppModule == NULL)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    *ppModule = NULL;
    
    /* Get a reader lock on the module list */
    if(CSSM_WRITER_LOCK == ModuleListLock)
    {
        LockRet = cssm_GetModuleListLock(CSSM_WRITER_LOCK);
    }
    else
    {
        LockRet = cssm_GetModuleListLock(CSSM_READER_LOCK);
    }
    
    if(CSSM_OK != LockRet)
    {
        retCode = LockRet;
        goto bailout;
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pModule = hModuleListHead;
    ModuleFound = CSSM_FALSE;
    while((NULL != pModule) &&
        (CSSM_FALSE == ModuleFound))
    {
        /* Test to see if the GUIDs match */
        if(CSSM_TRUE == CSSM_CompareGuids((*GuidPtr),
            pModule->GUID))
        {
            ModuleFound = CSSM_TRUE;
        }
        else
        {
            /* Move to the next record */
            pModule = pModule->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL == pModule)
    {
        retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
        goto droplock_bailout;
    }
    
    /* Obtain the proper lock on the record */
    if(CSSM_READER_LOCK == ModuleEntryLock)
    {
        /* Get a reader lock on this entry in the list */
        if ((retCode = cssm_ReaderLock (pModule->hModuleSWMRLock)) != CSSM_OK)
        {
            goto droplock_bailout;
        }
        else
        {
            /* make sure another thread hasn't changed the index value on us */
            if(CSSM_TRUE != CSSM_CompareGuids((*GuidPtr),
                pModule->GUID))
            {
                cssm_ReaderUnlock(pModule->hModuleSWMRLock);
                retCode = CSSMERR_CSSM_INVALID_GUID;
            }
        }
    }
    else
    {
        if(CSSM_WRITER_LOCK == ModuleEntryLock)
        {
            /* Get a writer lock on the record */
            if ((retCode = cssm_WriterLock (pModule->hModuleSWMRLock)) != CSSM_OK)           
            {
                goto droplock_bailout;
            }
            else
            {
                /* make sure another thread hasn't changed the index value on us */
                if(CSSM_TRUE != CSSM_CompareGuids((*GuidPtr),
                    pModule->GUID))
                {
                    cssm_WriterUnlock(pModule->hModuleSWMRLock);
                    retCode = CSSMERR_CSSM_INVALID_GUID;
                }
            }
        }
    }
    
    *ppModule = pModule;
    
droplock_bailout:
    if(CSSM_NO_LOCK == ModuleListLock)
    {
        /* Drop the reader lock on the module list. */
        cssm_ReaderUnlock(hModuleListSWMRLock);
    }
    else
    {
        if(CSSM_OK != retCode)
        {
            /* now we drop either a reader or writer lock */
            if(CSSM_READER_LOCK == ModuleListLock)
            {
                cssm_ReaderUnlock(hModuleListSWMRLock);
            }
            else
            {
                cssm_WriterUnlock(hModuleListSWMRLock);
            }
        }
    }
    
bailout:
    
    /* Return the error code */
    
    return retCode;
}



CSSM_RETURN
cssm_FindServiceAndMultiLock(CSSM_MODULE_HANDLE hService,
                             CSSM_LOCK_TYPE ModuleListLock,
                             cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                             CSSM_LOCK_TYPE ModuleEntryLock,
                             cssm_INTERNAL_SERVICE_LIST_PTR *ppService,
                             CSSM_LOCK_TYPE ServiceEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;
    CSSM_BOOL MatchFound;
    CSSM_MODULE_HANDLE hServiceIndex = cssm_ExtractServiceIndex(hService);
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;
    CSSM_LOCK_TYPE LockForModule;
    
    /* Check return parameter */
    if(NULL == ppService)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    /* We need at least a reader lock here, but that may not be what the caller
    wants to have returned.
    */
    if(CSSM_NO_LOCK != ModuleEntryLock)
    {
        LockForModule = ModuleEntryLock;
    }
    else
    {
        LockForModule = CSSM_READER_LOCK;
    }
    
    /* Fetch the module entry - locked as case demands */
    retCode = cssm_FindModuleAndMultiLock(hService,
        ModuleListLock,
        &pModule,
        LockForModule);
    if(CSSM_OK != retCode)
    {
        RETURN (CSSMERR_CSSM_INVALID_ADDIN_HANDLE)        
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pService = pModule->ServiceList;
    MatchFound = CSSM_FALSE;
    while((NULL != pService) &&
        (CSSM_FALSE == MatchFound))
    {
        /* test to see if we match indeces */
        if(pService->ServiceHandleIndex == hServiceIndex)
        {
            MatchFound = CSSM_TRUE;
        }
        else
        {
            pService = pService->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL != pService)
    {
        /* Obtain the proper lock on the record */
        if(CSSM_READER_LOCK == ServiceEntryLock)
        {
            /* Get a reader lock on this entry in the list */
            if ((retCode = cssm_ReaderLock (pService->hServiceSWMRLock)) == CSSM_OK)
            {
                /* make sure another thread hasn't changed the index value on us */
                if(pService->ServiceHandleIndex != hServiceIndex)
                {
                    cssm_ReaderUnlock(pService->hServiceSWMRLock);
                    retCode = CSSMERR_CSSM_INVALID_SERVICE_MASK;
                }
            }
        }
        else
        {
            if(CSSM_WRITER_LOCK == ServiceEntryLock)
            {
                /* Get a writer lock on the record */
                if ((retCode = cssm_WriterLock (pService->hServiceSWMRLock)) == CSSM_OK)
                {
                    /* make sure another thread hasn't changed the index value on us */
                    if(pService->ServiceHandleIndex != hServiceIndex)
                    {
                        cssm_WriterUnlock(pService->hServiceSWMRLock);
                        retCode = CSSMERR_CSSM_INVALID_SERVICE_MASK;
                    }
                }
            }
        }
    }
    else
    {
        retCode = CSSMERR_CSSM_INVALID_SERVICE_MASK;
    }
    
    if(CSSM_OK == retCode)
    {
        *ppService = pService;
        
        /* if they want a module record, we'll set it */
        if(NULL != ModuleRecord)
        {
            *ModuleRecord = pModule;
        }
        
        /* they may not want this entry locked */
        if(CSSM_NO_LOCK == ModuleEntryLock)
        {
            cssm_ReleaseModuleLock(pModule,
                LockForModule);
        }
    }
    else
    {
        /* error case - release all locks */
        cssm_ReleaseModuleLock(pModule,
            LockForModule);
        cssm_ReleaseModuleListLock(ModuleListLock);
    }
    
    /* Return the error code */
    return retCode;
}



CSSM_RETURN
cssm_FindServiceByGUIDAndSSIDAndMultiLock(const CSSM_GUID *pGUID,
                                          uint32 SubServiceID,
                                          CSSM_LOCK_TYPE ModuleListLock,
                                          cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                                          CSSM_LOCK_TYPE ModuleEntryLock,
                                          cssm_INTERNAL_SERVICE_LIST_PTR *ppService,
                                          CSSM_LOCK_TYPE ServiceEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;  
    CSSM_BOOL MatchFound;
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;
    CSSM_LOCK_TYPE LockForModule;
    
    /* Check return parameter */
    if(NULL == ppService)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    /* We need at least a reader lock here, but that may not be what the caller
    wants to have returned.
    */
    if(CSSM_NO_LOCK != ModuleEntryLock)
    {
        LockForModule = ModuleEntryLock;
    }
    else
    {
        LockForModule = CSSM_READER_LOCK;
    }
    
    /* Fetch the module entry - locked as case demands */
    retCode = cssm_FindModuleByGUIDAndMultiLock(pGUID,
        ModuleListLock,
        &pModule,
        LockForModule);
    if(CSSM_OK != retCode)
    {
        RETURN (CSSMERR_CSSM_MODULE_NOT_LOADED)
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pService = pModule->ServiceList;
    MatchFound = CSSM_FALSE;
    while((NULL != pService) &&
        (CSSM_FALSE == MatchFound))
    {
        /* test to see if we match indeces */
        if(pService->SubServiceID == SubServiceID)
        {
            MatchFound = CSSM_TRUE;
        }
        else
        {
            pService = pService->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL != pService)
    {
        /* Obtain the proper lock on the record */
        if(CSSM_READER_LOCK == ServiceEntryLock)
        {
            if ((retCode = cssm_ReaderLock (pService->hServiceSWMRLock)) == CSSM_OK)
            {
                /* make sure another thread hasn't changed the index value on us */
                if(pService->SubServiceID != SubServiceID)
                {
                    cssm_ReaderUnlock(pService->hServiceSWMRLock);
                    ERR(retCode = CSSMERR_CSSM_INVALID_SUBSERVICEID);
                }
            }
        }
        else
        {
            if(CSSM_WRITER_LOCK == ServiceEntryLock)
            {
                /* Get a writer lock on the record */
                
                if ((retCode = cssm_WriterLock (pService->hServiceSWMRLock)) == CSSM_OK)
                {
                    /* make sure another thread hasn't changed the index value on us */
                    if(pService->SubServiceID != SubServiceID)
                    {
                        cssm_WriterUnlock(pService->hServiceSWMRLock);
                        ERR(retCode = CSSMERR_CSSM_INVALID_SUBSERVICEID);
                    }
                }
            }
        }
    }
    else
    {
        ERR(retCode = CSSMERR_CSSM_INVALID_SUBSERVICEID);
    }
    
    if(CSSM_OK == retCode)
    {
        *ppService = pService;
        
        /* if they want a module record, we'll set it */
        if(NULL != ModuleRecord)
        {
            *ModuleRecord = pModule;
        }
        
        /* they may not want this entry locked */
        if(CSSM_NO_LOCK == ModuleEntryLock)
        {
            cssm_ReleaseModuleLock(pModule,
                LockForModule);
        }
    }
    else
    {
        /* error case - release all locks */
        cssm_ReleaseModuleLock(pModule,
            LockForModule);
        cssm_ReleaseModuleListLock(ModuleListLock);
    }
    
    
    /* Return the error code */
    return retCode;
}



CSSM_RETURN
cssm_FindAttachAndMultiLock(CSSM_MODULE_HANDLE hAttach,
                            CSSM_LOCK_TYPE ModuleListLock,
                            cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                            CSSM_LOCK_TYPE ModuleEntryLock,
                            cssm_INTERNAL_SERVICE_LIST_PTR *ServiceRecord,
                            CSSM_LOCK_TYPE ServiceEntryLock,
                            cssm_INTERNAL_ATTACH_LIST_PTR *ppAttach,
                            CSSM_LOCK_TYPE AttachEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;  
    CSSM_BOOL MatchFound;
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;
    cssm_INTERNAL_ATTACH_LIST_PTR pAttach = NULL;
    CSSM_LOCK_TYPE LockForService;
    
    /* Check return parameter */
    if(NULL == ppAttach)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    *ppAttach = NULL;
    
    /* We need at least a reader lock here, but that may not be what the caller
    wants to have returned.
    */
    if(CSSM_NO_LOCK != ServiceEntryLock)
    {
        LockForService = ServiceEntryLock;
    }
    else
    {
        LockForService = CSSM_READER_LOCK;
    }
    
    /* Fetch the service entry - locked as case demands */
    retCode = cssm_FindServiceAndMultiLock(hAttach,
        ModuleListLock,
        &pModule,
        ModuleEntryLock,
        &pService,
        LockForService);
    if(CSSM_OK != retCode)
    {
        RETURN (CSSMERR_CSSM_INVALID_ADDIN_HANDLE)
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pAttach = pService->AttachList;
    MatchFound = CSSM_FALSE;
    while((NULL != pAttach) &&
        (CSSM_FALSE == MatchFound))
    {
        /* test to see if we match indices */
        if(pAttach->AttachHandle == hAttach)
        {
            MatchFound = CSSM_TRUE;
        }
        else
        {
            pAttach = pAttach->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL != pAttach)
    {
        /* Obtain the proper lock on the record */
        if(CSSM_READER_LOCK == AttachEntryLock)
        {
            /* Get a reader lock on this entry in the list */
            if ((retCode = cssm_ReaderLock (pAttach->hAttachSWMRLock)) == CSSM_OK)
            {
                /* make sure another thread hasn't changed the index value on us */
                if(pAttach->AttachHandle != hAttach)
                {
                    cssm_ReaderUnlock(pAttach->hAttachSWMRLock);
                    retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
                }
            }
        }
        else
        {
            if(CSSM_WRITER_LOCK == AttachEntryLock)
            {
                /* Get a writer lock on the record */
                if ((retCode = cssm_WriterLock (pAttach->hAttachSWMRLock)) == CSSM_OK)
                {
                    /* make sure another thread hasn't changed the index value on us */
                    if(pAttach->AttachHandle != hAttach)
                    {
                        cssm_WriterUnlock(pAttach->hAttachSWMRLock);
                        retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
                    }
                }
            }
        }
    }
    else
    {
        retCode = CSSMERR_CSSM_INVALID_ADDIN_HANDLE;
    }
    
    if(CSSM_OK == retCode)
    {
        *ppAttach = pAttach;
        
        /* if they want a module record, we'll set it */
        if(NULL != ModuleRecord)
        {
            *ModuleRecord = pModule;
        }
        
        if(NULL != ServiceRecord)
        {
            *ServiceRecord = pService;
        }
        
        /* they may not want this entry locked */
        if(CSSM_NO_LOCK == ServiceEntryLock)
        {
            cssm_ReleaseServiceLock(pService,
                LockForService);
        }
        
    }
    else
    {
        /* error case - release all locks */
        cssm_ReleaseServiceLock(pService,
            LockForService);
        cssm_ReleaseModuleLock(pModule,
            ModuleEntryLock);
        cssm_ReleaseModuleListLock(ModuleListLock);
    }
    
    
    /* Return the error code */
    return retCode;
}

CSSM_RETURN
cssm_FindContextAndMultiLock(CSSM_CC_HANDLE hContext,
                             CSSM_LOCK_TYPE ModuleListLock,
                             cssm_INTERNAL_MODULE_LIST_PTR *ModuleRecord,
                             CSSM_LOCK_TYPE ModuleEntryLock,
                             cssm_INTERNAL_SERVICE_LIST_PTR *ServiceRecord,
                             CSSM_LOCK_TYPE ServiceEntryLock,
                             cssm_INTERNAL_ATTACH_LIST_PTR *AttachRecord,
                             CSSM_LOCK_TYPE AttachEntryLock,
                             cssm_INTERNAL_CONTEXT_PTR *ppContext,
                             CSSM_LOCK_TYPE ContextEntryLock)
{
    CSSM_RETURN retCode = CSSM_OK;  
    CSSM_BOOL MatchFound;
    cssm_INTERNAL_MODULE_LIST_PTR pModule = NULL;
    cssm_INTERNAL_SERVICE_LIST_PTR pService = NULL;
    cssm_INTERNAL_ATTACH_LIST_PTR pAttach = NULL;
    cssm_INTERNAL_CONTEXT_PTR pContext = NULL;
    CSSM_LOCK_TYPE LockForAttach;
    
    /* Check return parameter */
    if(NULL == ppContext)
    {
        RETURN (CSSMERR_CSSM_INVALID_POINTER)
    }
    
    *ppContext = NULL;
    
    /* We need at least a reader lock here, but that may not be what the caller
    wants to have returned.
    */
    if(CSSM_NO_LOCK != AttachEntryLock)
    {
        LockForAttach = AttachEntryLock;
    }
    else
    {
        LockForAttach = CSSM_READER_LOCK;
    }
    
    /* Fetch the service entry - locked as case demands */
    retCode = cssm_FindAttachAndMultiLock(CSSM_UINT64_HIGH(hContext),
        ModuleListLock,
        &pModule,
        ModuleEntryLock,
        &pService,
        ServiceEntryLock,
        &pAttach,
        LockForAttach);
    if(CSSM_OK != retCode)
    {
        RETURN (CSSMERR_CSSM_INVALID_CONTEXT_HANDLE)
    }
    
    /* Iterate down the list and find the record with a matching handle */
    pContext = pAttach->ContextList;
    MatchFound = CSSM_FALSE;
    while((NULL != pContext) &&
        (CSSM_FALSE == MatchFound))
    {
        /* test to see if we match indices */
        if(pContext->ContextHandle == hContext)
        {
            MatchFound = CSSM_TRUE;
        }
        else
        {
            pContext = pContext->Next;
        }
    }
    
    /* Did we find something? */
    if(NULL != pContext)
    {
        /* Obtain the proper lock on the record */
        if(CSSM_READER_LOCK == ContextEntryLock)
        {
            /* Get a reader lock on this entry in the list */
            if ((retCode = cssm_ReaderLock (pContext->hContextSWMRLock)) == CSSM_OK)
            {
                /* make sure another thread hasn't changed the index value on us */
                if(pContext->ContextHandle != hContext)
                {
                    cssm_ReaderUnlock(pContext->hContextSWMRLock);
                    retCode = CSSMERR_CSSM_INVALID_CONTEXT_HANDLE;
                }
            }
        }
        else
        {
            if(CSSM_WRITER_LOCK == ContextEntryLock)
            {
                /* Get a writer lock on the record */
                if ((retCode = cssm_WriterLock (pContext->hContextSWMRLock)) == CSSM_OK)
                {
                    /* make sure another thread hasn't changed the index value on us */
                    if(pContext->ContextHandle != hContext)
                    {
                        cssm_WriterUnlock(pContext->hContextSWMRLock);
                        retCode = CSSMERR_CSSM_INVALID_CONTEXT_HANDLE;
                    }
                }
            }
        }
    }
    else
    {
        retCode = CSSMERR_CSSM_INVALID_CONTEXT_HANDLE;
    }
    
    if(CSSM_OK == retCode)
    {
        *ppContext = pContext;
        
        /* if they want a module record, we'll set it */
        if(NULL != ModuleRecord)
        {
            *ModuleRecord = pModule;
        }
        
        if(NULL != ServiceRecord)
        {
            *ServiceRecord = pService;
        }
        
        if(NULL != AttachRecord)
        {
            *AttachRecord = pAttach;
        }
        
        /* they may not want this entry locked */
        if(CSSM_NO_LOCK == AttachEntryLock)
        {
            cssm_ReleaseAttachLock(pAttach,
                LockForAttach);
        }
        
    }
    else
    {
        /* error case - release all locks */
        cssm_ReleaseAttachLock(pAttach,
            LockForAttach);
        cssm_ReleaseServiceLock(pService,
            ServiceEntryLock);
        cssm_ReleaseModuleLock(pModule,
            ModuleEntryLock);
        cssm_ReleaseModuleListLock(ModuleListLock);
    }
    
    
    /* Return the error code */
    RETURN(retCode)
}



void cssm_ReleaseModuleLock(cssm_INTERNAL_MODULE_LIST_PTR pModule,
                            CSSM_LOCK_TYPE ltLockType)
{
    if(CSSM_READER_LOCK ==  ltLockType)
    {
        cssm_ReaderUnlock(pModule->hModuleSWMRLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK ==  ltLockType)
        {
            cssm_WriterUnlock(pModule->hModuleSWMRLock);
        }
    }
}




void cssm_ReleaseServiceLock(cssm_INTERNAL_SERVICE_LIST_PTR pService,
                             CSSM_LOCK_TYPE ltLockType)
{
    if(CSSM_READER_LOCK == ltLockType)
    {
        cssm_ReaderUnlock(pService->hServiceSWMRLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK == ltLockType)
        {
            cssm_WriterUnlock(pService->hServiceSWMRLock);
        }
    }
}





void cssm_ReleaseAttachLock(cssm_INTERNAL_ATTACH_LIST_PTR pAttach,
                            CSSM_LOCK_TYPE ltLockType)
{
    if(CSSM_READER_LOCK ==  ltLockType)
    {
        cssm_ReaderUnlock(pAttach->hAttachSWMRLock);
    }
    else
    {                                                              
        if(CSSM_WRITER_LOCK == ltLockType)
        {
            cssm_WriterUnlock(pAttach->hAttachSWMRLock);
        }
    }
}


void cssm_ReleaseContextLock(cssm_INTERNAL_CONTEXT_PTR pContext,
                             CSSM_LOCK_TYPE ltLockType)
{
    if ( ltLockType == CSSM_READER_LOCK )
    {
        cssm_ReaderUnlock( pContext->hContextSWMRLock );
    }
    else if ( ltLockType == CSSM_WRITER_LOCK )
    {
        cssm_WriterUnlock( pContext->hContextSWMRLock );
    }
    else
    {
        /* Do nothing since we are unlocking a "no lock" */
        ;
    }
}



CSSM_RETURN cssm_GetModuleListLock(CSSM_LOCK_TYPE ltLockType)
{
    CSSM_RETURN retCode = CSSM_OK;
    
    if(CSSM_READER_LOCK == ltLockType)
    {
        retCode = cssm_ReaderLock (hModuleListSWMRLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK ==  ltLockType)
        {
            retCode = cssm_WriterLock (hModuleListSWMRLock);
        }
        else
        {
            if(CSSM_NO_LOCK != ltLockType)
            {
                retCode = CSSMERR_CSSM_INTERNAL_ERROR;
            }
        }
    }
    
    RETURN (retCode)
}



void cssm_ReleaseModuleListLock(CSSM_LOCK_TYPE ltLockType)
{
    if(CSSM_READER_LOCK ==  ltLockType)
    {
        cssm_ReaderUnlock(hModuleListSWMRLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK == ltLockType)
        {
            cssm_WriterUnlock(hModuleListSWMRLock);
        }
    }
}



CSSM_RETURN cssm_GetPrivilegeSelectionLock(CSSM_LOCK_TYPE ltLockType)
{
    CSSM_RETURN retCode = CSSM_OK;
    
    if(CSSM_READER_LOCK == ltLockType)
    {
        retCode = cssm_ReaderLock (hPrivSelectionLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK ==  ltLockType)
        {
            retCode = cssm_WriterLock (hPrivSelectionLock);
        }
        else
        {
            if(CSSM_NO_LOCK != ltLockType)
            {
                retCode = CSSMERR_CSSM_INTERNAL_ERROR;
            }
        }
    }
    
    RETURN (retCode)
}



void cssm_ReleasePrivilegeSelectionLock(CSSM_LOCK_TYPE ltLockType)
{
    if(CSSM_READER_LOCK ==  ltLockType)
    {
        cssm_ReaderUnlock(hPrivSelectionLock);
    }
    else
    {
        if(CSSM_WRITER_LOCK == ltLockType)
        {
            cssm_WriterUnlock(hPrivSelectionLock);
        }
    }
}

CSSM_RETURN cssm_SetThreadPrivilege( CSSM_PRIVILEGE Privilege)
{
    CSSM_RETURN ret = CSSM_OK;
    CSSM_PRIVILEGE* threadprivilege = NULL;
    port_GetTlsValue( hThreadContext, (void **)&threadprivilege );

    if( threadprivilege != NULL )
        *threadprivilege = Privilege;
    else
    {
        threadprivilege = (CSSM_PRIVILEGE*) local_malloc( sizeof(CSSM_PRIVILEGE), NULL);
        if( threadprivilege != NULL)
        {
            *threadprivilege = Privilege;
        }
        else
            ERR( ret = CSSMERR_CSSM_MEMORY_ERROR );

    }
    if( CSSM_OK == ret )
        port_SetTlsValue( hThreadContext, (void *)threadprivilege );
        
    RETURN (ret)
}

void CSSMAPI cssm_FreeTLSPrivilege ( void *pValue )
{
    if( pValue != NULL )
    {
        local_free( pValue, NULL);
    }
}

CSSM_RETURN cssm_SetGlobalPrivilege( CSSM_PRIVILEGE Privilege)
{

    CSSM_RETURN ret = CSSM_OK;
    if( ERR( ret = cssm_GetPrivilegeSelectionLock( CSSM_WRITER_LOCK) ) == CSSM_OK )
    {
        cssm_GlobalPrivilege = Privilege;
        cssm_ReleasePrivilegeSelectionLock( CSSM_WRITER_LOCK );
    }
    else
    {
        ret = CSSMERR_CSSM_INTERNAL_ERROR;
    }

    RETURN ( ret)
}



CSSM_RETURN cssm_GetThreadPrivilege( CSSM_PRIVILEGE* Privilege)
{
    CSSM_RETURN ret = CSSM_OK;
    CSSM_PRIVILEGE* threadprivilege = NULL;

    port_GetTlsValue( hThreadContext, (void **)&threadprivilege );

    if( threadprivilege != NULL )
        *Privilege = *threadprivilege;

    RETURN (ret)
}



CSSM_RETURN cssm_GetGlobalPrivilege( CSSM_PRIVILEGE* Privilege)
{
    CSSM_RETURN ret = CSSM_OK;
    if( ERR( ret = cssm_GetPrivilegeSelectionLock( CSSM_READER_LOCK) ) == CSSM_OK )
    {
        *Privilege = cssm_GlobalPrivilege;
        cssm_ReleasePrivilegeSelectionLock( CSSM_READER_LOCK );
    }
    else
    {
        ret = CSSMERR_CSSM_INTERNAL_ERROR;
    }

    RETURN ( ret)
}



CSSM_RETURN
cssm_SetPVCPolicy(CSSM_PVC_MODE PvcPolicy)
{
    PvcMode = PvcPolicy;
    return CSSM_OK;
}


CSSM_PVC_MODE
cssm_GetPVCPolicy()
{
    return PvcMode;
}


/*
 * cssm_GetCSSMNativeServices
 *
 * Description: This function returns a mask of services known by this CSSM.
 *      This means that these are the types of service modules for which this
 *      CSSM can route function calls.
 *
 * Parameters:
 *      NativeServices (output) : The mask of services supported without use
 *                                of an EMM.
 *
 */

CSSM_RETURN
cssm_GetCSSMNativeServices(CSSM_SERVICE_MASK *NativeServices)
{
    CSSM_RETURN err = CSSM_OK;

    *NativeServices = CSSM_SERVICE_DL | CSSM_SERVICE_CSP | CSSM_SERVICE_CL |
                      CSSM_SERVICE_TP | CSSM_SERVICE_AC;

    return err;
}



#ifdef ISL_INCLUDED
CSSM_RETURN cssm_RetrieveManifestSectionAttribute( ISL_VERIFIED_MODULE_PTR VerifiedModulePtr, 
                                                   ISL_CONST_DATA AttribName,
                                                   ISL_CONST_DATA_PTR AttribValue)
{
    CSSM_RETURN ret = CSSM_OK;
    ISL_MANIFEST_SECTION_PTR pSection = NULL;

    if ( ( pSection = EISL_GetModuleManifestSection(
                                            VerifiedModulePtr ) ) == NULL )
    {
        ERR( ret = CSSMERR_CSSM_INTERNAL_ERROR );
    }
    else
    {
        if ( EISL_FindManifestSectionAttribute(
                                        pSection,
                                        AttribName,
                                        AttribValue ) != ISL_OK )
        {
            ERR( ret = CSSMERR_CSSM_INTERNAL_ERROR);
        }
    }
    RETURN ( ret )
}

CSSM_RETURN cssm_RetrieveSignerInfoAttribute( ISL_VERIFIED_MODULE_PTR VerifiedModulePtr, 
                                              ISL_CONST_DATA AttribName,
                                              ISL_CONST_DATA_PTR AttribValue)
{
    CSSM_RETURN ret = CSSM_OK;
    ISL_MANIFEST_SECTION_PTR pSection = NULL;

    if ( ( pSection = EISL_GetModuleManifestSection(
                                            VerifiedModulePtr ) ) == NULL )
    {
        ERR( ret = CSSMERR_CSSM_INTERNAL_ERROR );
    }
    else
    {
        if ( EISL_FindSignerInfoAttribute(EISL_GetManifestSignatureRoot(pSection),
                                        AttribName,
                                        AttribValue ) != ISL_OK )
        {
            ERR( ret = CSSMERR_CSSM_INTERNAL_ERROR);
        }
    }
    RETURN ( ret )
}

CSSM_RETURN cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc( ISL_VERIFIED_MODULE_PTR VerifiedCSSMModulePtr,
                                                                      ISL_VERIFIED_MODULE_PTR VerifiedCaller,
                                                                      CSSM_PVC_MODE* PvcPolicy )
{
    ISL_CONST_DATA SecAttrValue;
    ISL_CONST_DATA SecAttrName;
    uint8 SectionAttributeValue[64];
    CSSM_RETURN ret = CSSM_OK;
    const char *szSPIAttribName = "CDSA_PVC_SPI";
    const char*szAPIAttribName = "CDSA_PVC_API";
    CSSM_BOOL bCallerAPIExempt = CSSM_FALSE;
    CSSM_BOOL bCallerSPIExempt = CSSM_FALSE;
    CSSM_BOOL bCSSMAPIOFF = CSSM_FALSE;
    CSSM_BOOL bCSSMSPIOFF = CSSM_FALSE;
    CSSM_BITMASK mustBeSet;


    SecAttrName.Length = (uint32)strlen(szAPIAttribName);
    SecAttrName.Data = (uint8*)szAPIAttribName;
    SecAttrValue.Data = SectionAttributeValue;
    SecAttrValue.Length = sizeof(SectionAttributeValue);
    memset( SectionAttributeValue, 0, sizeof( SectionAttributeValue ));
    if( VerifiedCaller != NULL)
    {
        ERR(ret = cssm_RetrieveSignerInfoAttribute(VerifiedCaller,
                                                   SecAttrName,
                                                   &SecAttrValue));

        if( ret != CSSMERR_CSSM_INTERNAL_ERROR)

        {
            if( !memcmp( SecAttrValue.Data, "EXEMPT", strlen( "EXEMPT")) ) 
                bCallerAPIExempt = CSSM_TRUE;
        }
    }
    memset( SectionAttributeValue, 0, sizeof( SectionAttributeValue ));
    if( VerifiedCSSMModulePtr != NULL)
    {
        ERR(ret = cssm_RetrieveSignerInfoAttribute(VerifiedCSSMModulePtr,
                                                   SecAttrName,
                                                   &SecAttrValue));
        if( ret != CSSMERR_CSSM_INTERNAL_ERROR)
        {
            if( !memcmp( SecAttrValue.Data, "OFF", strlen( "OFF")) ) 
                bCSSMAPIOFF = CSSM_TRUE;
        }
    }
    SecAttrName.Length = (uint32)strlen(szSPIAttribName);
    SecAttrName.Data = (uint8*)szSPIAttribName;
    memset( SectionAttributeValue, 0, sizeof( SectionAttributeValue ));
    if( VerifiedCaller != NULL)
    {
        ERR(ret = cssm_RetrieveSignerInfoAttribute(VerifiedCaller, 
                                                   SecAttrName,
                                                   &SecAttrValue));

        if( ret != CSSMERR_CSSM_INTERNAL_ERROR)
        {
            if( !memcmp( SecAttrValue.Data, "EXEMPT", strlen( "EXEMPT")) )
                bCallerSPIExempt = CSSM_TRUE;
        }
    }
    memset( SectionAttributeValue, 0, sizeof( SectionAttributeValue ));
    if( VerifiedCSSMModulePtr != NULL)
    {
        ERR(ret = cssm_RetrieveSignerInfoAttribute(VerifiedCSSMModulePtr,
                                                   SecAttrName,
                                                   &SecAttrValue));
        if( ret != CSSMERR_CSSM_INTERNAL_ERROR)
        {
            if( !memcmp( SecAttrValue.Data, "OFF", strlen( "OFF")) )
                bCSSMSPIOFF = CSSM_TRUE;
        }
    }
    mustBeSet = 0;
    if( !bCSSMAPIOFF && !bCallerAPIExempt )
        mustBeSet |= CSSM_PVC_APP;
    if( !bCSSMSPIOFF && !bCallerSPIExempt )
        mustBeSet |= CSSM_PVC_SP;
    if( (*PvcPolicy & mustBeSet) == mustBeSet )
    {
        PvcMode = *PvcPolicy;
        ret = CSSM_OK;
    }
    else
    {
        ERR(ret = CSSMERR_CSSM_INVALID_PVC);
    }

    RETURN (ret)
}

CSSM_RETURN
cssm_VerifyCallerAndFillInPrivileges(cssm_CALLER_LIST_ENTRY_PTR Entry,
                                     const CSSM_DATA *Credential,
                                     const CSSM_DATA *Path,
                                     CSSM_KEY_HIERARCHY KeyHierarchy,
                                     ISL_VERIFIED_MODULE_PTR **VerModPtr,
                                     uint32 *NumSections)
{
    CSSM_RETURN err = CSSM_OK;
    CSSM_API_MEMORY_FUNCS MemFuncs;
    uint32 i;
    uint32 KeyType = 0;
    ISL_MANIFEST_SECTION_PTR ManSecPtr = NULL;

    ISL_CONST_DATA ModuleCredential;
    ISL_CONST_DATA ModulePath;
    ISL_CONST_DATA PrivName;

    uint8* USEEName = (uint8*)"CDSA_USEE";
    uint8* VendorTOGPrivTag = (uint8*)"CDSA_PRIV";

    uint32 NumPrivs = 0;
    uint32 *Privs = NULL;
    uint32 NumVendorTOGPrivs = 0;
    uint32* VendorTOGPrivs = NULL;
    uint32 NumVendorPrivs = 0;
    uint32 *VendorPrivs = NULL;
    uint32 NumTOGPrivs = 0;
    uint32 *TOGPrivs = NULL;

    cssm_VENDOR_PRIVILEGES_PTR tempVendorPrivs = NULL;

    MemFuncs.AllocRef = NULL;
    MemFuncs.calloc_func = internal_calloc;
    MemFuncs.free_func = internal_free;
    MemFuncs.malloc_func = internal_malloc;
    MemFuncs.realloc_func = internal_realloc;


    if((NULL != Credential) &&
       (NULL != Path))
    {
        ModuleCredential.Data = Credential->Data;
        ModuleCredential.Length = Credential->Length;
        ModulePath.Data = Path->Data;
        ModulePath.Length = Path->Length;

        if((CSSM_KEY_HIERARCHY_INTEG & KeyHierarchy) ||
           (CSSM_KEY_HIERARCHY_NONE == KeyHierarchy))
        {
            KeyType = CSSM_KEY_HIERARCHY_INTEG;
        }
        else
        {
            if(CSSM_KEY_HIERARCHY_EXPORT & KeyHierarchy)
            {
                KeyType = CSSM_KEY_HIERARCHY_EXPORT;
            }
            else
            {
                ERR(err = CSSMERR_CSSM_INVALID_KEY_HIERARCHY);
            }
        }

        /* verify the introducee for the key passed in */
        (*VerModPtr) = cssm_VerifyCredentialsAndLoadModules(ModuleCredential,
                                                            ModulePath,
                                                            KeyType,
                                                            CSSM_FALSE,
                                                            NumSections,
                                                            &MemFuncs);

        if(NULL != (*VerModPtr))
        {
            i = 0;

            /* we'll need to loop through each section of this manifest */
            while(i < (*NumSections))
            {
                ManSecPtr = EISL_GetModuleManifestSection(((*VerModPtr)[i]));

                if(NULL != ManSecPtr)
                {
                    PrivName.Data = USEEName;
                    PrivName.Length = strlen((char*)USEEName);

                    /* get the USEE privs from the manifest */
                    ERR(err =  cssm_ExtractPrivilege(ManSecPtr,
                                                     &NumPrivs,
                                                     &Privs,
                                                     PrivName));

                    if((CSSM_OK == err) &&
                       (0 < NumPrivs))
                    {

                        if(NULL == Entry->USEETags)
                        {
                            Entry->USEETags = Privs;
                            Entry->NumUSEETags = NumPrivs;

                        }
                       else
                        {
                            local_realloc(Entry->USEETags,
                                          (sizeof(uint32) *
                                           (Entry->NumUSEETags + NumPrivs)),
                                          NULL);
                            memcpy(&(Entry->USEETags[Entry->NumUSEETags]),
                                   Privs,
                                   sizeof(uint32) * NumPrivs);
                            Entry->NumUSEETags += NumPrivs;
                            local_free( Privs, NULL);
                            Privs = NULL;
                        }
                    }

                    if(CSSM_OK == err)
                    {
                        PrivName.Data = VendorTOGPrivTag;
                        PrivName.Length = strlen((char*)VendorTOGPrivTag);

                        /* get the privs from the manifest */
                        ERR(err =  cssm_ExtractPrivilege(ManSecPtr,
                                                         &NumVendorTOGPrivs,
                                                         &VendorTOGPrivs,
                                                         PrivName));
                        if(CSSM_OK == err)
                        {
                            /* break the privileges into TOG and
                               vendor-specific parts
                            */
                            ERR(err = cssm_FilterTOGAndVendorPrivs(NumVendorTOGPrivs,
                                                                   VendorTOGPrivs,
                                                                   &NumVendorPrivs,
                                                                   &VendorPrivs,
                                                                   &NumTOGPrivs,
                                                                   &TOGPrivs));

                            if((CSSM_OK == err) &&
                               (0 < NumTOGPrivs))
                            {
                                if(NULL == Entry->TOGPrivs)
                                {
                                    Entry->TOGPrivs = TOGPrivs;
                                    Entry->NumTOGPrivs = NumTOGPrivs;
                                }
                                else
                                {
                                    local_realloc(Entry->TOGPrivs,
                                                  (sizeof(uint32) *
                                                   (Entry->NumTOGPrivs + NumTOGPrivs)),
                                                  NULL);
                                    memcpy(&(Entry->TOGPrivs[Entry->NumTOGPrivs]),
                                           TOGPrivs,
                                           sizeof(uint32) * NumTOGPrivs);
                                    Entry->NumTOGPrivs += NumTOGPrivs;
                                }
                            }

                            if((CSSM_OK == err) &&
                               (0 < NumVendorPrivs))
                            {
                                /* now we construct a vendor privilege
                                   structure and insert it in the caller entry
                                */
                                tempVendorPrivs = local_calloc(sizeof(cssm_VENDOR_PRIVILEGES),
                                                               1,
                                                               NULL);
                                if(NULL != tempVendorPrivs)
                                {
                                    tempVendorPrivs->VendorPrivs = VendorPrivs;
                                    tempVendorPrivs->NumVendorPrivs = NumVendorPrivs;
                                    tempVendorPrivs->KeyHierarchy = KeyHierarchy;

                                    tempVendorPrivs->Next = Entry->VendorPrivs;
                                    Entry->VendorPrivs = tempVendorPrivs;
                                }
                                else
                                {
                                    ERR(err = CSSMERR_CSSM_MEMORY_ERROR);
                                }
                            }
                        }
                    }
                }

                i++;    /* move to the next manifest section */
            }
        }
        else
        {
            ERR(err = CSSMERR_CSSM_MODULE_MANIFEST_VERIFY_FAILED);
        }
    }
    else
    {
        ERR(err = CSSMERR_CSSM_INVALID_POINTER);
    }

    return err;
}



CSSM_RETURN cssm_CheckPVCTagsAndSetPVCPolicy(const CSSM_GUID* CallerGuid, 
                                             CSSM_PVC_MODE *PvcPolicy,
                                             CSSM_KEY_HIERARCHY KeyHierarchy,
                                             ISL_VERIFIED_MODULE_PTR* CssmExpVerifiedCredentials,
                                             uint32 NumExpSections)
{
    CSSM_RETURN ret = CSSM_OK;
    uint32 i;

    if (CallerGuid == NULL) 
    {
        /* If caller cannot be authenticated, check cssm's manifest section attribute 
           and see if CDSA_PVC_API tag is set to OFF or no tag. If so PvcPolicy could be
           0.CSSM cannot check the App.If CDSA_PVCSPI is OFF or no tag,
           PvcPolicy could be 2. So If the caller guid is NULL, and CSSM manifest has 
           CDSA_PVC_API and CDSA_PVC_SPI tags OFF or no tag the acceptable PvcPolicy value is 
           CSSM_PVC_
         */
        if( *PvcPolicy == (CSSM_PVC_APP| CSSM_PVC_SP) )
        {
            ret = CSSM_OK;
        }
        else
        {
            if( (KeyHierarchy == CSSM_KEY_HIERARCHY_NONE) || 
                (KeyHierarchy & CSSM_KEY_HIERARCHY_INTEG) )
            {
                if( cssm30_Credentials != NULL)
                {
                    ret = CSSMERR_CSSM_INVALID_PVC; /* if there's no error,
                                                       this will change.
                                                    */
                    i = 0;
                    while((CSSM_OK != ret) &&
                          (i < cssm30_NumSections))
                    {
                        ERR(ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(cssm30_Credentials[i],
                                                                                           NULL,
                                                                                           PvcPolicy));
                        i++;
                    }
                }
                else
                {
                    ret = CSSMERR_CSSM_INVALID_PVC;
                }

            }
            if( ret == CSSM_OK )
            {
                if( KeyHierarchy & CSSM_KEY_HIERARCHY_EXPORT )
                {
                    if( CssmExpVerifiedCredentials != NULL )
                    {
                        ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(  *CssmExpVerifiedCredentials,
                                                                                        NULL,
                                                                                        PvcPolicy);
                    }
                }
            }
                
        }
    }
    else 
    {
        uint32 NumAppIntegSections = 0;
        ISL_VERIFIED_MODULE_PTR* AppIntegCredentialPtr = NULL;
        uint32 NumAppExpSections = 0;
        ISL_VERIFIED_MODULE_PTR* AppExpCredentialPtr = NULL;
        CSSM_KEY_HIERARCHY tempKeyHierarchy;

        if( *PvcPolicy == (CSSM_PVC_APP| CSSM_PVC_SP) )
        {
            ret = CSSM_OK;
        }
        else
        {
            CSSM_GUID tempguid;
            port_memset( &tempguid, 0, sizeof(CSSM_GUID) );
            if( !port_memcmp( CallerGuid, &tempguid, (uint32)sizeof(CSSM_GUID) ))
            {
                ret = CSSMERR_CSSM_INVALID_GUID;    
            }
            else
            {
                if(CSSM_KEY_HIERARCHY_NONE == KeyHierarchy)
                {
                    tempKeyHierarchy = CSSM_KEY_HIERARCHY_INTEG;
                }
                else
                {
                    tempKeyHierarchy = KeyHierarchy;
                }

                if(CSSM_KEY_HIERARCHY_INTEG & tempKeyHierarchy)
                {

                    ERR(ret = cssm_VerifyCaller(CSSM_KEY_HIERARCHY_INTEG,
                                                &NumAppIntegSections,
                                                &AppIntegCredentialPtr,
                                                CallerGuid));
                }

                if((CSSM_OK == ret) &&
                   (CSSM_KEY_HIERARCHY_EXPORT & tempKeyHierarchy))
                {
                    ERR(ret = cssm_VerifyCaller(CSSM_KEY_HIERARCHY_EXPORT,
                                                &NumAppExpSections,
                                                &AppExpCredentialPtr,
                                                CallerGuid));
                }
            }

            if( ret == CSSM_OK)
            {
                if( ( KeyHierarchy == CSSM_KEY_HIERARCHY_NONE ) || 
                    ( KeyHierarchy & CSSM_KEY_HIERARCHY_INTEG ) )
                {
                    if( cssm30_Credentials != NULL)
                    {
                        i = 0;
                        ret = CSSMERR_CSSM_INVALID_PVC; /* if there's no error,
                                                           this will change.
                                                        */

                        /* first we check the CSSM manifest, then we'll check
                           the applications manifest
                        */
                        while((CSSM_OK != ret) &&
                              (i < cssm30_NumSections))
                        {
                            ERR(ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(cssm30_Credentials[i],
                                                                                               AppIntegCredentialPtr[0],
                                                                                               PvcPolicy));
                            i++;
                        }

                        if(CSSM_OK != ret)
                        {
                            i = 0;

                            while((CSSM_OK != ret) &&
                                  (i < NumAppIntegSections))
                            {
                                ERR(ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(cssm30_Credentials[0],
                                                                                                   AppIntegCredentialPtr[i],
                                                                                                   PvcPolicy));
                                i++;
                            }
                        }
                    }
                    else
                    {
                        ret = CSSMERR_CSSM_INVALID_PVC;
                    }
                }

                if(CSSM_OK == ret)
                {
                    if( KeyHierarchy & CSSM_KEY_HIERARCHY_EXPORT )
                    {
                        if( CssmExpVerifiedCredentials != NULL)
                        {
                            i = 0;
                            ret = CSSMERR_CSSM_INVALID_PVC; /* if there's no error,
                                                               this will change.
                                                            */

                            /* first we check the CSSM manifest, then we'll check
                               the applications manifest
                            */
                            while((CSSM_OK != ret) &&
                                  (i < NumExpSections))
                            {
                                ERR(ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(CssmExpVerifiedCredentials[i],
                                                                                                   AppExpCredentialPtr[0],
                                                                                                   PvcPolicy));
                                i++;
                            }

                            if(CSSM_OK != ret)
                            {
                                i = 0;

                                while((CSSM_OK != ret) &&
                                      (i < NumAppExpSections))
                                {
                                    ERR(ret = cssm_CheckCallerAndCssmManifestSectionAttributesForSPPvc(CssmExpVerifiedCredentials[0],
                                                                                                       AppExpCredentialPtr[i],
                                                                                                       PvcPolicy));
                                    i++;
                                }
                            }
                        }
                        else
                        {
                            ret = CSSMERR_CSSM_INVALID_PVC;
                        }
                    }
                }

                /* free memory used for app credentials */
                if((AppIntegCredentialPtr != NULL) &&
                   (NumAppIntegSections != 0))
                {
                    cssm_FreeVerifiedCredentials(AppIntegCredentialPtr,
                                                 NumAppIntegSections);
                    local_free( AppIntegCredentialPtr, NULL);
                    AppIntegCredentialPtr = NULL;
                    NumAppIntegSections = 0;
                }

                if((AppExpCredentialPtr != NULL) &&
                   (NumAppExpSections != 0))
                {
                    cssm_FreeVerifiedCredentials(AppExpCredentialPtr,
                                                 NumAppExpSections);
                    local_free( AppExpCredentialPtr, NULL);
                    AppExpCredentialPtr = NULL;
                    NumAppExpSections = 0;
                }
            }
        }
    }
    RETURN ( ret )
}



CSSM_RETURN
cssm_ExtractUseeAndPrivFromCssmManifest(ISL_VERIFIED_MODULE_PTR *VerModPtr,
                                        uint32 NumSections,
                                        uint32 *ModNumUseeTags,
                                        uint32** ModUseeTags,
                                        uint32 *ModNumTogPrivs,
                                        uint32** ModTogPrivs,
                                        cssm_VENDOR_PRIVILEGES_PTR* ModVendorPrivilegePtr,
                                        CSSM_KEY_HIERARCHY KeyHierarchy)
{
    CSSM_RETURN err = CSSM_OK;
    uint32 i =0;
    ISL_MANIFEST_SECTION_PTR ManSecPtr = NULL;
    ISL_CONST_DATA PrivName;
    uint8* USEEName = (uint8*)"CDSA_USEE";
    uint8* VendorTOGPrivName = (uint8*)"CDSA_PRIV";

    uint32 NumPrivs=0;
    uint32 *Privs = NULL;
    uint32 NumVendorTOGPrivs = 0;
    uint32* VendorTOGPrivs = NULL;
    uint32 NumVendorPrivs=0;
    uint32 *VendorPrivs=NULL;
    uint32 NumTOGPrivs=0;
    uint32 *TOGPrivs = NULL;

    cssm_VENDOR_PRIVILEGES_PTR tempVendorPrivs = NULL;

    i = 0;

    /* we'll need to loop through each section of this manifest */
    while(i < (NumSections))
    {
        ManSecPtr = EISL_GetModuleManifestSection(VerModPtr[i]);

        if(NULL != ManSecPtr)
        {
            if( KeyHierarchy == CSSM_KEY_HIERARCHY_EXPORT)
            {
                PrivName.Data = USEEName;
                PrivName.Length = strlen((char*)USEEName);

                /* get the USEE privs from the manifest */
                ERR(err =  cssm_ExtractPrivilege(ManSecPtr,
                                                 &NumPrivs,
                                                 &Privs,
                                                 PrivName));

                if((CSSM_OK == err) &&
                   (0 < NumPrivs))
                {
                    if(NULL == *ModUseeTags)
                    {
                        *ModUseeTags = Privs;
                        *ModNumUseeTags = NumPrivs;
                    }
                    else
                    {
                        local_realloc(*ModUseeTags,
                                      (sizeof(uint32) *
                                      (*ModNumUseeTags + NumPrivs)),
                                      NULL);
                        memcpy(&(*ModUseeTags[*ModNumUseeTags]),
                                 Privs,
                                 sizeof(uint32) * NumPrivs);
                               *ModNumUseeTags += NumPrivs;
                    }
                 }
            }
            
            if(CSSM_OK == err)
            {
                PrivName.Data = VendorTOGPrivName;
                PrivName.Length = strlen((char*)VendorTOGPrivName);

                /* get the privs from the manifest */
                ERR(err =  cssm_ExtractPrivilege( ManSecPtr,
                                                  &NumVendorTOGPrivs,
                                                  &VendorTOGPrivs,
                                                  PrivName ));
                if(CSSM_OK == err)
                {
                /* break the privileges into TOG and
                  vendor-specific parts
                 */
                     ERR(err = cssm_FilterTOGAndVendorPrivs(NumVendorTOGPrivs,
                                                            VendorTOGPrivs,
                                                            &NumVendorPrivs,
                                                            &VendorPrivs,
                                                            &NumTOGPrivs,
                                                            &TOGPrivs));

                    if((CSSM_OK == err) &&
                       (0 < NumTOGPrivs))
                    {
                        if(NULL == ModTogPrivs)
                        {
                            *ModTogPrivs = TOGPrivs;
                            *ModNumTogPrivs = NumTOGPrivs;
                        }
                        else
                        {
                            local_realloc(*ModTogPrivs,
                                          (sizeof(uint32) * (*ModNumTogPrivs + NumPrivs)),
                                          NULL);
                            memcpy(&(*ModTogPrivs[*ModNumTogPrivs]),
                                    Privs,
                                    sizeof(uint32) * NumPrivs);
                            ModNumTogPrivs += NumPrivs;
                        }
                    }

                    if((CSSM_OK == err) &&
                        (0 < NumVendorPrivs))
                    {
                        /* now we construct a vendor privilege
                           structure and insert it in the caller entry
                         */
                        tempVendorPrivs = local_calloc(sizeof(cssm_VENDOR_PRIVILEGES),
                                                            1,
                                                              NULL);
                        if(NULL != tempVendorPrivs)
                        {
                            tempVendorPrivs->VendorPrivs = VendorPrivs;
                            tempVendorPrivs->NumVendorPrivs = NumVendorPrivs;
                            tempVendorPrivs->KeyHierarchy = KeyHierarchy;

                            tempVendorPrivs->Next = *ModVendorPrivilegePtr;
                            *ModVendorPrivilegePtr = tempVendorPrivs;
                        }
                        else
                        {
                             ERR(err = CSSMERR_CSSM_MEMORY_ERROR);
                        }
                     }
                 }
             }
        }
        i++;    /* move to the next manifest section */
    }
    RETURN (err )
}
#endif
