/*-----------------------------------------------------------------------
 * File: CSSMINTGR.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
/*
 * This file contains the code used to do the cssm integrity checking related
   functions
 */

#include "precomp.h"


#ifdef ISL_INCLUDED

/*---------------------------------------------------------------
 *
 *Name: cssm_CheckFunctionTableInverifiedModules
 *
 *Description:
 *   This function checks if CSSM_REGISTRATION_INFO functions and functions in
 *   CSSM_MODULE_FUNCS structure are in the module
 *   
 *Parameters: 
 *   pVerifiedModule - Verified Module pointer
 *   AddInJT - pointer to CSSM_REGISTARION_INFO structre
 *Returns: CSSM_TRUE or CSSM_FALSE
 *
 *-------------------------------------------------------------*/
CSSM_BOOL
cssm_CheckFunctionTableInverifiedModules(uint32 NumVerifiedModules,
                                         ISL_VERIFIED_MODULE_PTR *pVerifiedModules,
                                         CSSM_MODULE_FUNCS_PTR ModuleFuncs)
{
    uint32 i, j;
    CSSM_BOOL InVerifiedModule = CSSM_TRUE;
    ISL_STATUS eislReturn;

    if(cssm_GetPVCPolicy() & CSSM_PVC_SP)
    {
        /* do parameter checking - make sure all the supposed memory is there */
        if((NULL != pVerifiedModules) &&
           (CSSM_FALSE == port_IsBadReadPtr(ModuleFuncs,
                                            sizeof(CSSM_MODULE_FUNCS))) &&
           (CSSM_FALSE == port_IsBadReadPtr(ModuleFuncs->ServiceFuncs,
                                            (sizeof(CSSM_PROC_ADDR) * ModuleFuncs->NumberOfServiceFuncs))))
        {
            i = 0;

            /* check that all addresses given are valid */
            while((CSSM_TRUE == InVerifiedModule) &&
                  (i < ModuleFuncs->NumberOfServiceFuncs))
            {
                if(NULL != ModuleFuncs->ServiceFuncs[i])
                {
                    j = 0;
                    InVerifiedModule = CSSM_FALSE;

                    while((j < NumVerifiedModules) &&
                          (CSSM_FALSE == InVerifiedModule))
                    {
                        eislReturn = EISL_CheckAddressWithinModule(pVerifiedModules[j],
                                                                   (ISL_FUNCTION_PTR)ModuleFuncs->ServiceFuncs[i]);
                        if(ISL_OK == eislReturn)
                        {
                            InVerifiedModule = CSSM_TRUE;
                        }

                        j++;
                    }
                }

                i++;
            }
        }
        else
        {
            InVerifiedModule = CSSM_FALSE;
        }
    }
    RETURN (InVerifiedModule)
}



#endif

