/*-----------------------------------------------------------------------
 * File: EMMINTRN.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
 

#ifndef EMM_INTRN_H
#define EMM_INTRN_H
#include "cssmport.h"
#include "cssmlock.h"
#include "internal.h"

#ifdef ISL_INCLUDED
#include "eislapi.h"
#endif


#define cssm_IsCSSMMask(ModuleType) ((ModuleType) & (CSSM_SERVICE_CSSM | \
                                                     CSSM_SERVICE_CSP | \
                                                     CSSM_SERVICE_DL | \
                                                     CSSM_SERVICE_CL | \
                                                     CSSM_SERVICE_TP | \
                                                     CSSM_SERVICE_AC))

/*
 * Internal EMM functions.
 */
CSSM_RETURN CSSMAPI
cssm_GetEMMManagerRecordByGUID(const CSSM_GUID *ModuleGUID,
                               cssm_INTERNALMANAGERLIST_PTR *ppEMMRecord);
CSSM_RETURN CSSMAPI
cssm_GetEMMManagerRecordByType(CSSM_SERVICE_MASK ModuleManagerType,
                               cssm_INTERNALMANAGERLIST_PTR *ppEMMRecord);

CSSM_RETURN CSSMAPI
cssm_GetEMMManagerRecordsByType(CSSM_SERVICE_MASK ModuleManagerType,
                                uint32 *NumManagers,
                                cssm_INTERNALMANAGERLIST_PTR *ManagerArray);

CSSM_RETURN CSSMAPI cssm_EMMManagerLoad ( CSSM_SERVICE_MASK ManagerType,
                                        const CSSM_GUID *CallerGuid,
                                        CSSM_GUID_PTR ManagerGUID,
                                        cssm_INTERNALMANAGERLIST_PTR *ppEMMRecord,
                                        const CSSM_VERSION *Version,
                                        CSSM_KEY_HIERARCHY KeyHierarchy);
CSSM_RETURN CSSMAPI
cssm_checkEMMAndLoad(const CSSM_GUID *ModuleGuid,
                     const CSSM_GUID *CallerGuid,
                     CSSM_GUID_PTR ManagerGUID,
                     CSSM_SERVICE_TYPE ModuleType,
                     cssm_INTERNALMANAGERLIST_PTR *ppEMMRecord,
                     const CSSM_VERSION *Version,
                     CSSM_KEY_HIERARCHY KeyHierarchy);

CSSM_RETURN CSSMAPI cssm_EMMManagerDetach (CSSM_GUID_PTR pManagerGUID, 
                                           cssm_INTERNALMANAGERLIST_PTR* pEMMRecord);                               

CSSM_RETURN CSSMAPI cssm_EMMManagerUnload (const CSSM_GUID *pManagerGUID);
CSSM_RETURN CSSMAPI cssm_NewEMMManagerRecord (const CSSM_GUID *GUID);
CSSM_RETURN CSSMAPI cssm_UpdateEMMManagerRecord (
                                 const CSSM_GUID *GUID,
                                 const CSSM_MANAGER_REGISTRATION_INFO *FunctionTable);
CSSM_RETURN CSSMAPI cssm_RemoveEMMManagerRecord (const CSSM_GUID *GUID);



CSSM_RETURN cssm_RegisterDispatchTable (CSSM_SERVICE_MASK ModuleManagerType);
                                                                                  
CSSM_RETURN CSSMAPI cssm_DeregisterManagerServices (const CSSM_GUID* GUID);

CSSM_RETURN CSSMAPI
cssm_DeliverModuleManagerEvent(
                      const CSSM_MANAGER_EVENT_NOTIFICATION *EventDescription);


#ifdef ISL_INCLUDED

CSSM_RETURN CSSMAPI
cssm_EMMLoadAndCheckFromCredAndPath(CSSM_DATA EMMCredential,
                                    CSSM_DATA EMMSearchPath,
                                    CSSM_KEY_HIERARCHY KeyType,
                                    CSSM_BOOL LoadModule,
                                    uint32 *NumEmmSections,
                                    ISL_VERIFIED_MODULE_PTR** pEmmVerifiedmModule);


CSSM_RETURN
cssm_EMMLoadAndCheckFromGUID(const CSSM_GUID *ModuleGuid,
                             CSSM_KEY_HIERARCHY KeyType,
                             CSSM_BOOL LoadModule,
                             uint32 *NumEmmSections,
                             ISL_VERIFIED_MODULE_PTR** pEmmVerifiedmModule);


CSSM_RETURN cssm_LoadEMMUsingExportOrIntegrityKeys(
                                            CSSM_DATA EMMCredential,
                                            CSSM_DATA EMMSearchPath,
                                            CSSM_GUID_PTR EmmGuid,
                                            const CSSM_GUID* CallerGuid,
                                            CSSM_KEY_HIERARCHY KeyHierarchy,
                                            uint32* NumEmmSections,
                                            ISL_VERIFIED_MODULE_PTR** VerifiedModulePtrArr,
                                            CSSM_MANAGER_REGISTRATION_INFO_PTR ManagerInfo);

CSSM_RETURN CSSMAPI
cssm_EmmManagerAuthenticate(CSSM_KEY_HIERARCHY KeyHierarchy,
                            const CSSM_GUID *CallerGuid,
                            ISL_VERIFIED_MODULE_PTR pVerifiedEmmModule,
                            CSSM_MANAGER_REGISTRATION_INFO_PTR ManagerRegistrationInfo);

#endif
#endif /* CSSM_EMM_H */
