/*-----------------------------------------------------------------------
 * File: CDSACLENCPRODUCT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */


#include <assert.h>
#include <string.h>
#ifdef VMS
#include "cssmport.h"
#else
#include "port/cssmport.h"
#endif
#include "cssmtype.h"
#include "cssmerr.h"
#include "mds_schema.h"

#include "mds.h"
#include "mds_util.h"
#include "convert.h"
#include "internal.h"

#define __ModuleID              ( 0 )
#define __SSID                  ( 1 )
#define __ProductDesc           ( 2 )
#define __ProductVendor         ( 3 )
#define __ProductVersion        ( 4 )
#define __ProductFlags          ( 5 )
#define __StandardDesc          ( 6 )
#define __StandardVersion       ( 7 )

typedef struct _mdsu_cdsa_cl_enc_prod_attr_state
{
    CSSM_DB_ATTRIBUTE_DATA DbRecordAttributes[MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES];
    CSSM_DATA AttributeValues[MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_MDSU_PRINTABLE_GUID_LENGTH];
    uint8 PrintProductVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
    uint8 PrintStandardVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
}
_MDSU_CDSA_CL_ENC_PROD_ATTR_STATE;

static CSSM_RETURN CSSMAPI __MDSU_ConstructCDSACLEncProductAttributes(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaData,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFillValues,
                            void **ppAttrState )
{
    CSSM_RETURN rv = CSSM_OK;
    const MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA *pProduct = 
                        (const MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA*)pSchemaData;
    _MDSU_CDSA_CL_ENC_PROD_ATTR_STATE *pState = NULL;
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( ( bFillValues == CSSM_FALSE ) ||
            ( ( bFillValues == CSSM_TRUE ) && pContext && pSchemaData && DbRecord ) );

    /* Allocate the memory for the attributes */
    pState = MDSU_malloc( pContext, sizeof(_MDSU_CDSA_CL_ENC_PROD_ATTR_STATE) );
    if ( pState == NULL )
    {
        ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
    }
    else
    {
        /* Clear the structure */
        memset( pState,
                0,
                sizeof(*pState) );
        DbRecordAttributes = pState->DbRecordAttributes;

        /* Fill in the record information */
        DbRecord->DataRecordType = 
                            MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE;
        DbRecord->SemanticInformation = 0;
        DbRecord->NumberOfAttributes = 
                            MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES;
        DbRecord->AttributeData = DbRecordAttributes;

        /* Fill in the attribute info */
        DbRecordAttributes[__ModuleID].Info = s_MdsAttrInfo_ModuleId;
        DbRecordAttributes[__SSID].Info = s_MdsAttrInfo_SSID;
        DbRecordAttributes[__ProductDesc].Info = s_MdsAttrInfo_ProductDesc;
        DbRecordAttributes[__ProductVendor].Info = s_MdsAttrInfo_ProductVendor;
        DbRecordAttributes[__ProductVersion].Info = s_MdsAttrInfo_ProductVersion;
        DbRecordAttributes[__ProductFlags].Info = s_MdsAttrInfo_ProductFlags;
        DbRecordAttributes[__StandardDesc].Info = s_MdsAttrInfo_StandardDesc;
        DbRecordAttributes[__StandardVersion].Info = s_MdsAttrInfo_StandardVersion;

        if ( bFillValues == CSSM_TRUE )
        {
            /* Connect the attribute value array to the attribute array */
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = &pState->AttributeValues[i];
                DbRecordAttributes[i].NumberOfValues = 1;
            }

            /* Fill in the record contents */
            /* ModuleID */
            _MDSU_GetPrintableGUID( &pProduct->ModuleID,
                                    (char*)pState->PrintableGuid );
            DbRecordAttributes[__ModuleID].Value->Data =
                                                    pState->PrintableGuid;
            DbRecordAttributes[__ModuleID].Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;

            /* SSID */
            DbRecordAttributes[__SSID].Value->Length = sizeof(uint32);
            DbRecordAttributes[__SSID].Value->Data = (uint8*)&pProduct->SSID;

            /* ProductDesc */
            DbRecordAttributes[__ProductDesc].Value->Data = 
                                                    (uint8*)pProduct->ProductDesc;
            DbRecordAttributes[__ProductDesc].Value->Length =
                                            strlen( pProduct->ProductDesc ) + 1;

            /* ProductVendor */
            DbRecordAttributes[__ProductVendor].Value->Length = 
                                            strlen( pProduct->ProductVendor ) + 1;
            DbRecordAttributes[__ProductVendor].Value->Data =
                                                (uint8*)pProduct->ProductVendor;

            /* ProductVersion */
            _MDSU_GetPrintableVersion(
                                &pProduct->ProductVersion,
                                (char*)pState->PrintProductVersion );
            DbRecordAttributes[__ProductVersion].Value->Data =
                                                pState->PrintProductVersion;
            DbRecordAttributes[__ProductVersion].Value->Length = 
                            strlen( (char*)pState->PrintProductVersion ) + 1;

            /* ProductFlags */
            DbRecordAttributes[__ProductFlags].Value->Length = sizeof(uint32);
            DbRecordAttributes[__ProductFlags].Value->Data =
                                                (uint8*)&pProduct->ProductFlags;

            /* StandardDesc */
            DbRecordAttributes[__StandardDesc].Value->Length = 
                                            strlen( pProduct->StandardDesc ) + 1;
            DbRecordAttributes[__StandardDesc].Value->Data =
                                                (uint8*)pProduct->StandardDesc;

            /* StandardVersion */
            _MDSU_GetPrintableVersion(
                                &pProduct->StandardVersion,
                                (char*)pState->PrintStandardVersion );
            DbRecordAttributes[__StandardVersion].Value->Data =
                                            pState->PrintStandardVersion;
            DbRecordAttributes[__StandardVersion].Value->Length = 
                        strlen( (char*)pState->PrintStandardVersion ) + 1;
        }
        else
        {
            for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
            {
                DbRecordAttributes[i].Value = NULL;
                DbRecordAttributes[i].NumberOfValues = 0;
            }
        }
    } /* Allocate attributes array */

    *ppAttrState = pState;

    return rv;

}

static CSSM_RETURN CSSMAPI __MDSU_DestructCDSACLEncProductAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            CSSM_BOOL bFreeAll,
                            void *pAttrState )
{
    CSSM_DB_ATTRIBUTE_DATA *DbRecordAttributes;
    uint32 i;

    assert( pContext && DbRecord );

    /* Free the buffers allocated for the attribute values */
    DbRecordAttributes = DbRecord->AttributeData;
    if ( bFreeAll == CSSM_TRUE )
    {
        for ( i = 0; i < DbRecord->NumberOfAttributes; i++ )
        {
            if ( DbRecordAttributes[i].Value )
            {
                if ( DbRecordAttributes[i].Value->Data )
                {
                    MDSU_free( pContext,
                               DbRecordAttributes[i].Value->Data );
                }
                MDSU_free( pContext,
                           DbRecordAttributes[i].Value );
            }
        }
    }

    /* Free the attribute array */
    MDSU_free( pContext,
               pAttrState );

    return CSSM_OK;
}

typedef struct _mdsu_cdsa_cl_enc_prod_pred_state
{
    CSSM_SELECTION_PREDICATE Predicates[MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES];
    CSSM_DATA PredicateValues[MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES];
    uint8 PrintableGuid[_MDSU_PRINTABLE_GUID_LENGTH];
    uint8 PrintProductVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
    uint8 PrintStandardVersion[_MDSU_PRINTABLE_VERSION_LENGTH];
}
_MDSU_CDSA_CL_ENC_PROD_PRED_STATE;

static CSSM_RETURN CSSMAPI __MDSU_ConstructCDSACLEncProductPredicate(
                            MDSU_CONTEXT *pContext,
                            const void *pSchemaTemplate,
                            MDSU_TEMPLATE_MASK ValidTemplates,
                            CSSM_QUERY *pQuery,
                            void **ppPredState )
{
    CSSM_RETURN rv = CSSM_OK;
    const MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA *pProduct = 
                    (const MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA*)pSchemaTemplate;
    _MDSU_CDSA_CL_ENC_PROD_PRED_STATE *pState = NULL;
    CSSM_SELECTION_PREDICATE *pPredicates = NULL;
    uint32 uNumPredicates;
    uint32 uIndex = 0;
    uint32 i;

    assert( pContext && 
            pQuery &&
            ppPredState &&
            ( ( pSchemaTemplate && ValidTemplates ) ||
              ( !pSchemaTemplate && !ValidTemplates ) ) );

    /* Count the predicates */
    _MDSU_CountBits( ValidTemplates, uNumPredicates );

    /* Allocate the predicate state and link the value structures */
    if ( uNumPredicates != 0 )
    {
        pState = MDSU_malloc( pContext, sizeof(*pState) );
        if ( pState == NULL )
        {
            ERR( rv = CSSMERR_CSSM_MEMORY_ERROR );
        }
        else
        {
            memset( pState,
                    0,
                    sizeof(*pState) );
            pPredicates = pState->Predicates;

            /* Allocate a single attribute value struct per attribute */
            for ( i = 0; i < uNumPredicates; i++ )
            {
                pPredicates[i].Attribute.Value =
                                            &pState->PredicateValues[i];
                pPredicates[i].Attribute.NumberOfValues = 1;
            }
        } /* Allocate and link predicate values */
    } /* uNumPredicates != 0 */

    if ( rv == CSSM_OK )
    {
        /* Fill in the query parameters */
        pQuery->RecordType = MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE;
        pQuery->Conjunctive = CSSM_DB_AND;
        pQuery->NumSelectionPredicates = uNumPredicates;
        pQuery->QueryLimits.TimeLimit = CSSM_QUERY_TIMELIMIT_NONE;
        pQuery->QueryLimits.SizeLimit = CSSM_QUERY_SIZELIMIT_NONE;
        pQuery->QueryFlags = 0;
        pQuery->SelectionPredicate = pPredicates;
        for ( uIndex = 0; uIndex < uNumPredicates; uIndex++ )
        {
            pPredicates[uIndex].DbOperator = CSSM_DB_EQUAL;
        }

        /* Fill in the predicate data */
        uIndex = 0;
        if ( ValidTemplates & MDSU_CLEP_MODULE_ID )
        {
            /* ModuleID */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ModuleId;
            _MDSU_GetPrintableGUID( &pProduct->ModuleID,
                                    (char*)pState->PrintableGuid );
            pPredicates[uIndex].Attribute.Value->Data = pState->PrintableGuid;
            pPredicates[uIndex].Attribute.Value->Length = 
                                strlen( (char*)pState->PrintableGuid ) + 1;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CLEP_SSID )
        {
            /* SSID */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_SSID;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        sizeof(pProduct->SSID) ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pProduct->SSID;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CLEP_PRODUCT_DESC )
        {
            /* ProductDesc */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ProductDesc;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pProduct->ProductDesc ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pProduct->ProductDesc;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_CLEP_PRODUCT_VENDOR )
        {
            /* ProductVendor */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ProductVendor;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pProduct->ProductVendor ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pProduct->ProductVendor;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_CLEP_PRODUCT_VERSION )
        {
            /* ProductVersion */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ProductVersion;
            _MDSU_GetPrintableVersion( &pProduct->ProductVersion,
                                       (char*)pState->PrintProductVersion );
            pPredicates[uIndex].Attribute.Value->Data =
                                                pState->PrintProductVersion;
            pPredicates[uIndex].Attribute.Value->Length = 
                            strlen( (char*)pState->PrintProductVersion ) + 1;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_CLEP_PRODUCT_FLAGS )
        {
            /* ProductFlags */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_ProductFlags;
            pPredicates[uIndex].Attribute.Value->Length = sizeof(uint32);
            pPredicates[uIndex].Attribute.Value->Data = 
                                                    (uint8*)&pProduct->ProductFlags;
            uIndex++;
        }

        if ( ValidTemplates & MDSU_CLEP_STANDARD_DESC )
        {
            /* StandardDesc */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_StandardDesc;
            pPredicates[uIndex].Attribute.Value->Length = 
                                        strlen( pProduct->StandardDesc ) + 1 ;
            pPredicates[uIndex].Attribute.Value->Data =
                                                (uint8*)pProduct->StandardDesc;
            uIndex++;
        }
        if ( ValidTemplates & MDSU_CLEP_STANDARD_VERSION )
        {
            /* StandardVersion */
            pPredicates[uIndex].Attribute.Info = s_MdsAttrInfo_StandardVersion;
            _MDSU_GetPrintableVersion( &pProduct->StandardVersion,
                                       (char*)pState->PrintStandardVersion );
            pPredicates[uIndex].Attribute.Value->Data =
                                                pState->PrintStandardVersion;
            pPredicates[uIndex].Attribute.Value->Length = 
                            strlen( (char*)pState->PrintStandardVersion ) + 1;
            uIndex++;
        }
    } /* Initial allocation successful */

    *ppPredState = (void*)pState;

    return rv;
}

static CSSM_RETURN CSSMAPI __MDSU_DestructCDSACLEncProductPredicate(
                            MDSU_CONTEXT *pContext,
                            CSSM_QUERY *pQuery,
                            void *pPredState )
{
    assert( pContext && pQuery );

    /* Free the attribute array */
    if ( pPredState != NULL )
    {
        MDSU_free( pContext, pPredState );
    }

    return CSSM_OK;
}

static CSSM_RETURN CSSMAPI __MDSU_ConvertCDSACLEncProductAttributes(
                            MDSU_CONTEXT *pContext,
                            CSSM_DB_RECORD_ATTRIBUTE_DATA *DbRecord,
                            void *pSchemaData )
{
    CSSM_RETURN crReturn;
    MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA *pProduct = 
                            (MDSU_CDSA_CL_ENC_PRODUCT_SCHEMA*)pSchemaData;
    const CSSM_DB_ATTRIBUTE_DATA *DbAttr = NULL;

    assert( pSchemaData && DbRecord && pContext );

    DbAttr = DbRecord->AttributeData;

    /* ModuleID */
    crReturn = _MDSU_GetStructuredGUID( 
                                (char*)DbAttr[__ModuleID].Value->Data,
                                &pProduct->ModuleID );

    /* SSID */
    pProduct->SSID = *(uint32*)DbAttr[__SSID].Value->Data;

    /* ProductDesc */
	if ( DbAttr[__ProductDesc].Value->Data != NULL )
        strncpy( pProduct->ProductDesc,
                (char*)DbAttr[__ProductDesc].Value->Data,
				DbAttr[__ProductDesc].Value->Length );
	else
		strcpy( pProduct->ProductDesc, "" );

    /* ProductVendor */
	if ( DbAttr[__ProductVendor].Value->Data != NULL )
        strncpy( pProduct->ProductVendor,
                (char*)DbAttr[__ProductVendor].Value->Data,
				DbAttr[__ProductVendor].Value->Length );
	else
		strcpy ( pProduct->ProductVendor, "" );

    /* ProductVersion */
    _MDSU_GetStructuredVersion( (char*)DbAttr[__ProductVersion].Value->Data,
                                &pProduct->ProductVersion );

    /* ProductFlags */
    pProduct->ProductFlags = *(uint32*)DbAttr[__ProductFlags].Value->Data;

    /* StandardDesc */
	if ( DbAttr[__StandardDesc].Value->Data != NULL )
        strncpy( pProduct->StandardDesc,
                (char*)DbAttr[__StandardDesc].Value->Data,
				DbAttr[__StandardDesc].Value->Length );
	else
		strcpy( pProduct->StandardDesc, "" );

    /* StandardVersion */
    _MDSU_GetStructuredVersion( (char*)DbAttr[__StandardVersion].Value->Data,
                                &pProduct->StandardVersion );

    return CSSM_OK;
}

const IfMdsuSchemaManipulator IfiMdsuCdsadirCLEncProductSchema =
{
    __MDSU_ConstructCDSACLEncProductAttributes,
    __MDSU_DestructCDSACLEncProductAttributes,
    __MDSU_ConstructCDSACLEncProductPredicate,
    __MDSU_DestructCDSACLEncProductPredicate,
    __MDSU_ConvertCDSACLEncProductAttributes,
    MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE
};

