/*-----------------------------------------------------------------------
 * File: CSSMCCF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Cryptographic Context APIs. The
 * CSSM_DEFINE_FUNC and __CSSM_PASTE2 macros must be defined
 * by the file that includes this header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif
#ifndef __CSSM_PASTE2
#error "The __CSSM_PASTE2(x,y) macro must be defined!"
#endif

/* Use this macro to define the name of CSP functions that have and explicit
 * CSP indication in the API name.
 */
# define _CSSM_EXPLICIT_NAME( name ) __CSSM_PASTE2( CSP_, name )

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateSignatureContext), (
						CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_KEY *Key,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateSymmetricContext), (
					 	CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						CSSM_ENCRYPT_MODE Mode,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_KEY *Key,
						const CSSM_DATA *InitVector,
						CSSM_PADDING Padding,
						void *Reserved,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateDigestContext), (
						CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateMacContext), (
						CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						const CSSM_KEY *Key,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateRandomGenContext), (
					    CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						const CSSM_CRYPTO_DATA *Seed,
						uint32 Length,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateAsymmetricContext), (
					    CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_KEY *Key,
						CSSM_PADDING Padding,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateDeriveKeyContext), (
						CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						CSSM_KEY_TYPE DeriveKeyType,
						uint32 DeriveKeyLengthInBits,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_KEY *BaseKey,
						uint32 IterationCount,
						const CSSM_DATA *Salt,
						const CSSM_CRYPTO_DATA *Seed,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreateKeyGenContext), (
						CSSM_CSP_HANDLE CSPHandle,
						CSSM_ALGORITHMS AlgorithmID,
						uint32 KeySizeInBits,
						const CSSM_CRYPTO_DATA *Seed,
						const CSSM_DATA *Salt,
						const CSSM_DATE *StartDate,
						const CSSM_DATE *EndDate,
						const CSSM_DATA *Params,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( _CSSM_EXPLICIT_NAME(CreatePassThroughContext), (
						CSSM_CSP_HANDLE CSPHandle,
						const CSSM_KEY *Key,
						CSSM_CC_HANDLE *NewContextHandle ) );

CSSM_DEFINE_FUNCTION( GetContext, (
						CSSM_CC_HANDLE CCHandle,
						CSSM_CONTEXT_PTR *Context ) );

CSSM_DEFINE_FUNCTION( FreeContext, (
						CSSM_CONTEXT_PTR Context ) );

CSSM_DEFINE_FUNCTION( SetContext, (
						CSSM_CC_HANDLE CCHandle,
						const CSSM_CONTEXT *Context ) );

CSSM_DEFINE_FUNCTION( DeleteContext, (
						CSSM_CC_HANDLE CCHandle ) );

CSSM_DEFINE_FUNCTION( GetContextAttribute, (
						const CSSM_CONTEXT *Context,
						uint32 AttributeType,
						CSSM_CONTEXT_ATTRIBUTE_PTR *ContextAttribute ) );

CSSM_DEFINE_FUNCTION( UpdateContextAttributes, (
						CSSM_CC_HANDLE CCHandle,
						uint32 NumberAttributes,
						const CSSM_CONTEXT_ATTRIBUTE *ContextAttributes ) );

CSSM_DEFINE_FUNCTION( DeleteContextAttributes, (
						CSSM_CC_HANDLE CCHandle,
						uint32 NumberOfAttributes,
						const CSSM_CONTEXT_ATTRIBUTE *ContextAttributes ) );

# undef _CSSM_EXPLICIT_NAME
