/*-----------------------------------------------------------------------
 * File: CSSMDLF.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Data Storage Library Service Provider module.
 * The CSSM_DEFINE_FUNC macro must be defined by the file that includes this
 * header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif

/* Change the value of the handle type depending on the type of functions 
 * being declared.
 */
#ifdef _CSSM_MDS_DEFINITIONS
#define _DL_HANDLE_TYPE MDS_HANDLE MdsHandle
#define _DLDB_HANDLE_TYPE MDS_DB_HANDLE MdsDbHandle
#define _DB_HANDLE_PTR_TYPE CSSM_DB_HANDLE *hMds
#else
#define _DL_HANDLE_TYPE CSSM_DL_HANDLE DLHandle
#define _DLDB_HANDLE_TYPE CSSM_DL_DB_HANDLE DLDBHandle
#define _DB_HANDLE_PTR_TYPE CSSM_DB_HANDLE *DbHandle
#endif

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( DbOpen, (
						_DL_HANDLE_TYPE,
						const char *DbName,
						const CSSM_NET_ADDRESS *DbLocation,   
						CSSM_DB_ACCESS_TYPE AccessRequest,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const void *OpenParameters,
						_DB_HANDLE_PTR_TYPE ) );

CSSM_DEFINE_FUNCTION( DbClose, (
						_DLDB_HANDLE_TYPE ) );

#ifndef _CSSM_MDS_DEFINITIONS
CSSM_DEFINE_FUNCTION( DbCreate, (
						_DL_HANDLE_TYPE,
						const char *DbName,
						const CSSM_NET_ADDRESS *DbLocation,   
						const CSSM_DBINFO *DBInfo,
						CSSM_DB_ACCESS_TYPE AccessRequest,
						const CSSM_RESOURCE_CONTROL_CONTEXT *CredAndAclEntry,
						const void *OpenParameters,
						_DB_HANDLE_PTR_TYPE ) ); 

CSSM_DEFINE_FUNCTION( DbDelete, (
						_DL_HANDLE_TYPE,
						const char *DbName,
						const CSSM_NET_ADDRESS *DbLocation,
						const CSSM_ACCESS_CREDENTIALS *AccessCred ) );

CSSM_DEFINE_FUNCTION( Authenticate, (
						_DLDB_HANDLE_TYPE,
						CSSM_DB_ACCESS_TYPE AccessRequest,
						const CSSM_ACCESS_CREDENTIALS *AccessCred ) );

#endif /* ifndef _CSSM_MDS_DEFINITIONS */


#ifndef _CSSM_MDS_DEFINITIONS
/*
 * Extensibility Functions 
 */

/* Chg200 addition */

CSSM_DEFINE_FUNCTION( GetDbAcl, (
						_DLDB_HANDLE_TYPE,
						const CSSM_STRING *SelectionTag,
						uint32 *NumberOfAclInfos,
						CSSM_ACL_ENTRY_INFO_PTR *AclInfos ) );

CSSM_DEFINE_FUNCTION( ChangeDbAcl, (
						_DLDB_HANDLE_TYPE,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_ACL_EDIT *AclEdit ) );

CSSM_DEFINE_FUNCTION( GetDbOwner, (
						_DLDB_HANDLE_TYPE,
						CSSM_ACL_OWNER_PROTOTYPE_PTR Owner ) );


CSSM_DEFINE_FUNCTION( ChangeDbOwner, (
						_DLDB_HANDLE_TYPE,
						const CSSM_ACCESS_CREDENTIALS *AccessCred,
						const CSSM_ACL_OWNER_PROTOTYPE *NewOwner ) );

/* Chg200 addition end */
#endif /* ifndef _CSSM_MDS_DEFINITIONS */


CSSM_DEFINE_FUNCTION( GetDbNames, (
						_DL_HANDLE_TYPE,
						CSSM_NAME_LIST_PTR *NameList ) );

CSSM_DEFINE_FUNCTION( GetDbNameFromHandle, (
						_DLDB_HANDLE_TYPE,
						char ** DbName ) );

CSSM_DEFINE_FUNCTION( FreeNameList, (
						_DL_HANDLE_TYPE,
						CSSM_NAME_LIST_PTR NameList ) );  

/*
 * Data Record operations API.
 */

CSSM_DEFINE_FUNCTION( DataInsert, (
						_DLDB_HANDLE_TYPE,
						CSSM_DB_RECORDTYPE RecordType,
						const CSSM_DB_RECORD_ATTRIBUTE_DATA *Attributes, 
						const CSSM_DATA *Data,
						CSSM_DB_UNIQUE_RECORD_PTR *UniqueId ) );

CSSM_DEFINE_FUNCTION( DataDelete, (
						_DLDB_HANDLE_TYPE,
						const CSSM_DB_UNIQUE_RECORD *UniqueRecordIdentifier ) );

CSSM_DEFINE_FUNCTION( DataModify, (
						_DLDB_HANDLE_TYPE,
						CSSM_DB_RECORDTYPE RecordType,
						CSSM_DB_UNIQUE_RECORD_PTR UniqueRecordIdentifier,
						const CSSM_DB_RECORD_ATTRIBUTE_DATA *AttributesToBeModified,
						const CSSM_DATA *DataToBeModified,
						CSSM_DB_MODIFY_MODE ModifyMode ) );

CSSM_DEFINE_FUNCTION( DataGetFirst, (
						_DLDB_HANDLE_TYPE,
						const CSSM_QUERY *Query,
						CSSM_HANDLE_PTR ResultsHandle,
						CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes,
						CSSM_DATA_PTR Data,
						CSSM_DB_UNIQUE_RECORD_PTR *UniqueId ) );

CSSM_DEFINE_FUNCTION( DataGetNext, (
						_DLDB_HANDLE_TYPE,
						CSSM_HANDLE ResultsHandle,
						CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes, 
						CSSM_DATA_PTR Data,
						CSSM_DB_UNIQUE_RECORD_PTR *UniqueId ) );

CSSM_DEFINE_FUNCTION( DataAbortQuery, (
						_DLDB_HANDLE_TYPE,
						CSSM_HANDLE ResultsHandle ) );

CSSM_DEFINE_FUNCTION( DataGetFromUniqueRecordId, (
						_DLDB_HANDLE_TYPE,
						CSSM_DB_UNIQUE_RECORD_PTR UniqueRecord,
						CSSM_DB_RECORD_ATTRIBUTE_DATA_PTR Attributes,
						CSSM_DATA_PTR Data ) );

CSSM_DEFINE_FUNCTION( FreeUniqueRecord, (
						_DLDB_HANDLE_TYPE,
						CSSM_DB_UNIQUE_RECORD_PTR UniqueRecord ) );

CSSM_DEFINE_FUNCTION( CreateRelation, (
						CSSM_DL_DB_HANDLE DLDBHandle,
						CSSM_DB_RECORDTYPE RelationID,
						const char *RelationName,
						uint32 NumberOfAttributes,
						const CSSM_DB_SCHEMA_ATTRIBUTE_INFO *pAttributeInfo,
						uint32 NumberOfIndexes,
						const CSSM_DB_SCHEMA_INDEX_INFO *pIndexInfo ) );

CSSM_DEFINE_FUNCTION( DestroyRelation, (
						CSSM_DL_DB_HANDLE DLDBHandle,
						CSSM_DB_RECORDTYPE RelationID ) );

#ifndef _CSSM_MDS_DEFINITIONS
/*
 * Extensibility Function 
 */
CSSM_DEFINE_FUNCTION( PassThrough, (
						_DLDB_HANDLE_TYPE, 
						uint32 PassThroughId,
						const void *InputParams,
						void **OutputParams ) ); 
#endif /* ifndef _CSSM_MDS_DEFINITIONS */

#undef _DL_HANDLE_TYPE
#undef _DLDB_HANDLE_TYPE
#undef _DB_HANDLE_PTR_TYPE
