/*-----------------------------------------------------------------------
 * File: CSSMHRS.H
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _CSSMHRS_H
#define _CSSMHRS_H    

/* API Functions */
#ifdef __cplusplus
extern "C" {
#endif


/*************************************************************************/
/*** Human Recognition Services ******************************************/
/*************************************************************************/

CSSM_RETURN CSSMAPI CSSM_HRS_FreeBIRHandle(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle );

CSSM_RETURN CSSMAPI CSSM_HRS_GetBIRFromHandle(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle,
                        CSSM_HRS_BIR_PTR *BIR );

CSSM_RETURN CSSMAPI CSSM_HRS_GetHeaderFromHandle(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle,
                        CSSM_HRS_BIR_HEADER_PTR Header );

CSSM_RETURN CSSMAPI CSSM_HRS_EnableEvents(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_MODULE_EVENT_MASK *Events );

CSSM_RETURN CSSMAPI CSSM_HRS_SetGUICallbacks(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_GUI_STREAMING_CALLBACK GuiStreamingCallback,
                        void *GuiStreamingCallbackCtx,
                        CSSM_HRS_GUI_STATE_CALLBACK GuiStateCallback,
                        void *GuiStateCallbackCtx );

CSSM_RETURN CSSMAPI CSSM_HRS_CancelGUICallbacks(
                        CSSM_HANDLE ModuleHandle );

CSSM_RETURN CSSMAPI CSSM_HRS_SetStreamCallback(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_STREAM_CALLBACK StreamCallback,
                        void *StreamCallbackCtx );

CSSM_RETURN CSSMAPI CSSM_HRS_CancelStreamCallbacks(
                        CSSM_HANDLE ModuleHandle );

CSSM_RETURN CSSMAPI CSSM_HRS_StreamInputOutput(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_DATA_PTR InMessage,
                        CSSM_DATA_PTR OutMessage );

CSSM_RETURN CSSMAPI CSSM_HRS_Capture(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        CSSM_HRS_BIR_HANDLE_PTR CapturedBIR,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

CSSM_RETURN CSSMAPI CSSM_HRS_CreateTemplate(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *CapturedBIR,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                        const CSSM_DATA *Payload );

CSSM_RETURN CSSMAPI CSSM_HRS_Process(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *CapturedBIR,
                        CSSM_HRS_BIR_HANDLE_PTR ProcessedBIR );

CSSM_RETURN CSSMAPI CSSM_HRS_VerifyMatch(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE *AdaptedBIR,
                        CSSM_BOOL *Result,
                        CSSM_HRS_FAR_PTR FARAchieved,
                        CSSM_HRS_FRR_PTR FRRAchieved,
                        CSSM_DATA_PTR *Payload );

CSSM_RETURN CSSMAPI CSSM_HRS_IdentifyMatch(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_IDENTIFY_POPULATION *Population,
                        CSSM_BOOL Binning,
                        uint32 MaxNumberOfResults,
                        uint32 *NumberOfResults,
                        CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                        sint32 Timeout );

CSSM_RETURN CSSMAPI CSSM_HRS_Enroll(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                        const CSSM_DATA *Payload,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

CSSM_RETURN CSSMAPI CSSM_HRS_Verify(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR AdaptedBIR,
                        CSSM_BOOL *Result,
                        CSSM_HRS_FAR_PTR FARAchieved,
                        CSSM_HRS_FRR_PTR FRRAchieved,
                        CSSM_DATA_PTR *Payload,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

CSSM_RETURN CSSMAPI CSSM_HRS_Identify(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_IDENTIFY_POPULATION *Population,
                        CSSM_BOOL Binning,
                        uint32 MaxNumberOfResults,
                        uint32 *NumberOfResults,
                        CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

CSSM_RETURN CSSMAPI CSSM_HRS_Import(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_DATA *InputData,
                        CSSM_HRS_BIR_BIOMETRIC_DATA_FORMAT InputFormat,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        CSSM_HRS_BIR_HANDLE_PTR ConstructedBIR );

CSSM_RETURN CSSMAPI CSSM_HRS_SetPowerMode(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_POWER_MODE PowerMode );

CSSM_RETURN CSSMAPI CSSM_HRS_DbOpen(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName,
                        CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                        CSSM_HRS_DB_HANDLE_PTR DbHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

CSSM_RETURN CSSMAPI CSSM_HRS_DbClose(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle );

CSSM_RETURN CSSMAPI CSSM_HRS_DbCreate(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName,
                        CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                        CSSM_HRS_DB_HANDLE_PTR DbHandle );

CSSM_RETURN CSSMAPI CSSM_HRS_DbDelete(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName );

CSSM_RETURN CSSMAPI CSSM_HRS_DbSetCursor(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

CSSM_RETURN CSSMAPI CSSM_HRS_DbFreeCursor(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

CSSM_RETURN CSSMAPI CSSM_HRS_DbStoreBIR(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *BIRToStore,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        CSSM_GUID_PTR Uuid );

CSSM_RETURN CSSMAPI CSSM_HRS_DbGetBIR(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue,
                        CSSM_HRS_BIR_HANDLE_PTR RetrievedBIR,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

CSSM_RETURN CSSMAPI CSSM_HRS_DbGetNextBIR(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor,
                        CSSM_HRS_BIR_HANDLE_PTR RetievedBIR,
                        CSSM_GUID_PTR Uuid );

CSSM_RETURN CSSMAPI CSSM_HRS_DbQueryBIR(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_HRS_INPUT_BIR *BIRToQuery,
                        CSSM_GUID_PTR Uuid );

CSSM_RETURN CSSMAPI CSSM_HRS_DbDeleteBIR(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue );


/*************************************************************************/


/*************************************************************************/
/*** Pointer Types For Above Functions ***********************************/
/*************************************************************************/

/* Define a function pointer type for each API function. The pointer types 
 * are named _CSSM_HRS_XXX_PTR, where XXX is the value of the API.
 */
typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_FreeBIRHandle_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_GetBIRFromHandle_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle,
                        CSSM_HRS_BIR_PTR *BIR );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_GetHeaderFromHandle_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_HANDLE BIRHandle,
                        CSSM_HRS_BIR_HEADER_PTR Header );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_EnableEvents_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_MODULE_EVENT_MASK *Events );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_SetGUICallbacks_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_GUI_STREAMING_CALLBACK GuiStreamingCallback,
                        void *GuiStreamingCallbackCtx,
                        CSSM_HRS_GUI_STATE_CALLBACK GuiStateCallback,
                        void *GuiStateCallbackCtx );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_CancelGUICallbacks_PTR)(
                        CSSM_HANDLE ModuleHandle );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_SetStreamCallback_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_STREAM_CALLBACK StreamCallback,
                        void *StreamCallbackCtx );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_CancelStreamCallbacks_PTR)(
                        CSSM_HANDLE ModuleHandle );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_StreamInputOutput_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_DATA_PTR InMessage,
                        CSSM_DATA_PTR OutMessage );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Capture_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        CSSM_HRS_BIR_HANDLE_PTR CapturedBIR,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_CreateTemplate_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *CapturedBIR,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                        const CSSM_DATA *Payload );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Process_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *CapturedBIR,
                        CSSM_HRS_BIR_HANDLE_PTR ProcessedBIR );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_VerifyMatch_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE *AdaptedBIR,
                        CSSM_BOOL *Result,
                        CSSM_HRS_FAR_PTR FARAchieved,
                        CSSM_HRS_FRR_PTR FRRAchieved,
                        CSSM_DATA_PTR *Payload );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_IdentifyMatch_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_IDENTIFY_POPULATION *Population,
                        CSSM_BOOL Binning,
                        uint32 MaxNumberOfResults,
                        uint32 *NumberOfResults,
                        CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                        sint32 Timeout );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Enroll_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
                        const CSSM_DATA *Payload,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Verify_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *StoredTemplate,
                        CSSM_HRS_BIR_HANDLE_PTR AdaptedBIR,
                        CSSM_BOOL *Result,
                        CSSM_HRS_FAR_PTR FARAchieved,
                        CSSM_HRS_FRR_PTR FRRAchieved,
                        CSSM_DATA_PTR *Payload,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Identify_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_FAR *MaxFARRequested,
                        const CSSM_HRS_FRR *MaxFRRRequested,
                        const CSSM_BOOL *FARPrecedence,
                        const CSSM_HRS_INPUT_BIR *ProcessedBIR,
                        const CSSM_HRS_IDENTIFY_POPULATION *Population,
                        CSSM_BOOL Binning,
                        uint32 MaxNumberOfResults,
                        uint32 *NumberOfResults,
                        CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
                        sint32 Timeout,
                        CSSM_HRS_BIR_HANDLE_PTR AuditData );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_Import_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_DATA *InputData,
                        CSSM_HRS_BIR_BIOMETRIC_DATA_FORMAT InputFormat,
                        CSSM_HRS_BIR_PURPOSE Purpose,
                        CSSM_HRS_BIR_HANDLE_PTR ConstructedBIR );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_SetPowerMode_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_POWER_MODE PowerMode );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbOpen_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName,
                        CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                        CSSM_HRS_DB_HANDLE_PTR DbHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbClose_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbCreate_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName,
                        CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
                        CSSM_HRS_DB_HANDLE_PTR DbHandle );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbDelete_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const uint8 *DbName );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbSetCursor_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbFreeCursor_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbStoreBIR_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        const CSSM_HRS_INPUT_BIR *BIRToStore,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        CSSM_GUID_PTR Uuid );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbGetBIR_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue,
                        CSSM_HRS_BIR_HANDLE_PTR RetrievedBIR,
                        CSSM_HRS_DB_CURSOR_PTR Cursor );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbGetNextBIR_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_CURSOR_PTR Cursor,
                        CSSM_HRS_BIR_HANDLE_PTR RetievedBIR,
                        CSSM_GUID_PTR Uuid );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbQueryBIR_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_HRS_INPUT_BIR *BIRToQuery,
                        CSSM_GUID_PTR Uuid );

typedef CSSM_RETURN (CSSMAPI *_CSSM_HRS_DbDeleteBIR_PTR)(
                        CSSM_HANDLE ModuleHandle,
                        CSSM_HRS_DB_HANDLE DbHandle,
                        const CSSM_GUID *KeyValue );


/*************************************************************************/


#ifdef __cplusplus
}
#endif

#endif
