/*-----------------------------------------------------------------------
 * File: CSSMHRSF.H
 *
 * Copyright (c) 1999-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Function declarations for the Human Recognition Service Provider module.
 * The CSSM_DEFINE_FUNC, __CSSM_PASTE2 and __CSSM_PASTE3 macros must be 
 * defined by the file that includes this header.
 */

/* Check to make sure that all macros were defined */
#ifndef CSSM_DEFINE_FUNCTION
#error "The CSSM_DEFINE_FUNCTION(name,params) macro must be defined!"
#endif

/*************************************************************************/
/*** Function declarations ***********************************************/
/*************************************************************************/

CSSM_DEFINE_FUNCTION( FreeBIRHandle, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_BIR_HANDLE BIRHandle ) );

CSSM_DEFINE_FUNCTION( GetBIRFromHandle, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_BIR_HANDLE BIRHandle,
						CSSM_HRS_BIR_PTR *BIR ) );

CSSM_DEFINE_FUNCTION( GetHeaderFromHandle, (
						CSSM_HANDLE ModuleHandle, 
						CSSM_HRS_BIR_HANDLE BIRHandle,
						CSSM_HRS_BIR_HEADER_PTR Header ) );

CSSM_DEFINE_FUNCTION( EnableEvents, (
						CSSM_HANDLE ModuleHandle, 
						CSSM_HRS_MODULE_EVENT_MASK *Events ) );

CSSM_DEFINE_FUNCTION( SetGUICallbacks, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_GUI_STREAMING_CALLBACK GuiStreamingCallback,
						void *GuiStreamingCallbackCtx,
						CSSM_HRS_GUI_STATE_CALLBACK  GuiStateCallback,
						void *GuiStateCallbackCtx ) );

CSSM_DEFINE_FUNCTION( CancelGUICallbacks, (
                        CSSM_HANDLE  ModuleHandle ) );

CSSM_DEFINE_FUNCTION( SetStreamCallback, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_STREAM_CALLBACK StreamCallback,
						void *StreamCallbackCtx ) );

CSSM_DEFINE_FUNCTION( CancelStreamCallbacks, (
                        CSSM_HANDLE  ModuleHandle ) );

CSSM_DEFINE_FUNCTION( StreamInputOutput, (
						CSSM_HANDLE ModuleHandle,
						CSSM_DATA_PTR InMessage,
						CSSM_DATA_PTR OutMessage ) );

CSSM_DEFINE_FUNCTION( Capture, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_BIR_PURPOSE Purpose,
						CSSM_HRS_BIR_HANDLE_PTR CapturedBIR,
						sint32 Timeout,
						CSSM_HRS_BIR_HANDLE_PTR AuditData ) );

CSSM_DEFINE_FUNCTION( CreateTemplate, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_INPUT_BIR *CapturedBIR,
						const CSSM_HRS_INPUT_BIR *StoredTemplate,
						CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
						const CSSM_DATA *Payload ) );

CSSM_DEFINE_FUNCTION( Process, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_INPUT_BIR *CapturedBIR,
						CSSM_HRS_BIR_HANDLE_PTR ProcessedBIR ) );

CSSM_DEFINE_FUNCTION( VerifyMatch, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_FAR *MaxFARRequested,
						const CSSM_HRS_FRR *MaxFRRRequested,
						const CSSM_BOOL *FARPrecedence,
						const CSSM_HRS_INPUT_BIR *ProcessedBIR,
						const CSSM_HRS_INPUT_BIR *StoredTemplate,
						CSSM_HRS_BIR_HANDLE *AdaptedBIR,
						CSSM_BOOL *Result,
						CSSM_HRS_FAR_PTR FARAchieved,
						CSSM_HRS_FRR_PTR FRRAchieved,
						CSSM_DATA_PTR  *Payload ) );

CSSM_DEFINE_FUNCTION( IdentifyMatch, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_FAR *MaxFARRequested,
						const CSSM_HRS_FRR *MaxFRRRequested,
						const CSSM_BOOL *FARPrecedence,
						const CSSM_HRS_INPUT_BIR *ProcessedBIR,
						const CSSM_HRS_IDENTIFY_POPULATION *Population,
						CSSM_BOOL  Binning,
						uint32 MaxNumberOfResults,
						uint32 *NumberOfResults,
						CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
						sint32 Timeout ) );

CSSM_DEFINE_FUNCTION( Enroll, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_BIR_PURPOSE Purpose,
						const CSSM_HRS_INPUT_BIR *StoredTemplate,
						CSSM_HRS_BIR_HANDLE_PTR NewTemplate,
						const CSSM_DATA *Payload,
						sint32 Timeout,
						CSSM_HRS_BIR_HANDLE_PTR AuditData ) );

CSSM_DEFINE_FUNCTION( Verify, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_FAR *MaxFARRequested,
						const CSSM_HRS_FRR *MaxFRRRequested,
						const CSSM_BOOL *FARPrecedence,
						const CSSM_HRS_INPUT_BIR *StoredTemplate,
						CSSM_HRS_BIR_HANDLE_PTR AdaptedBIR,
						CSSM_BOOL *Result,
						CSSM_HRS_FAR_PTR FARAchieved,
						CSSM_HRS_FRR_PTR FRRAchieved,
						CSSM_DATA_PTR *Payload,
						sint32 Timeout,
						CSSM_HRS_BIR_HANDLE_PTR AuditData ) );

CSSM_DEFINE_FUNCTION( Identify, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_FAR *MaxFARRequested,
						const CSSM_HRS_FRR *MaxFRRRequested,
						const CSSM_BOOL *FARPrecedence,
						const CSSM_HRS_INPUT_BIR *ProcessedBIR,
						const CSSM_HRS_IDENTIFY_POPULATION *Population,
						CSSM_BOOL Binning,
						uint32 MaxNumberOfResults,
						uint32 *NumberOfResults,
						CSSM_HRS_CANDIDATE_ARRAY_PTR *Candidates,
						sint32 Timeout,
						CSSM_HRS_BIR_HANDLE_PTR AuditData ) );

CSSM_DEFINE_FUNCTION( Import, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_DATA *InputData,
						CSSM_HRS_BIR_BIOMETRIC_DATA_FORMAT InputFormat,
						CSSM_HRS_BIR_PURPOSE Purpose,
						CSSM_HRS_BIR_HANDLE_PTR ConstructedBIR ) );

CSSM_DEFINE_FUNCTION( SetPowerMode, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_POWER_MODE PowerMode ) );

CSSM_DEFINE_FUNCTION( DbOpen, (
						CSSM_HANDLE ModuleHandle,
						const uint8 *DbName,
						CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
						CSSM_HRS_DB_HANDLE_PTR DbHandle,
						CSSM_HRS_DB_CURSOR_PTR Cursor ) );

CSSM_DEFINE_FUNCTION( DbClose, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_HANDLE DbHandle ) );

CSSM_DEFINE_FUNCTION( DbCreate, (
						CSSM_HANDLE ModuleHandle,
						const uint8 *DbName,
						CSSM_HRS_DB_ACCESS_TYPE AccessRequest,
						CSSM_HRS_DB_HANDLE_PTR DbHandle ) );

CSSM_DEFINE_FUNCTION( DbDelete, (
						CSSM_HANDLE ModuleHandle,
						const uint8 *DbName ) );

CSSM_DEFINE_FUNCTION( DbSetCursor, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_HANDLE DbHandle,
						const CSSM_GUID *KeyValue,
						CSSM_HRS_DB_CURSOR_PTR Cursor ) );

CSSM_DEFINE_FUNCTION( DbFreeCursor, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_CURSOR_PTR Cursor ) );

CSSM_DEFINE_FUNCTION( DbStoreBIR, (
						CSSM_HANDLE ModuleHandle,
						const CSSM_HRS_INPUT_BIR *BIRToStore,
						CSSM_HRS_DB_HANDLE DbHandle,
						CSSM_GUID_PTR Uuid ) );

CSSM_DEFINE_FUNCTION( DbGetBIR, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_HANDLE DbHandle,
						const CSSM_GUID *KeyValue,
						CSSM_HRS_BIR_HANDLE_PTR RetrievedBIR,
						CSSM_HRS_DB_CURSOR_PTR Cursor ) );

CSSM_DEFINE_FUNCTION( DbGetNextBIR, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_CURSOR_PTR Cursor,
						CSSM_HRS_BIR_HANDLE_PTR RetievedBIR,
						CSSM_GUID_PTR Uuid ) );

CSSM_DEFINE_FUNCTION( DbQueryBIR, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_HANDLE DbHandle,
						const CSSM_HRS_INPUT_BIR *BIRToQuery,
						CSSM_GUID_PTR Uuid ) );

CSSM_DEFINE_FUNCTION( DbDeleteBIR, (
						CSSM_HANDLE ModuleHandle,
						CSSM_HRS_DB_HANDLE DbHandle,
						const CSSM_GUID *KeyValue ) );
