/*-----------------------------------------------------------------------
 * File: CSSMKRSPI.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _CSSMKRSPI_H
#define _CSSMKRSPI_H    

#include "cssmtype.h"
#include "cssmspi.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef CSSM_RETURN (*CSSM_PRIV_FUNC_PTR)(CSSM_CC_HANDLE hContext, CSSM_BOOL Priv);

typedef struct cssm_spi_krsp_funcs {
	CSSM_RETURN (CSSMAPI *GetPolicyInfo) (
		    CSSM_CC_HANDLE CCHandle,
			uint32 *EncryptionProhibited,
			uint32 *WorkFactor);
    CSSM_RETURN (CSSMAPI *CreateRecoveryEnablementContext(
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_KR_PROFILE_PTR LocalProfile,
			CSSM_KR_PROFILE_PTR RemoteProfile,
			CSSM_CC_CONTEXT *NewContext );
	CSSM_RETURN (CSSMAPI *CreateRecoveryRegistrationContext(
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_CC_HANDLE *NewHandle );
	CSSM_RETURN (CSSMAPI *CreateRecoveryRequestContext(
			CSSM_KRSP_HANDLE KRSPHandle,
			const CSSM_KR_PROFILE *LocalProfile,
			CSSM_CC_HANDLE *NewContext );
	CSSM_RETURN (CSSMAPI *SetEnterpriseRecoveryPolicy(
			char *RecoveryPolicyFileName,
			const CSSM_CRYPTO_DATA *OldPassPhrase, 
			const CSSM_CRYPTO_DATA *NewPassPhrase );
	CSSM_RETURN (CSSMAPI *RegistrationRequest) (
			CSSM_CC_HANDLE RecoveryRegistrationContext,
			CSSM_DATA_PTR KRInData,
			CSSM_CRYPTO_DATA_PTR UserCallback,
			uint8 KRFlags,
			uint32 *EstimatedTime,
			CSSM_HANDLE_PTR ReferenceHandle);
	CSSM_RETURN (CSSMAPI *RegistrationRetrieve) (
			CSSM_KRSP_HANDLE hKRSP,
			CSSM_HANDLE ReferenceHandle,
			uint32 *EstimatedTime,
			CSSM_KR_PROFILE_PTR KRProfile);
	CSSM_RETURN (CSSMAPI *RecoveryRequest) (
			CSSM_CC_HANDLE RecoveryRequestContext,
			const CSSM_DATA *KRInData,
			const CSSM_CRYPTO_DATA *UserCallback,
			uint32 *EstimatedTime,
			const CSSM_HANDLE *ReferenceHandle );
	CSSM_RETURN (CSSMAPI *RecoveryRetrieve) (
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_HANDLE_PTR ReferenceHandle,
			uint32 *EstimatedTime,
			CSSM_HANDLE_PTR CacheHandle,
			uint32 *NumberOfRecoveredKeys );
	CSSM_RETURN (CSSMAPI *GetRecoveredObject) (
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_HANDLE_PTR CacheHandle,
			uint32 IndexInResults,
			CSSM_CSP_HANDLE CSPHandle,
			const CSSM_CRYPTO_DATA *PassPhrase,
			CSSM_KEY_PTR RecoveredKey,
			uint32 Flags,
			CSSM_DATA_PTR OtherInfo );
	CSSM_RETURN (CSSMAPI *RecoveryRequestAbort) (
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_HANDLE CacheHandle );
    CSSM_RETURN (CSSMAPI *QueryPolicyInfo(
			const uint32 AlgorithmID, 
			const uint32 Mode, 
			const uint32 Class,
			CSSM_POLICY_INFO_PTR *PolicyInfoData ); 
    CSSM_RETURN (CSSMAPI *GenerateRecoveryFields (
			CSSM_CC_HANDLE KRSPHandle,
			CSSM_CC_HANDLE CryptoContext,
			CSSM_DATA_PTR KRSPOptions,
			uint32 KRFlags,
			CSSM_DATA_PTR KRFields );
    CSSM_RETURN (CSSMAPI *ProcessRecoveryFields (
			CSSM_CC_HANDLE KeyRecoveryContext,
			CSSM_CC_HANDLE CryptoContext,
			CSSM_DATA_PTR KRSPOptions,
			uint32 KRFlags,
			CSSM_DATA_PTR KRFields );
	CSSM_RETURN (CSSMAPI *FreePolicyInfo(
		    CSSM_KR_POLICY_INFO_PTR PolicyInfoData);
    CSSM_RETURN (CSSMAPI *PassThrough(
			CSSM_KRSP_HANDLE KRSPHandle,
			CSSM_CC_HANDLE KeyRecoveryContext, 
			CSSM_CC_HANDLE CryptoContext,
			uint32 PassThroughId,
			const CSSM_DATA *InputParams,
			CSSM_DATA_PTR Output);
	//CSSM_RETURN (CSSMAPI *PassPrivFunc)(CSSM_PRIV_FUNC_PTR SetContextPriv);
} CSSM_SPI_KRSP_FUNCS, *CSSM_SPI_KRSP_FUNCS_PTR;

#ifdef __cplusplus
}
#endif

#endif
