/*-----------------------------------------------------------------------
 * File: SUBTRACT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: subtract.c
   Computes t = t - a
   assuming t>=a
*/

#include "eicl.h"
#include "dss.h"

/* return t = t-a */
void __stdcall EICL_Subtract (DSAData *t, DSAData *a)
{
  int			pos;
  register EICLWord	*aval, *tval, msb, sum;
  EICLWord		aneg, carry;
  long			length;

  aval=a->value;
  tval=t->value;
  length = a->length;

  carry=1;
  for(pos=0; pos<length; ++pos) {
    sum=(tval[pos] & 0x7FFFFFFF) + ((aneg=~aval[pos]) & 0x7FFFFFFF) + carry;
    msb=(tval[pos] >> 31) + (aneg >> 31) + (sum >> 31);
    carry = (msb & 2)!=0;
    if (msb & 1)
      tval[pos]=sum | 0x80000000;
    else
      tval[pos]=sum & 0x7FFFFFFF;
  }
/* ... just add the carry-out (0/1) for the residue, if any */
  length = t->length;
  for (; pos<length; ++pos) {
    sum=(tval[pos] & 0x7FFFFFFF) + carry + 0x7FFFFFFF;
    msb=(tval[pos] >> 31) + (sum >> 31) + 1;
    carry = (msb & 2)!=0;
    if (msb & 1)
      tval[pos]=sum | 0x80000000;
    else
      tval[pos]=sum & 0x7FFFFFFF;
  }
/* compute the length of t */
  --pos;
  while (pos>0 && tval[pos]==0)
    --pos;
  t->length = pos + 1;
}
