/*-----------------------------------------------------------------------
 * File: DIV.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: div.c
   Multi-precision integer division.
   Only the quotient is returned.
*/

#include "eicl.h"
#include "dss.h"
#include "dsskg.h"

#define ShiftLeftOneBit(w) {register int cmp=w.length;\
			    w.value[cmp]=0;\
			    while (cmp-->0) {\
			      w.value[cmp+1]|=w.value[cmp] >> 31;\
			      w.value[cmp]   =w.value[cmp] << 1;\
			    }\
			    cmp=w.length;\
			    w.length=(w.value[cmp]!=0) ? cmp+1 : cmp;\
			   }

/* A simple restoring division */
void __stdcall	EICL_Div (DSAData *dividend, DSAData *divisor, DSAData *divide)
{
  register EICLWord	bitmask;
  register int		cnt, blength;
  DSAData		rem;
  EICLWord		*tval, *aval, remarray[MODULUS];

  blength=divisor->length;
/* Save time: if dividend<divisor then the result is obviously zero! */
  tval=divide->value;
  divide->length=1;
  for (cnt=0; cnt<MODULUS; ++cnt)
    tval[cnt] = 0;
  if (dividend->length<blength) return;

/* start with zero remainder. Note that, Subtract routine *MUST* be able to
deal with denormalized numbers. "rem" is a denormalized 'zero'! */
  rem.value=remarray;
  rem.length=blength;
  aval = dividend->value;
  for (cnt=0; cnt<MODULUS; ++cnt)
    remarray[cnt]=0;
/* start looping */
  for (cnt=dividend->length-1; cnt>=0; --cnt) {
    tval[cnt]=0;
    for (bitmask=0x80000000; bitmask!=0; bitmask>>=1) {
      remarray[0]|= (aval[cnt] & bitmask)!=0;     /* get the next a bit  */
      tval[cnt]<<=1;                   /* adjust the quotient             */
      if (EICL_Compare (&rem, divisor)>=0) {   /* subtract and shift left         */
	    EICL_Subtract (&rem, divisor);
	    tval[cnt] |= 1;                /* it divides! put 1 into quotient */
      }
      ShiftLeftOneBit (rem);           /* and shift partial remainder     */
    }
  }
/* This function discards the remainder. */
  cnt=dividend->length-1;
/* discard the leading 0's and find the normalized length of the result */
  while (cnt>0 && tval[cnt]==0)
    --cnt;
  divide->length=cnt+1;
}
