/*-----------------------------------------------------------------------
 * File: EEA.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* Module name: eea.c
   Extended Euclidean Algorithm using Berlekamp's version.
   On return:
     b*upp - a*vpp = (-1)(k-1)*rpp
   where k is the iteration number when the partial remainder becomes 0.
   if k is odd
	      a*a^-1 = 1 mod b    ----->   a^-1 = b - vpp
	      b*b^-1 = 1 mod a    ----->   b^-1 = t
   if k is even
	      a*a^-1 = 1 mod b    ----->   a^-1 = s
	      b*b^-1 = 1 mod a    ----->   b^-1 = a - upp
*/

#include "eicl.h"
#include "dss.h"
#include "dsskg.h"
#include "eiclproc.h"


void __stdcall	EICL_EEA (DSAData *a, DSAData *b, DSAData *vpp, DSAData *upp, DSAData *rpp, int *k)
{
  DSAData	q, r, rp, u, up, v, vp;
  EICLWord	qarray[2*MODULUS],
  		rarray[2*MODULUS], rparray[2*MODULUS],
		uarray[2*MODULUS], uparray[2*MODULUS],
		varray[2*MODULUS], vparray[2*MODULUS];

/* fill the arrays with 0's */
  for (*k=0; *k<2*MODULUS; ++*k)
    qarray[*k]=rarray[*k]=rparray[*k]=
    	uarray[*k]=uparray[*k]=varray[*k]=vparray[*k] = 0;
/* value initialization */
  q.value  = qarray;
  r.value  = rarray;
  rp.value = rparray;
  u.value  = uarray;
  up.value = uparray;
  v.value  = varray;
  vp.value = vparray;

/* Initialize the variables */
  *k=0;
  upp->value[0]=0;    upp->length=1;
  up.value[0]=1;      up.length=1;
  vpp->value[0]=1;    vpp->length=1;
  vp.value[0]=0;      vp.length=1;

  EICL_MoveWord (a,  rpp);			/* rpp = a       */
  EICL_MoveWord (b, &rp);

  /* start looping on a zero rp (remainder) */
  while (!EICL_IsZero (rp)) {
    EICL_Div (rpp, &rp, &q);

    EICL_Mul (&q, &up, &u);
    EICL_Add (&u, upp, &u);
    while (u.length>1 && u.value[u.length-1]==0)
        --u.length;

    EICL_Mul (&q, &vp, &v);
    EICL_Add (&v, vpp, &v);
    while (v.length>1 && v.value[v.length-1]==0)
        --v.length;

    EICL_Rem (rpp, &rp, &r);
/* discard the leading 0's since ICL_Rem does not do this */
    while (r.length>1 && rarray[r.length-1]==0)
    	--r.length;

    EICL_MoveWord (&up, upp);
    EICL_MoveWord (&u, &up);

    EICL_MoveWord (&vp, vpp);
    EICL_MoveWord (&v, &vp);

    EICL_MoveWord (&rp, rpp);
    EICL_MoveWord (&r, &rp);
    
    ++*k;
  }
/* Now,   b*upp - a*vpp = (-1)^(k-1)*rpp   */
}
