/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: cfgcssm.c,v $
 * Revision 1.1.2.4  2000/08/31  18:05:51  Thomas_Woodburn
 *     Merged in Intel CDSA Version 2.0 Release 3.11.
 *
 * Revision 1.1.2.3  2000/08/07  14:41:38  Thomas_Woodburn
 *     Merged in Intel CDSA Version 2.0 Release 3.1.
 *
 * Revision 1.1.2.2  2000/06/28  21:00:04  Thomas_Woodburn
 *     Initial port to Tru64 UNIX.
 *
 * Revision 1.1.2.1  2000/06/28  20:24:39  Thomas_Woodburn
 *     Created from Intel CDSA Version 2.0 Release 3.0.
 *
 * $EndLog$
 */
#ifndef WIN32
#ifndef VMS
#pragma ident "@(#)$RCSfile: cfgcssm.c,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:05:51 $"
#endif
#endif
/*-----------------------------------------------------------------------
 * File: CFGCSSM.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

//config.cpp: implementation file
//
#include "isl_internal.h"
#include "isl_def.h"
#include "isl_cssm.h"
#include "aal.h"
#include "cssmapi_typecast.h"
#include "mds_util_api.h"
#include <isl_memtrack.h>

extern ISL_ARCHIVE_CONFIG_METHODS    ArchiveConfigMethods;
extern ISL_ARCHIVE_METHODS            ArchiveMethods;
extern ISL_MANIFEST_SECTION_METHODS    ManifestSectionMethods;
extern ISL_SIG_SECTION_METHODS        SignedListMethods;
extern ISL_DIGEST_METHODS            JarMD5Methods;
extern ISL_DIGEST_METHODS            JarMD2Methods;
extern ISL_DIGEST_METHODS            JarSHA1Methods;
extern ISL_GET_DATA_METHODS            FileGetDataMethods;
extern ISL_GET_DATA_METHODS            ExecutableFileGetDataMethods;
extern ISL_GET_DATA_METHODS            URLGetMethods;
extern ISL_SIGNATURE_CONFIG_METHODS    PKCS7SignatureConfigMethods;
extern ISL_DIGEST_METHODS            PKCS7MD5Methods;
extern ISL_DIGEST_METHODS            PKCS7SHA1Methods;
extern ISL_DIGEST_METHODS            PKCS7MD2Methods;
extern ISL_SIGNATURE_METHODS            PKCS7SignatureMethods;
extern ISL_SIGN_VERIFY_METHODS        PKCS1RSASHA1Methods;
extern ISL_SIGN_VERIFY_METHODS        PKCS1RSAMD5Methods;
extern ISL_SIGN_VERIFY_METHODS        PKCS1RSAMD2Methods;
extern ISL_SIGN_VERIFY_METHODS        DSAMethods;
extern ISL_SIGN_VERIFY_METHODS        PKCS7DSAMethods;
extern ISL_SIGN_VERIFY_METHODS        PKCS1RSAMethods;
extern ISL_CERTIFICATE_METHODS        X509CertMethods;
extern ISL_SIGNER_METHODS            PKCS7SignerMethods;


extern void * isl_ConfigAllocateMemory(
    ISL_ARCHIVE_CONFIG    *Context,
    uint32                Size);

extern void isl_ConfigFreeMemory(
    ISL_ARCHIVE_CONFIG    *Context,
    void *memory);

extern void isl_ConfigureISL_CleanUp(
    ISL_ARCHIVE_CONFIG *Context);

//memory functions used by CSSM
#ifdef _DEBUG
#include <crtdbg.h>
#endif

void* CSSMAPI
_app_malloc(uint32 size,
           void *palloc_ref
#ifdef CSSM_MEMTRACK_ON
           , const char *szFilename,
           uint32 u32LineNumber
#endif
            )
{
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
        return _malloc_dbg( size, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
        return malloc( size );
#endif
}

void* CSSMAPI
_app_calloc(uint32 num_elements,
           uint32 size_element,
           void *palloc_ref
#ifdef CSSM_MEMTRACK_ON
           , const char *szFilename,
           uint32 u32LineNumber
#endif
           )
{
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    return _calloc_dbg( num_elements, size_element, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
    return calloc(num_elements, size_element );
#endif
}


void* CSSMAPI
_app_realloc(void * ptr,
            uint32 new_size,
            void *palloc_ref
#ifdef CSSM_MEMTRACK_ON
            , const char *szFilename,
            uint32 u32LineNumber
#endif
            )
{
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    return _realloc_dbg( ptr, new_size, _NORMAL_BLOCK, szFilename, u32LineNumber );
#else
    return realloc(ptr, new_size);
#endif
}

void CSSMAPI
_app_free(void * ptr,
         void *palloc_ref
#ifdef CSSM_MEMTRACK_ON
         , const char *szFilename,
         uint32 u32LineNumber
#endif
         )
{
#if defined(_DEBUG) && defined(CSSM_MEMTRACK_ON) && defined(WIN32)
    _free_dbg( ptr, _NORMAL_BLOCK );
#else
    free(ptr);
#endif
    //return;
}


#ifdef COMPAQ_FIX
static /* Avoid conflicts with the same symbol in mrm's config.cpp. */
#endif /* COMPAQ_FIX */
CSSM_API_MEMORY_FUNCS g_memoryFX = {_app_malloc, _app_free, _app_realloc, _app_calloc, NULL};

#include <isl_memtrack.h>

CSP_LIST_ITEM_PTR isl_FindAvailableCSPs(
    ISL_ARCHIVE_CONFIG *pArchiveConfig,
    CSSM_CONTEXT_TYPE    contextType,
    CSSM_ALGORITHMS    algorithmType,
    uint32    *numberOfModuleInfo,
    ISL_CSSM_METHODS *pCSSM)
{
    // loop through all the CSP's and gather info on each one
    CSSM_RETURN retValue;
    MDSUTIL_LIST_PTR pGUIDList = NULL;
    CSP_LIST_ITEM_PTR pModuleList = NULL;
    uint32 iModule, iService, iSubService, iCapability;

    *numberOfModuleInfo = 0;

    if ((retValue = MDSUTIL_ListModules(CSSM_SERVICE_CSP, FALSE, &pGUIDList)) != CSSM_OK)
    {
        return NULL;
    }
        

    for(iModule = 0; iModule < pGUIDList->NumberItems; iModule++)
    {
        MDSUTIL_MODULE_INFO_PTR pInfo;
        if ((retValue = MDSUTIL_GetModuleInfo (&(pGUIDList->Items[iModule].SubserviceUid.Guid),
                                    CSSM_SERVICE_CSP,
                                    0,                                    
                                    CSSM_USEE_NONE,
                                    &pInfo)) != CSSM_OK)
        {
            return NULL;
        }

        for(iService = 0; iService < pInfo->NumberOfServices; iService++)
        {
            MDSUTIL_SERVICE_INFO_PTR pServiceInfo;
            MDSUTIL_CSPSUBSERVICE_PTR pSubServiceList;
            pServiceInfo = &pInfo->ServiceList[iService];
            
    
            pSubServiceList = (MDSUTIL_CSPSUBSERVICE_PTR)(pServiceInfo->SubServiceList->SubServiceInfo);
    
            for(iSubService = 0; 
                            iSubService < pServiceInfo->NumberOfSubServices;
                            iSubService++)
            {
                uint32 numCapabilities = 0;
                CSSM_CONTEXT_PTR capabilityList;
                MDSUTIL_CSPSUBSERVICE_PTR pSubService;

                pSubService = &pSubServiceList[iSubService];
            
                if(pSubService->CspType == CSSM_CSP_SOFTWARE)
                {
                    numCapabilities = pSubService->SubServiceInfo.SoftwareCspSubService.NumberOfCapabilities;
                    capabilityList = pSubService->SubServiceInfo.SoftwareCspSubService.CapabilityList;
                }
                else if (pSubService->CspType == CSSM_CSP_HARDWARE)
                {
                    numCapabilities = pSubService->SubServiceInfo.HardwareCspSubService.NumberOfCapabilities;
                    capabilityList = pSubService->SubServiceInfo.HardwareCspSubService.CapabilityList;
                }
                else
                {
                    MDSUTIL_FreeModuleInfo(pInfo);
                    MDSUTIL_FreeModuleList(pGUIDList);
                    return pModuleList;

                }

                
                for(iCapability =0 ; 
                                iCapability < numCapabilities; 
                                iCapability++)
                {

                    CSSM_CONTEXT_PTR pCapability;
                    pCapability = &capabilityList[iCapability];
                    // Right now there is no way to tell an asymetric keygen alg
                    // from a symetric one. So we have to hard code the two asymetric algs
                    // we know about...RSA and DSA
            
                    if(    (pCapability->ContextType == contextType ) 
                        &&(pCapability->AlgorithmType == algorithmType)) 
                    {
                    

                        // make an entry for this CSP, since it provides the 
                        //specified capabilities
                        CSP_LIST_ITEM_PTR pEntry = (CSP_LIST_ITEM_PTR)isl_ConfigAllocateMemory(
                                                pArchiveConfig,sizeof(CSP_LIST_ITEM));
                        if (!pEntry) {
                            MDSUTIL_FreeModuleInfo(pInfo);
                            MDSUTIL_FreeModuleList(pGUIDList);
                            return pModuleList;
                        }
                        pEntry->SubServiceID = pSubService->SubServiceId;
                        pEntry->Version.Major = pInfo->CompatibleCSSMVersion.Major;
                        pEntry->Version.Minor = pInfo->CompatibleCSSMVersion.Minor;
                        memcpy((void*)&pEntry->Guid, 
                            (void*)&(pGUIDList->Items[iModule].SubserviceUid.Guid),
                            sizeof(CSSM_GUID)); 
                        pEntry->Next = NULL;
                        //add the new entry to the head of the module info list
                        if (!pModuleList) 
                            pModuleList = pEntry;
                        else {
                            pEntry->Next = pModuleList;
                            pModuleList = pEntry;
                        }
                        *numberOfModuleInfo = *numberOfModuleInfo + 1;
                        break;
                    }
                } //for( iCapability
            } //for(iSubService
        }//for(iService
        MDSUTIL_FreeModuleInfo(pInfo);
    }// for(iModule

    // clean up
    MDSUTIL_FreeModuleList(pGUIDList);
    return pModuleList;
}

ISL_CSSM_MODULE_HANDLE * isl_GetCSPHandles(
    ISL_ARCHIVE_CONFIG *Context,
    CSP_LIST_ITEM_PTR pModuleInfoList,
    uint32 numberOfModuleInfo,
    uint32 *numberOfCSPHandles,
    ISL_CSSM_METHODS    *pCSSM) 
{
    
    CSP_LIST_ITEM_PTR pModuleInfo;
    ISL_CSSM_MODULE_HANDLE *pCSPHandles = NULL;
    ISL_CSSM_MODULE_HANDLE *hCSP;
    
    *numberOfCSPHandles = 0;

    //allocate memory
    pCSPHandles = (ISL_CSSM_MODULE_HANDLE *)isl_ConfigAllocateMemory(Context,
                    sizeof(ISL_CSSM_MODULE_HANDLE) * numberOfModuleInfo);
    if (!pCSPHandles) {
        return NULL;
    }

    
    hCSP = pCSPHandles;
   
    //attach to each module in the list
    pModuleInfo = pModuleInfoList;
    while (pModuleInfo) {
            if (pCSSM->CSSM_ModuleLoad(
                          &pModuleInfo->Guid,
                          CSSM_KEY_HIERARCHY_NONE,
                          NULL,
                          NULL) == CSSM_OK) {
                if (pCSSM->CSSM_ModuleAttach (
                                        &pModuleInfo->Guid, 
                                        &pModuleInfo->Version, 
                                        &g_memoryFX,
                                        pModuleInfo->SubServiceID,                
                                        CSSM_SERVICE_CSP,
                                        0,
                                        CSSM_KEY_HIERARCHY_NONE,
                                        NULL,
                                        0,
                                        NULL,
                                        &hCSP->Handle) == CSSM_OK) {
                    *numberOfCSPHandles = *numberOfCSPHandles + 1;
                    memcpy(&hCSP->Guid, &pModuleInfo->Guid, sizeof(CSSM_GUID));
                    hCSP++;
                } else {
                    //Error attaching CSP
                    pCSSM->CSSM_ModuleUnload(
                                        &pModuleInfo->Guid,
                                        0,
                                        NULL);
                }
            }        
            pModuleInfo = pModuleInfo->Next;
    }
    return pCSPHandles;
}
        
ISL_CLASS_PTR isl_CreateDigestClass(
    ISL_ARCHIVE_CONFIG *pArchiveConfig,
    CSSM_CONTEXT_TYPE contextType,
    CSSM_ALGORITHMS algorithmType,
    ISL_CSSM_METHODS *pCSSMMethods,
    CSP_LIST_PTR CspList)
    
{
    ISL_CLASS_PTR pClass= NULL;
    CSP_LIST_ITEM_PTR pModuleInfoList = NULL;
    uint32 numberOfModuleInfo;
    uint32 numberOfCSPHandles, i;
    CSP_LIST_ITEM_PTR current, next;
#ifndef VMS
    CSP_LIST ;
#endif
    if (!CspList) {
        //find all csps that support the specified digest algorithm
        pModuleInfoList = isl_FindAvailableCSPs(pArchiveConfig,
                                    contextType, 
                                    algorithmType, 
                                    &numberOfModuleInfo, 
                                    pCSSMMethods);
        if (!pModuleInfoList)
            return NULL;
    }
    else {
        pModuleInfoList = CspList->Items;
        numberOfModuleInfo = CspList->NumberOfItems;
    }


    //allocate memory
    pClass = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pClass)
        return NULL;
    
    //allocate memory
    pClass->ClassContext = (void *)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_CLASS_CONTEXT));
    if (!pClass->ClassContext) {
        return NULL;
    }
    

    ((ISL_DIGEST_CLASS_CONTEXT*)(pClass->ClassContext))->CssmMethods = pCSSMMethods;
    
    ((ISL_DIGEST_CLASS_CONTEXT*)(pClass->ClassContext))->CSPList.ModuleHandles = 
        isl_GetCSPHandles(
            pArchiveConfig, 
            pModuleInfoList, 
            numberOfModuleInfo, 
            &numberOfCSPHandles,
            pCSSMMethods);

    ((ISL_DIGEST_CLASS_CONTEXT*)(pClass->ClassContext))->CSPList.NumberOfHandles = numberOfCSPHandles;

    //free memory
    current = pModuleInfoList;
    for (i=0; i<numberOfModuleInfo; i++) {
        next = current->Next;
        isl_ConfigFreeMemory(pArchiveConfig, current);
        current = next;
    }

    return pClass;
}

ISL_CLASS_PTR isl_CreateSignatureClass(
    ISL_ARCHIVE_CONFIG        *pArchiveConfig,
    uint32                    contextType,
    uint32                    algorithmType,
    ISL_CSSM_METHODS        *pCSSMMethods,
    CSP_LIST_PTR CspList)
{

    ISL_CLASS_PTR pClass= NULL;
    CSP_LIST_ITEM_PTR pModuleInfoList = NULL;
    uint32 numberOfModuleInfo, i;
    uint32 numberOfCSPHandles;
    CSP_LIST_ITEM_PTR current, next;

    if (!CspList) {
        //find all csps that support the specified digest algorithm
        pModuleInfoList = isl_FindAvailableCSPs(pArchiveConfig,
                                    contextType, 
                                    algorithmType, 
                                    &numberOfModuleInfo, 
                                    pCSSMMethods);
        if (!pModuleInfoList)
            return NULL;
    }
    else {
        pModuleInfoList = CspList->Items;
        numberOfModuleInfo = CspList->NumberOfItems;
    }
    //allocate memory for the class structure
    pClass = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pClass)
        return NULL;
    
    //allocate memory for the class context
    pClass->ClassContext = (void *)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_SIGN_VERIFY_CLASS_CONTEXT));
    if (!pClass->ClassContext) {
        return NULL;
    }

    ((ISL_SIGN_VERIFY_CLASS_CONTEXT*)(pClass->ClassContext))->CssmMethods = pCSSMMethods;

    //save the memory functions that are used by CSSM in the class context
    ((ISL_SIGN_VERIFY_CLASS_CONTEXT*)(pClass->ClassContext))->MemoryFuncs = &g_memoryFX;
    
    

    //save the csp handles list in the class context
    ((ISL_SIGN_VERIFY_CLASS_CONTEXT*)(pClass->ClassContext))->CSPList.ModuleHandles =
            isl_GetCSPHandles(
                pArchiveConfig, 
                pModuleInfoList, 
                numberOfModuleInfo, 
                &numberOfCSPHandles,
                pCSSMMethods);
    ((ISL_SIGN_VERIFY_CLASS_CONTEXT*)(pClass->ClassContext))->CSPList.NumberOfHandles = numberOfCSPHandles;
    //free memory
    current = pModuleInfoList;
    for (i=0; i<numberOfModuleInfo; i++) {
        next = current->Next;
        isl_ConfigFreeMemory(pArchiveConfig, current);
        current = next;
    }


    
    return pClass;
}

//free resource that was used by signature method
void isl_FreeSignatureMethod(ISL_ARCHIVE_CONFIG *Context, ISL_SIGN_VERIFY_METHODS *methods) {
    uint32 i, count;
    ISL_CSSM_MODULE_HANDLE *hCSP;
    ISL_SIGN_VERIFY_CLASS_CONTEXT *pClassContext;
    
    if (!methods)
        return;

    if (methods->ServiceMethods.Class) {
        if (methods->ServiceMethods.Class->ClassContext) {
            pClassContext = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)
                            methods->ServiceMethods.Class->ClassContext;    
            //detach from the CSPs
            count = pClassContext->CSPList.NumberOfHandles;
            hCSP = pClassContext->CSPList.ModuleHandles;
            for (i = 0; i<count; i++) {
                if (hCSP[i].Handle)
                    pClassContext->CssmMethods->CSSM_ModuleDetach (hCSP[i].Handle);
                    pClassContext->CssmMethods->CSSM_ModuleUnload(
                                                      &hCSP[i].Guid,
                                                      NULL,
                                                      NULL);
            }
            isl_ConfigFreeMemory(Context,pClassContext->CSPList.ModuleHandles);
            isl_ConfigFreeMemory(Context,pClassContext->SignatureMethodList.Methods);
            isl_ConfigFreeMemory(Context, pClassContext);
        }
        isl_ConfigFreeMemory(Context,methods->ServiceMethods.Class);
        methods->ServiceMethods.Class = NULL;
    }
    isl_ConfigFreeMemory(Context,methods);
}

//free resource that was used by digest method 
void isl_FreeDigestMethod(ISL_ARCHIVE_CONFIG *Context, ISL_DIGEST_METHODS *methods) {
    uint32 i, count;
    ISL_CSSM_MODULE_HANDLE *hCSP;
    ISL_DIGEST_CLASS_CONTEXT *pClassContext;
    if (!methods)
        return;

    if (methods->ServiceMethods.Class) {
        if (methods->ServiceMethods.Class->ClassContext) {
            pClassContext = (ISL_DIGEST_CLASS_CONTEXT *)
                            methods->ServiceMethods.Class->ClassContext;
            //detach from the CSPs
            count = pClassContext->CSPList.NumberOfHandles;
            hCSP = pClassContext->CSPList.ModuleHandles;
            for (i = 0; i<count; i++) {
                if (hCSP[i].Handle)
                    pClassContext->CssmMethods->CSSM_ModuleDetach (hCSP[i].Handle);
                    pClassContext->CssmMethods->CSSM_ModuleUnload(
                                                      &hCSP[i].Guid,
                                                      NULL,
                                                      NULL);
            }
            isl_ConfigFreeMemory(Context, pClassContext->CSPList.ModuleHandles);
            isl_ConfigFreeMemory(Context,
                methods->ServiceMethods.Class->ClassContext);
        }
        isl_ConfigFreeMemory(Context,methods->ServiceMethods.Class);
        methods->ServiceMethods.Class = NULL;
    }
    isl_ConfigFreeMemory(Context,methods);
}



FARPROC isl_FindCssmFuncAddress(AAL_FUNC_TABLE_PTR CssmFuncTable, 
                              uint32 NumCssmFuncTable, 
                              char *FunctionName)
{

    uint32 i;
    for (i=0; i<NumCssmFuncTable; i++) {
        if (!strcmp(CssmFuncTable[i].Name, FunctionName))
                     return (int(*)())CssmFuncTable[i].Address;   /* ifdef VMS, apply cast */
    }
    return NULL;
}

ISL_CSSM_METHODS * isl_BuildCssmFuncTable(ISL_ARCHIVE_CONFIG_PTR pArchiveConfig, 
                                          AAL_FUNC_TABLE_PTR CssmFuncTable, 
                                          uint32 NumCssmFuncTable) {

      //allocate memory pCSSM
    ISL_CSSM_METHODS *pCSSMMethods;

    pCSSMMethods = (ISL_CSSM_METHODS_PTR)isl_ConfigAllocateMemory(pArchiveConfig,
                    sizeof(ISL_CSSM_METHODS));
    if (!pCSSMMethods) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }                        

    pCSSMMethods->CSSM_ModuleLoad= (CSSM_ModuleLoad_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_ModuleLoad");
    if (!pCSSMMethods->CSSM_ModuleLoad) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_ModuleUnload= (CSSM_ModuleUnload_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_ModuleUnload");
    if (!pCSSMMethods->CSSM_ModuleUnload) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_ModuleAttach= (CSSM_ModuleAttach_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_ModuleAttach");
    if (!pCSSMMethods->CSSM_ModuleAttach) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_ModuleDetach= (CSSM_ModuleDetach_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_ModuleDetach");
    if (!pCSSMMethods->CSSM_ModuleDetach) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_DigestDataInit= (CSSM_DigestDataInit_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_DigestDataInit");
    if (!pCSSMMethods->CSSM_DigestDataInit) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_DigestDataUpdate = (CSSM_DigestDataUpdate_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_DigestDataUpdate");
    if (!pCSSMMethods->CSSM_DigestDataUpdate) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_DigestDataFinal = (CSSM_DigestDataFinal_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_DigestDataFinal");
    if (!pCSSMMethods->CSSM_DigestDataFinal) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_CSP_CreateSignatureContext = (CSSM_CSP_CreateSignatureContext_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_CSP_CreateSignatureContext");
    if (!pCSSMMethods->CSSM_CSP_CreateSignatureContext) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_CSP_CreateDigestContext = (CSSM_CSP_CreateDigestContext_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_CSP_CreateDigestContext");
    if (!pCSSMMethods->CSSM_CSP_CreateDigestContext) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_DeleteContext = (CSSM_DeleteContext_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_DeleteContext");
    if (!pCSSMMethods->CSSM_DeleteContext) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_VerifyData = (CSSM_VerifyData_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable, 
                                                            NumCssmFuncTable,
                                                            "CSSM_VerifyData");
    if (!pCSSMMethods->CSSM_VerifyData) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_SignData = (CSSM_SignData_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable,
                                                            NumCssmFuncTable,
                                                            "CSSM_SignData");
    if (!pCSSMMethods->CSSM_SignData) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
    pCSSMMethods->CSSM_CSP_ObtainPrivateKeyFromPublicKey = (CSSM_CSP_ObtainPrivateKeyFromPublicKey_ptr)
                                    isl_FindCssmFuncAddress(CssmFuncTable,
                                                            NumCssmFuncTable,
#ifdef VMS
                                                            "CSSM_CSP_ObtainPrivateKeyFromPu");
#else
                                                            "CSSM_CSP_ObtainPrivateKeyFromPublicKey");
#endif
    if (!pCSSMMethods->CSSM_CSP_ObtainPrivateKeyFromPublicKey) {
        isl_ConfigFreeMemory(pArchiveConfig, pCSSMMethods);
        return NULL;
    }
     
    return pCSSMMethods;
}

ISL_STATUS isl_AutoArchiveConfig(
    ISL_ARCHIVE_CONFIG_PTR pArchiveConfig,
    AAL_FUNC_TABLE_PTR CssmFuncTable,
    uint32 NumCssmFuncTable,
    CSP_LIST_PTR CspList) 
{
    ISL_SERVICE_CLASS ServiceClass;
    ISL_CONST_DATA AlgID,HumanName;
    ISL_CSSM_METHODS *pCSSMMethods = NULL;
    ISL_CLASS_PTR pClass = NULL;
    ISL_DIGEST_METHODS *pMyJarMD5HashMethods = NULL;
    ISL_DIGEST_METHODS *pMyJarSHA1HashMethods = NULL;
    ISL_DIGEST_METHODS *pMyJarMD2HashMethods = NULL;
    ISL_GET_DATA_METHODS *pMyFileGetDataMethods = NULL;
    ISL_GET_DATA_METHODS *pMyExecutableFileGetDataMethods = NULL;
    ISL_GET_DATA_METHODS *pMyURLGetMethods = NULL;
    


    if (!pArchiveConfig ||
        !CssmFuncTable || NumCssmFuncTable < 0)
        return ISL_FAIL;

    pCSSMMethods = isl_BuildCssmFuncTable(pArchiveConfig, CssmFuncTable,NumCssmFuncTable );
    if (!pCSSMMethods)
        return ISL_FAIL;

    //allocate memory
    pClass = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(
                            pArchiveConfig,
                            sizeof(ISL_CLASS));
    if (!pClass)
        goto FAIL;

    //allocate memory
    pClass->ClassContext = (void *)isl_ConfigAllocateMemory(pArchiveConfig, 
                            sizeof(ISL_CONFIG_CLASS_CONTEXT));
    if (!pClass->ClassContext)  
        goto FAIL;
    
    //save cssm function pointers in the class context in order to free the resource
    //at "recycle" call.
    ((ISL_CONFIG_CLASS_CONTEXT*)(pClass->ClassContext))->CssmMethods = pCSSMMethods; 
    
    pArchiveConfig->Class = pClass;


    
    // add md5 hash algorithm
    pMyJarMD5HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));

    if (!pMyJarMD5HashMethods) 
        goto FAIL;

    memcpy((void*)pMyJarMD5HashMethods, (void*)&JarMD5Methods, sizeof(ISL_DIGEST_METHODS));
    pMyJarMD5HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_MD5,
            pCSSMMethods, 
            CspList);
    if (!pMyJarMD5HashMethods->ServiceMethods.Class) {
        //No csp supports MD5 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyJarMD5HashMethods);
        pMyJarMD5HashMethods = NULL;
    }
    else {
        pMyJarMD5HashMethods->ServiceMethods.Class->ClassMethods = pMyJarMD5HashMethods;
        (*pMyJarMD5HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ( (*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyJarMD5HashMethods) != ISL_OK)
            goto FAIL;
    }

    // add sha-1 hash algorithm 
    pMyJarSHA1HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));
    if (!pMyJarSHA1HashMethods) 
        goto FAIL;

    memcpy((void*)pMyJarSHA1HashMethods, (void*)&JarSHA1Methods, sizeof(ISL_DIGEST_METHODS));
    pMyJarSHA1HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_SHA1,
            pCSSMMethods,
            CspList);
    if (!pMyJarSHA1HashMethods->ServiceMethods.Class) {
        //No csp supports SHA1 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyJarSHA1HashMethods);
        pMyJarSHA1HashMethods = NULL;
    }
    else {
        pMyJarSHA1HashMethods->ServiceMethods.Class->ClassMethods = pMyJarSHA1HashMethods;
        (*pMyJarSHA1HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyJarSHA1HashMethods) != ISL_OK)
            goto FAIL;
    }

    // add md2 hash algorithm
    pMyJarMD2HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));
    if (!pMyJarMD2HashMethods)
        goto FAIL;

    memcpy((void*)pMyJarMD2HashMethods, (void*)&JarMD2Methods, sizeof(ISL_DIGEST_METHODS));
    pMyJarMD2HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_MD2,
            pCSSMMethods,
            CspList);
    if (!pMyJarMD2HashMethods->ServiceMethods.Class) {
        //No csp supports MD2 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyJarMD2HashMethods);
        pMyJarMD2HashMethods = NULL;
    }
    else {
        pMyJarMD2HashMethods->ServiceMethods.Class->ClassMethods = pMyJarMD2HashMethods;
        (*pMyJarMD2HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyJarMD2HashMethods) != ISL_OK)
            goto FAIL;
    }

    
    // add "file" get data algorithm 
    pMyFileGetDataMethods = (ISL_GET_DATA_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_GET_DATA_METHODS));
    if (!pMyFileGetDataMethods) 
        goto FAIL;
    memcpy((void*)pMyFileGetDataMethods, (void*)&FileGetDataMethods, sizeof(ISL_GET_DATA_METHODS));
    pMyFileGetDataMethods->ServiceMethods.Class = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pMyFileGetDataMethods->ServiceMethods.Class) {
        isl_ConfigFreeMemory(pArchiveConfig, pMyFileGetDataMethods);
        pMyFileGetDataMethods = NULL;
    }
    else {
        pMyFileGetDataMethods->ServiceMethods.Class->ClassMethods = pMyFileGetDataMethods;
        (*pMyFileGetDataMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*) pMyFileGetDataMethods) != ISL_OK)
            goto FAIL;
    }

    /* add "dll" get data algorithm */
    pMyExecutableFileGetDataMethods = (ISL_GET_DATA_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_GET_DATA_METHODS));
    if (!pMyExecutableFileGetDataMethods) 
        goto FAIL;
    memcpy((void*)pMyExecutableFileGetDataMethods, (void*)&ExecutableFileGetDataMethods, sizeof(ISL_GET_DATA_METHODS));
    pMyExecutableFileGetDataMethods->ServiceMethods.Class = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pMyExecutableFileGetDataMethods->ServiceMethods.Class) {
        isl_ConfigFreeMemory(pArchiveConfig, pMyExecutableFileGetDataMethods);
        pMyExecutableFileGetDataMethods = NULL;
    }
    else {
        pMyExecutableFileGetDataMethods->ServiceMethods.Class->ClassMethods = pMyExecutableFileGetDataMethods;
        (*pMyExecutableFileGetDataMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*) pMyExecutableFileGetDataMethods) != ISL_OK)
            goto FAIL;
    }

#ifdef WIN32
    /* XXX "url" methods are currently only implemented on WIN32 */
    /* add "url" get data algorithm */
    pMyURLGetMethods = (ISL_GET_DATA_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_GET_DATA_METHODS));
    if (!pMyURLGetMethods) 
        goto FAIL;

    memcpy((void*)pMyURLGetMethods, (void*)&URLGetMethods, sizeof(ISL_GET_DATA_METHODS));
    pMyURLGetMethods->ServiceMethods.Class = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pMyURLGetMethods->ServiceMethods.Class) {
        isl_ConfigFreeMemory(pArchiveConfig, pMyURLGetMethods);
        pMyURLGetMethods = NULL;
    }
    else {
        pMyURLGetMethods->ServiceMethods.Class->ClassMethods = pMyURLGetMethods;
        (*pMyURLGetMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if((*ArchiveConfigMethods.AddAlgorithm)(pArchiveConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*) pMyURLGetMethods) != ISL_OK)
            goto FAIL;
    }
#endif

    return ISL_OK;

FAIL:
    if (pCSSMMethods)
        isl_ConfigFreeMemory(pArchiveConfig, (void *)pCSSMMethods);

    if (pClass) {
        if (pClass->ClassContext)
            isl_ConfigFreeMemory(pArchiveConfig, (void *)pClass->ClassContext);
        isl_ConfigFreeMemory(pArchiveConfig, (void *)pClass);
    }

    if (!pMyJarMD5HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyJarMD5HashMethods);

    if (!pMyJarSHA1HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyJarSHA1HashMethods);

    if (!pMyJarMD2HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyJarMD2HashMethods);

    if (!pMyFileGetDataMethods){
        isl_ConfigFreeMemory(pArchiveConfig,pMyFileGetDataMethods->ServiceMethods.Class);
        isl_ConfigFreeMemory(pArchiveConfig,pMyFileGetDataMethods);
    }

    if (!pMyExecutableFileGetDataMethods){
        isl_ConfigFreeMemory(pArchiveConfig,pMyExecutableFileGetDataMethods->ServiceMethods.Class);
        isl_ConfigFreeMemory(pArchiveConfig,pMyExecutableFileGetDataMethods);
    }

    if (!pMyURLGetMethods){
        isl_ConfigFreeMemory(pArchiveConfig,pMyURLGetMethods->ServiceMethods.Class);
        isl_ConfigFreeMemory(pArchiveConfig,pMyURLGetMethods);
    }


    return ISL_FAIL;

    
}


ISL_STATUS isl_AutoSignatureConfig(
    ISL_SIGNATURE_CONFIG_PTR pSignatureConfig,
    AAL_FUNC_TABLE_PTR CssmFuncTable,
    uint32 NumCssmFuncTable,
    CSP_LIST_PTR CspList) 
{
    ISL_STATUS status;
    ISL_SERVICE_CLASS ServiceClass;
    ISL_CLASS_PTR pClass;
    ISL_CONST_DATA AlgID,HumanName;
    ISL_SIGN_VERIFY_CLASS_CONTEXT *pSignClassCtx;
    ISL_SIGN_VERIFY_CLASS_CONTEXT *pClassCtx;
    ISL_CSSM_METHODS    *pCSSMMethods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyDSAMethods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyPKCS1RSASHA1Methods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyPKCS1RSAMD5Methods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyPKCS1RSAMD2Methods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyPKCS1RSAMethods = NULL;
    ISL_SIGN_VERIFY_METHODS *pMyPKCS7DSAMethods = NULL;
    ISL_DIGEST_METHODS *pMyPKCS7MD5HashMethods = NULL;
    ISL_DIGEST_METHODS *pMyPKCS7SHA1HashMethods = NULL;
    ISL_DIGEST_METHODS *pMyPKCS7MD2HashMethods = NULL;
    ISL_ARCHIVE_CONFIG_PTR pArchiveConfig = NULL;

    if (!pSignatureConfig ||
        !CssmFuncTable || NumCssmFuncTable < 0)
        return ISL_FAIL;

    pArchiveConfig = pSignatureConfig->ArchiveConfig;
    if (!pArchiveConfig)
        return ISL_FAIL;

    pCSSMMethods = isl_BuildCssmFuncTable(pArchiveConfig, CssmFuncTable,NumCssmFuncTable );
    if (!pCSSMMethods)
        return ISL_FAIL;

    pClass = (ISL_CLASS_PTR)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_CLASS));
    if (!pClass) 
        goto FAIL;
    
    pClass->ClassContext = (void *)isl_ConfigAllocateMemory(pArchiveConfig, 
                            sizeof(ISL_CONFIG_CLASS_CONTEXT));
    if (!pClass->ClassContext)  
        goto FAIL;

    //save cssm function pointers in the class context in order to free the resource
    //at "recycle" call.
    ((ISL_CONFIG_CLASS_CONTEXT*)(pClass->ClassContext))->CssmMethods = pCSSMMethods; 
    pSignatureConfig->Class = pClass;
    
    
    pMyPKCS7SHA1HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));
    if (!pMyPKCS7SHA1HashMethods)
        goto FAIL;

    pMyPKCS7MD5HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));
    if (!pMyPKCS7MD5HashMethods) 
        goto FAIL;

    pMyPKCS7MD2HashMethods = (ISL_DIGEST_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_DIGEST_METHODS));
    if (!pMyPKCS7MD2HashMethods) 
        goto FAIL;


    pMyPKCS7DSAMethods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyPKCS7DSAMethods)
        goto FAIL;

    pMyPKCS1RSAMethods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyPKCS1RSAMethods) 
        goto FAIL;


    pMyDSAMethods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyDSAMethods)
        goto FAIL;


    pMyPKCS1RSAMD5Methods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig,
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyPKCS1RSAMD5Methods) 
        goto FAIL;


    pMyPKCS1RSASHA1Methods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyPKCS1RSASHA1Methods) 
        goto FAIL;

    pMyPKCS1RSAMD2Methods = (ISL_SIGN_VERIFY_METHODS*)isl_ConfigAllocateMemory(
                            pArchiveConfig, 
                            sizeof(ISL_SIGN_VERIFY_METHODS));
    if (!pMyPKCS1RSAMD2Methods)
        goto FAIL;


    // add sha1 hash algorithm 
    memcpy((void*)pMyPKCS7SHA1HashMethods, (void*)&PKCS7SHA1Methods, sizeof(ISL_DIGEST_METHODS));
    pMyPKCS7SHA1HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_SHA1,
            pCSSMMethods,
            CspList);
    if (!pMyPKCS7SHA1HashMethods->ServiceMethods.Class) {
        //No csp supports SHA1 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS7SHA1HashMethods);
        pMyPKCS7SHA1HashMethods = NULL;
    }
    else {
        pMyPKCS7SHA1HashMethods->ServiceMethods.Class->ClassMethods = pMyPKCS7SHA1HashMethods;
        (*pMyPKCS7SHA1HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyPKCS7SHA1HashMethods) != ISL_OK) 
            goto FAIL;
    }

        

    // add md5 hash algorithm
    memcpy((void*)pMyPKCS7MD5HashMethods, (void*)&PKCS7MD5Methods, sizeof(ISL_DIGEST_METHODS));
    pMyPKCS7MD5HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_MD5,
            pCSSMMethods,
            CspList);
    if (!pMyPKCS7MD5HashMethods->ServiceMethods.Class) {
        //No csp supports MD5 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS7MD5HashMethods);
        pMyPKCS7MD5HashMethods = NULL;
    }
    else {
        pMyPKCS7MD5HashMethods->ServiceMethods.Class->ClassMethods = pMyPKCS7MD5HashMethods;
        (*pMyPKCS7MD5HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyPKCS7MD5HashMethods) != ISL_OK)
            goto FAIL;
    }


    // add md2 hash algorithm
    memcpy((void*)pMyPKCS7MD2HashMethods, (void*)&PKCS7MD2Methods, sizeof(ISL_DIGEST_METHODS));
    pMyPKCS7MD2HashMethods->ServiceMethods.Class = 
        isl_CreateDigestClass(
            pArchiveConfig, 
            CSSM_ALGCLASS_DIGEST, 
            CSSM_ALGID_MD2,
            pCSSMMethods,
            CspList);

    if (!pMyPKCS7MD2HashMethods->ServiceMethods.Class){
        //No csp supports MD2 algorithm
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS7MD2HashMethods);
        pMyPKCS7MD2HashMethods = NULL;
    }
    else {
        pMyPKCS7MD2HashMethods->ServiceMethods.Class->ClassMethods = pMyPKCS7MD2HashMethods;
        (*pMyPKCS7MD2HashMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyPKCS7MD2HashMethods) != ISL_OK)
            goto FAIL;
    }

    // add PKCS7 DSA algorithm 
    //Note : Since Intel CSP only support combination algorithms(i.e. CSSM_ALGID_DSAWithSHA1)
    //      type for signing, we'll use CSSM_ALGCLASS_KEYGEN context type to search for CSPs 
    //      that support CSSM_ALGID_DSA instead of using CSSM_ALGCLASS_SIGNATURE context type
    memcpy((void*)pMyPKCS7DSAMethods, (void*)&PKCS7DSAMethods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyPKCS7DSAMethods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_SIGNATURE, 
            CSSM_ALGID_SHA1WithDSA,
            pCSSMMethods,
            CspList);
    if (!pMyPKCS7DSAMethods->ServiceMethods.Class) {
        isl_ConfigFreeMemory(pArchiveConfig,pMyPKCS7DSAMethods); 
        pMyPKCS7DSAMethods = NULL;
    }
    else {
        pMyPKCS7DSAMethods->ServiceMethods.Class->ClassMethods = pMyPKCS7DSAMethods;
        pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS7DSAMethods->ServiceMethods.Class->ClassContext;
        pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_SIGN_METHOD));
        pSignClassCtx->SignatureMethodList.NumberOfMethods = 1;
        pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7SHA1HashMethods;
        pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyDSAMethods;
        (*pMyPKCS7DSAMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        status = (*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyPKCS7DSAMethods);
        if (status != ISL_OK) {
            return status;
        }
    }
    
    

    //add PKCS1 RSA algorithm 
    //Note : Since Intel CSP only support combination algorithms(i.e. CSSM_ALGID_MD5WithRSA)
    //      type for signing, we'll use CSSM_ALGCLASS_KEYGEN context type to search for CSPs 
    //      that support CSSM_ALGID_RSA insteads of using CSSM_ALGCLASS_SIGNATURE context type
    memcpy((void*)pMyPKCS1RSAMethods, (void*)&PKCS1RSAMethods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyPKCS1RSAMethods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_KEYGEN, 
            CSSM_ALGID_RSA,
            pCSSMMethods,
            CspList);
    if (!pMyPKCS1RSAMethods->ServiceMethods.Class){
        isl_ConfigFreeMemory(pArchiveConfig,pMyPKCS1RSAMethods);
        pMyPKCS1RSAMethods = NULL;
    }
    else {
        pMyPKCS1RSAMethods->ServiceMethods.Class->ClassMethods = pMyPKCS1RSAMethods;
        pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSAMethods->ServiceMethods.Class->ClassContext;
        pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, 3*sizeof(ISL_DIGEST_SIGN_METHOD));
        pSignClassCtx->SignatureMethodList.NumberOfMethods = 3;
        pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7SHA1HashMethods;
        pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyPKCS1RSASHA1Methods;
        pSignClassCtx->SignatureMethodList.Methods[1].DigestMethods = pMyPKCS7MD2HashMethods;
        pSignClassCtx->SignatureMethodList.Methods[1].SignMethods = pMyPKCS1RSAMD2Methods;
        pSignClassCtx->SignatureMethodList.Methods[2].DigestMethods = pMyPKCS7MD5HashMethods;
        pSignClassCtx->SignatureMethodList.Methods[2].SignMethods = pMyPKCS1RSAMD5Methods;

        (*pMyPKCS1RSAMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
        if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
            ServiceClass,
            AlgID,
            HumanName,
            (ISL_SERVICE_CLASS_METHODS*)pMyPKCS1RSAMethods) != ISL_OK) 
            goto FAIL;
    }


    // add PKCS1 DSA/SHA1 signature algorithm 
    memcpy((void*)pMyDSAMethods, (void*)&DSAMethods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyDSAMethods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_SIGNATURE,
            CSSM_ALGID_SHA1WithDSA,
            pCSSMMethods,
            CspList);
    if (pMyDSAMethods->ServiceMethods.Class) {
        pMyDSAMethods->ServiceMethods.Class->ClassMethods = pMyDSAMethods;
        pClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyDSAMethods->ServiceMethods.Class->ClassContext;
        //add it to the configuration only if the algorithm is supported
        //by a csp
        if (pClassCtx->CSPList.NumberOfHandles) {
            pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyDSAMethods->ServiceMethods.Class->ClassContext;
            pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_SIGN_METHOD));
            pSignClassCtx->SignatureMethodList.NumberOfMethods = 1;
            pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7SHA1HashMethods;
            pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyPKCS7DSAMethods;

            (*pMyDSAMethods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
            if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
                ServiceClass,
                AlgID,
                HumanName,
                (ISL_SERVICE_CLASS_METHODS*)pMyDSAMethods)!= ISL_OK)
                goto FAIL;
        }
        else {
            isl_ConfigFreeMemory(pArchiveConfig, pMyDSAMethods);
            pMyDSAMethods = NULL;
        }
    }
    else {
        isl_ConfigFreeMemory(pArchiveConfig, pMyDSAMethods);
        pMyDSAMethods = NULL;
    }

    // add PKCS1 RSA/MD5 signature algorithm 
    memcpy((void*)pMyPKCS1RSAMD5Methods, (void*)&PKCS1RSAMD5Methods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyPKCS1RSAMD5Methods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_SIGNATURE,
            CSSM_ALGID_MD5WithRSA,
            pCSSMMethods,
            CspList); 
    if (pMyPKCS1RSAMD5Methods->ServiceMethods.Class) {
        pMyPKCS1RSAMD5Methods->ServiceMethods.Class->ClassMethods = pMyPKCS1RSAMD5Methods;
        pClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSAMD5Methods->ServiceMethods.Class->ClassContext;
        //add it to the configuration only if the algorithm is supported
        //by a csp
        if (pClassCtx->CSPList.NumberOfHandles) {
            //save my pkcs1 rsa/md5 methods in the class context
            pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSAMD5Methods->ServiceMethods.Class->ClassContext;
            pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_SIGN_METHOD));
            pSignClassCtx->SignatureMethodList.NumberOfMethods = 1;
            pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7MD5HashMethods;
            pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyPKCS1RSAMethods;

            (*pMyPKCS1RSAMD5Methods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
            if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
                ServiceClass,
                AlgID,
                HumanName,
                (ISL_SERVICE_CLASS_METHODS*)pMyPKCS1RSAMD5Methods) != ISL_OK) 
                goto FAIL;
        }
        else {
            isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSAMD5Methods);
            pMyPKCS1RSAMD5Methods = NULL;
        }
    }
    else {
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSAMD5Methods);
        pMyPKCS1RSAMD5Methods = NULL;
    }

    // add PKCS1 RSA/SHA1 signature algorithm 
    memcpy((void*)pMyPKCS1RSASHA1Methods, (void*)&PKCS1RSASHA1Methods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyPKCS1RSASHA1Methods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_SIGNATURE,
            CSSM_ALGID_SHA1WithRSA,
            pCSSMMethods,
            CspList); 
    if (pMyPKCS1RSASHA1Methods->ServiceMethods.Class) {
        pMyPKCS1RSASHA1Methods->ServiceMethods.Class->ClassMethods = pMyPKCS1RSASHA1Methods;
        pClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSASHA1Methods->ServiceMethods.Class->ClassContext;
        //add it to the configuration only if the algorithm is supported
        //by a csp
        if (pClassCtx->CSPList.NumberOfHandles) {
            pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSASHA1Methods->ServiceMethods.Class->ClassContext;
            pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_SIGN_METHOD));
            pSignClassCtx->SignatureMethodList.NumberOfMethods = 1;
            pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7SHA1HashMethods;
            pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyPKCS1RSAMethods;

            (*pMyPKCS1RSASHA1Methods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
            if((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
                ServiceClass,
                AlgID,
                HumanName,
                (ISL_SERVICE_CLASS_METHODS*)pMyPKCS1RSASHA1Methods) != ISL_OK) 
                goto FAIL;

        }
        else {
            isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSASHA1Methods);
            pMyPKCS1RSASHA1Methods = NULL;
        }
    }
    else {
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSASHA1Methods);
        pMyPKCS1RSASHA1Methods = NULL;
    }

    // add PKCS1 RSA/MD2 signature algorithm 
    memcpy((void*)pMyPKCS1RSAMD2Methods, (void*)&PKCS1RSAMD2Methods, sizeof(ISL_SIGN_VERIFY_METHODS));
    pMyPKCS1RSAMD2Methods->ServiceMethods.Class = 
        isl_CreateSignatureClass(
            pArchiveConfig,
            CSSM_ALGCLASS_SIGNATURE,
            CSSM_ALGID_MD2WithRSA,
            pCSSMMethods,
            CspList); 
    if (pMyPKCS1RSAMD2Methods->ServiceMethods.Class) {
        pMyPKCS1RSAMD2Methods->ServiceMethods.Class->ClassMethods = pMyPKCS1RSAMD2Methods;
        pClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSAMD2Methods->ServiceMethods.Class->ClassContext;
        //add it to the configuration only if the algorithm is supported
        //by a csp
        if (pClassCtx->CSPList.NumberOfHandles) {
            pSignClassCtx = (ISL_SIGN_VERIFY_CLASS_CONTEXT *)pMyPKCS1RSAMD2Methods->ServiceMethods.Class->ClassContext;
            pSignClassCtx->SignatureMethodList.Methods = (ISL_DIGEST_SIGN_METHOD*)isl_ConfigAllocateMemory(pArchiveConfig, sizeof(ISL_DIGEST_SIGN_METHOD));
            pSignClassCtx->SignatureMethodList.NumberOfMethods = 1;
            pSignClassCtx->SignatureMethodList.Methods[0].DigestMethods = pMyPKCS7MD2HashMethods;
            pSignClassCtx->SignatureMethodList.Methods[0].SignMethods = pMyPKCS1RSAMethods;

            (*pMyPKCS1RSAMD2Methods->ServiceMethods.id)(&ServiceClass, &AlgID, &HumanName);
            if ((*PKCS7SignatureConfigMethods.AddAlgorithm)(pSignatureConfig,
                ServiceClass,
                AlgID,
                HumanName,
                (ISL_SERVICE_CLASS_METHODS*)pMyPKCS1RSAMD2Methods) != ISL_OK) 
                goto FAIL;
        }
        else {
            isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSAMD2Methods);
            pMyPKCS1RSAMD2Methods = NULL;

        }

    }
    else {
        isl_ConfigFreeMemory(pArchiveConfig, pMyPKCS1RSAMD2Methods);
        pMyPKCS1RSAMD2Methods = NULL;
    }

    return ISL_OK;

FAIL:
    if (pCSSMMethods)
        isl_ConfigFreeMemory(pArchiveConfig, (void *)pCSSMMethods);

    if (pClass) {
        if (pClass->ClassContext)
            isl_ConfigFreeMemory(pArchiveConfig, (void *)pClass->ClassContext);
        isl_ConfigFreeMemory(pArchiveConfig, (void *)pClass);
    }

    if (!pMyDSAMethods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyDSAMethods);

    if (!pMyPKCS1RSASHA1Methods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyPKCS1RSASHA1Methods);

    if (!pMyPKCS1RSAMD5Methods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyPKCS1RSAMD5Methods);

    if (!pMyPKCS1RSAMD2Methods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyPKCS1RSAMD2Methods);

    if (!pMyPKCS1RSAMethods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyPKCS1RSAMethods);

    if (!pMyPKCS7DSAMethods)
        isl_FreeSignatureMethod(pArchiveConfig, pMyPKCS7DSAMethods);

    if (!pMyPKCS7MD5HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyPKCS7MD5HashMethods);

    if (!pMyPKCS7SHA1HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyPKCS7SHA1HashMethods);

    if (!pMyPKCS7MD2HashMethods)
        isl_FreeDigestMethod(pArchiveConfig, pMyPKCS7MD2HashMethods);

    return ISL_FAIL;
}

    



