/*-----------------------------------------------------------------------
 * File: X509CERT.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
#include "ber_der.h"
#ifndef EISL
#include "integapi.h"
#endif
#include "ber_bn.h"
#include "isl_internal.h"
#include <isl_memtrack.h>

#ifndef EISL
extern ISL_SIGNATURE_CONFIG_METHODS PKCS7SignatureConfigMethods;
extern ISL_GET_DATA_METHODS getCSSMDataMethods;
#endif

#if (!defined(EISL) || defined(DEBUG))
#define REPORT_CER_BER_EXPAND_ITEM(Context, Data, Length, Item)\
	isl_X509Cert_BER_ExpandItemError(Context, Data, Length, Item)
#else
#define REPORT_CER_BER_EXPAND_ITEM(Context, Data, Length, Item)
#endif


#if (!defined(EISL) || defined(DEBUG))
#define REPORT_CER_ALLOCATE_MEMORY(Context, Size)\
	isl_X509Cert_AllocateMemoryError(Context, Size);
#else
#define REPORT_CER_ALLOCATE_MEMORY(Context, Size)
#endif

#define REPORT_CER_BER_COUNT_ITEMS(x,y,z)


/*
**	X509 certificate class
*/
/* service methods */
#ifndef EISL
static void 
X509id(
	ISL_SERVICE_CLASS *algClass,				
	ISL_CONST_DATA *algID,				
	ISL_CONST_DATA *serviceName)
{
	*algClass = ISL_ServiceParseCertificate;
	algID->Data = (uint8 *)"cer";
	algID->Length = strlen((char *)algID->Data);;
	serviceName->Data = (uint8 *)"X509";
	serviceName->Length = strlen((char *)serviceName->Data);
}
#endif
/*
**	X509 certificate class
*/
static
void * isl_CertAllocateMemory(
	ISL_CERTIFICATE		*Context,
	uint32				Size)
{
	struct isl_memory_buffer * item;

	if (!Context) return NULL;

#ifdef EISL
	item = malloc(sizeof(struct isl_memory_buffer));
#else
	item = ISL_MALLOC_WRAPPER(Context->MemoryMethods->Malloc, 
		Context->MemoryContext, 
		sizeof(struct isl_memory_buffer));
#endif

	if (!item) return NULL;

#ifdef EISL
	item->Buffer = malloc(Size);
#else
	item->Buffer = ISL_MALLOC_WRAPPER(Context->MemoryMethods->Malloc, 
		Context->MemoryContext, 
		Size);
#endif

	if (!item->Buffer) {
#ifdef EISL
		free(item);
#else
		ISL_FREE_WRAPPER(Context->MemoryMethods->Free, Context->MemoryContext, item);
#endif
		return NULL;
	}

	item->Next = Context->Buffers;
	Context->Buffers = item;
	memset(item->Buffer, 0, Size);
	return item->Buffer;
}

#if (!defined(EISL) || defined(DEBUG))
static 
void isl_CertReportError(
	ISL_CERTIFICATE		*Context,
	ISL_ERROR_CODE		ErrCode,
	char *				ErrStr)
{
	ISL_ERROR_NODE_PTR	pErrNode = NULL;

	ASSERT(Context);
	pErrNode = isl_CertAllocateMemory(Context, sizeof(ISL_ERROR_NODE));
	if (!pErrNode) return;
	memset(pErrNode, 0, sizeof(ISL_ERROR_NODE));
	pErrNode->ErrCode = ErrCode;
	if (ErrStr) {
		pErrNode->ErrStr = isl_CertAllocateMemory(Context, sizeof(uint8) * 1024);
		if(!pErrNode->ErrStr) return;
		memset(pErrNode->ErrStr, 0, 1024);
		memcpy(pErrNode->ErrStr, ErrStr, 1024);
	}

	pErrNode->Next = Context->ErrStack;
	Context->ErrStack = pErrNode;
	return;
}
#endif

#if (!defined(EISL) || defined(DEBUG))
STATIC
void 
isl_X509Cert_BER_ExpandItemError(
	ISL_CERTIFICATE		*Context,
	const uint8				*Data,
	uint32				Length,
	BER_PARSED_ITEM		*Item)
{
	gErrStr = isl_GetErrorStr(BER_EXPAND_ITEM);
	sprintf(gErrBuf, gErrStr, Data, Length, Item);
	isl_CertReportError(Context, ISL_INVALID_BER_ENCODING, gErrBuf);
}
#endif

#if (!defined(EISL) || defined(DEBUG))
STATIC
void 
isl_X509Cert_AllocateMemoryError(
	ISL_CERTIFICATE		*Context,
	uint32				Size)
{
	gErrStr = isl_GetErrorStr(ISL_ALLOCATE_MEMORY);
	sprintf(gErrBuf, gErrStr, Context, Size);
	isl_CertReportError(Context, ISL_NO_MEMORY, gErrBuf);
}
#endif

/* class methods */
STATIC
ISL_SIZE 
isl_X509Cert_SizeofObject()			/* returns sizeof object */
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x00);
	return sizeof(struct isl_certificate);
}
	
/* object methods */

STATIC
ISL_STATUS
isl_X509Cert_Initialize(
		ISL_CERTIFICATE *Context,			/* memory for internal rep of cert */
		ISL_MEMORY_CONTEXT *MemoryContext,	/* memory allocation object */
		const ISL_MEMORY_METHODS *MemoryMethods,	/* memory allocation methods */
		ISL_CONST_DATA Cert)					/* external rep of certificate */
{
	const uint8 *end, *position, *tmp, *attribute, *attributeField;
	uint8 *buffer, *pkcs7buffer;
	BER_PARSED_ITEM whole, certinfo, keyAlgID, field, extension;
	uint32 i;
	size_t size;
	uint32 count;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x01);

	if (!Context) return ISL_FAIL;
#ifndef EISL
	if (!MemoryContext || !MemoryMethods) return ISL_FAIL;
#endif

	memset(Context, 0, sizeof(ISL_CERTIFICATE));

	pkcs7buffer = 0;
	Context->MemoryContext = MemoryContext;
	Context->MemoryMethods = (void*) MemoryMethods;


	if (!Cert.Data) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_INITIALIZE);
		sprintf(gErrBuf, gErrStr, Context, MemoryContext, MemoryMethods, Cert.Data);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;

	}

    buffer = isl_CertAllocateMemory(Context, Cert.Length);
	if (!buffer) { 
		REPORT_CER_ALLOCATE_MEMORY(Context, Cert.Length);
		return ISL_FAIL;
	}

	memcpy(buffer, Cert.Data, Cert.Length);
	Context->theCert.Length = Cert.Length;
	Context->theCert.Data = buffer;

	if (BER_ExpandItem(buffer, Cert.Length, &whole) != buffer + Cert.Length) {
		REPORT_CER_BER_EXPAND_ITEM(Context, buffer, Cert.Length, &whole);
		return ISL_FAIL;
	}
	if (whole.Tag[0] != (BER_SEQUENCE | BER_CONSTRUCTED)) { 
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_ALLOCATE_MEMORY);
		sprintf(gErrBuf, gErrStr, Context, Cert.Length);
		isl_CertReportError(Context, ISL_NO_MEMORY, gErrBuf);
#endif
		return ISL_FAIL;
	}
	end = BER_ExpandItem(whole.Content, whole.ContentLength, &certinfo);
	if (!end) {
		REPORT_CER_BER_EXPAND_ITEM(Context, whole.Content, whole.ContentLength, &certinfo);
		return ISL_FAIL;
	}
	Context->certificateInfo.Data = whole.Content;
	Context->certificateInfo.Length = BER_SizeofObject(whole.Content);
	position = BER_ExpandItem(certinfo.Content, certinfo.ContentLength, &field);
	if (!position) {
		REPORT_CER_BER_EXPAND_ITEM(Context, certinfo.Content, certinfo.ContentLength, &field);
		return ISL_FAIL;
	}
	// version number is EXTERNAL context specific 0 tag
	if (field.Tag[0] == (BER_CLASS_CONTEXTSPECIFIC | BER_CONSTRUCTED | 0)) {	/* skip version number */
		tmp = BER_ExpandItem(position, end - position, &field);
		if (!tmp) {
			REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
			return ISL_FAIL;
		}
		position = tmp;
	}
	Context->serialNumber.Data = field.Tag;
	Context->serialNumber.Length = position - field.Tag;
	/* skip cert signature algID */
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	/* pick up issuer */
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	Context->issuer.Data = field.Tag;
	Context->issuer.Length = position - field.Tag;
	/* skip validity */
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	/* pick up subject name	*/
	position = tmp;
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	Context->subject.Data = field.Tag;
	Context->subject.Length = position - field.Tag;
	/* pick get subjectpublickey */
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	Context->key.Data = field.Tag;
	Context->key.Length = position - field.Tag;
	/* pick algorithm ID and parameters out of subjectpublic key */
	tmp = BER_ExpandItem(field.Content, end - field.Content, &keyAlgID);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, field.Content, end - field.Content, &keyAlgID);
		return ISL_FAIL;
	}
	end = tmp;
	/* pick out OID */
	position = BER_ExpandItem(keyAlgID.Content, end - keyAlgID.Content, &field);
	if (!position) {
		REPORT_CER_BER_EXPAND_ITEM(Context,keyAlgID.Content, end - keyAlgID.Content, &field);
		return ISL_FAIL;
	}
	Context->algorithmID.Data = field.Tag;
	Context->algorithmID.Length = position - field.Tag;
	/* pick out parameters */
	tmp = BER_ExpandItem(position, end - position, &field);
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	Context->algorithmParameters.Data = field.Tag;
	Context->algorithmParameters.Length = position - field.Tag;
	/* find any certificate extensions */
	Context->attributeCount = 0;
	Context->attributes = 0;
	position = (uint8*) Context->key.Data + Context->key.Length;
	end = (uint8*) Context->certificateInfo.Data + Context->certificateInfo.Length;
	while (position < end) {
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x03);
		tmp = BER_ExpandItem(position, end - position, &field);
		if (!tmp) { 
			REPORT_CER_BER_EXPAND_ITEM(Context, position, end - position, &field);
			return ISL_FAIL;
		}
		position = tmp;
		/* X509V3 extensions are context specific tag 3 external sequence of extensions */
		if (field.Tag[0] == (BER_CLASS_CONTEXTSPECIFIC | BER_CONSTRUCTED | 3)) {
			/* strip external tag */
			tmp = BER_ExpandItem(field.Content, field.ContentLength, &field);
			if (!tmp) {
				REPORT_CER_BER_EXPAND_ITEM(Context, field.Content, field.ContentLength, &field);
				return ISL_FAIL;
			}
			position = tmp;
			count = BER_CountItems(field.Content, field.ContentLength);
			if (count <= 0) {
				REPORT_CER_BER_COUNT_ITEMS(Context, field.Content, field.ContentLength);
				return ISL_FAIL;
			}
			Context->attributeCount = count;
			Context->attributes = isl_CertAllocateMemory(Context, 
				Context->attributeCount * 2 * sizeof(ISL_CONST_DATA));
			if (!Context->attributes) {
				REPORT_CER_ALLOCATE_MEMORY(
					Context, 
					Context->attributeCount * 2 * sizeof(ISL_CONST_DATA));
				return ISL_FAIL;
			}
			break;
		}
	}
	if (Context->attributes) {
		attribute = field.Content;
		for (i = 0; i < Context->attributeCount; i++) {
            NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x04);

            tmp = BER_ExpandItem(attribute, end - attribute, &extension);
			if (!tmp) {
				REPORT_CER_BER_EXPAND_ITEM(Context, attribute, end - attribute, &extension);
				return ISL_FAIL;
			}
			attribute = tmp;
			Context->attributes[i].Name.Data = extension.Content; /* include tag and length */
			attributeField = BER_ExpandItem(extension.Content, 
				attribute - extension.Content, &extension);
			if (!attributeField) {
				REPORT_CER_BER_EXPAND_ITEM(
					Context, 
					extension.Content, 
					attribute - extension.Content, 
					&extension);
				return ISL_FAIL;
			}
			if (extension.Tag[0] != BER_OBJECT_IDENTIFIER) {
#if (!defined(EISL) || defined(DEBUG))
				gErrStr = isl_GetErrorStr(ISL_CERT_INITIALIZE);
				sprintf(gErrBuf, gErrStr, Context, MemoryContext, MemoryMethods, Cert.Data);
				isl_CertReportError(Context, ISL_INVALID_BER_ENCODING, gErrBuf);
#endif
				return ISL_FAIL;

			}
			Context->attributes[i].Name.Length = attributeField - 
				Context->attributes[i].Name.Data;
			tmp = BER_ExpandItem(attributeField, 
				attribute - attributeField, &extension);   /* skip over critical field */
			if (!tmp) {
				REPORT_CER_BER_EXPAND_ITEM(
					Context,
					attributeField, 
					attribute - attributeField, 
					&extension);
				return ISL_FAIL;
			}
			attributeField = tmp;
			if (extension.Tag[0] == BER_BOOLEAN) {		   /* field only present if true */
				tmp = BER_ExpandItem(attributeField,
					attribute - attributeField, &extension);
				if (!tmp) {
					REPORT_CER_BER_EXPAND_ITEM(
						Context,
						attributeField,
						attribute - attributeField, 
						&extension);
					return ISL_FAIL;
				}
				attributeField = tmp;
			}
			if (extension.Tag[0] != BER_OCTET_STRING) {
#if (!defined(EISL) || defined(DEBUG))
				gErrStr = isl_GetErrorStr(ISL_CERT_INITIALIZE);
				sprintf(gErrBuf, gErrStr, Context, MemoryContext, MemoryMethods, Cert.Data);
				isl_CertReportError(Context, ISL_INVALID_BER_ENCODING, gErrBuf);
#endif
				return ISL_FAIL;
			}
			Context->attributes[i].Value.Data = extension.Content;  /* strip octet string wrapper */
			Context->attributes[i].Value.Length = extension.ContentLength;
			if (attributeField != attribute) {
#if (!defined(EISL) || defined(DEBUG))
				gErrStr = isl_GetErrorStr(ISL_CERT_INITIALIZE);
				sprintf(gErrBuf, gErrStr, Context, MemoryContext, MemoryMethods, Cert.Data);
				isl_CertReportError(Context, ISL_INVALID_BER_ENCODING, gErrBuf);
#endif
				return ISL_FAIL;
			}
		}
	}
	
	/* construct the pkcs7 signer id (Seq(issuer, serial number)) */
	size = Context->issuer.Length + Context->serialNumber.Length;
	Context->pkcs7id.Length = size + BER_LengthOfLength(size) + 1;	/* content + length + tag */
	pkcs7buffer = isl_CertAllocateMemory(Context, Context->pkcs7id.Length);
	if (!pkcs7buffer) {
		REPORT_CER_ALLOCATE_MEMORY(Context, Context->pkcs7id.Length);
		return ISL_FAIL;
	}

	Context->pkcs7id.Data = pkcs7buffer;
	*pkcs7buffer++ = BER_SEQUENCE | BER_CONSTRUCTED;
	pkcs7buffer += BER_OutputLength(size, pkcs7buffer);
	memcpy(pkcs7buffer, Context->issuer.Data, Context->issuer.Length);
	pkcs7buffer += Context->issuer.Length;
	memcpy(pkcs7buffer, Context->serialNumber.Data, Context->serialNumber.Length);

	return ISL_OK;
}


#ifndef EISL
STATIC
ISL_STATUS 
isl_X509Cert_Flatten(
		ISL_CERTIFICATE *Context,
		ISL_CONST_DATA_PTR ExternalCert)
{
	if (!Context) return ISL_FAIL;

	*ExternalCert = Context->theCert;
	return ISL_OK;
}
#endif

STATIC
ISL_STATUS 
isl_X509Cert_Recycle(ISL_CERTIFICATE *Context)
{
	ISL_MEMORY_BUFFER_PTR item;
	ISL_MEMORY_BUFFER_PTR temp;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x05);
	if (!Context) return ISL_FAIL;
	/* recycle all memory buffers we allocated (do this last) */
	item = Context->Buffers;
	while(item) {
#ifdef EISL
		free(item->Buffer);
#else
		ISL_FREE_WRAPPER(Context->MemoryMethods->Free, Context->MemoryContext, item->Buffer);
#endif
		temp = item;
		item = item->Next;
#ifdef EISL
		free(temp);
#else
		ISL_FREE_WRAPPER(Context->MemoryMethods->Free, Context->MemoryContext, temp);
#endif
	}
	/* clear all the pointers,etc */
	memset(Context, 0, sizeof(ISL_CERTIFICATE));
	return ISL_OK;
}

STATIC
ISL_STATUS 
isl_X509Cert_GetID(ISL_CERTIFICATE *Context, ISL_CONST_DATA_PTR SignerID)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x06);

	if (!Context) return ISL_FAIL;

	if (!SignerID) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_INITIALIZE);
		sprintf(gErrBuf, gErrStr, Context, SignerID);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	*SignerID = Context->pkcs7id;
	return ISL_OK;
}

STATIC
ISL_STATUS
isl_X509Cert_GetKey(ISL_CERTIFICATE *Context, ISL_CONST_DATA_PTR Key)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x07);
	if (!Context) return ISL_FAIL;

	if (!Key) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_GET_KEY);
		sprintf(gErrBuf, gErrStr, Context, Key);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	*Key = Context->key;
	return ISL_OK;
}

STATIC
ISL_STATUS
isl_X509Cert_GetSubject(ISL_CERTIFICATE *Context, ISL_CONST_DATA_PTR SubjectID)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x08);

	if (!Context) return ISL_FAIL;

	if (!SubjectID) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_GET_SUBJECT);
		sprintf(gErrBuf, gErrStr, Context, SubjectID);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	*SubjectID = Context->subject;
	return ISL_OK;
}

STATIC
ISL_STATUS 
isl_X509Cert_GetIssuer(ISL_CERTIFICATE *Context, ISL_CONST_DATA_PTR IssuerID)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x09);

	if (!Context) return ISL_FAIL;

	if (!IssuerID) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_GET_ISSUER);
		sprintf(gErrBuf, gErrStr, Context, IssuerID);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	*IssuerID = Context->issuer;
	return ISL_OK;
}

STATIC
ISL_STATUS 
isl_X509Cert_GetAlgID(ISL_CERTIFICATE *Context, ISL_CONST_DATA *AlgID)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0a);

	if (!Context) return ISL_FAIL;

	if (!AlgID) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_GET_ALG_ID);
		sprintf(gErrBuf, gErrStr, Context, AlgID);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}
	*AlgID = Context->algorithmID;
	return ISL_OK;
}

STATIC
ISL_STATUS
isl_ParseKey(
	ISL_KEY_BLOCK		*Key,
	ISL_CONST_DATA_PTR	algorithmID,
	ISL_CONST_DATA_PTR	algorithmParameters,
	ISL_KEY_BLOCK		*keyBlock)
{
	BER_PARSED_ITEM		field;
	const uint8			*endkey;
	const uint8			*end;
	const uint8			*position;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0b);
	/* pick apart key into algorithm, parameters, and key block */
	endkey = BER_ExpandItem((uint8 *)Key->Data, Key->Length, &field);
	if (!endkey) 
		return ISL_FAIL;
	end = BER_ExpandItem(field.Content, endkey - field.Content, &field);
	if (!end) 
		return ISL_FAIL;
	/* pick out OID */
	position = BER_ExpandItem(field.Content, end - field.Content, &field);
	if (!position) 
		return ISL_FAIL;
	if (algorithmID)
	{
		algorithmID->Data = field.Tag;
		algorithmID->Length = position - field.Tag;
	}
	/* pick out parameters */
	position = BER_ExpandItem(position, end - position, &field);
	if (!position) 
		return ISL_FAIL;
	if (algorithmParameters)
	{
		algorithmParameters->Data = field.Tag;
		algorithmParameters->Length = position - field.Tag;
	}
	/* pick out key block from bitstring */
	position = BER_ExpandItem(end, endkey - end, &field);
	if (field.Tag[0] != BER_BIT_STRING)
		return ISL_FAIL;
	if (keyBlock)
	{
		keyBlock->Data = field.Content + 1;
		keyBlock->Length = field.ContentLength - 1;
	}
	return ISL_OK;
}

STATIC
ISL_STATUS
isl_ParseKeyBlock(
	ISL_KEY_BLOCK		keyBlock,
	ISL_CONST_DATA_PTR	algorithmParameters,
	EICLData				dssKey[])
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0c);
	/* handle parameter and non-parameter DSA key blocks */
	if (algorithmParameters && algorithmParameters->Data[0] == (BER_SEQUENCE | BER_CONSTRUCTED)) {
		if (BER_Decode((uint8*) algorithmParameters->Data, 
				algorithmParameters->Length, dssKey, 3) != 3)
			return ISL_FAIL;
		dssKey[3].length = BER_BerToVeryUnLong((uint8 *)keyBlock.Data, dssKey[3].value, 4*MODULUS);
		if (dssKey[3].length == 0)	
			return ISL_FAIL;
	} else {
		if (BER_Decode((uint8 *)keyBlock.Data, keyBlock.Length, dssKey, 4) != 4)
			return ISL_FAIL;
	}

	return ISL_OK;
}

STATIC
ISL_STATUS 
isl_X509Cert_Verify(ISL_CERTIFICATE *Context, ISL_SIGNATURE_CONFIG *Config, ISL_KEY_BLOCK *Key)
{
	BER_PARSED_ITEM field;
	const uint8 *tmp, *position;
	uint8 *end;
	ISL_CONST_DATA algorithmID, algorithmParameters, signature;
	ISL_KEY_BLOCK keyBlock;
#ifdef EISL
	uint8	work[6][MODULUS*4];
	EICLData rs[2], dssKey[4];
#else
	ISL_ERROR_CODE err;
	ISL_SIGN_VERIFY_METHODS *methods;
#endif

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0d);

	if (!Context) return ISL_FAIL;

#ifdef EISL
if ( Config || !Key)
#else
if (!Config || !Key )
#endif
	{
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_VERIFY);
		sprintf(gErrBuf, gErrStr, Context, Config, Key);
		isl_CertReportError(Context, ISL_INVALID_PARAMETER, gErrBuf);
#endif
		return ISL_FAIL;
	}

	if (isl_ParseKey(Key, &algorithmID, &algorithmParameters, &keyBlock) != ISL_OK)
		return ISL_FAIL;

	/* verify the certificate proper with the key block */
	position = (uint8*) Context->certificateInfo.Data + Context->certificateInfo.Length;
	end = (uint8*) Context->theCert.Data + Context->theCert.Length;
	tmp = BER_ExpandItem(position, end - position, &field); /* skip algorithm ID */
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context,position, end - position, &field);
		return ISL_FAIL;
	}
	position = tmp;
	tmp = BER_ExpandItem(position, end - position, &field); /* pick up signature */
	if (!tmp) {
		REPORT_CER_BER_EXPAND_ITEM(Context,position, end - position, &field);
		return ISL_FAIL;
	}		
	signature.Data = field.Content + 1;
	signature.Length = field.ContentLength - 1;


#ifdef EISL
	rs[0].value = &work[0][0];		rs[0].length = MODULUS * 4;
	rs[1].value = &work[1][0];		rs[1].length = MODULUS * 4;

	dssKey[0].value = &work[2][0];	dssKey[0].length = MODULUS * 4;
	dssKey[1].value = &work[3][0];	dssKey[1].length = MODULUS * 4;
	dssKey[2].value = &work[4][0];	dssKey[2].length = MODULUS * 4;
	dssKey[3].value = &work[5][0];	dssKey[3].length = MODULUS * 4;

	if (BER_Decode((uint8*) signature.Data, signature.Length, rs, 2) != 2) {
		return ISL_FAIL;
	}
	if (isl_ParseKeyBlock(keyBlock, &algorithmParameters, dssKey) != ISL_OK)
		return ISL_FAIL;
	
	if (EICL_DSSVerify((EICLData*)&Context->certificateInfo, 
			&rs[0], &rs[1], (EICLPublicKey *) dssKey) != 0) 
	{ 
		return ISL_FAIL;
	}
	return ISL_OK;
#else
	methods = (ISL_SIGN_VERIFY_METHODS*)
		(*PKCS7SignatureConfigMethods.FindAlgorithm)(Config, algorithmID);
	if (!methods) {
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_SIGNATURE_CONFIG_FIND_ALGORITHM);
		sprintf(gErrBuf, gErrStr, Config, algorithmID);
		isl_CertReportError(Context, ISL_ALGORITHM_NOT_FOUND, gErrBuf);
#endif
		return ISL_FAIL;
	}
	err = (*methods->Verify)(algorithmParameters,
		&keyBlock, 
		signature,
		NULL,					/* DigestMethods are filled in by combination OID */
		algorithmParameters,	/* digest parameters, too  */
		&getCSSMDataMethods,	/* immediate data specified by ISL_CONST_DATA */
		Context->certificateInfo);
	if (err != ISL_NO_ERROR) 
	{
#if (!defined(EISL) || defined(DEBUG))
		gErrStr = isl_GetErrorStr(ISL_CERT_VERIFY);
		sprintf(gErrBuf, gErrStr, Context, Config, Key);
		isl_CertReportError(Context, ISL_INVALID_CERTIFICATE, gErrBuf);
#endif
		return ISL_FAIL;
	}
	return ISL_OK;
#endif
}

typedef struct isl_cert_attr_iterator {
	ISL_CERTIFICATE *Cert;
	uint32 NodeCount;
} ISL_CERT_ATTR_ITERATOR;


STATIC
ISL_ITERATOR_PTR 
EISL_CreateCertificateAttributeEnumerator(ISL_CERTIFICATE *Context)		/* pointer to certificate */
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0e);

	if (!Context) return NULL;

#if (!defined(EISL) || defined(DEBUG))
	gErrStr = isl_GetErrorStr(ISL_CERT_CREATE_ATTRIBUTE_ENUM);
	sprintf(gErrBuf, gErrStr, Context);
	isl_CertReportError(Context, ISL_UNIMPLEMENTED, gErrBuf);
#endif
	return NULL;
}

STATIC
ISL_STATUS
EISL_GetNextCertificateAttribute(ISL_ITERATOR_PTR Context,
	ISL_CONST_DATA_PTR Attribute,
	ISL_CONST_DATA_PTR Value)
{
	ISL_CERT_ATTR_ITERATOR *Iterator; 
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x0f);
	if (!Context) return ISL_FAIL;
	Iterator = (ISL_CERT_ATTR_ITERATOR *) Context;
	memset(Attribute, 0, sizeof(ISL_CONST_DATA));
	memset(Value, 0, sizeof(ISL_CONST_DATA));
#if (!defined(EISL) || defined(DEBUG))
	gErrStr = isl_GetErrorStr(ISL_CERT_GET_NEXT_ATTRIBUTE);
	sprintf(gErrBuf, gErrStr, Context);
	isl_CertReportError(Iterator->Cert, ISL_UNIMPLEMENTED, gErrBuf);
#endif
	return ISL_FAIL;
}

STATIC
ISL_STATUS
EISL_RecycleCertificateAttributeEnumerator(ISL_ITERATOR_PTR Context)
{
	ISL_CERT_ATTR_ITERATOR *Iterator; 

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x10);

    if (!Context) return ISL_FAIL;
	Iterator = (ISL_CERT_ATTR_ITERATOR *) Context;
#if (!defined(EISL) || defined(DEBUG))
	gErrStr = isl_GetErrorStr(ISL_CERT_RECYCLE_ATTRIBUTE_ENUM);
	sprintf(gErrBuf, gErrStr, Context);
	isl_CertReportError(Iterator->Cert, ISL_UNIMPLEMENTED, gErrBuf);
#endif
	return ISL_FAIL;
}

//#define NUM_CERT_ATTRIBUTES 8
#ifdef EISL
ISL_STATUS 
EISL_FindCertificateAttribute(
	ISL_VERIFIED_CERTIFICATE_PTR Context,
	ISL_CONST_DATA Name,
	ISL_CONST_DATA_PTR Value)
{
	ISL_ITERATOR_PTR	pIterator = NULL;
	ISL_CONST_DATA		testName = {0, NULL}, testValue = {0, NULL};

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_X509CERT_TAG,0x11);
	pIterator = EISL_CreateCertificateAttributeEnumerator(Context);
	if (!pIterator)
		return ISL_FAIL;

	while (ISL_OK == EISL_GetNextCertificateAttribute(pIterator, &testName, &testValue))
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_CERT_TAG,0x01);

		if ( IS_EQUAL(Name, testName) )
		{
			memcpy(Value, &testValue, sizeof(ISL_CONST_DATA));
			EISL_RecycleCertificateAttributeEnumerator(pIterator);
			return ISL_OK;
		}
	}

	EISL_RecycleCertificateAttributeEnumerator(pIterator);
	return ISL_OK;
}
#endif

#ifndef EISL
ISL_CERTIFICATE_METHODS X509CertMethods = {
	isl_X509Cert_SizeofObject,
	isl_X509Cert_Initialize,
	isl_X509Cert_Flatten,
	isl_X509Cert_Recycle,
	isl_X509Cert_GetID,
	isl_X509Cert_GetKey,
	isl_X509Cert_GetSubject,
	isl_X509Cert_GetIssuer,
	isl_X509Cert_GetAlgID,
	isl_X509Cert_Verify,
	EISL_CreateCertificateAttributeEnumerator,
	EISL_GetNextCertificateAttribute,
	EISL_RecycleCertificateAttributeEnumerator
};
ISL_PARSE_CERT_METHODS X509ParseCertMethods = {
	{ X509id },
	{ isl_X509Cert_SizeofObject,
		isl_X509Cert_Initialize,
		isl_X509Cert_Flatten,
		isl_X509Cert_Recycle,
		isl_X509Cert_GetID,
		isl_X509Cert_GetKey,
		isl_X509Cert_GetSubject,
		isl_X509Cert_GetIssuer,
		isl_X509Cert_GetAlgID,
		isl_X509Cert_Verify,
		EISL_CreateCertificateAttributeEnumerator,
		EISL_GetNextCertificateAttribute,
		EISL_RecycleCertificateAttributeEnumerator
	}
};
#endif
