/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: module.c,v $
 * Revision 1.1.2.4  2000/08/31  18:06:23  Thomas_Woodburn
 *     Merged in Intel CDSA Version 2.0 Release 3.11.
 *
 * Revision 1.1.2.3  2000/08/07  14:42:09  Thomas_Woodburn
 *     Merged in Intel CDSA Version 2.0 Release 3.1.
 *
 * Revision 1.1.2.2  2000/06/28  21:00:07  Thomas_Woodburn
 *     Initial port to Tru64 UNIX.
 *
 * Revision 1.1.2.1  2000/06/28  20:25:21  Thomas_Woodburn
 *     Created from Intel CDSA Version 2.0 Release 3.0.
 *
 * $EndLog$
 */
#ifndef WIN32
#ifndef VMS
#pragma ident "@(#)$RCSfile: module.c,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:06:23 $"
#endif
#endif
/*-----------------------------------------------------------------------
 * File: MODULE.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

/*
 * Crendential and Attribute Verification Services
 *
 * EISL_VerifyAndLoadModuleAndCredentials
 * EISL_VerifyLoadedModuleAndCredentials
 * EISL_RecycleVerifiedModuleCredentials
 */
#include <sys/stat.h>
#include "eislapi.h"
#include "isl_internal.h"
#include <isl_memtrack.h>
#include "callouts.h"
#ifndef VMS
#define PATH_SEPARATOR    ';'
#endif
#define    MAX_CERTS 5

/*
 * isl_InitArchiveContextFromCredentials
 * 
 * This function builds a ISL_SIGNATURE_CONTEXT from an external
 * representation including the ISL_ARCHIVE_CONTEXT and returns
 * a pointer the to the ISL_SIGNATURE_CONTEXT
 */
ISL_STATUS
isl_InitArchiveContextFromCredentials(
    ISL_ARCHIVE_CONTEXT_PTR pArchive, 
    ISL_CONST_DATA Credentials)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_ARCHIVE_TAG,0x05);
    ASSERT(pArchive);
    
    return Archive_InitializeFromPath (
                                    pArchive,
                                    NULL,
                                    Credentials);
}

ISL_STATUS
isl_InitArchiveContextFromCredentialsBuffer(
    ISL_ARCHIVE_CONTEXT_PTR pArchive, 
    ISL_CONST_DATA Credentials,
    ISL_CONST_DATA ModuleSearchPath)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_ARCHIVE_TAG,0x05);
    ASSERT(pArchive);

    return Archive_InitializeFromMemory (
                                    pArchive,
                                    NULL,
                                    Credentials,
                                    ModuleSearchPath);
}

static
ISL_MANIFEST_SECTION_PTR
InitModuleVerification(
    const ISL_ARCHIVE_CONTEXT_PTR Context,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Issuer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_VERIFIED_CERTIFICATE_CHAIN_PTR    CertChain;
    ISL_VERIFIED_CERTIFICATE_PTR        SigCert;
    ISL_VERIFIED_SIGNATURE_ROOT_PTR        SigRoot;
    ISL_MANIFEST_SECTION_PTR            ManSect;
    uint32 NumCerts = 0;
    ISL_VERIFIED_CERTIFICATE_PTR        Certs[MAX_CERTS];
    ISL_SIGNER_CONTEXT_PTR pSignerContext = NULL;
    ISL_SIGNATURE_LIST_PTR pSigList = NULL;
    uint32 found = 0;
    ISL_CONST_DATA RootPublicKey;
    uint8 szPublicKey[1024];

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x02);

    CertChain = isl_AllocateMemory(Context, sizeof(ISL_VERIFIED_CERTIFICATE_CHAIN));
    if (!CertChain) {
        return NULL;
    }

    memset(CertChain, 0, sizeof(ISL_VERIFIED_CERTIFICATE_CHAIN));
    memset (&RootPublicKey, 0, sizeof (ISL_CONST_DATA));

    /*
     * We should verify the PublicKey, in case specfied by the user.
     */
    RootPublicKey.Length = 1024;
    RootPublicKey.Data = szPublicKey;
    EISL_RetrieveRootIssuerKey((ISL_CALLOUT_DATA*)&RootPublicKey);

    /*
     * If user has not specified rootKey, then it should be Signer's key.
     * Verify it.
     */
    if ((PublicKey.Data != NULL) && (memcmp(RootPublicKey.Data, PublicKey.Data, PublicKey.Length)))
    {
        SigRoot = isl_CreateVerifiedSignatureRoot(Context, Signer, PublicKey);
    
        if (SigRoot == NULL)
            goto FAIL;
    }

    pSigList = Context->Signatures;
    if (!Signer.Data) {
        while (pSigList) {
            if (!pSigList->Signature || !pSigList->Signature->Signers)
                break;

            pSignerContext = pSigList->Signature->Signers;
            while (pSignerContext) {
                if (isl_CreateCertificateChain(CertChain, Context, 
                            pSignerContext->SignerID, Issuer, RootPublicKey) == ISL_OK){
                    found = 1;
                    break;
                }
                pSignerContext = pSignerContext->Next;
            }
            pSigList = pSigList->Next;
        }
    } else {
        /*
         * If signer id is specified, match the signer id.
         */
        while (pSigList) {
            if (!pSigList->Signature || !pSigList->Signature->Signers)
                break;

            pSignerContext = pSigList->Signature->Signers;
            while (pSignerContext) {
                if (IS_EQUAL(pSignerContext->SignerID, Signer)) {
                    if (isl_CreateCertificateChain(CertChain, Context, 
                                Signer,    Issuer, RootPublicKey) == ISL_OK){
                        found = 1;
                        break;
                    }                                            
                }
                pSignerContext = pSignerContext->Next;
            }
            pSigList = pSigList->Next;
        }
    }

    if (found == 0)
        goto FAIL;

    NumCerts = EISL_CopyCertificateChain(CertChain, Certs, MAX_CERTS);
    if (NumCerts > 0)
    {
        SigCert = Certs[NumCerts-1];
        SigRoot = isl_CreateVerifiedSignatureRootWithCertificate(Context, SigCert);
    }
    else
    {
        SigRoot = isl_CreateVerifiedSignatureRoot(Context, Signer, PublicKey);
    } 
    if (SigRoot == NULL)
        goto FAIL;
    SigRoot->CertChain = CertChain;

    ManSect = EISL_FindManifestSection(SigRoot, SectionName);
    if (ManSect == NULL) {
        goto FAIL;
    }

    return ManSect;

FAIL:
    (void) isl_RecycleArchiveContext(Context);
    free(Context);
    return NULL;
}


ISL_STATUS
isl_GetPublicKeyAndIssuerFromCertificate (
    const ISL_CONST_DATA Certificate,
    ISL_CONST_DATA_PTR Issuer,
    ISL_CONST_DATA_PTR PublicKey,
    ISL_CERTIFICATE_PTR theCert)
{
    memset(PublicKey, 0, sizeof(ISL_CONST_DATA));
    memset(Issuer, 0, sizeof(ISL_CONST_DATA));

    if (ISL_OK != isl_X509Cert_Initialize(
                    theCert, 
                    NULL, 
                    NULL, 
                    Certificate))
    {
        isl_X509Cert_Recycle(theCert);
        return ISL_FAIL;
    }

    if (ISL_OK != isl_X509Cert_GetKey(
                    theCert,
                    PublicKey))
    {
        isl_X509Cert_Recycle(theCert);
        return ISL_FAIL;
    }
    if (ISL_OK != isl_X509Cert_GetSubject(
                    theCert,
                    Issuer))
    {
        isl_X509Cert_Recycle(theCert);
        return ISL_FAIL;
    }

    return ISL_OK;
}

#ifdef _IBM
/*
 * EISL_VerifyDataAndCredentials
 *
 * This function verifies the signature specified by the full file system path name 
 * Credentials using the public key. If the signature has more than one signer, 
 * the Signer parameter selects the appropriate signer. It then verifies the object 
 * referenced by the manifest section with the specified section.  If successful, 
 * a verified module object pointer is returned. Otherwise, NULL is returned. 
 *
 * Certificates embedded in the PKCS#7 signature as well as free-standing X509 
 * certificates in the credentials directory may be used in the certificate chain.
 */

ISL_VERIFIED_MODULE_PTR 
EISL_VerifyDataAndCredentials(
    const ISL_CONST_DATA Credentials,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA PublicKey)
{

    ISL_VERIFIED_SIGNATURE_ROOT_PTR SigRoot = NULL;
    ISL_MANIFEST_SECTION_PTR        ManSect = NULL;
    ISL_VERIFIED_MODULE_PTR            Module  = NULL;
    ISL_VERIFIED_CERTIFICATE_PTR    SigCert = NULL;
    uint32 NumCerts = 0;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x00);
    /* This code is awaiting certificate library coding */
    ISL_VERIFIED_CERTIFICATE_CHAIN_PTR    CertChain = NULL;
    ISL_VERIFIED_CERTIFICATE_PTR    Certs[MAX_CERTS];

    CertChain = EISL_CreateCertificateChain(Signer,PublicKey,Credentials);
    if (CertChain == NULL)
        return NULL;

    NumCerts = EISL_CopyCertificateChain(CertChain, Certs, MAX_CERTS);
    if (NumCerts <= 0)
        return NULL;

    SigCert = Certs[NumCerts-1];
    SigRoot = EISL_CreateVerifiedSignatureRootWithCertificate(Credentials, SigCert);
    if (SigRoot == NULL)
        return NULL;

    SigRoot->CertChain = CertChain;

    ManSect = EISL_FindManifestSection(SigRoot,SectionName);
    if (ManSect == NULL)
        return NULL;

    Module = EISL_VerifyData(ManSect);
    return (Module);
}
#endif


static
ISL_VERIFIED_MODULE_PTR
VerifyAndLoadModuleAndCredentials(
    const ISL_ARCHIVE_CONTEXT_PTR pArchive,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Issuer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_MANIFEST_SECTION_PTR    ManSect;
    ISL_VERIFIED_MODULE_PTR        Module;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x01);

    
    if ((ManSect = InitModuleVerification (
                        pArchive,
                        SectionName,
                        Signer,
                        Issuer,
                        PublicKey)) == NULL)
        return NULL;

    Module = EISL_VerifyAndLoadModule(ManSect);
    return (Module);
}

/*
 * EISL_VerifyAndLoadModuleAndCredentials
 *
 * This function verifies the signature specified by the full file system path name 
 * Credentials using the public key. If the signature has more than one signer, 
 * the Signer parameter selects the appropriate signer. It then verifies the object 
 * referenced by the manifest section with the specified section.  If successful, 
 * a verified module object pointer is returned. Otherwise, NULL is returned. 
 *
 * If the module is not already loaded, the object code is verified as an object 
 * module object using filesystem reads to obtain the image without loading it. 
 * If verified, the module is loaded.
 *
 * If the module is already loaded, it is verified in memory. 
 *
 * Certificates embedded in the PKCS#7 signature as well as free-standing X509 
 * certificates in the credentials directory may be used in the certificate chain.
 * 
 * This function combines many smaller functions into one call for a common case.  
 * If greater flexibility is needed, a series of calls that includes
 *
 * EISL_CreateCertificateChain
 * EISL_CopyCertificateChain 
 * EISL_CreateSignatureRootWithCertificate
 * EISL_FindManifestSection 
 * EISL_VerifyAndLoadModule 
 *
 * provides the same functionality. 
 */

ISL_VERIFIED_MODULE_PTR 
EISL_VerifyAndLoadModuleAndCredentialsWithCertificate(
    const ISL_CONST_DATA Credentials,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Certificate)
{
    ISL_CERTIFICATE theCert;
    ISL_CONST_DATA PublicKey;
    ISL_CONST_DATA Issuer;
    ISL_VERIFIED_MODULE_PTR pModule;
    ISL_ARCHIVE_CONTEXT_PTR pArchive;

    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));
    memset(&PublicKey, 0, sizeof(ISL_CONST_DATA));
    if (Certificate.Data != NULL) {
        if (ISL_OK != isl_GetPublicKeyAndIssuerFromCertificate (
                                Certificate, &Issuer, &PublicKey, &theCert))
            return NULL;
    }

        
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));

    if (ISL_OK != isl_InitArchiveContextFromCredentials(pArchive, Credentials))
        return NULL;

    pModule = VerifyAndLoadModuleAndCredentials(
                    pArchive,
                    SectionName,
                    Signer,
                    Issuer,
                    PublicKey);

    if (Certificate.Data != NULL)
        isl_X509Cert_Recycle(&theCert);
    return pModule;
}

ISL_VERIFIED_MODULE_PTR 
EISL_VerifyAndLoadModuleAndCredentialDataWithCertificate(
    const ISL_CONST_DATA CredentialsImage,
    const ISL_CONST_DATA ModuleSearchPath,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Certificate)
{
    ISL_CERTIFICATE theCert;
    ISL_CONST_DATA PublicKey;
    ISL_CONST_DATA Issuer;
    ISL_VERIFIED_MODULE_PTR pModule;
    ISL_ARCHIVE_CONTEXT_PTR pArchive;

    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));
    memset(&PublicKey, 0, sizeof(ISL_CONST_DATA));
    if (Certificate.Data != NULL) {
        if (ISL_OK != isl_GetPublicKeyAndIssuerFromCertificate (
                                Certificate, &Issuer, &PublicKey, &theCert))
            return NULL;
    }

        
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));

    if (ISL_OK != isl_InitArchiveContextFromCredentialsBuffer(pArchive, CredentialsImage,
                                                    ModuleSearchPath))
        return NULL;

    pModule = VerifyAndLoadModuleAndCredentials(
                    pArchive,
                    SectionName,
                    Signer,
                    Issuer,
                    PublicKey);

    if (Certificate.Data != NULL)
        isl_X509Cert_Recycle(&theCert);
    return pModule;
}

ISL_VERIFIED_MODULE_PTR 
EISL_VerifyAndLoadModuleAndCredentials(
    const ISL_CONST_DATA Credentials,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_ARCHIVE_CONTEXT_PTR pArchive;
    ISL_CONST_DATA Issuer;
    
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));
    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));

    if (ISL_OK != isl_InitArchiveContextFromCredentials(pArchive, Credentials))
        return NULL;

    return(
        VerifyAndLoadModuleAndCredentials(
            pArchive,
            SectionName,
            Signer,
            Issuer,
            PublicKey));
}

/*
 * The following API accepts manifest in a buffer and manifest path 
 * as parameters instead of manifest file.
 */ 
ISL_VERIFIED_MODULE_PTR 
EISL_VerifyAndLoadModuleAndCredentialData(
    const ISL_CONST_DATA CredentialsImage,
    const ISL_CONST_DATA ModuleSearchPath,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_ARCHIVE_CONTEXT_PTR pArchive;
    ISL_CONST_DATA Issuer;
    
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));
    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));

    if (ISL_OK != isl_InitArchiveContextFromCredentialsBuffer(pArchive, CredentialsImage,
                                                    ModuleSearchPath))
        return NULL;

    return(
        VerifyAndLoadModuleAndCredentials(
            pArchive,
            SectionName,
            Signer,
            Issuer,
            PublicKey));
}


static
ISL_VERIFIED_MODULE_PTR
VerifyLoadedModuleAndCredentials(
    const ISL_ARCHIVE_CONTEXT_PTR pArchive,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Issuer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_MANIFEST_SECTION_PTR    ManSect;
    ISL_VERIFIED_MODULE_PTR        Module;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x02);

    if ((ManSect = InitModuleVerification (
                        pArchive,
                        SectionName,
                        Signer,
                        Issuer,
                        PublicKey)) == NULL)
        return NULL;

    Module = EISL_VerifyLoadedModule(ManSect);
    return (Module);
}


ISL_VERIFIED_MODULE_PTR
EISL_VerifyLoadedModuleAndCredentialsWithCertificate(
    const ISL_CONST_DATA Credentials,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Certificate)
{
    ISL_CERTIFICATE theCert;
    ISL_CONST_DATA PublicKey;
    ISL_CONST_DATA Issuer;
    ISL_VERIFIED_MODULE_PTR pModule;
    ISL_ARCHIVE_CONTEXT_PTR pArchive;


    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));
    memset(&PublicKey, 0, sizeof(ISL_CONST_DATA));
    if (Certificate.Data != NULL) {
        if (ISL_OK != isl_GetPublicKeyAndIssuerFromCertificate (
                                Certificate, &Issuer, &PublicKey, &theCert))
            return NULL;
    }

    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));

    if (ISL_OK != isl_InitArchiveContextFromCredentials(pArchive, Credentials))
        return NULL;


    pModule = VerifyLoadedModuleAndCredentials(
                            pArchive,
                            SectionName,
                            Signer,
                            Issuer,
                            PublicKey);

    if (Certificate.Data != NULL)
        isl_X509Cert_Recycle(&theCert);

    return pModule;
}

ISL_VERIFIED_MODULE_PTR
EISL_VerifyLoadedModuleAndCredentialDataWithCertificate(
    const ISL_CONST_DATA CredentialsImage,
    const ISL_CONST_DATA ModuleSearchPath,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA Certificate)
{
    ISL_CERTIFICATE theCert;
    ISL_CONST_DATA PublicKey;
    ISL_CONST_DATA Issuer;
    ISL_VERIFIED_MODULE_PTR pModule;
    ISL_ARCHIVE_CONTEXT_PTR pArchive;

    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));
    memset(&PublicKey, 0, sizeof(ISL_CONST_DATA));
    if (Certificate.Data != NULL) {
        if (ISL_OK != isl_GetPublicKeyAndIssuerFromCertificate (
                                Certificate, &Issuer, &PublicKey, &theCert))
            return NULL;
    }

        
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));

    
    if (ISL_OK != isl_InitArchiveContextFromCredentialsBuffer(pArchive, CredentialsImage,
                                                    ModuleSearchPath))
        return NULL;

    pModule = VerifyLoadedModuleAndCredentials(
                            pArchive,
                            SectionName,
                            Signer,
                            Issuer,
                            PublicKey);

    if (Certificate.Data != NULL)
        isl_X509Cert_Recycle(&theCert);
    return pModule;
}
    
/*
 * EISL_VerifyLoadedModuleAndCredentials
 *
 * This function verifies the signature of the manifest section corresponding 
 * to the object module using the public key.  If successful, 
 * a verified module object pointer is returned. Otherwise, NULL is returned.
 *
 * The object code is verified using the loaded image in memory.  
 *
 * Certificates embedded in the PKCS#7 signature as well as free-standing 
 * X509 certificates in the credentials directory may be used in the certificate chain.
 *
 * This function combines many smaller functions into one call for a common case.  
 * If greater flexibility is needed, a series of calls that includes 
 *
 * EISL_CreateCertificateChain 
 * EISL_CopyCertificateChain 
 * EISL_CreateSignatureRootWithCertificate 
 * EISL_FindManifestSection
 * EISL_VerifyLoadedModule 
 * 
 * provides the same functionality.  
 *
 * Cleanup is done by ISL_RecycleModuleCredentials.
 */

ISL_VERIFIED_MODULE_PTR
EISL_VerifyLoadedModuleAndCredentials(
    const ISL_CONST_DATA Credentials,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_ARCHIVE_CONTEXT_PTR pArchive;
    ISL_CONST_DATA Issuer;
    
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));
    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));

    if (ISL_OK != isl_InitArchiveContextFromCredentials(pArchive, Credentials))
        return NULL;


    return(
        VerifyLoadedModuleAndCredentials(
            pArchive,
            SectionName,
            Signer,
            Issuer,
            PublicKey));
}

/*
 * The following API accepts manifest in a buffer and manifest path 
 * as parameters instead of manifest file.
 */ 
ISL_VERIFIED_MODULE_PTR 
EISL_VerifyLoadedModuleAndCredentialData(
    const ISL_CONST_DATA CredentialsImage,
    const ISL_CONST_DATA ModuleSearchPath,
    const ISL_CONST_DATA SectionName,
    const ISL_CONST_DATA Signer,
    const ISL_CONST_DATA PublicKey)
{
    ISL_ARCHIVE_CONTEXT_PTR pArchive;
    ISL_CONST_DATA Issuer;
    
    pArchive = malloc(sizeof(ISL_ARCHIVE_CONTEXT));
    if (NULL == pArchive)
        return NULL;
    memset(pArchive, 0, sizeof(ISL_ARCHIVE_CONTEXT));
    memset(&Issuer, 0, sizeof(ISL_CONST_DATA));

    
    if (ISL_OK != isl_InitArchiveContextFromCredentialsBuffer(pArchive, CredentialsImage,
                                                    ModuleSearchPath))
        return NULL;

    return(
        VerifyLoadedModuleAndCredentials(
            pArchive,
            SectionName,
            Signer,
            Issuer,
            PublicKey));
}



/*
 * EISL_RecycleVerifiedModuleCredentials
 *
 * This function destroys and recycles the memory for the module verification object
 * as well as its containing Signature Root Object and Certificate Chain Object, 
 * and all subordinate objects.  It should be the last call which references the 
 * module verification object.  It should also follow any recycle calls for iterators 
 * and certificate objects.  All pointers to certificates, manifest sections, iterators, 
 * and the information returned by iterators are invalid after this call is completed.
 */

ISL_STATUS 
EISL_RecycleVerifiedModuleCredentials(
    const ISL_VERIFIED_MODULE_PTR Verification)
{
#ifdef VMS /* taken from v3_13 code stream */
   ISL_STATUS status = ISL_OK;
   ISL_ARCHIVE_CONTEXT_PTR pArchive;
   struct isl_verify_continuation *pContinuation;
   ISL_SIGNER_CONTEXT_PTR pSigners;

#endif
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x03);

    if (Verification)
    {        
        if (0 == Verification->RefCount) {
            if (NULL == Verification->ManSect || NULL == Verification->ManSect->Parent ||
                NULL == Verification->ManSect->Parent->Signatures || 
                NULL == Verification->ManSect->Parent->Signatures->Signature)
                return ISL_FAIL;

#ifdef VMS /* taken from v3_13 code stream */

			pArchive = Verification->ManSect->Parent;

			if (Verification->Path.Data)
				isl_FreeMemory(pArchive, (void *) Verification->Path.Data);

			if (pContinuation = Verification->Continuation) {
				if (pContinuation->DigestContext)
					isl_FreeMemory(pArchive, pContinuation->DigestContext);

				if (pContinuation->GetDataContext) {
					if (isl_GetDllRecycleMemory(pContinuation->GetDataContext) != ISL_OK)
						status = ISL_FAIL;
					isl_FreeMemory(pArchive, pContinuation->GetDataContext);
				}

				isl_FreeMemory(pArchive, pContinuation);
			}

			/*
			 * Must free Verification before recycling pSigners
			 * because the recycling frees pArchive.
			 */
			pSigners = Verification->ManSect->Parent->Signatures->Signature->Signers;
			isl_FreeMemory(pArchive, Verification);
			if (EISL_RecycleVerifiedSignatureRoot(pSigners) != ISL_OK)
				status = ISL_FAIL;

			return status;
#else
            return EISL_RecycleVerifiedSignatureRoot(Verification->ManSect->Parent->Signatures->Signature->Signers);
#endif
        } else {
            Verification->RefCount--;
            return ISL_OK;
        }
    }

    return ISL_FAIL;
}


static uint8 *
GetModuleName (ISL_CONST_DATA RelativeName)
{
    uint8 *ptr = NULL;
    uint8 *ModuleName = NULL;
    uint32 NameLength = 0;

    /*
     * Strip off the releative address stuff
     */
    ptr = (unsigned char *)(RelativeName.Data + RelativeName.Length);
    while ((*ptr != '/' || *ptr != '\\') && (ptr != RelativeName.Data))
        ptr--;

    /* Skip '/' or '\\' */
    if (ptr != RelativeName.Data)    
        ptr++;
        
    NameLength = RelativeName.Length - (ptr - RelativeName.Data);
    if ((ModuleName = (unsigned char *) calloc (1, NameLength + 1)) == NULL)
        return NULL;
    
    strncpy ((char *)ModuleName, (char *)ptr, NameLength);
    return ModuleName;
}


/*
 * SetModulePath
 */

static ISL_STATUS
SetModulePath(
    ISL_MODULE_PTR pModule)
{
    ISL_ARCHIVE_CONTEXT_PTR        pArchive = NULL;
    ISL_MANIFEST_SECTION_PTR    pManSect = NULL;
    unsigned char        *ptr     = NULL;
    unsigned char        *parameterPtr         = NULL;
    unsigned long        parameterLength     = 0;
    ISL_SECTION_PTR SectionInfo;
    ISL_CONST_DATA    Path;
    struct _stat st;
    const uint8 *SearchPtr;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x04);
    ASSERT(pModule);
    
    pManSect = pModule->ManSect;
    pArchive = pManSect->Parent;
    SectionInfo = &pManSect->SectionInfo;
    pModule->Path.Length = pArchive->SearchPath.Length + SectionInfo->Name.Length + 2;
    
    ptr = isl_AllocateMemory(pArchive, sizeof(uint8)*pModule->Path.Length);
    if (NULL == ptr) 
        return ISL_FAIL;

    /*
     * Separate the protocol and parameter.
     */
    parameterPtr = memchr(SectionInfo->Name.Data,        /* find protocol separator */
                    ':', 
                    SectionInfo->Name.Length);
    if (NULL == parameterPtr) 
        return ISL_FAIL;

    /* We should really check to make sure the protocol is executable */
    /* FIX LATER */
    if (strncmp ((const char *)SectionInfo->Name.Data, "executable", 10))
        return ISL_FAIL;

    parameterPtr++;                                                /* skip over : */
    parameterLength = SectionInfo->Name.Length;                    /* length of Name */
    parameterLength -= parameterPtr - SectionInfo->Name.Data;    /* subtract off length of executable: */


    SearchPtr = pArchive->SearchPath.Data;
    Path.Data = pArchive->SearchPath.Data;
    Path.Length = pArchive->SearchPath.Length;
    while (SearchPtr) {
        SearchPtr = memchr (Path.Data, PATH_SEPARATOR, Path.Length);
        if (SearchPtr != NULL) {
            Path.Length = SearchPtr - Path.Data;
            SearchPtr++;            /* Skip the PATH_SEPARATOR */
        } else {    
            /*
             * get rid of the null character at the end of search path, if any.
             */
            if (Path.Data[Path.Length-1] == '\0')
                Path.Length--;
        }
        
        pModule->Path.Data = ptr;
        memcpy(ptr, Path.Data, Path.Length);
        ptr = ptr + Path.Length;
#ifdef COMPAQ_FIX
                if( ptr[-1] != DIRECTORY_SEPARATOR )                 /* look back */
                   *ptr++ = DIRECTORY_SEPARATOR;
#else  /* COMPAQ_FIX */
        *ptr++ = '\\';
#endif /* COMPAQ_FIX */
        memcpy(ptr, parameterPtr, parameterLength);          /* copy relative path */
        ptr = ptr + parameterLength;                         /* move to end of string */
        *ptr = '\0';                                         /* null terminate */
        pModule->Path.Length = ptr - pModule->Path.Data;
        /* 
         * stat the file and find out if we have found it.
         * If yes, then return else continue searching the
         * search path.
         */
        if ((_stat((const char *)pModule->Path.Data, &st)) == 0)
            return ISL_OK;
        Path.Data = SearchPtr;
        Path.Length = pArchive->SearchPath.Length - (SearchPtr - pArchive->SearchPath.Data);
    }
    return ISL_FAIL;
}

static ISL_STATUS
VerifyModuleCommon( ISL_MODULE_PTR    pModule,
                    uint32            Memory)
{
    ISL_GET_DATA_SERVICE_CONTEXT    *context = NULL;
    ISL_DATA                        data     = {0, NULL};
    EICLSHAState                    state;
    EICLSHADigest                   digest; 
    EICLData                        message  = {0, NULL};
    ISL_ALG_INFO_PTR                pNode    = NULL;
#ifdef TEST
    FILE                            *fp      =  NULL;
    ISL_DATA_PTR                    encoded  = NULL;
    ISL_DATA                        decoded  = {0, NULL};
#endif

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x09);

    if (NULL == pModule) {
#ifdef MEDUSA_DEBUG
        _asm int 3;
#endif
        return ISL_FAIL;
    }

    (void)EICL_SHABegin(&state);
    context = (ISL_GET_DATA_SERVICE_CONTEXT*)
              isl_AllocateMemory(    pModule->ManSect->Parent, 
                                    isl_GetDllSizeofObject());
    if (!context) {
#ifdef MEDUSA_DEBUG
        _asm int 3;
#endif
        REPORT_ALLOCATE_MEMORY_ERROR(pModule->ManSect->Parent, isl_GetDllSizeofObject());
        return ISL_FAIL;
    }

    if (Memory) {
#ifdef VMS
        if (isl_GetDllInitMemory(context,(void *)pModule->Path.Data) != ISL_OK) {
#else
        if (isl_GetDllInitMemory(context, pModule->hModule) != ISL_OK) {
#endif
#ifdef MEDUSA_DEBUG
            _asm int 3;
#endif
            return ISL_FAIL;
        }
    } else {
        if (isl_GetDllInit(context, pModule->Path) != ISL_OK)
            return ISL_FAIL;
    }

#ifdef TEST
    if (Memory) {
        fp = fopen("loaded.bin","wb");
        if (!fp)
            return ISL_FAIL;
    } else {
        fp = fopen("nloaded.bin","wb");
        if (!fp)
            return ISL_FAIL;
    }
#endif

    do {
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x10);
        if (Memory) {
            if (isl_GetDllUpdateMemory(context, (ISL_CONST_DATA_PTR)&data) != ISL_OK) {
#ifdef MEDUSA_DEBUG
                _asm int 3;
#endif    
                return ISL_FAIL;
            }
        } else {
            if (isl_GetDllUpdate(context, (ISL_CONST_DATA_PTR)&data) != ISL_OK)
                return ISL_FAIL;
        }

        if (data.Length != 0) {
            message.value = data.Data;
            message.length = data.Length;
#ifdef TEST
            if (fwrite(message.value, message.length, 1, fp) != 1)
                return ISL_FAIL;
#endif
            (void)EICL_SHAProcess(&message, &state);
        }
    } while (data.Length != 0); 
#ifdef TEST
    fclose(fp);
#endif
    (void)EICL_SHAEnd(&state, digest);

    if (Memory) {
        if (isl_GetDllRecycleMemory(context) != ISL_OK) {
#ifdef MEDUSA_DEBUG
            _asm int 3;
#endif    
            return ISL_FAIL;
        }
    } else {
        if (isl_GetDllRecycle(context) != ISL_OK)
            return ISL_FAIL;
    }

    pNode = isl_FindSHA(pModule->ManSect->SectionInfo.Algorithm);
    if (!pNode)
#ifdef MEDUSA_DEBUG
        _asm int 3;
#endif    
        return ISL_FAIL;

#ifdef TEST
    fp = fopen("sha.txt","wb");
    if (!fp)
        return ISL_FAIL;

    /* Calculated SHA Digest */
    decoded.Data = digest;
    decoded.Length = 20;
    encoded = IslUtil_Base64Encode(&decoded);
    fprintf(fp,"eisl:\t\t");
    if (fwrite(encoded->Data, encoded->Length, 1, fp) != 1)
        return ISL_FAIL;
    fprintf(fp,"\n");
    decoded.Data = pNode->Hash->Value.Data;
    decoded.Length = 20;

    encoded = IslUtil_Base64Encode(&decoded);
    fprintf(fp,"manifest:\t");
    if (fwrite(encoded->Data, encoded->Length, 1, fp) != 1)
        return ISL_FAIL;
    fprintf(fp,"\n");
    fclose(fp);
#endif

    if (pNode->Hash->Value.Length != 20 ||
        memcmp(pNode->Hash->Value.Data, digest, 20) != 0)
#ifdef MEDUSA_DEBUG
        _asm int 3;
#endif    
        return ISL_FAIL;

    return ISL_OK;
}


static ISL_STATUS
VerifyModule(
    ISL_MODULE_PTR pModule)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x07);
    return VerifyModuleCommon (pModule, 1);
}


static ISL_STATUS
VerifyModulePath(
    ISL_MODULE_PTR pModule)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MANIFEST_TAG,0x08);

    return VerifyModuleCommon (pModule, 0);
}

/*
 * EISL_VerifyAndLoadModule
 */
ISL_VERIFIED_MODULE_PTR 
EISL_VerifyAndLoadModule(
    const ISL_MANIFEST_SECTION_PTR Section)
{    
    ISL_ARCHIVE_CONTEXT_PTR        pArchive = NULL;
    ISL_VERIFIED_MODULE_PTR        pModule  = NULL;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x05);
    if (!Section)
        return NULL;

    pArchive = Section->Parent;
    ASSERT(pArchive);

    pModule = (ISL_VERIFIED_MODULE_PTR) isl_AllocateMemory(pArchive, sizeof (ISL_VERIFIED_MODULE));
    if (!pModule)
    {
        /* need to set error */
        return NULL;
    }
    pModule->RefCount = 0;
    pModule->Continuation = NULL;
    pModule->ManSect = Section;
    if (SetModulePath(pModule) != ISL_OK)
        return NULL;

    /* verify not loaded module */
    if (VerifyModulePath(pModule) != ISL_OK)
        return NULL;
    /* load module */
    pModule->hModule = isl_LoadLibraryEx(
#ifdef VMS
                                              (char *)(pModule->Path.Data),           /* path to module */
#else
                                              (const char *) (pModule->Path.Data),    /* path to module */
#endif
                        NULL,                                 /* required to be NULL */
                        0);                                   /* default behavior */
    
    if (!pModule->hModule)
    {
        /* need to set error */
        return NULL;
    }
    /* verify loaded module */
    if (VerifyModule(pModule) != ISL_OK)
    {
        (void)isl_FreeLibrary(pModule->hModule);
        return NULL;
    }
    return (pModule);
}

/*
 * EISL_VerifyLoadedModule
 */
ISL_VERIFIED_MODULE_PTR 
EISL_VerifyLoadedModule(
    const ISL_MANIFEST_SECTION_PTR Section)
{
    ISL_MODULE_PTR            pModule = NULL;
    ISL_ARCHIVE_CONTEXT_PTR pArchive = NULL;
    uint8    *ModuleName;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_MODULE_TAG,0x06);

    if (!Section)
        return NULL;

    pArchive = Section->Parent;
    pModule = (ISL_VERIFIED_MODULE_PTR) isl_AllocateMemory(pArchive,sizeof (ISL_VERIFIED_MODULE));
    if (!pModule)
        return NULL;
    pModule->RefCount = 0;
    pModule->Continuation = NULL;

    if ((ModuleName = GetModuleName(Section->Parameters)) == NULL)
        return NULL;

    pModule->hModule = isl_GetModuleHandle((LPCSTR)ModuleName);
    
    free (ModuleName);
    
    if (!pModule->hModule)
        return NULL;

    pModule->ManSect = Section;
    
    if (SetModulePath(pModule) != ISL_OK)
        return NULL;

    /* verify loaded module */
    if (VerifyModule(pModule) != ISL_OK)
        return NULL;
    return pModule;
}


ISL_VERIFIED_MODULE_PTR
EISL_DuplicateVerifiedModulePtr (                
                            ISL_VERIFIED_MODULE_PTR pModule)
{
    if (pModule)
        pModule->RefCount++;

    return pModule;
}
