/*
 * @DEC_COPYRIGHT@
 */
/*
 * OpenVMS History
 *
 *  29-Nov-2001	Carol Kelly
 *     - Implement CheckDataAddressWithinModule for OpenVMS
 *
 */
/*
 * HISTORY
 * $Log: seclink.c,v $
 * Revision 1.1.2.4  2000/08/31  18:06:29  Thomas_Woodburn
 *    Merged in Intel CDSA Version 2.0 Release 3.11.
 *
 * Revision 1.1.2.3  2000/08/07  14:42:16  Thomas_Woodburn
 *    Merged in Intel CDSA Version 2.0 Release 3.1.
 *
 * Revision 1.1.2.2  2000/06/28  21:00:12  Thomas_Woodburn
 *    Initial port to Tru64 UNIX.
 *
 * Revision 1.1.2.1  2000/06/28  20:25:29  Thomas_Woodburn
 *    Created from Intel CDSA Version 2.0 Release 3.0.
 *
 * $EndLog$
 */
#ifndef WIN32
#ifndef VMS
#pragma ident "@(#)$RCSfile: seclink.c,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:06:29 $"
#endif
#endif
/*-----------------------------------------------------------------------
 * File: SECLINK.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

/* 
 * Secure Linkage Methods
 *
 */

#include "eislapi.h"
#include "isl_internal.h"
#include <isl_memtrack.h>
#ifdef __digital__
#include "compaq/dprintf.h"
#endif

#define CHECK_ADDRESS      1
#define CHECK_DATAADDRESS   2

/*
 * EISL_LocateProcedureAddress
 *
 * This function returns the address of a function whose name is given in the loaded module.  
 * The address may be checked prior to calling the function to make sure that the address 
 * is actually a valid address within a verified object code module by 
 * EISL_CheckAddressWithinModule.
 */

ISL_FUNCTION_PTR
EISL_LocateProcedureAddress(
   ISL_VERIFIED_MODULE_PTR Module,
   ISL_CONST_DATA Name)
{
#ifdef COMPAQ_FIX
   CSSM_PROC_ADDR Address;
#endif /* COMPAQ_FIX */
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x00);
   if (!Module)
      return NULL;

#ifdef COMPAQ_FIX
   if (port_GetProcAddress(Module->hModule, (const char *)Name.Data,
            &Address) != CSSM_OK)
      return NULL;
   return (ISL_FUNCTION_PTR) Address;
#else  /* COMPAQ_FIX */
   return ( (ISL_FUNCTION_PTR) GetProcAddress(
                               Module->hModule,   // handle to DLL module  
                 (const char *)Name.Data          // name of function 
                        ) );
#endif /* COMPAQ_FIX */
}

/*
 * EISL_GetLibHandle
 */
void * 
EISL_GetLibHandle(
   ISL_VERIFIED_MODULE_PTR Module)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x01);
   if (!Module)
      return NULL;

   return (Module->hModule);
}


static ISL_STATUS 
CheckDataAddressWithinModule(
   ISL_VERIFIED_MODULE_PTR Verification,
   void * Address,
   int type)
#if defined (WIN32)
{
   unsigned char *         pImage = NULL;
   PIMAGE_NT_HEADERS      pNTHeader = NULL;
   PIMAGE_SECTION_HEADER   SecHead = NULL;
   uint32               SectionCount = 0;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x05);
   if (!Verification)
      return ISL_FAIL;

   if (!Address)
      return ISL_FAIL;

   pImage = Verification->hModule;
   if (!pImage)
      return ISL_FAIL;

   pNTHeader = isl_getNTHeader(pImage);
   if (!pNTHeader)
      return ISL_FAIL;

   SecHead = IMAGE_FIRST_SECTION(pNTHeader);
   if (!SecHead)
      return ISL_FAIL;

   for(SectionCount = 0;
      SectionCount< (DWORD) pNTHeader->FileHeader.NumberOfSections;
      SectionCount++, SecHead++)
   {   
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x06);

      /* code for deciding what gets digested */
      if ((type == CHECK_DATAADDRESS && IS_DATA_SECTION(SecHead))
            || (type == CHECK_ADDRESS && IS_DIGESTED_SECTION(SecHead)))
      {
         const unsigned char *start = NULL;
         const unsigned char *end   = NULL;

         start = pImage + SecHead->VirtualAddress;
         end   = start +  SecHead->Misc.VirtualSize;

         if ( ((const unsigned char *)Address >=   start) &&
             ((const unsigned char *)Address <=   end) )
             return ISL_OK;
      }
   }

   return ISL_FAIL;
}
#elif defined (VMS)
{

 char ImageName[40];
 char PathName[255];
 int ret;
 char *lastslash;
 char *extension;
 void *StartAddress;
 void *EndAddress;


    if (!Verification)
       return ISL_FAIL;

    if (!Address)
       return ISL_FAIL;

    strncpy( ImageName, (const char *)Verification->Path.Data, Verification->Path.Length );
    ImageName[Verification->Path.Length] = '\0';

    lastslash = strrchr( ImageName, '/' );
    extension = strrchr( ImageName, '.' );
    *extension = '\0';

    strcpy( ImageName, ++lastslash );

    ret = port_ComputeAddressInModule( ImageName, &StartAddress, &EndAddress );

    if( ret != ISL_OK )
       return ISL_FAIL;

    if( (StartAddress <= Address) && (Address <= EndAddress) ){
       return ISL_OK;
    }
    else return ISL_FAIL;
}
#elif defined (__digital__)

   DPRINTF("isl", "(CheckDataAddressWithinModule) Address = 0x%p\n", Address);
   return ISL_OK;
}
#endif

/*
 * EISL_CheckAddressWithinModule
 *
 * The address is checked against the list of address ranges that are 
 * valid addresses within the module.
 */
ISL_STATUS 
EISL_CheckAddressWithinModule(
   ISL_VERIFIED_MODULE_PTR Verification,
   ISL_FUNCTION_PTR Address)
{   
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x02);

   return CheckDataAddressWithinModule(
                  Verification,
                  (void *)Address,
                  CHECK_ADDRESS);
}


/*
 * EISL_CheckDataAddressWithinModule
 *
 * The address is checked against the list of address ranges that are 
 * valid writable addresses within the module.
 */
ISL_STATUS 
EISL_CheckDataAddressWithinModule(
   ISL_VERIFIED_MODULE_PTR Verification,
   void * Address)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x05);

   return CheckDataAddressWithinModule(
                  Verification,
                  Address,
                  CHECK_DATAADDRESS);
}


/*
 * EISL_GetModuleManifestSection
 *
 * This function returns the manifest section that was used to verify the module.
 */

ISL_MANIFEST_SECTION_PTR 
EISL_GetModuleManifestSection(
   ISL_VERIFIED_MODULE_PTR Module)
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_SECLINK_TAG,0x04);
   if (!Module)
      return NULL;

   return (Module->ManSect);
}
