/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: util.c,v $
 * Revision 1.1.2.4  2000/08/31  18:06:34  Thomas_Woodburn
 * 	Merged in Intel CDSA Version 2.0 Release 3.11.
 *
 * Revision 1.1.2.3  2000/08/07  14:42:20  Thomas_Woodburn
 * 	Merged in Intel CDSA Version 2.0 Release 3.1.
 *
 * Revision 1.1.2.2  2000/06/28  21:00:14  Thomas_Woodburn
 * 	Initial port to Tru64 UNIX.
 *
 * Revision 1.1.2.1  2000/06/28  20:25:36  Thomas_Woodburn
 * 	Created from Intel CDSA Version 2.0 Release 3.0.
 *
 * $EndLog$
 */
#ifndef WIN32
#ifndef VMS
#pragma ident "@(#)$RCSfile: util.c,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:06:34 $"
#endif
#endif
/*-----------------------------------------------------------------------
 * File: UTIL.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 

#include <stdio.h>
#include <sys/stat.h>
#include "isl_internal.h"
#include "islutil.h"
#include <isl_memtrack.h>

ISL_ALG_INFO_PTR
isl_FindSHA(
	ISL_ALG_INFO_PTR	pAlgList)
{
	ISL_ALG_INFO_PTR	pNode = NULL;
    uint8               sha1[8];
    uint8               sha2[8];
    uint8               sha3[8];
    uint32              sha1_len = 0;
    uint32              sha2_len = 0;
    uint32              sha3_len = 0;
    
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x01);
    sha1[sha1_len++]='S';
    sha1[sha1_len++]='H';
    sha1[sha1_len++]='A';

    sha2[sha2_len++]='S';
    sha2[sha2_len++]='H';
    sha2[sha2_len++]='A';
    sha2[sha2_len++]='1';

    sha3[sha3_len++]='S';
    sha3[sha3_len++]='H';
    sha3[sha3_len++]='A';
    sha3[sha3_len++]='-';
    sha3[sha3_len++]='1';

	for(pNode = pAlgList;
		pNode;
		pNode = pNode->Next)
	{
        NAMELOOP( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x02);
		if ( (	pNode->AlgName.Length == 3 && 
				memcmp(pNode->AlgName.Data,sha1,sha1_len) == 0) ||
			 (	pNode->AlgName.Length == 4 &&
				memcmp(pNode->AlgName.Data,sha2,sha2_len) == 0 ) ||
			 (  pNode->AlgName.Length == 5 &&
				memcmp(pNode->AlgName.Data,sha3,sha3_len) == 0))
			break;
	}
	return pNode;
}

ISL_STATUS
isl_ParseFilePath(
    ISL_CONST_DATA      FilePath,
    ISL_CONST_DATA_PTR  FileDir,
    ISL_CONST_DATA_PTR  FileName,
    ISL_CONST_DATA_PTR  FileTitle,
    ISL_CONST_DATA_PTR  FileExt)
{
	const uint8	*ptr;
	const uint8	*oldptr;
	const uint8 *endptr;
	const uint8	*pBS;
	const uint8	*pFS;
	uint32	ptrlen;

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x03);
	if (!FilePath.Data) return ISL_FAIL;

	ptr = FilePath.Data;
	oldptr = FilePath.Data;
	endptr = FilePath.Data + FilePath.Length;
	while (ptr)	
	{
		oldptr = ++ptr;
		ptrlen = endptr - ptr;
		pBS = memchr(oldptr, '\\', ptrlen);
		pFS = memchr(oldptr, '/', ptrlen);
		ptr = (pBS > pFS) ? pBS : pFS;
	}

	ptr = memchr(oldptr, '.', endptr - oldptr);
	ptr = (ptr)? ptr : endptr;
	if (FileExt)
	{
	   FileExt->Length = (endptr - ptr) - 1;
	   FileExt->Data = ptr + 1;
	}
	if (FileTitle)
	{
	   FileTitle->Length = ptr - oldptr;
	   FileTitle->Data = oldptr;
	}
	if (FileName)
	{
	   FileName->Length = endptr - oldptr;
	   FileName->Data = oldptr;
	}
	if (FileDir)
	{
	   FileDir->Length = oldptr - FilePath.Data; 
	   FileDir->Data = FilePath.Data;
	}
	return ISL_OK;
}

#ifndef VMS
#pragma auto_inline(off)
#endif
HMODULE
isl_LoadLibraryEx(
    LPCSTR lpLibFileName,
    HANDLE hFile,
    DWORD dwFlags
    )
{ 
#ifdef COMPAQ_FIX
	HMODULE hLib;
#endif /* COMPAQ_FIX */
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x0a);
#ifdef COMPAQ_FIX
	if (port_LoadLibrary(lpLibFileName, &hLib) != CSSM_OK)
		return NULL;
	return hLib;
#else  /* COMPAQ_FIX */
	return LoadLibraryEx(lpLibFileName, hFile, dwFlags);
#endif /* COMPAQ_FIX */
}
#ifndef VMS
#pragma auto_inline(on)

#pragma auto_inline(off)
#endif
BOOL
isl_FreeLibrary(
    HMODULE hLibModule
    )
{
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x0b);
#ifdef COMPAQ_FIX
	if (port_FreeLibrary(hLibModule) != CSSM_OK)
		return FALSE;
	return TRUE;
#else  /* COMPAQ_FIX */
	return FreeLibrary(hLibModule);
#endif /* COMPAQ_FIX */
}
#ifndef VMS
#pragma auto_inline(on)

#pragma auto_inline(off)
#endif
HMODULE
isl_GetModuleHandle(
    LPCSTR lpModuleName
    )
{
#ifdef COMPAQ_FIX
	HMODULE hModule;
#endif /* COMPAQ_FIX */
    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG,0x0c);
#ifdef COMPAQ_FIX
	if (port_GetModuleHandle(lpModuleName, &hModule) != CSSM_OK)
		return NULL;
	return hModule;
#else  /* COMPAQ_FIX */
	return GetModuleHandle(lpModuleName);
#endif /* COMPAQ_FIX */
}
#ifndef VMS
#pragma auto_inline(on)
#endif

#ifndef EISL
uint32 isl_GetKeyModulus(uint8* pKeyData, uint32 keyDataLength)
{
#define	MAX_KEY_PARTS 2
	uint32 outDataCount;
	BER_PARSED_ITEM berData[MAX_KEY_PARTS];
	const uint8 *pContent;

	pKeyData = (uint8 *)BER_ExpandItem(pKeyData, keyDataLength,berData);
	if ((pKeyData != NULL) && (*berData[0].Tag == BER_CONSTRUCTED_SEQUENCE)) {
		outDataCount = BER_ExpandSet(berData[0].Content, berData[0].ContentLength,
											  MAX_KEY_PARTS, 0, NULL, berData);
		
		//skip the leading zeros
		if (berData[0].ContentLength) {
			pContent = berData[0].Content;
			while (*pContent == 0) {
				berData[0].ContentLength--;
				pContent++;
			}
			return berData[0].ContentLength * 8;
		}
	}
	return 0;
}
#endif

void isl_InitHeaderInfo (ISL_HEADER_INFO_PTR pHeaderInfo)

{
	pHeaderInfo->MajorVersion = MANIFEST_MAJOR;
	pHeaderInfo->MinorVersion = MANIFEST_MINOR;
	pHeaderInfo->Attribute = NULL;
	pHeaderInfo->AttributeCount = 0;
}


ISL_STATUS
isl_LoadFile(
	ISL_ARCHIVE_CONTEXT_PTR	pMem,
	ISL_CONST_DATA		Filename,
	ISL_CONST_DATA_PTR	Filedata,
	time_t				*LastModTime)
{
	char				NTfilename[MAX_PATH+1];
	FILE *fi = NULL;
	struct _stat st;
    char                flags[3];

    NAMEPROC( ISL_DSCM_TAG, ISL_EISL_TAG, ISL_UTIL_TAG, 0x10);

	if (!Filedata)
		return ISL_FAIL;

	/* Null terminate the file name and load the archive into memory */
	memcpy(NTfilename, Filename.Data, Filename.Length);
	NTfilename[Filename.Length] = '\0';

	/* open it for read */
    flags[0] = 'r';
    flags[1] = 'b';
    flags[2] = '\0';
	fi = fopen(NTfilename,flags);
	if (!fi){
		return ISL_FAIL;
	}
	/* stat the file and find out how much memory we need */
	if ((_stat(NTfilename,&st)) != 0){
		return ISL_FAIL;
	}
	Filedata->Length = st.st_size;
	if (LastModTime)
		*LastModTime = st.st_mtime;
	/* allocate enough memory to hold the image */
	Filedata->Data = isl_AllocateMemory(pMem, Filedata->Length);
	if (! Filedata->Data){
		REPORT_ALLOCATE_MEMORY_ERROR(pMem, Filedata->Length);
		return ISL_FAIL;
	}
	/* read in the file */
	if ((fread((void*)Filedata->Data,1,Filedata->Length,fi)) != Filedata->Length){
		isl_FreeMemory(pMem, (void*)Filedata->Data);
		fclose(fi);
		return ISL_FAIL;
	}
	/* otherwise, it worked */
	fclose(fi);
	return ISL_OK;
}

/*
 * This function compares the two strings. This function returns TRUE even if one
 * of the strings does not include the NULL character.
 */
BOOL
is_equal (ISL_CONST_DATA Name, ISL_CONST_DATA String)

{
	if (String.Length == Name.Length) {
		if (!memcmp(Name.Data, String.Data, String.Length))
			return TRUE;
	} else if (String.Length == Name.Length-1) {
		if ((Name.Data[Name.Length] == '\0') && (!memcmp(Name.Data, String.Data, String.Length)))
			return TRUE;
	} else if (Name.Length == String.Length-1) {
		if ((String.Data[String.Length] == '\0') && (!memcmp(Name.Data, String.Data, Name.Length)))
		return TRUE;
	}

	return FALSE;
}
