/*-----------------------------------------------------------------------
 * File: ISLTYPE.H
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

#ifndef _ISLTYPE_H
#define _ISLTYPE_H

#include <stdlib.h>
#include <stdio.h>


#ifdef WIN32
#pragma warning(disable:4201 4514 4214 4115) 
#include <windows.h>
#pragma warning(disable:4201 4514 4214 4115) 
#endif

/* Basic Types */
#ifdef VMS
#include <ints.h>
#else
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef short sint16;
typedef unsigned int uint32;
typedef int sint32;
#endif

/*
**	public data structures
*/
typedef long	ISL_ERROR_CODE;
typedef size_t	ISL_SIZE;

typedef void (*ISL_FUNCTION_PTR)(void);

typedef struct isl_data {
	uint32 Length;
	uint8  *Data;
} ISL_DATA, *ISL_DATA_PTR;

typedef struct isl_const_data {
	uint32 Length;
	const uint8 *Data;
} ISL_CONST_DATA, *ISL_CONST_DATA_PTR;

typedef enum isl_status {
    ISL_OK = 0,
    ISL_FAIL = -1
} ISL_STATUS;

/* codes for class of service */
typedef enum {
	ISL_ServiceGetData				= 0,
	ISL_ServiceDigest				= 2,
	ISL_ServiceSignVerify			= 3,
	ISL_ServiceParseCertificate		= 4,
	ISL_ServiceFindVerify			= 5
} ISL_SERVICE_CLASS;
/*
**	class structure
**
**	This is probably what the method vector should be (perhaps next API?).
**	For now, we'll just add a pointer to the class structure in each method 
**	vector amd live with it.
**	Notable additions are the ability to define class variables in the
**  the class context.  The super link may be useful for inheritance. 
*/
typedef struct isl_class {
	void *ClassMethods;					/* pointer to class methods */
	void *ClassProtectedMethods;		/* private methods */
	void *ClassContext;					/* pointer to class variables */
	struct isl_class *SuperClass;		/* optional in case we want to use inheritance */
	char *ClassName;					/* optional human-readable class name */
	char **MethodNames;					/* optional method name vector */
} ISL_CLASS, *ISL_CLASS_PTR;
/*
**	memory allocation service class methods
*/
typedef void ISL_MEMORY_CONTEXT;

typedef void *(*ISL_MALLOC)(		/* returns address of allocated memory/NULL */
	ISL_MEMORY_CONTEXT *Context, 	/* context for memory manager */
	ISL_SIZE Size);

typedef void (*ISL_FREE)(
	ISL_MEMORY_CONTEXT *Context, 	/* context for memory manager */
	void *Memory);					/* address of memory to free */

typedef struct isl_memory_methods{
	/* class methods */
	ISL_SIZE (*SizeofObject)();	/* returns sizeof object */
	/* object methods */
	ISL_STATUS (*Initialize)(
		ISL_MEMORY_CONTEXT *MemoryContext);
	ISL_MALLOC Malloc;
	ISL_FREE Free;
	void (*Recycle)(						/* destructor for memory management object */
		ISL_MEMORY_CONTEXT *Context);
} ISL_MEMORY_METHODS;

typedef struct isl_file_methods{
    FILE * (*fopen)(const char *,
        const char *);
    size_t (*fread)(void *, 
        size_t, 
        size_t, 
        FILE *);
    int (*fclose)(FILE *);
} ISL_FILE_METHODS;
 
typedef struct isl_service_methods{
	ISL_MEMORY_CONTEXT *MemoryContext;	/* memory for IntegrityContext */	
	ISL_MEMORY_METHODS *MemoryMethods;	/* malloc, free methods */
    ISL_FILE_METHODS   *FileMethods;
} ISL_SERVICE_METHODS;



/* opaque Types */
typedef struct isl_signer_context			  	*ISL_VERIFIED_SIGNATURE_ROOT_PTR;
typedef struct isl_verified_certificate_chain	*ISL_VERIFIED_CERTIFICATE_CHAIN_PTR;
typedef struct isl_certificate					*ISL_VERIFIED_CERTIFICATE_PTR;
typedef struct isl_manifest_section				*ISL_MANIFEST_SECTION_PTR;
typedef struct isl_verified_module				*ISL_VERIFIED_MODULE_PTR;
typedef struct isl_loaded_module				*ISL_LOADED_MODULE_PTR;
typedef struct isl_verified_module				*ISL_MODULE_PTR;
typedef struct isl_iterator						*ISL_ITERATOR_PTR;
typedef struct isl_signer_context				*ISL_SIGNER_CONTEXT_PTR;
typedef ISL_CONST_DATA							ISL_KEY_BLOCK;
typedef struct isl_certificate					ISL_CERTIFICATE, *ISL_CERTIFICATE_PTR;
typedef struct isl_archive_config				ISL_ARCHIVE_CONFIG;
typedef struct isl_archive_context 				ISL_ARCHIVE_CONTEXT;
typedef struct isl_signature_config				ISL_SIGNATURE_CONFIG;
typedef struct isl_signature_context 			ISL_SIGNATURE_CONTEXT;
typedef struct isl_signature_list	 			ISL_SIGNATURE_LIST;
typedef struct isl_get_data_service_context		ISL_GET_DATA_SERVICE_CONTEXT;
typedef struct isl_digest_service_context 		ISL_DIGEST_SERVICE_CONTEXT;
typedef struct isl_sign_service_context			ISL_SIGN_SERVICE_CONTEXT;
typedef struct isl_digest_methods				ISL_DIGEST_METHODS;
typedef struct isl_sign_verify_methods			ISL_SIGN_VERIFY_METHODS;
typedef struct isl_certificate_methods			ISL_CERTIFICATE_METHODS;
typedef struct isl_service_methods				ISL_SERVICE_METHODS;
typedef struct isl_sig_section					ISL_SIG_SECTION;
typedef struct isl_archive_methods				ISL_ARCHIVE_METHODS;
typedef struct isl_sig_section_list 			ISL_SIG_SECTION_LIST;
typedef struct isl_archive_context			    *ISL_ARCHIVE_CONTEXT_PTR;
typedef struct isl_archive_config			    *ISL_ARCHIVE_CONFIG_PTR;
typedef struct isl_archive_methods			    *ISL_ARCHIVE_METHODS_PTR;
typedef struct isl_signature_context		    *ISL_SIGNATURE_CONTEXT_PTR;
typedef struct isl_signature_list	 			*ISL_SIGNATURE_LIST_PTR;
typedef struct isl_signer_context			    *ISL_SIGNATURE_ROOT_PTR;
typedef struct isl_signature_methods		    *ISL_SIGNATURE_METHODS_PTR;	
typedef struct isl_signature_config			    *ISL_SIGNATURE_CONFIG_PTR;
typedef struct isl_manifest					    *ISL_MANIFEST_PTR;
typedef struct isl_manifest_section			    *ISL_MANIFEST_SECTION_PTR;
typedef struct isl_manifest_section_methods     *ISL_MANIFEST_SECTION_METHODS_PTR;
typedef struct isl_alg_info					    *ISL_ALG_INFO_PTR;
typedef struct isl_hash_info				    *ISL_HASH_INFO_PTR;
typedef struct isl_sig_section_list			    *ISL_SIG_SECTION_LIST_PTR;
typedef struct isl_sig_section				    *ISL_SIG_SECTION_PTR;
typedef struct isl_sig_section_methods		    *ISL_SIG_SECTION_METHODS_PTR;	
typedef struct isl_siginfo					    *ISL_SIGINFO_PTR;
typedef struct isl_name_value				    *ISL_NAME_VALUE_PTR;
typedef struct isl_signers					    *ISL_SIGNERS_PTR;
typedef struct isl_memory_buffer			    *ISL_MEMORY_BUFFER_PTR;
typedef struct isl_digest_methods			    *ISL_DIGEST_METHODS_PTR;
typedef struct isl_get_data_methods			    *ISL_GET_DATA_METHODS_PTR;
typedef struct isl_sign_verify_methods		    *ISL_SIGN_VERIFY_METHODS_PTR;
typedef struct isl_algorithm_list_item		    *ISL_ALGORITHM_LIST_ITEM_PTR;
typedef struct isl_archive_config_methods	    *ISL_ARCHIVE_CONFIG_METHODS_PTR;
typedef struct isl_memory_methods			    *ISL_MEMORY_METHODS_PTR;
typedef struct isl_signature_config_methods     *ISL_SIGNATURE_CONFIG_METHODS_PTR;
typedef struct isl_certificate_list			    *ISL_CERTIFICATE_LIST_PTR;
typedef struct isl_error_context				*ISL_ERROR_CONTEXT;
typedef ISL_SIGNATURE_CONTEXT				    ISL_SIGNATURE_ROOT;

/* Do not edit or remove the following string. 
 * It should be expanded at check-in by version control.
 */
static const char sccs_id_INC_ISLTYPE_H[] = { "Fix ME" }; 

#endif
