// 
// @DEC_COPYRIGHT@
//
// HISTORY
// $Log: cdsa_sign.cpp,v $
// Revision 1.1.2.4  2000/08/31  18:07:52  Thomas_Woodburn
// 	Merged in Intel CDSA Version 2.0 Release 3.11.
//
// Revision 1.1.2.3  2000/08/07  14:43:34  Thomas_Woodburn
// 	Merged in Intel CDSA Version 2.0 Release 3.1.
//
// Revision 1.1.2.2  2000/06/28  21:00:29  Thomas_Woodburn
// 	Initial port to Tru64 UNIX.
//
// Revision 1.1.2.1  2000/06/28  20:26:59  Thomas_Woodburn
// 	Created from Intel CDSA Version 2.0 Release 3.0.
//
// $EndLog$
// 
/*
** #pragma ident "@(#)$RCSfile: cdsa_sign.cpp,v $ $Revision: 1.1.2.4 $ (DEC) $Date: 2000/08/31 18:07:52 $"
*/
/*-----------------------------------------------------------------------
 * File: CDSA_SIGN.CPP
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

 
// standard includes
#include <iostream.h>
#include <string.h>
#ifdef WIN32
#include <direct.h>
#endif
#include <stdio.h>
#include <sys/stat.h>

// workflow includes
#include "workflow.h"

#include "cssmport.h"

// local definitions/globals
#define MYFAIL 0
#define MYPASS 1

#if defined(WIN32)
static const char *dllExt = ".dll";
static const char *exeExt = ".exe";
#elif defined(UNIX)
static const char *dllExt = ".so";
static const char *exeExt = "";
#elif defined (VMS)
static const char *dllExt = ".exe";
static const char *exeExt = ".exe";
#endif


int SignObjects(CObjectList *list, 
                char *SignedManifestName,
                char *DllPath,
                char *CertPath,
                char *PasswordStr,
                WF_CERTGROUP *CertGroup,
                CAttributeList *SignerAtt)
{
    FILE* fn;
    WF_CONST_DATA Password, SignersCertBytes, SignerName;
    struct _stat st;
    char Tmp[MAX_PATH], ManifestPath[MAX_PATH];

    // read Signer's certificate from the file
	// open it for read
	fn = fopen(CertPath,"rb");
	if (!fn)
		return MYFAIL;
	// stat the file and find out how much memory we need
	if ((_stat(CertPath,&st)) != 0)
		return MYFAIL;
	// allocate enough memory to hold the certificate
	SignersCertBytes.Length = st.st_size;
	SignersCertBytes.Data = (unsigned char*)malloc(SignersCertBytes.Length);
	if (! SignersCertBytes.Data)
        return MYFAIL;
	// read the certificate data
	if ((fread((uint8 *)SignersCertBytes.Data,SignersCertBytes.Length,1,fn)) != 1)
        {
	   free((void*)SignersCertBytes.Data);
 	   fclose(fn);
	   return MYFAIL;
	}
	fclose(fn);

	// setup password and signer name data
        Password.Length = strlen(PasswordStr);
	Password.Data = (unsigned char*)PasswordStr;
        SignerName.Length = strlen(SignedManifestName);
        SignerName.Data = (unsigned char*)SignedManifestName;

#ifdef VMS
        sprintf( ManifestPath, "%s%c%s.esw", getenv( "CDSA_SIGN" ), DIRECTORY_SEPARATOR, SignedManifestName);
#else     
	// find out where we are
	getcwd(Tmp, MAX_PATH);
    
	// create manifest
    sprintf( ManifestPath, "%s%c%s%c%s.esw", Tmp, DIRECTORY_SEPARATOR,
	     DllPath, DIRECTORY_SEPARATOR, SignedManifestName);
#endif
    WF_CONST_DATA Name;
    Name.Data = (unsigned char*)SignedManifestName;
    Name.Length = strlen(SignedManifestName);
    CManifest manifest;
    if(manifest.Init(list,ManifestPath)!=WF_OK)
        return MYFAIL;
    if(manifest.SignAllSections(&SignersCertBytes, &Password, &Name, CertGroup, SignerAtt)!=ISL_OK)
        return MYFAIL;
    if(manifest.Commit()!=WF_OK)
        return MYFAIL;

    free((void*)SignersCertBytes.Data);
	
    return MYPASS;
}

int SignManifest(char *ManifestPath,
                 char *CertPath,
                 char *PasswordStr,
                 WF_CERTGROUP *CertGroup,
                 CAttributeList *SignerAtt)
{
/* prep for signing */
    FILE* fn;
    WF_CONST_DATA Password, SignersCertBytes, SignerName;
	struct _stat st;
    
    // read Signer's certificate from the file
	// open it for read
	fn = fopen(CertPath,"rb");
	if (!fn)
		return MYFAIL;
	// stat the file and find out how much memory we need
	if ((_stat(CertPath,&st)) != 0)
		return MYFAIL;
	// allocate enough memory to hold the certificate
	SignersCertBytes.Length = st.st_size;
	SignersCertBytes.Data = (unsigned char*)malloc(SignersCertBytes.Length);
	if (! SignersCertBytes.Data)
        return MYFAIL;
	// read the certificate data
	if ((fread((uint8 *)SignersCertBytes.Data,SignersCertBytes.Length,1,fn)) != 1)
    {
		free((void*)SignersCertBytes.Data);
		fclose(fn);
		return MYFAIL;
	}
	fclose(fn);

	// setup password and signer name data
    Password.Length = strlen(PasswordStr);
	Password.Data = (unsigned char*)PasswordStr;
    SignerName.Length = strlen("Export Agent");
    SignerName.Data = (unsigned char*)"Export Agent";

/* construct manifest */
    CManifest manifest;
/* init manifest from path */
    if(manifest.Init(ManifestPath)!=WF_OK)
        return MYFAIL;
/* sign manifest with new signer */
    if(manifest.SignAllSections(&SignersCertBytes, &Password, &SignerName, CertGroup, SignerAtt)!=ISL_OK)
        return MYFAIL;
/* commit a record to disk */
    if(manifest.Commit()!=WF_OK)
        return MYFAIL;

    free((void*)SignersCertBytes.Data);
	
    return MYPASS;

}


void main(int argc, char* argv[])
{
    WF_CONST_DATA Value;
    CAttributeList SignerAtt;
    CAttributeList *pSignerAtt = NULL;

    char filename[MAX_PATH], filepath[MAX_PATH];
    char Buffer[WF_MAX_STRING], RtnURL[WF_MAX_STRING];

    memset( Buffer, 0, WF_MAX_STRING);
    memset( RtnURL, 0, WF_MAX_STRING);
    
    if( argc !=9 && argc != 12 )
    {
        printf( "%s\n%s\n%s%s%s%s\n\n%s\n%s%s%s\n",
                "Usage:",
                "Integrity signing:",
                "cdsa_sign <dll name> <dll path> ",
                "<(E)EMM/(A)ADDIN/(C)CSSM/(X)APP EXE/(D)APP DLL/(G)GENERIC DLL>/(I)Installer", 
                "<signer_cert_path> <password> <cert chain filepath> CDSA_GUID CDSA_ACCESS_TAG( for Installers only- XX for others)", 
                "CDSA_PVC_API(XX if no value) CDSA_PVC_SPI(XX if no value) CDSA_PRIV(XX if no value)",
                "Export signing (requires integrity signature first):"
                "cdsa_sign <manifest path> ", 
                "<signer_cert_path> <password> <cert chain filepath> ",
                "<CDSA_USEE> <CDSA_PRIV>(XX if no value) CDSA_PVC_API(XX if no value) CDSA_PVC_SPI(XX if no value");
        exit(1);
    }

/* USEE */
    if( argc == 9)
    {
        WF_CONST_DATA Attribute = {strlen("CDSA_USEE"),(unsigned char*)"CDSA_USEE"};
        WF_CONST_DATA Value = {strlen(argv[5]),(unsigned char*)argv[5]};
        CAttributeListNode a1(&Attribute, &Value);
        SignerAtt.AddNode( &a1 );
        if( strcmpi( argv[6], "XX" ))
        {
            WF_CONST_DATA Attribute1 = {strlen("CDSA_PRIV"),(unsigned char*)"CDSA_PRIV"};
            WF_CONST_DATA Value1 = {strlen(argv[6]),(unsigned char*)argv[6]};
            CAttributeListNode a2(&Attribute1, &Value1);
            SignerAtt.AddNode( &a2 );
        }
       
        if( strcmp( argv[7], "XX"))
        {
			WF_CONST_DATA pvcapiAttribute = {strlen("CDSA_PVC_API"),(unsigned char*)"CDSA_PVC_API"};
			WF_CONST_DATA pvcapiValue = {strlen(argv[7]),(unsigned char*)argv[7]};
			CAttributeListNode a1(&pvcapiAttribute, &pvcapiValue);
			SignerAtt.AddNode( &a1 );
        }                
        if( strcmp( argv[8], "XX"))
        {
				WF_CONST_DATA pvcspiAttribute = {strlen("CDSA_PVC_SPI"),(unsigned char*)"CDSA_PVC_SPI"};
				WF_CONST_DATA pvcspiValue = {strlen(argv[8]),(unsigned char*)argv[8]};
				CAttributeListNode a2(&pvcspiAttribute, &pvcspiValue);
				SignerAtt.AddNode( &a2 );
        }
        pSignerAtt = &SignerAtt;
    }   
    else if (argc == 12)
    {
        WF_BOOL bpvcapi = WF_FALSE;
        WF_BOOL bpvcspi = WF_FALSE;
        WF_BOOL bpriv = WF_FALSE;
       
        if( strcmp( argv[9], "XX"))
        {
			WF_CONST_DATA pvcapiAttribute = {strlen("CDSA_PVC_API"),(unsigned char*)"CDSA_PVC_API"};
			WF_CONST_DATA pvcapiValue = {strlen(argv[9]),(unsigned char*)argv[9]};
			CAttributeListNode a1(&pvcapiAttribute, &pvcapiValue);
			SignerAtt.AddNode( &a1 );
			bpvcapi = WF_TRUE;
        }                
        if( strcmp( argv[10], "XX"))
        {
				WF_CONST_DATA pvcspiAttribute = {strlen("CDSA_PVC_SPI"),(unsigned char*)"CDSA_PVC_SPI"};
				WF_CONST_DATA pvcspiValue = {strlen(argv[10]),(unsigned char*)argv[10]};
				CAttributeListNode a2(&pvcspiAttribute, &pvcspiValue);
				SignerAtt.AddNode( &a2 );
				bpvcspi = WF_TRUE;
        }
        if( strcmp( argv[11], "XX"))
        {
            WF_CONST_DATA privAttribute = {strlen("CDSA_PRIV"),(unsigned char*)"CDSA_PRIV"};
            WF_CONST_DATA privValue = {strlen(argv[11]),(unsigned char*)argv[11]};
            CAttributeListNode a3(&privAttribute, &privValue);
            SignerAtt.AddNode( &a3 );
            bpriv = WF_TRUE;
        }
        if( bpvcapi  || bpvcspi || bpriv )
            pSignerAtt = &SignerAtt;
        else
            pSignerAtt = NULL;
        
    }
/* CERT CHAIN */
    int N;
    FILE* fp2;
    
    if( argc > 9 )
        fp2 = fopen(argv[6], "rb");
    else
        fp2 = fopen(argv[4], "rb");

    fscanf(fp2, "%d", &N);

    WF_DATA Certs[10];
    WF_CERTGROUP CertGroup;

    for(int i=0; i < N && i < 10; i++)
    {
        struct _stat st;
        char CertPath[MAX_PATH];
        memset(CertPath, 0, MAX_PATH);
        fscanf(fp2, "%s", CertPath);
        _stat(CertPath,&st);
        Certs[i].Data = (uint8*)malloc(st.st_size);
        Certs[i].Length = st.st_size;
        FILE *fp3 = fopen(CertPath, "rb");
        fread(Certs[i].Data, 
              Certs[i].Length,
              1,
              fp3);
        fclose(fp3);
    }
    fclose(fp2);

    CertGroup.NumCerts = i;    
    CertGroup.GroupList.CertList = (CSSM_DATA*)Certs;
    CertGroup.Reserved = 0;
	if( argc != 9 )
	{
		// create object lists
#ifdef COMPAQ_FIX
		sprintf( filepath, "%s%c%s", argv[2], DIRECTORY_SEPARATOR, filename );
#else  /* COMPAQ_FIX */
		sprintf( filepath, "%s\\%s", argv[2], filename );
#endif /* COMPAQ_FIX */

		if( ((argv[3][0] == 'A')||(argv[3][0] == 'a')))
		{
			WF_CONST_DATA Attribute;
			sprintf( filename, "%s%s", argv[1], dllExt );
			Attribute.Length = strlen ( "CDSA_MODULE");
			Attribute.Data = (unsigned char*)"CDSA_MODULE";
			Value.Length = strlen( "ADDIN");
			Value.Data = ( unsigned char*)"ADDIN";
			CAttributeListNode a1(&Attribute, &Value);

			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );
			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if( SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
			   printf("Failed to sign manifest\n");
		}
		else if( ((argv[3][0] == 'G')||(argv[3][0] == 'g')))
		{
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			sprintf( filename, "%s%s", argv[1], dllExt );
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a1(&mfAttribute, &mfAttribValue);
			CAttributeList al1( &a1 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if(	SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}
		else if( ((argv[3][0] == 'E')||(argv[3][0] == 'e')))
		{
			sprintf( filename, "%s%s", argv[1], dllExt );
			WF_CONST_DATA Attribute;
			Attribute.Length = strlen ( "CDSA_MODULE");
			Attribute.Data = (unsigned char*)"CDSA_MODULE";
			Value.Length = strlen( "EMM");
			Value.Data = ( unsigned char*)"EMM";
			CAttributeListNode a1(&Attribute, &Value);
			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if( SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}
		else if(  ((argv[3][0] == 'X')||(argv[3][0] == 'x')))
		{
			sprintf( filename, "%s%s", argv[1], exeExt );
			WF_CONST_DATA Attribute;
			Attribute.Length = strlen ( "CDSA_MODULE");
			Attribute.Data = (unsigned char*)"CDSA_MODULE";
			Value.Length = strlen( "APP");
			Value.Data = ( unsigned char*)"APP";
			CAttributeListNode a1(&Attribute, &Value);
			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if( SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}
		else if(  ((argv[3][0] == 'I')||(argv[3][0] == 'i')))
		{
			sprintf( filename, "%s%s", argv[1], exeExt );
			WF_CONST_DATA Attribute;
			Attribute.Length = strlen ( "CDSA_MODULE");
			Attribute.Data = (unsigned char*)"CDSA_MODULE";
			Value.Length = strlen( "APP");
			Value.Data = ( unsigned char*)"APP";
			CAttributeListNode a1(&Attribute, &Value);
			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );
			WF_CONST_DATA Attribute1;
			WF_CONST_DATA AttribValue1;
			Attribute1.Length = strlen( "CDSA_ACCESS_TAG");
			Attribute1.Data = (unsigned char*)"CDSA_ACCESS_TAG";
			AttribValue1.Length = strlen( argv[8]);
			AttribValue1.Data = (unsigned char*)argv[8];
			CAttributeListNode a3(&Attribute1, &AttribValue1);
			al1.AddNode( &a3 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if( SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}

		else if( ((argv[3][0] == 'D')||(argv[3][0] == 'd')))
		{
			sprintf( filename, "%s%s", argv[1], dllExt );
			WF_CONST_DATA Attribute;
			Attribute.Length = strlen ( "CDSA_MODULE");
			Attribute.Data = (unsigned char*)"CDSA_MODULE";
			Value.Length = strlen( "APP");
			Value.Data = ( unsigned char*)"APP";
			CAttributeListNode a1(&Attribute, &Value);
			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if( SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}
		else /* CSSM */
		{
			sprintf( filename, "%s%s", argv[1], dllExt );
			WF_CONST_DATA Attribute = {strlen("CDSA_MODULE"),(unsigned char*)"CDSA_MODULE"};
			WF_CONST_DATA Value = {strlen("CSSM"),(unsigned char*)"CSSM"};
			CAttributeListNode a1(&Attribute, &Value);
			CAttributeList al1( &a1 );
			WF_CONST_DATA mfAttribute;
			WF_CONST_DATA mfAttribValue;
			mfAttribute.Length = strlen( "CDSA_GUID");
			mfAttribute.Data = (unsigned char*)"CDSA_GUID";
			mfAttribValue.Length = strlen( argv[7]);
			mfAttribValue.Data = (unsigned char*)argv[7];
			CAttributeListNode a2(&mfAttribute, &mfAttribValue);
			al1.AddNode( &a2 );

			CObjectRef objref( argv[1], RefExecutable, filepath, filename, &al1 );
			CObjectListNode n1( &objref );
			CObjectList list1( &n1 );
			if (SignObjects(&list1,argv[1], argv[2], argv[4], argv[5], &CertGroup, pSignerAtt) == MYPASS)
				printf("Signed Manifest\n");
			else
				printf("Failed to sign manifest\n");

		}
	}
    else
    {
        if( SignManifest( argv[1], argv[2], argv[3], &CertGroup, pSignerAtt ) == MYPASS)
	            printf("Signed Manifest\n");
			else
	            printf("Failed to sign manifest\n");

    }


}
