/*-----------------------------------------------------------------------
 * File: CL_PUBLIC.C
 *
 * Copyright (c) 1995-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */
/*
 * INTEL CONFIDENTIAL 
 * This file, software, or program is supplied under the terms of a 
 * license agreement or nondisclosure agreement with Intel Corporation 
 * and may not be copied or disclosed except in accordance with the 
 * terms of that agreement. This file, software, or program contains 
 * copyrighted material and/or trade secret information of Intel 
 * Corporation, and must be treated as such. Intel reserves all rights 
 * in this material, except as the license agreement or nondisclosure 
 * agreement specifically indicate. 
 */
/* 
 * WARNING: EXPORT RESTRICTED. 
 * This software listing contains cryptographic methods and technology. 
 * It is export restricted by the Office of Defense Trade Controls, United 
 * States Department of State and cannot be downloaded or otherwise 
 * exported or re-exported (i) into (or to a national or resident of) Cuba, 
 * Iraq, Libya, Yugoslavia, North Korea, Iran, Syria or any other country 
 * to which the US has embargoed goods; or (ii) to anyone on the US 
 * Treasury Department's list of Specially Designated Nationals or the US 
 * Commerce Department's Table of Denial Orders. By downloading or using 
 * this product, you are agreeing to the foregoing and you are representing 
 * and warranting that you are not located in, under the control of, or a 
 * national or resident of any such country or on any such list. 
 */


/*
 * cl_public.c
 * Routines related to the CL tests which may be used by other applications
 */

/* CSSM includes */
#include "cssm.h"
#include "guids.h"
#include "cssmtype.h"
#include "mds_util_api.h"

#include <assert.h>
#include <stdio.h>
#include <cssmMemTrack.h>

CSSM_RETURN 
CL_TEST_PUBLIC_CreateNewCert(	CSSM_CL_HANDLE CLHandle, 
                    CSSM_CSP_HANDLE CSPHandle,
					CSSM_FIELD_PTR field_ptr, 
					uint32 num_fields, 
                    CSSM_ALGORITHMS algid,
                    CSSM_KEY_PTR private_keyptr,
                    CSSM_ACCESS_CREDENTIALS *AccessCred,
                    CSSM_UPCALLS MemoryFunctions,
                    CSSM_DATA_PTR *new_cert)
{

	CSSM_CC_HANDLE		SignCCHandle = 0;
	CSSM_RETURN 		retVal = CSSM_OK;
    CSSM_DATA           newcerttemplate;

	*new_cert = NULL;
    newcerttemplate.Data = NULL;
    newcerttemplate.Length = 0;

    retVal = CSSM_CSP_CreateSignatureContext (CSPHandle, 
											algid, 
											AccessCred, 
											private_keyptr,
											&SignCCHandle );    
    
	if (CSSM_OK != retVal || SignCCHandle == 0) 
	{/* CreateSignatureContext failed */
		return retVal;
	}
    
    /* CertCreateTemplate/CertSign */
    retVal = CSSM_CL_CertCreateTemplate(CLHandle, num_fields, field_ptr,  &newcerttemplate);
    if (CSSM_OK != retVal)
    {
        if (NULL != newcerttemplate.Data) 
        {
            CSSM_FREE_WRAPPER(MemoryFunctions.free_func, CLHandle, newcerttemplate.Data);
        }
        CSSM_DeleteContext(SignCCHandle);    
        return retVal;
    }
    *new_cert = CSSM_MALLOC_WRAPPER(MemoryFunctions.malloc_func, CLHandle, sizeof(CSSM_DATA));
    if (NULL == *new_cert)
    {
        if (NULL != newcerttemplate.Data) 
        {
            CSSM_FREE_WRAPPER(MemoryFunctions.free_func, CLHandle, newcerttemplate.Data);
        }
        CSSM_DeleteContext(SignCCHandle);    
        return CSSMERR_CL_MEMORY_ERROR;
    }
    retVal = CSSM_CL_CertSign(CLHandle, SignCCHandle, &newcerttemplate, NULL, 0, *new_cert);
    
    CSSM_FREE_WRAPPER(MemoryFunctions.free_func, CLHandle, newcerttemplate.Data);
    CSSM_DeleteContext(SignCCHandle);    
    return retVal;
    
}
