/*-----------------------------------------------------------------------
 * File: SIGNERLIST.H
 *
 * Copyright (c) 1998-2000 Intel Corporation. All rights reserved.
 *-----------------------------------------------------------------------
 */

// SignerList.h
//
// This file contains the class definition for the CSignerList
//  and CSignerListNode classes. These classes implement a list
//  of signers which can contain a list of counter-signers.
//
//////////////////////////////////////////////////////////////////////

#ifndef SIGNERLIST_H
#define SIGNERLIST_H

// mrm includes
#include "mrm_defs.h"
#include "wf_type.h"

class DllExport CSignerList;

//////////////////////////////////////////////////////////////////////
// Class description: CSignerListNode  
//  Members:
//      mCounterSigners(public)      
//          pointer to a list of counter-signers
//      mNext(public)
//          pointer to the next node in the list
//      mSigBlock(public)
//          the signature block associated with the signing operation
//  Methods:
//      none
/////////////////////////////////////////////////////////////////////
class DllExport CSignerListNode  
{
public:
    // public vars
	CSignerList *mCounterSigners;
	CSignerListNode *mNext;
	WF_CONST_DATA mSigBlock;
	
	// constructors
	CSignerListNode(const CSignerListNode *node);
	CSignerListNode(WF_SIGNATURE_CONTEXT *Context);
	CSignerListNode();
	~CSignerListNode();
private:
    // private vars
};


//////////////////////////////////////////////////////////////////////
// Class description: CSignatureGraph  
//  Members:
//      mHead(public)      
//          pointer to the head node of the list
//      mTail(public)      
//          pointer to the tail node of the list
//      mNumberOfNodes(public)
//          count of the number of nodes in the list
//  Methods:
//      AddNode(public)
//          appends a node to the end of the list
/////////////////////////////////////////////////////////////////////
class CSignerList  
{
public:
    CSignerListNode *mHead;
	CSignerListNode *mTail;
	int mNumberOfNodes;

	// constructors
	CSignerList(const CSignerList *graph);
	CSignerList(CSignerListNode *node);
	CSignerList();
	~CSignerList();

	// public methods
	WF_RETURN AddNode( CSignerListNode *NodeToAdd );

};

#endif // !def SIGNERLIST_H
